<?xml version="1.0" encoding="UTF-8"?>
<!--
    Skeleton xsl html renderer. Generates skeleton HTML. Should be imported to any stylesheet generating HTML for the project,
    the importing stylesheet should then overide various templates to provide appropriate content for that page.

    All templates have the root node ("/") as the context node when they are called.
-->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:xg="http://www.hyfinity.com/xgate">
    <xsl:output method="xml" indent="yes" encoding="ISO-8859-1"/>
    <!--
        Generates the skeleton html for the screen, lays out the tables that make up the page calling appropriate templates to fill in page content.
    -->
    <xsl:template name="hyfinity_com_skin_html" match="/">
        <html>
            <head>
                <!-- Fill in the contents of the <head> html element -->
                <xsl:call-template name="head_html"/>
                <!-- Fill in the client side scripts -->
                <xsl:call-template name="client_scripts"/>
            </head>
            <xsl:call-template name="body"/>
        </html>
    </xsl:template>
    <xsl:template name="body">
        <body>
            <!-- Main Screen Sections -->
            <!-- Fill in the contents of the page header, i.e. logo -->
            <xsl:call-template name="header"/>
            <!-- Fill in the section title -->
            <xsl:call-template name="section_title_container"/>
            <!-- Fill in the content of the top menu -->
            <xsl:call-template name="top_menu"/>

            <!-- Fill in the main body -->
            <xsl:call-template name="main_body"/>



            <xsl:call-template name="additional_content"/>

        </body>
    </xsl:template>



    <!-- outputs any additional content that should be placed directly under the body tag -->
    <xsl:template name="additional_content">
        <div id="status_message">
        </div>

        <!-- create the hidden div to use for drag and drop -->
        <div id="xpathBox2" style="visibility : hidden; position:absolute; background:yellow;z-index:10;"></div>

        <!-- draggable dividers -->
        <div id="temp_divider" class="temp_divider" onselectstart="return false;"></div>

        <!-- create the hidden div to use for popup messages -->
        <div id="tipDiv" style="position:absolute; top:-100px; left:-300px; width : 300px; height : 100px; visibility:hidden;">
            <div class="messageTop" id="messageTop"></div>
            <iframe title="tipFrame" id="tipFrame" name="tipFrame" src="about:blank" style="width :296px; height : 100px;"></iframe>
            <div id="messageDiv" style="" onmouseover="inToolTip()" onmouseout="outToolTip()"></div>
            <div class="messageBottom" id="messageBottom"></div>
        </div>
        <div id="context_menu" onmousedown="event.cancelBubble = true;" onclick="hideContextMenu();">
            <a id="context_menu_copy_rule" href="#" onclick="menuCopyRule(); return false;">Copy Rule</a>
            <a id="context_menu_paste_rule" href="#" onclick="menuPasteRule(); return false;">Paste Rule</a>
            <div id="context_menu_rule_options">
                <hr/>
                <a id="context_menu_delete_rule" href="#" onclick="menuDeleteRule(); return false;">Delete Rule</a>
            </div>
            <div id="context_menu_result_options">
                <hr/>
                <a id="context_menu_delete_result" href="#" onclick="contextMenuTarget.rule.deleteResult(contextMenuTarget.displayID); return false;">Delete Action</a>
            </div>
            <div id="context_menu_condition_options">
                <hr/>
                <a id="context_menu_delete_condition" href="#" onclick="contextMenuTarget.parent.deleteChild(contextMenuTarget.displayID); return false;">Delete Condition</a>
            </div>

        </div>
    </xsl:template>

    <!--
        Place holder for client side scripts.
    -->
    <xsl:template name="client_scripts">

    </xsl:template>
    <!--
        Provides the contents for the HTML <head> element
    -->
    <xsl:template name="head_html">
        <title>
            <xsl:choose>
                <xsl:when test="/xg:message/xg:header/xg:attributes[xg:attribute[@xg:name = 'pattern_id' and . != ''] and xg:attribute[@xg:name = 'node_id' and . != '']]">
                    <!-- TODO: these are the ids of the pattern/node -  we should be showing the names!!!!!! -->
                    <xsl:value-of select="/xg:message/xg:header/xg:attributes/xg:attribute[@xg:name = 'pattern_id']"/>
                    <xsl:text>-</xsl:text>
                    <xsl:value-of select="/xg:message/xg:header/xg:attributes/xg:attribute[@xg:name = 'node_id']"/>
                    <xsl:text> - </xsl:text>
                </xsl:when>
                <xsl:when test="/xg:message/xg:header/xg:attributes/xg:attribute[@xg:name = 'document'] != ''">
                    <xsl:value-of select="/xg:message/xg:header/xg:attributes/xg:attribute[@xg:name = 'document']"/>
                    <xsl:text> - </xsl:text>
                </xsl:when>
            </xsl:choose>
            RuleMaker - Hyfinity® - SOA Development Simplified
        </title>
        <meta http-equiv="Content-Type" content="text/xml; charset=UTF-8"/>
        <link rel="stylesheet" type="text/css" href="css/hyfinity_com.css"/>
        <link rel="stylesheet" type="text/css" href="css/hyfinity_apps.css"/>
        <link rel="stylesheet" type="text/css" href="css/hyfinity_colours.css"/>
        <link rel="shortcut icon" href="images/favicon_rulemaker.ico" />
        <script language="javascript" src="../xde/js/webmakerChildWin.js">//WebMakerWinManager</script>
        <script type="text/javascript" src="../formmaker/js/dojo/dojo/dojo.js">//include dojo for ajax functions</script>
    </xsl:template>
    <!--
        Provides HTML content for the header at the top of the screen.
    -->
    <xsl:template name="header">
        <div id="tagline">
            <xsl:call-template name="tagline"/>
        </div>
        <xsl:call-template name="logo"/>
        <!--<a href="index.html"/>-->
    </xsl:template>
    <!--
        Need a separate template for the logo to enable the service call helper window to override display of the top strip
    -->
    <xsl:template name="logo">
        <div id="logo">
        </div>
    </xsl:template>

    <!--
        Provides HTML content for the tagline.
    -->
    <xsl:template name="tagline"/>
    <!--
        Provides HTML content for the menu at the top of the screen.
    -->
    <xsl:template name="top_menu">
        <div class="top_menu_primary">
        </div>
    </xsl:template>
    <!--
        Provides HTML content for the menu at the left of the screen.
    -->
    <xsl:template name="left_menu">
        <a href="">News &amp; Events</a>
        <a href="">Downloads</a>
        <a href="">About Us</a>
        <a href="">Contact Us</a>
    </xsl:template>
    <!--
        Provides HTML content fof the header in the working part of the screen.
    -->
    <xsl:template name="header_screen_context">
        <p>Header Screen Context Goes Here</p>
    </xsl:template>
    <!--
        Builds the main body of the page adding a header, footer and content area.
    -->
    <xsl:template name="main_body">
        <!-- Fill in the main body -->
        <xsl:call-template name="main_body_content"/>
    </xsl:template>
    <xsl:template name="main_body_content">
        <div id="main_body">
            <xsl:call-template name="section_body"/>
            <!--FIll in the contents of the page footer -->
            <xsl:call-template name="footer"/>
        </div>
    </xsl:template>
    <!--
        Builds the section heading within main body.
    -->
    <xsl:template name="section_title_container">
        <div id="section_title">
        </div>
    </xsl:template>
    <!--
        Builds the section heading within main body.
    -->
    <xsl:template name="section_body">
        <!--<div class="thin_divider"></div>
            <p>hfbeuvnewipcmw,ecfv</p>
            <div class="thick_divider"></div>-->
            An innovative range of products from hyfinity™ includes PxP. Every organisation has different processes, systems, vocabularies and cultures. This can very often be a significant barrier when participating in B2B solutions. PxP addresses these issues using innovative techniques based on web services, XML Patterns and Rules Engines.
PxP is a sophisticated set of Web Services for enabling Peer-to-Peer communications using XML technology. Horizontal and Vertical standards are emerging at rapid pace and the ability to incorporate, leverage and adapt to these standards requires technology that is designed for purpose.

    </xsl:template>
    <!--
        Provides HTML content for the footer.
    -->
    <xsl:template name="footer">
        <div id="footer">
            <xsl:text disable-output-escaping="yes"><![CDATA[Copyright © 2001-2010 Hyfinity® Limited. All Rights Reserved.]]></xsl:text>
        </div>
    </xsl:template>



    <!-- Start of templates relating to outputting XML content into a textarea.
         This should ensure that all data is escaped as needed for use inside a textarea
         To use: <xsl:apply-templates select="??" mode="copy-to-textarea"/>
         -->

    <xsl:template match="*" mode="copy-to-textarea">
        <xsl:copy>
            <xsl:apply-templates select="@*" mode="copy-to-textarea"/>
            <xsl:apply-templates select="node()" mode="copy-to-textarea"/>
        </xsl:copy>
    </xsl:template>

    <xsl:template match="*[local-name() = 'textarea']" mode="copy-to-textarea">
        <xsl:text disable-output-escaping="no">&lt;</xsl:text>
        <xsl:value-of select="name(.)"/>
        <xsl:for-each select="@*">
            <xsl:text> </xsl:text>
            <xsl:value-of select="name(.)"/>
            <xsl:text disable-output-escaping="no">="</xsl:text>
            <xsl:value-of select="."/>
            <xsl:text disable-output-escaping="no">"</xsl:text>
        </xsl:for-each>
        <xsl:text disable-output-escaping="no">&gt;</xsl:text>

        <xsl:apply-templates select="node()" mode="copy-to-textarea"/>

        <xsl:text disable-output-escaping="no">&lt;/</xsl:text>
        <xsl:value-of select="name(.)"/>
        <xsl:text disable-output-escaping="no">&gt;</xsl:text>

    </xsl:template>

    <xsl:template match="@*" mode="copy-to-textarea">
        <xsl:attribute name="{name()}" namespace="{namespace-uri()}">
            <xsl:call-template name="process-text-for-copy-to-textarea">
                <xsl:with-param name="text" select="."/>
            </xsl:call-template>
        </xsl:attribute>
    </xsl:template>

    <xsl:template match="text()" mode="copy-to-textarea" priority="10">
        <xsl:call-template name="process-text-for-copy-to-textarea">
            <xsl:with-param name="text" select="."/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="comment() | processing-instruction()" mode="copy-to-textarea">
        <xsl:copy-of select="."/>
    </xsl:template>

    <!-- when content is placed in a textarea, the browser seems to automatically
         process escaped characters (eg &lt;).  This means that trying to read the textarea
         contents as XML fails, because it is invalid.
         Therefore we need to double escape any special characters (eg &amp;lt;) to ensure
         the XML is still valid when read on the client -->
    <xsl:template name="process-text-for-copy-to-textarea">
        <xsl:param name="text"/>
        <xsl:call-template name="replace-string">
            <xsl:with-param name="text">
                <xsl:call-template name="replace-string">
                    <xsl:with-param name="text">
                        <xsl:call-template name="replace-string">
                            <xsl:with-param name="text">
                                <xsl:call-template name="replace-string">
                                    <xsl:with-param name="text" select="$text"/>
                                    <xsl:with-param name="replace">&amp;</xsl:with-param>
                                    <xsl:with-param name="with">&amp;amp;</xsl:with-param>
                                </xsl:call-template>
                            </xsl:with-param>
                            <xsl:with-param name="replace">&lt;</xsl:with-param>
                            <xsl:with-param name="with">&amp;lt;</xsl:with-param>
                        </xsl:call-template>
                    </xsl:with-param>
                    <xsl:with-param name="replace">&gt;</xsl:with-param>
                    <xsl:with-param name="with">&amp;gt;</xsl:with-param>
                </xsl:call-template>
            </xsl:with-param>
            <xsl:with-param name="replace">&quot;</xsl:with-param>
            <xsl:with-param name="with">&amp;quot;</xsl:with-param>
        </xsl:call-template>
    </xsl:template>

    <!-- replaces all occurrences of the given 'replace' string with the contents of the 'with'
         string in the 'text' string -->
    <xsl:template name="replace-string">
        <xsl:param name="text"/>
        <xsl:param name="replace"/>
        <xsl:param name="with"/>
        <xsl:choose>
            <xsl:when test="contains($text,$replace)">
                <xsl:value-of select="substring-before($text,$replace)"/>
                <xsl:value-of select="$with"/>
                <xsl:call-template name="replace-string">
                    <xsl:with-param name="text" select="substring-after($text,$replace)"/>
                    <xsl:with-param name="replace" select="$replace"/>
                    <xsl:with-param name="with" select="$with"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$text"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

</xsl:stylesheet>
