/*
 * DisplayUitls.js
 *
 * Utility functions for altering the display of HTML componenets from javascript
 *
 * Company: Hyfinity Ltd
 * Copyright (c) 2003
 */


/**
 * Generic function for toggling the display of a component.
 * @param targetComponent the HTML component (or its ID) to show/hide
 * @param method An optional string specifying the required visibility of the
 *               component ('show' or 'hide')
 */
function toggleComponent(targetComponent, method, animate)
{
    var component
    if (typeof(targetComponent) == 'string')
        component = document.getElementById(targetComponent);
    else
        component = targetComponent;
    if ((component != null) && (typeof(component) != 'undefined'))
    {
        if ((method == null) || (typeof(method) == 'undefined'))
        {
            if (getCurrentStyle(component, 'display') == 'none')
            {
                showComponent(component, animate);
            }
            else
            {
                hideComponent(component, animate);
            }
        }
        else
        {
            if (method == 'hide')
            {
                hideComponent(component, animate);
            }
            else
            {
                showComponent(component, animate);
            }
        }
    }
}

if (typeof(dojo) != 'undefined')
{
    try
    {
        dojo.require("dojo.fx");
    }
    catch (e)
    {}
}

/*generic utility function for hiding a specific component*/
function hideComponent(component, animate)
{
    if (component == null)
        return;

    if ((animate == true) && (typeof(dojo) != 'undefined'))
    {
        dojo.fx.wipeOut({node: component, duration: 250}).play();
    }
    else
    {
        if ((typeof(component._oldCSSDisplay) == 'undefined') && (component.style.display != 'none'))
            component._oldCSSDisplay = component.style.display;

        component.style.visibility = 'hidden';
        component.style.display = 'none';
    }
}
/*generic utility function for showing a particular component*/
function showComponent(component, animate)
{
    if (component == null)
        return;

    if ((animate == true) && (typeof(dojo) != 'undefined'))
    {
        dojo.fx.wipeIn({node: component, duration: 250}).play();
    }
    else
    {
        component.style.visibility = 'visible';
        if (component.style.display == 'none')
        {
            if (component._oldCSSDisplay != undefined)
                component.style.display= component._oldCSSDisplay;
            else
                component.style.display = 'block';
        }
    }
}

/* utility function for disabling a component
 * may want to extend this so children components are disabled as well, ie if a div component is passed in*/
function disableComponent(component, deep)
{
    if ((component == null) || (typeof(component) == 'undefined') || (typeof(component.tagName) == 'undefined'))
        return;

    if ((component.tagName == 'INPUT') || (component.tagName == 'TEXTAREA') || (component.tagName == 'SELECT'))
    {
        if (typeof(dojo) != 'undefined')
            dojo.addClass(component, 'readOnlyTxt');
        component.disabled=true;
    }
    else if (component.tagName == 'A')
    {
        if (component.getAttribute("_oldHref") == null)
        {
            component.setAttribute("_oldHref", component.href)
        }
        component.removeAttribute('href');
        component.disabled=true;
        if (typeof(dojo) != 'undefined')
            dojo.addClass(component, 'disabledText');
    }
    else
    {
        component.disabled=true;
        if (typeof(dojo) != 'undefined')
            dojo.addClass(component, 'disabledText');
    }

    if (deep && component.hasChildNodes())
    {
        for (var i = 0; i < component.childNodes.length; ++i)
        {
            disableComponent(component.childNodes.item(i), deep);
        }
    }
}

/* utility function for enabling a component*/
function enableComponent(component, deep)
{
    if ((component == null) || (typeof(component) == 'undefined') || (typeof(component.tagName) == 'undefined'))
        return;

    component.disabled=false;

    if ((component.tagName == 'INPUT') || (component.tagName == 'TEXTAREA') || (component.tagName == 'SELECT'))
    {
        if (typeof(dojo) != 'undefined')
            dojo.removeClass(component, 'readOnlyTxt');
    }
    else if (component.tagName == 'A')
    {
        if (component.getAttribute("_oldHref") != null)
        {
            var oldHref = component.getAttribute("_oldHref")
            component.removeAttribute("_oldHref");
            component.href = oldHref;
        }
        if (typeof(dojo) != 'undefined')
            dojo.removeClass(component, 'disabledText');
    }
    else
    {
        if (typeof(dojo) != 'undefined')
            dojo.removeClass(component, 'disabledText');
    }

    if (deep && component.hasChildNodes())
    {
        for (var i = 0; i < component.childNodes.length; ++i)
        {
            enableComponent(component.childNodes.item(i), deep);
        }
    }
}

/**
 * Returns the current value for the given style property on the given object
 * @param object The HTML component to get the style for.
 * @param property The name of the CSS property to return.
 */
function getCurrentStyle(object, property)
{
    var currentDisplay;
    if (window.getComputedStyle)
        //currentDisplay = document.defaultView.getComputedStyle(object,null).getPropertyValue(property);
        currentDisplay = document.defaultView.getComputedStyle(object,null)[property];
    else if (object.currentStyle)
        currentDisplay = object.currentStyle[property];
    return currentDisplay;
}

/**
 * Sets the specified property in the definition of the given CSS class/rule.
 * @param classname The name of the class to apply the rpoperty to. eg '.myclass'
 *          This is actually the CSS selector for the CSS rule, so could be more complicated,
 *          eg '#abd #def .xyz'
 * @param property The name of the CSS property to defined for this rule.
 * @param value The value to give the specified property.
 */
function setClassSetting(classname, property, value)
{
    var found = false;
    var ssheet;
    for (var si = 0; si < document.styleSheets.length; ++si)
    {
        ssheet = document.styleSheets[si];
        var rules;
        if(ssheet.cssRules)
            rules = ssheet.cssRules;
        else if (ssheet.rules)
            rules = ssheet.rules;

        for (var i = 0; i < rules.length; ++i)
        {
            if (rules.item(i).selectorText == classname)
            {
                found = true;
                rules.item(i).style[property] = value;
            }
        }
    }
    if (!found)
    {
        if (ssheet.addRule)
        {
            ssheet.addRule(classname, property + " : " + value + ";");
            return false;
        }
        else if (ssheet.insertRule)
        {
            ssheet.insertRule(classname + " {" + property + " : " + value + ";}", ssheet.cssRules.length);
        }
    }
}

/**
 * Moves the given object to the specified location
 * @param obj the HTML component to position - the 'top' and 'left' style properties
 *              of this object will be adjusted, so it should have absolute positioning
 * @param coords An object indicating where to position the component.
 *              This must have x and y properties.  For ease of use with teh getMouseCoords
 *              method, a 10pc offset will be applied to the x value.
 * @param keepInViewport (optional) a boolean value indicating whether the supplied coords
 *              should be adjusted to make sure that the object will be fully visible.
 *              If not provided, true will be assumed.
 */
function moveObjToCoords(obj, coords, keepInViewport)
{
    obj.style.top = coords.y + 'px';
    obj.style.left = (Number(coords.x) + Number(10)) + 'px';

    if ((typeof(keepInViewport) == 'undefined') || keepInViewport)
    {
        var newCoords = getComponentPosition(obj);
        var viewportSize = getViewportSize();

        //allow 10 px extra gap for shadows etc
        if ((newCoords.y + newCoords.height + 10) > viewportSize.height)
        {
            obj.style.top = (viewportSize.height - newCoords.height - 10) + 'px';
        }

        if ((newCoords.x + newCoords.width + 10) > viewportSize.width)
        {
            obj.style.left = (viewportSize.width - newCoords.width - 10) + 'px';
        }
    }
}

/* Functions for adjustable dividers. */
var dividerMode = 'stop'

var widthAdjustTarget;
var fixedPoint;
var directionMode;
var dividerMoveStartX;
var currentWidth;
var moveDivider;

function dividerPressed(target, mode, e)
{
    if (typeof(mode) == 'undefined')
    {
        mode = 'left';
    }

    var evt = (e) ? e : window.event;
    var evtSource = (evt.srcElement) ? evt.srcElement : evt.target;
    if (evtSource.nodeType == 3) // defeat Safari bug
        evtSource = evtSource.parentNode;

    dividerMode = 'move';
    widthAdjustTarget = target;
    directionMode = mode;
    currentWidth = -1;

    if (directionMode == 'left')
    {
        fixedPoint = getLeftPosition(widthAdjustTarget);
        /*if (getCurrentStyle(widthAdjustTarget, 'display') == 'none')
            fixedPoint = getLeftPosition(evtSource);*/
    }
    else
    {
        fixedPoint = getRightPosition(widthAdjustTarget);
        /*if (getCurrentStyle(widthAdjustTarget, 'display') == 'none')
            fixedPoint = getRightPosition(evtSource);*/
    }

    moveDivider = document.getElementById('temp_divider');

    moveDivider.style.top = getTopPosition(widthAdjustTarget) + 'px';
    //moveDivider.style.height = widthAdjustTarget.offsetHeight;
    moveDivider.style.height = evtSource.offsetHeight + 'px';

    var pos = getMouseCoords(evt);

    dividerMoveStartX = pos.x;

    moveDivider.style.left = (pos.x - 5) + 'px';
    moveDivider.style.display = 'block';

    if(evt.preventDefault){
        evt.preventDefault();
    }
    evt.cancelBubble=true;
    evt.returnValue = false;

}

function dividerMove(e, base)
{
    //alert(dividerMode);
    if (dividerMode == 'move')
    {
        var evt;
        var target;
        if (!e)
        {
            evt = window.event;
            target = evt.srcElement;
        }
        else //netscape
        {
            evt = e;
            target = evt.target;
        }
        var pos = getMouseCoords(evt, base);

        if(evt.preventDefault){
            evt.preventDefault();
        }
        evt.cancelBubble=true;
        evt.returnValue = false;

        if (pos.x != dividerMoveStartX)
        {
            if (directionMode == 'left')
            {
                currentWidth = pos.x - fixedPoint;
            }
            else
            {
                currentWidth = fixedPoint - pos.x;
            }
        }
        else
            currentWidth = -1;

        moveDivider.style.left = (pos.x - 5) + 'px';
    }
}

function dividerReleased()
{
    dividerMode = 'stop';

    if (currentWidth > 0)
    {
        showComponent(widthAdjustTarget);
        updateDividerSize();
    }

    if (moveDivider != null)
    {
        moveDivider.style.display = 'none';
    }
}

function updateDividerSize()
{
    //make sure we have a unit definition
    currentWidth = "" + currentWidth;
    if ((currentWidth.indexOf('%') != currentWidth.length - 1) && (currentWidth.indexOf('px') != currentWidth.length - 2))
        currentWidth = parseInt(currentWidth) + 'px';

    widthAdjustTarget.style.width = currentWidth;
    currentWidth = -1;
}

function dividerDoubleClicked(target)
{
    widthAdjustTarget = target;
    var tempWidth = parseInt(getComputedWidth(widthAdjustTarget));
    if (tempWidth <= 2)  //ie sometimes reports a width of 1 when hidden so allow some leway
    {
        if (widthAdjustTarget.hiddenDividerWidth)
            currentWidth = widthAdjustTarget.hiddenDividerWidth;
        else
            currentWidth = '200'; //fallback
        updateDividerSize();
        widthAdjustTarget.style.visibility = 'visible';
    }
    else
    {
        currentWidth = 0;
        updateDividerSize();
        widthAdjustTarget.hiddenDividerWidth = tempWidth;
        widthAdjustTarget.style.visibility = 'hidden';
    }
}

function dividerIsHidden(target)
{
    var tempWidth = parseInt(getComputedWidth(target));
    if (tempWidth <= 2)  //ie sometimes reports a width of 1 when hidden so allow some leway
        return true;
    else
        return false;
}

/**
 * Returns an object containing the coordinates of the mouse from the given event.
 * The returned object will have two properties, 'x' and 'y'
 */
function getMouseCoords(e, base)
{
    var coords= new Object();

    var evt
    if (!e)
    {
        evt = window.event;
    }
    else //netscape
    {
        evt = e;
    }

    //get the mouse coords
    if (evt.pageX || evt.pageY)
    {
        //Gecko based
        coords.x = evt.pageX;
        coords.y = evt.pageY;
    }
    else if (evt.clientX || evt.clientY)
    {
        coords.x = evt.clientX
        coords.y = evt.clientY
        if ((document.body) && (document.body.scrollLeft || document.body.scrollTop))
        {
            //IE 4, 5, and 6 (Non standards compliant mode)
            coords.x += document.body.scrollLeft;
            coords.y += document.body.scrollTop;
        }
        else if ((document.documentElement) &&
                 ((document.documentElement.scrollLeft) ||
                 (document.documentElement.scrollTop)))
        {
            //IE 6 (Standards compliant mode)
            coords.x += document.documentElement.scrollLeft;
            coords.y += document.documentElement.scrollTop;
        }
    }

    if ((typeof(base) != 'undefined') && (typeof(base.x) != 'undefined') && (typeof(base.y) != 'undefined'))
    {
        coords.x += base.x;
        coords.y += base.y;
    }

    return coords;

}

/**
 * Returns the location of the left of the given object.
 * If the screen parameter is set to true, then the returned value will be in screen coords.
 * ie it adjusts for scrollable regions, to allow comparison with mouse coordinates
 * returned from getMouseCoords method
 */
function getLeftPosition(obj, screen)
{
    return getLeft(obj, screen);
    /*var ol=obj.offsetLeft;
    while ((obj=obj.offsetParent) != null)
    {
        ol += obj.offsetLeft;
        if (screen)
        {
            if (obj.scrollLeft != 0)
            {
                ot -= obj.scrollLeft;
            }
        }
    }
    return ol;*/
}

function getRightPosition(obj, screen)
{
    return getLeftPosition(obj, screen) + obj.offsetWidth;
}

/**
 * Returns the location of the top of the given object.
 * If the screen parameter is set to true, then the returned value will be in screen coords.
 * ie it adjusts for scrollable regions, to allow comparison with mouse coordinates
 * returned from getMouseCoords method
 */
function getTopPosition(obj, screen)
{
    return getTop(obj, screen);

    /*var ot=obj.offsetTop;
    while ((obj=obj.offsetParent) != null)
    {
        ot += obj.offsetTop;
        if (screen)
        {
            if (obj.scrollTop != 0)
            {
                ot -= obj.scrollTop;
            }
        }
    }
    return ot;*/
}


function getLeft(element, screen) {
    var left = 0, absoluteAncestor = false, computedStyle = (document.defaultView && document.defaultView.getComputedStyle), currentStyle = (element.currentStyle);
//	so long as the element has an positioning context...
    while (element.offsetParent) {
//		add the left offset of the element
        left += element.offsetLeft

        if (screen)
        {
            if (element.scrollLeft != 0)
            {
                left -= element.scrollLeft;
            }
        }

//		set the element to the element which provided the positioning context
        element = element.offsetParent;
//		if the element hasLayout (IE-only property)
        if (currentStyle && element.currentStyle.hasLayout && element.nodeName.toLowerCase() != 'html') {
//			add the width of the left border
            left += element.clientLeft;
//			and if it's absolutely positioned, flag that we've got an absolutely positioned ancestor
            if (element.currentStyle['position'] == 'absolute') absoluteAncestor = true;
//		otherwise, if it's a browser that supports computedStyle & the element is absolutely positioned
        } else if (computedStyle && document.defaultView.getComputedStyle(element, "").getPropertyValue('position') == 'absolute') {
//			add the left border of the element
            left += parseInt(document.defaultView.getComputedStyle(element, "").getPropertyValue('border-left-width'));
//			and flag that we've got an absolutely positioned ancestor
            absoluteAncestor = true;
        }
    }
//	if the browser supports currentStyle (i.e. is IE) and we found an absolutely positioned ancestor, add any left margin on the BODY
    if (!absoluteAncestor && currentStyle) return left += document.getElementsByTagName('BODY')[0].offsetLeft;
//	otherwise, if we found an absolutely positioned ancestor and the browser supports computed style add any left border from the BODY
    else if (!absoluteAncestor && computedStyle) return left += parseInt(document.defaultView.getComputedStyle(document.getElementsByTagName('BODY')[0], "").getPropertyValue('border-left-width'));
    else return left;
}

function getTop(element, screen) {
    var top = 0, absoluteAncestor = false, computedStyle = (document.defaultView && document.defaultView.getComputedStyle), currentStyle = (element.currentStyle);
//	so long as the element has an positioning context...
    while (element.offsetParent) {
//		add the top offset of the element
        top += element.offsetTop

        if (screen)
        {
            if (element.scrollTop != 0)
            {
                top -= element.scrollTop;
            }
        }

//		set the element to the element which provided the positioning context
        element = element.offsetParent;
//		if the element hasLayout (IE-only property)
        if (currentStyle && element.currentStyle.hasLayout && (element.nodeName.toLowerCase() != 'html')) {
//			add the width of the top border
            top += element.clientTop;
//			and if it's absolutely positioned, flag that we've got an absolutely positioned ancestor
            if (element.currentStyle['position'] == 'absolute') absoluteAncestor = true;
//		otherwise, if it's a browser that supports computedStyle & the element is absolutely positioned
        } else if (computedStyle && document.defaultView.getComputedStyle(element, "").getPropertyValue('position') == 'absolute') {
//			add the top border of the element
            top += parseInt(document.defaultView.getComputedStyle(element, "").getPropertyValue('border-top-width'));
//			and flag that we've got an absolutely positioned ancestor
            absoluteAncestor = true;
        }
    }
//	if the browser supports currentStyle (i.e. is IE) and we found an absolutely positioned ancestor, add any top margin on the BODY
    if (!absoluteAncestor && currentStyle) return top += document.getElementsByTagName('BODY')[0].offsetTop;
//	otherwise, if we found an absolutely positioned ancestor and the browser supports computed style add any top border from the BODY
    else if (!absoluteAncestor && computedStyle) top += parseInt(document.defaultView.getComputedStyle(document.getElementsByTagName('BODY')[0], "").getPropertyValue('border-top-width'));
    return top;
}

/**
 * Returns the current height of the given source component
 * @param source The DOM component to return the current computed height for
 * @return a float containing the current height value (pixels)
 */
function getComputedHeight(source)
{
    var compHeight
    if (dojo.isIE)
    {
        compHeight = source.offsetHeight;
    }
    else
    {
        compHeight = getCurrentStyle(source,"height");
    }
    return parseFloat(compHeight);
}

/**
 * Returns the current width of the given source component
 * @param source The DOM component to return the current computed width for
 * @return a float containing the current width value (pixels)
 */
function getComputedWidth(source)
{
    var compWidth
    if (dojo.isIE < 9)
    {
        compWidth = source.offsetWidth;
    }
    else
    {
        compWidth = getCurrentStyle(source,"width");
    }
    //special handling for opera which seems to return 0 from getCurrentStyle for the width for some reason.
    if ((parseFloat(compWidth) == 0) && (source.offsetWidth) && (source.offsetWidth > 0))
        compWidth = source.offsetWidth;

    return parseFloat(compWidth);
}

/**
 * Returns the current location coordinates of the given object.
 * The returned object will have four properties, 'x' and 'y' which give
 * the position of the top left corner, and 'width' and 'height'
 * If the screen parameter is set to true, the returned values are in 'screen coords', ie they are
 * adjusted for scrollable regions, to allow comparison with the mouse corrdinates returned from
 * the getMouseCoords method.
 */
function getComponentPosition(obj, screen)
{
    var objCoords = new Object();
    objCoords.x = getLeftPosition(obj, screen);
    objCoords.y = getTopPosition(obj, screen);
    objCoords.width = obj.offsetWidth;
    objCoords.height = obj.offsetHeight;
    return objCoords;
}

/**
 * Returns the size of the available window viewport
 * @return an object containing two properties, width and height.
 */
function getViewportSize()
{
    var size = new Object();
    if (window.innerWidth)
    {
        size.width = window.innerWidth;
        size.height = window.innerHeight;
    }
    else if (document.documentElement && document.documentElement.clientWidth)
    {
        size.width = document.documentElement.clientWidth;
        size.height = document.documentElement.clientHeight;
    }
    else if (document.body && document.body.clientWidth)
    {
        size.width = document.body.clientWidth;
        size.height = document.body.clientHeight;
    }
    return size;
}



/**
 * Adjusts the scrollTop and scrollLeft properties of the given container
 * to try and make the entry visible on the screen.
 * @param container The HTML component that contains the entry, and has overflow set to scroll.
 * @param entry The HTML component within the container that should be shown.
 */
function scrollViewToShow(container, entry)
{
    if ((container == null) || (typeof(container) == 'undefined') ||
        (entry == null) || (typeof(entry) == 'undefined'))
    {
        //not recieved the required details so can't continue
        return;
    }

    var cScrollTop = container.scrollTop;
    var cPos = getComponentPosition(container, false);
    var cClientHeight = container.clientHeight;
    var cScrollLeft = container.scrollLeft;
    var cClientWidth = container.clientWidth;

    var ePos = getComponentPosition(entry, false);
    var eClientWidth = entry.clientWidth;

    if (((cScrollTop + cPos.y + cClientHeight) < ePos.y) || (ePos.y < (cScrollTop + cPos.y)))
    { //hidden off the top or bottom of the scrolling window, so need to adjust scroll top
        container.scrollTop = (ePos.y - cPos.y - (cClientHeight/2));
    }

    if (eClientWidth < cClientWidth)
    {  //entry width less than window width so show in the middle
        if (((cScrollLeft + cPos.x + cClientWidth) < (ePos.x + eClientWidth)) || (ePos.x < (cScrollLeft + cPos.x)))
        {
            container.scrollLeft = (ePos.x - cPos.x - ((cClientWidth - eClientWidth)/2));
        }
    }
    else
    { //entry width greater than window width so show left hand section
        if (((cScrollLeft + cPos.x + cClientWidth) < (ePos.x + eClientWidth)) || (ePos.x < (cScrollLeft + cPos.x)))
        {
            container.scrollLeft = (ePos.x - cPos.x - 10);
        }
    }
}



/**
 * Sets the opacity of the given object.
 * @param obj The object to set the opactiry on.
 * @param opacity The opacity value to set - 0.0 = transparent, 1.0 = opaque.
 */
function setOpacity(obj, opacity)
{
    if (obj != null)
    {
        obj.style.filter = 'Alpha(Opacity=' + opacity * 100 + ')'; //IE
        obj.style.opacity = opacity; //Others
    }

}


/**
 * Clears the opacity setting on the given object.
 */
function clearOpacity(obj)
{
    //TODO: this will clear out all filters, not just opacity, so need to change.
    if (obj != null)
    {
        obj.style.filter = '';
        obj.style.opacity = 1;
    }
}




/**
 * The scrollable div manager is used to provide extra scrolling features
 * for divs with the overflow setting set to scroll.
 * It will enable automatic scrolling of the container when the user has the mouse button
 * pressed, and moves the mouse towards the top or bottom of the container.
 * ie when they are doing a drag and drop operation.
 */
scrollableDivManager =
{
    scrollEnabled : false,
    DETECTION_OFFSET : 20,
    SCROLL_AMOUNT : 10
}

/**
 * Adds a container to be managed by the scriollable div manager
 */
scrollableDivManager.attachContainer = function(cont)
{
    //check that the container is set to scroll
    if (getCurrentStyle(cont, 'overflow') == 'scroll')
    {
        WinManager.attachEvent(cont, 'onmousemove', scrollableDivManager.mouseMove);
        WinManager.attachEvent(cont, 'onmouseout', scrollableDivManager.mouseOut);
        cont.setAttribute('scrollableDivContainer', 'true');
    }
}


scrollableDivManager.mouseMove = function(e)
{
    if (!e) { e = window.event; }

    window.clearTimeout(scrollableDivManager.timeoutID);

    var elem = (e.srcElement) ? e.srcElement : e.target;

    while ((elem != null) && (elem.getAttribute('scrollableDivContainer') == null))
    {
        elem = elem.parentNode;
    }

    if (elem == null)
        return;

    //check that the mouse button is pressed
    //if ((e.button == 1) || (e.which == 1))
    if (scrollableDivManager.scrollEnabled)
    {
        //alert(elem.getAttribute('id'));
        var mouseCoords = getMouseCoords(e);
        var elemCoords = getComponentPosition(elem, false);

        scrollableDivManager.scrollContainer = elem;

        if (mouseCoords.y < (elemCoords.y + scrollableDivManager.DETECTION_OFFSET))
        {
            elem.scrollTop = elem.scrollTop - scrollableDivManager.SCROLL_AMOUNT;
            scrollableDivManager.timeoutID = window.setTimeout("scrollableDivManager.mouseHeldVertical(-"+scrollableDivManager.SCROLL_AMOUNT+")", 5);
        }
        else if (mouseCoords.y > ((elemCoords.y + elemCoords.height) - scrollableDivManager.DETECTION_OFFSET))
        {
            elem.scrollTop = elem.scrollTop + scrollableDivManager.SCROLL_AMOUNT;
            scrollableDivManager.timeoutID = window.setTimeout("scrollableDivManager.mouseHeldVertical("+scrollableDivManager.SCROLL_AMOUNT+")", 5);
        }
        else if (mouseCoords.x < (elemCoords.x + scrollableDivManager.DETECTION_OFFSET))
        {
            elem.scrollLeft = elem.scrollLeft - scrollableDivManager.SCROLL_AMOUNT;
            scrollableDivManager.timeoutID = window.setTimeout("scrollableDivManager.mouseHeldHorizontal(-"+scrollableDivManager.SCROLL_AMOUNT+")", 5);
        }
        else if (mouseCoords.x > ((elemCoords.x + elemCoords.width) - scrollableDivManager.DETECTION_OFFSET))
        {
            elem.scrollLeft = elem.scrollLeft + scrollableDivManager.SCROLL_AMOUNT;
            scrollableDivManager.timeoutID = window.setTimeout("scrollableDivManager.mouseHeldHorizontal("+scrollableDivManager.SCROLL_AMOUNT+")", 5);
        }



    }
}

scrollableDivManager.mouseOut = function(e)
{
    window.clearTimeout(scrollableDivManager.timeoutID);
}

scrollableDivManager.mouseHeldVertical = function(changeAmount)
{
    scrollableDivManager.scrollContainer.scrollTop = scrollableDivManager.scrollContainer.scrollTop + changeAmount;
    scrollableDivManager.timeoutID = window.setTimeout("scrollableDivManager.mouseHeldVertical("+changeAmount+")", 5);
}

scrollableDivManager.mouseHeldHorizontal = function(changeAmount)
{
    scrollableDivManager.scrollContainer.scrollLeft = scrollableDivManager.scrollContainer.scrollLeft + changeAmount;
    scrollableDivManager.timeoutID = window.setTimeout("scrollableDivManager.mouseHeldHorizontal("+changeAmount+")", 5);
}


//-------------------------------------------------------------------------

/**
 * MenuManager provides support for drop down menu functionality.
 */
var menuManager = new Object();

menuManager.showMenu = function(menuID, source)
{
    if (menuManager.timeoutMenu == menuID)
    {
        menuManager.maintainMenu(menuID);
    }
    else
    {
        var menuComponent = document.getElementById(menuID + '_menu');
        //find the lcoation of the source object, to position the menu under
        var sourcePos = getComponentPosition(source, false);
        menuComponent.style.position = 'absolute';
        menuComponent.style.left = sourcePos.x + 15;
        menuComponent.style.top = sourcePos.y + sourcePos.height + 5;

        showComponent(menuComponent);
    }
}

menuManager.hideMenu = function(menuID)
{
    menuManager.timeoutMenu = menuID;
    menuManager.timeoutID = window.setTimeout("menuManager._hide('"+menuID+"')", 100);

}

menuManager.maintainMenu = function(menuID)
{
    window.clearTimeout(menuManager.timeoutID);
    menuManager.timeoutMenu = null;
}

menuManager._hide = function(menuID)
{
    menuManager.timeoutMenu = null;
    var menuComponent = document.getElementById(menuID + '_menu');
    hideComponent(menuComponent);
}


// TODO: Ideally should use the actual browser detection functinality, but studio is IE only for now...
var is_ie = true;

// This stuff is copied from the runtime displayUtils file that is used within the FM generated apps
// should probably look at reusing the existing file rather than having another copy here!!!

/**
 * Root object to contain all hyfinity functions.
 * This should remove the chance of a name conflict with other functionality.
 *
 * TODO: At some point we need to update all existing functionality to use this namespace.
 */
if (typeof(hyf) == 'undefined')
    var hyf =
    {
        version : '1.0'
    };


/**
 * The hyf.textarea namespace contains all functionality relating to textarea field controls.
 */
hyf.textarea =
{
    desc : "Handles functionality relating to textareas, eg auto expanding fields.",
    offsetBuffer : (is_ie) ? 2 : ((is_opera) ? 4 : 0)
};

/**
 * Handles automatically updating the size of the supplied textarea to support the current content.
 *
 * @param evt The (keypress) event object that triggered the call.
 * @param field The textarea object to adjust
 * @param min The minim number of rows to show in the textarea.
 * @param max The maximum number of rows to expand the textarea to.
 */
hyf.textarea.adjustHeight = function(evt, field, min, max)
{
    if ((typeof(dojo) != 'undefined') && (dojo.isWebKit)) return;

    if ((evt != null) && (typeof(evt) != 'undefined'))
        var charCode = (evt.which) ? evt.which : event.keyCode;

    //ignore the arrow keys and shift/ctrl key
    if ((charCode >= 37 && charCode <= 40) || (charCode == 16) || (charCode == 17) || (evt && evt.ctrlKey))
    {
        return;
    }

    if ((max == null) || typeof(max) == 'undefined')
        max = Number.MAX_VALUE;
    if ((min == null) || typeof(min) == 'undefined')
        min = 1;

    //if the delete or backspace keys have been pressed then check whether to remove a row
    if ((field.rows > min) && ((charCode == 8) || (charCode == 46)))
    {
        if (is_ie)  //does this work for other browsers???
        {
            var needToAddBack = false;
            while ((field.rows >= min ) && (!hyf.textarea.hasScroll(field)))
            {
                field.rows--;
                //needToAddBack = true;
            }
            //alert('hello');
            if (needToAddBack)
                field.rows++;
        }
        else
        {
            //find a guess of the number of rows of text
            var numRows = hyf.textarea.countNumTextRows(field);

            //set based on this
            field.rows = (numRows < min) ? min : ((numRows <= max) ? numRows : max);
        }
    }
    //otherwise look at adding a row
    else if ((field.rows < max ) && (hyf.textarea.hasScroll(field)) && !((charCode == 8) || (charCode == 46)))
    {
        if (is_ie || is_gecko)
        {
            while ((field.rows < max ) && (hyf.textarea.hasScroll(field)))
            {
                field.rows++;
            }
        }
        else
        {
            //find a guess of the number of rows of text
            var numRows = hyf.textarea.countNumTextRows(field);

            //set based on this
            field.rows = (numRows < min) ? min : ((numRows <= max) ? numRows : max);
        }
    }
};

/**
 * Tries to determine whether the given textarea field currently has a scroll bar or not.
 * @param field The textarea object.
 * @return true or false depending on whether a scrollbar is currently required.
 *
 * @private
 */
hyf.textarea.hasScroll = function(field)
{
    //attempt to scroll to the bottom
    field.scrollTop = field.scrollHeight;
    //check if the field has been scrolled at all
    if (field.scrollTop != 0)
    {
        //field.value = content;
        return true;
    }
    else
    {
        //field.value = content;
        return false;
    }
}


/**
 * Returns an estimate of the number of rows used by the text content in the
 * provided textarea.
 * @param field The textarea object.
 *
 * @private
 */
hyf.textarea.countNumTextRows = function(field)
{
    //count number of line breaks
    var re = /\r\n|\r|\n/g;
    var numBreaks = 0;
    var lineStart = 0;
    while (re.exec(field.value))
    {
        numBreaks++;
        //for each line of text between physical breaks, try to work out how many lines it wraps to
        var newLineStart = re.lastIndex;
        var lineString = field.value.substring(lineStart, newLineStart);
        if (lineString.length > field.cols)
            numBreaks += Math.ceil(lineString.length / field.cols);

        lineStart = newLineStart;
    }

    var lineString = field.value.substring(lineStart);
    if (lineString.length == 0)
        return Number(numBreaks) + Number(1);
    else
        return Math.ceil(lineString.length / field.cols) + Number(numBreaks);

};



/**
 * Collapses all the whitespace in the given string, and then converts any remaining
 * spaces to underscores.
 * This can be used to make the users entered value a valid id string.
 */
function spacesToUnderscores(value)
{
    value = String(value);
    //remove whitespace characters from the start of the string
    value = value.replace(/^\s+/g,'');
    //remove whitespace characters from the end of the string
    value = value.replace(/\s+$/g, '');
    //collapse whitespace in the middle
    value = value.replace(/\s+/g, ' ');

    //convert any remaining whitespace to underscores
    value = value.replace(/\s/g, '_');

    return value;

}

/**
 * Generic method that cancesl propagation of the provided event object.
 * This should hopefully handle all browsers
 */
function cancelEvent(evt)
{
    evt.cancelBubble = true;
    if (evt.preventDefault)
        evt.preventDefault();
    if (evt.stopPropagation)
        evt.stopPropagation();
}

