/**
 * Instance TreeView Functions
 * Provides the needed functions for handling the visibility of the
 * input and output instance treevies for the rulebase viewers.
 *
 * @author Gerard Smyth
 * @copyright (c) 2005
 * Company Hyfinity Ltd.
 */
function InstanceTreeviewManager(inputXMLString, inputContainerID, inputDividerID,
                                 outputXMLString, outputContainerID, outputDividerID,
                                 rulebase, id, state)
{

    this.displayID = id;

    this.inputContainerTD = document.getElementById(inputContainerID);
    this.inputContainer = null;
    for (var i = 0; i < this.inputContainerTD.childNodes.length; ++i)
    {
        if (this.inputContainerTD.childNodes.item(i).nodeType == 1)
        {
            this.inputContainer = this.inputContainerTD.childNodes.item(i)
            break;
        }
    }
    this.inputDivider = document.getElementById(inputDividerID);
    this.outputContainerTD = document.getElementById(outputContainerID);
    this.outputContainer = null;
    for (var i = 0; i < this.outputContainerTD.childNodes.length; ++i)
    {
        if (this.outputContainerTD.childNodes.item(i).nodeType == 1)
        {
            this.outputContainer = this.outputContainerTD.childNodes.item(i)
            break;
        }
    }
    this.outputDivider = document.getElementById(outputDividerID);

    this.rulebase = rulebase;
    this.rulebase.instanceManager = this;

    this.inputDoc = null;
    this.inputDetail = new Object();
    if ((inputXMLString != null) && (inputXMLString != ''))
    {
        this.inputDoc = getXMLDocument(inputXMLString);
        Sarissa.setXpathNamespaces(this.inputDoc, this.rulebase.getNamespaceString());
        this.rulebase.addNamespacesFromDocument(this.inputDoc);
    }

    this.outputDoc = null;
    this.outputDetail = new Object();
    if ((outputXMLString != null) && (outputXMLString != ''))
    {
        this.outputDoc = getXMLDocument(outputXMLString);
        Sarissa.setXpathNamespaces(this.outputDoc, this.rulebase.getNamespaceString());
        this.rulebase.addNamespacesFromDocument(this.outputDoc);
    }


    //set up the treeview XSL
    this.treeviewXSL = getXMLDocumentFromURL("xsl/InstanceTreeView.xsl");
    //this.treeviewXSL.load("xsl/InstanceTreeView.xsl");
    // create an instance of XSLTProcessor
    this.XSLProcessor = new XSLTProcessor();
    this.XSLProcessor.importStylesheet(this.treeviewXSL);

    //Stores the set of DIV elements in the treeviews corresponding to the currently highlighted elements
    this.currentInputMatches = new Array();
    this.currentOutputMatches = new Array();


    //Stores whether the input and output treeviews have been generated yet
    this.inputGenerated = false;
    this.outputGenerated = false;

    /*
     * Setup class methods
     */
    if (typeof(__InstanceTreeviewManagerPrototypeCalled) == 'undefined')
    {
        __InstanceTreeviewManagerPrototypeCalled = true;

        InstanceTreeviewManager.prototype.toggleInputVisibility = toggleInputVisibility;
        InstanceTreeviewManager.prototype.toggleOutputVisibility = toggleOutputVisibility;
        InstanceTreeviewManager.prototype.setInput = setInput;
        InstanceTreeviewManager.prototype.setOutput = setOutput;
        InstanceTreeviewManager.prototype.genInput = genInput;
        InstanceTreeviewManager.prototype.genOutput = genOutput;
        InstanceTreeviewManager.prototype.chooseInput = chooseInput;
        InstanceTreeviewManager.prototype.chooseOutput = chooseOutput;

        InstanceTreeviewManager.prototype.getJavascriptIdentifier = getInstanceManagerJavascriptIdentifier;
        InstanceTreeviewManager.prototype.getStateXML = getInstanceManagerStateXML;

        InstanceTreeviewManager.prototype.refreshNamespaces = refreshInstanceManagerNamespaces

        InstanceTreeviewManager.prototype.evaluateXPathAgainstInput = evaluateXPathAgainstInput;
        InstanceTreeviewManager.prototype.evaluateXPathAgainstOutput = evaluateXPathAgainstOutput;

        InstanceTreeviewManager.prototype.showMatchesOnInput = showMatchesOnInput;
        InstanceTreeviewManager.prototype.showMatchesOnOutput = showMatchesOnOutput;
        InstanceTreeviewManager.prototype.clearInputMatches = clearInputMatches;
        InstanceTreeviewManager.prototype.clearOutputMatches = clearOutputMatches;

        InstanceTreeviewManager.prototype.findDOMNodeUniqueID = findDOMNodeUniqueID;
        InstanceTreeviewManager.prototype.getPreviousSiblingCount = getPreviousSiblingCount;
        InstanceTreeviewManager.prototype.loadInstanceDocuments = loadInstanceDocuments;

        InstanceTreeviewManager.prototype.ensureMaintainDocuments = ensureMaintainDocuments;
        InstanceTreeviewManager.prototype.saveInput = saveInput;
        InstanceTreeviewManager.prototype.saveOutput = saveOutput;

    }

    //default the treeviews to be hidden, and any initial content to be displayed in them
    this.toggleInputVisibility();
    this.toggleOutputVisibility();

    //parse state info
    if (state != null);
    {
        var input = state.selectSingleNode("/hy:rulebase_state/hy:instances/hy:input");
        if (input != null)
        {
            if (input.selectSingleNode("*") != null)
            {
                this.inputDoc = getXMLDocument(input.selectSingleNode("*"));
                Sarissa.setXpathNamespaces(this.inputDoc, this.rulebase.getNamespaceString());
                this.inputGenerated = false;
            }

            this.inputDetail.xstore = input.getAttribute("xstore");
            this.inputDetail.collection = input.getAttribute("collection");
            this.inputDetail.document = input.getAttribute("document");
            this.inputDetail.xstoreID = input.getAttribute("xstoreID");
            this.inputDetail.collectionID = input.getAttribute("collectionID");
            this.inputDetail.documentID = input.getAttribute("documentID");

            if ((input.getAttribute('width') != null) && (input.getAttribute('width') != ''))
                this.inputContainerTD.hiddenDividerWidth = input.getAttribute('width');

            if (input.getAttribute('visible') == 'true')
                this.toggleInputVisibility();

            if (((this.inputDetail.xstoreID != null) &&
                 (this.inputDetail.collectionID != null) &&
                 (this.inputDetail.documentID != null)) ||
                ((this.inputDetail.xstore != null) &&
                 (this.inputDetail.collection != null) &&
                 (this.inputDetail.document != null)))
            {
                this.inputLoadNeeded = true;
            }

            if ((input.getAttribute('import_namespaces') != null) && (input.getAttribute('import_namespaces') == 'true'))
                this.inputDetail.importNamespaces = true;


        }


        var output = state.selectSingleNode("/hy:rulebase_state/hy:instances/hy:output");
        if (output != null)
        {
            if (output.selectSingleNode("*") != null)
            {
                this.outputDoc = getXMLDocument(output.selectSingleNode("*"));
                Sarissa.setXpathNamespaces(this.outputDoc, this.rulebase.getNamespaceString());
                this.outputGenerated = false;
            }

            this.outputDetail.xstore = output.getAttribute("xstore");
            this.outputDetail.collection = output.getAttribute("collection");
            this.outputDetail.document = output.getAttribute("document");
            this.outputDetail.xstoreID = output.getAttribute("xstoreID");
            this.outputDetail.collectionID = output.getAttribute("collectionID");
            this.outputDetail.documentID = output.getAttribute("documentID");

            if ((output.getAttribute('width') != null) && (output.getAttribute('width') != ''))
                this.outputContainerTD.hiddenDividerWidth = output.getAttribute('width');

            if (output.getAttribute('visible') == 'true')
                this.toggleOutputVisibility();

            if (((this.outputDetail.xstoreID != null) &&
                 (this.outputDetail.collectionID != null) &&
                 (this.outputDetail.documentID != null)) ||
                ((this.outputDetail.xstore != null) &&
                 (this.outputDetail.collection != null) &&
                 (this.outputDetail.document != null)))
            {
                this.outputLoadNeeded = true;
            }

            if ((output.getAttribute('import_namespaces') != null) && (output.getAttribute('import_namespaces') == 'true'))
                this.outputDetail.importNamespaces = true;
        }
        this.loadInstanceDocuments();
    }

}


/**
 * Toggles visibility of the input instance document
 */
function toggleInputVisibility()
{
    if (this.inputDoc != null)
    {
        if (!this.inputGenerated)
        {
            this.genInput();
        }
    }
    else
    {
        this.inputContainer.innerHTML = "Please select a document to display";
    }
    dividerDoubleClicked(this.inputContainerTD);

    checkStyleMethodForRulesDisplay();
}

/**
 * Generates the input treeview from the currently set input document
 */
function genInput()
{
    /*var logicsheetDoc = Sarissa.getDomDocument();
    logicsheetDoc.loadXML(this.rulebase.getLogicsheetXML(false));*/
    //var logicsheetDoc = getXMLDocument(this.rulebase.getLogicsheetXML(false));
    //var logicsheetDoc = getXMLDocument(hyf.ns.getNamespaceDocument());
    //this.XSLProcessor.setParameter(null, "namespace_document", logicsheetDoc);

    this.XSLProcessor.setParameter(null, "output_prefix", "input_document");

    var xmlToTransform = '<container><namespaces>'+hyf.ns.getNamespaceDocument()+'</namespaces>'
                        + '<document>' + xmlToString(this.inputDoc) + '</document></container>';
    var docToTransform = getXMLDocument(xmlToTransform);

    var tv = this.XSLProcessor.transformToDocument(docToTransform);

    if (((this.inputDetail.xstoreID != null) &&
         (this.inputDetail.collectionID != null) &&
         (this.inputDetail.documentID != null)) ||
        ((this.inputDetail.xstore != null) &&
         (this.inputDetail.collection != null) &&
         (this.inputDetail.document != null)))
    {
        //this.inputContainer.innerHTML = '<div class="TreeList">' + xmlToString(tv) + '</div>';
        this.inputContainer.innerHTML = xmlToString(tv);
    }
    else
    {
        var mes = '<div id="input_temp_message" class="DocumentTempMessage"><a href="javascript:'+this.displayID+'.saveInput();">This document is not saved.<br/>Click here to store permanently</a>.</div>';
        //this.inputContainer.innerHTML = mes + '<div class="TreeList">' + xmlToString(tv) + '</div>';
        this.inputContainer.innerHTML = mes + xmlToString(tv);
    }
    this.inputGenerated = true;
}


/**
 * Toggles visibility of the output instance document
 */
function toggleOutputVisibility()
{
    if (this.outputDoc != null)
    {
        if (!this.outputGenerated)
        {
            this.genOutput();
        }
    }
    else
    {
        this.outputContainer.innerHTML = "Please select a document to display";
    }
    dividerDoubleClicked(this.outputContainerTD);

    checkStyleMethodForRulesDisplay();
}

/**
 * Generates the output treeview from the currently set output document
 */
function genOutput()
{
    /*var logicsheetDoc = Sarissa.getDomDocument();
    logicsheetDoc.loadXML(this.rulebase.getLogicsheetXML(false));*/
    //var logicsheetDoc = getXMLDocument(this.rulebase.getLogicsheetXML(false));
    //var logicsheetDoc = getXMLDocument(hyf.ns.getNamespaceDocument());
    //this.XSLProcessor.setParameter(null, "namespace_document", logicsheetDoc);

    this.XSLProcessor.setParameter(null, "output_prefix", "output_document");

    var xmlToTransform = '<container><namespaces>'+hyf.ns.getNamespaceDocument()+'</namespaces>'
                        + '<document>' + xmlToString(this.outputDoc) + '</document></container>';
    var docToTransform = getXMLDocument(xmlToTransform);

    var tv = this.XSLProcessor.transformToDocument(docToTransform);

    if (((this.outputDetail.xstoreID != null) &&
         (this.outputDetail.collectionID != null) &&
         (this.outputDetail.documentID != null)) ||
        ((this.outputDetail.xstore != null) &&
         (this.outputDetail.collection != null) &&
         (this.outputDetail.document != null)))
    {
        //this.outputContainer.innerHTML = '<div class="TreeList">' + xmlToString(tv) + '</div>';
        this.outputContainer.innerHTML = xmlToString(tv);
    }
    else
    {
        var mes = '<div id="output_temp_message" class="DocumentTempMessage"><a href="javascript:'+this.displayID+'.saveOutput();">This document is not saved.<br/>Click here to store permanently</a>.</div>';
        //this.outputContainer.innerHTML = mes + '<div class="TreeList">' + xmlToString(tv) + '</div>';
        this.outputContainer.innerHTML = mes + xmlToString(tv);
    }


    this.outputGenerated = true;
}

/**
 * Refrezsh the namespace information defined on the instance documents / treeviews
 */
function refreshInstanceManagerNamespaces()
{
    //Update the namespaces defined for evaluating xpaths
    if (this.inputDoc != null)
    {
        Sarissa.setXpathNamespaces(this.inputDoc, this.rulebase.getNamespaceString());
    }
    if (this.outputDoc != null)
    {
        Sarissa.setXpathNamespaces(this.outputDoc, this.rulebase.getNamespaceString());
    }

    //refresh the generated treeviews to ensure the xpaths pick up the new namespaces
    if (this.outputGenerated)
    {
        this.genOutput();
    }
    if (this.inputGenerated)
    {
        this.genInput();
    }

}


/**
 * Evaluates the given xpath against the input document and returns the results.
 * @param xpath A String containing the xpath to evaluate.
 * @param context An optional string conatining the XPath to use as the context point for the main xpath
 * @return An array containing all the matched nodes - or null if an error occurred
 */
function evaluateXPathAgainstInput(xpath, context)
{
    if (this.inputDoc != null)
    {
        try
        {
            var contextPoints = new Array();
            //find all context points
            if ((context != null) && (typeof(context) != 'undefined') && (xpath.indexOf('/') != 0))
            {
                var contexts = this.inputDoc.selectNodes(context);
                for (var i = 0; i < contexts.length; ++i)
                {
                    contextPoints[contextPoints.length] = contexts.item(i);
                }
            }
            else
            {
                contextPoints[contextPoints.length] = this.inputDoc.documentElement;
            }

            var matches = new Array();
            for (var i = 0; i < contextPoints.length; ++i)
            {
                tempMatches = contextPoints[i].selectNodes(xpath);
                for (var match = 0; match < tempMatches.length; ++match)
                {
                    if (!arrayContains(matches, tempMatches.item(match)))
                        matches[matches.length] = tempMatches.item(match);
                }
            }

            return matches;
        }
        catch (e)
        {
            return null;
        }
    }
    else
        return null;
}

/**
 * Evaluates the given xpath against the output document and returns the results.
 * @param xpath A String containing the xpath to evaluate.
 * @param context An optional string conatining the XPath to use as the context point for the main xpath
 * @return A NodeList containing all the matched nodes - or null if an error occurred.
 */
function evaluateXPathAgainstOutput(xpath, context)
{
    if (this.outputDoc != null)
    {
        try
        {
            var contextPoints = new Array();
            //find all context points
            if ((context != null) && (typeof(context) != 'undefined'))
            {
                var contexts = this.outputDoc.selectNodes(context);
                for (var i = 0; i < contexts.length; ++i)
                {
                    contextPoints[contextPoints.length] = contexts.item(i);
                }
            }
            else
            {
                contextPoints[contextPoints.length] = this.outputDoc.documentElement;
            }

            var matches = new Array();
            for (var i = 0; i < contextPoints.length; ++i)
            {
                tempMatches = contextPoints[i].selectNodes(xpath);
                for (var match = 0; match < tempMatches.length; ++match)
                {
                    if (!arrayContains(matches, tempMatches.item(match)))
                        matches[matches.length] = tempMatches.item(match);
                }
            }

            return matches;
        }
        catch (e)
        {
            return null;
        }
    }
    else
        return null;
}


/**
 * Highlights all nodes on the input treeview that match the given xpath
 * @param xpath A String containing the XPath to show the matches for
 * @param show A boolean indicating whether the treeview should be shown if it is currently hidden.
 * @param context An optional string conatining the XPath to use as the context point for the main xpath
 */
function showMatchesOnInput(xpath, show, context)
{
    if (!this.inputGenerated)
    {
        this.genInput();
    }

    this.clearInputMatches();

    var matches = this.evaluateXPathAgainstInput(xpath, context);
    if ((matches != null) && matches.length > 0)
    {
        //add all the matching treeview elements to the array
        for (var i = 0; i < matches.length; ++i)
        {
           this.currentInputMatches[this.currentInputMatches.length] =
                    document.getElementById("input_document" + this.findDOMNodeUniqueID(matches[i]));
        }

        for (var j = 0; j < this.currentInputMatches.length; ++j)
        {
            this.currentInputMatches[j].className = "selected_treeview_entry";
            if (j == 0)
            {
                scrollViewToShow(this.inputContainer, this.currentInputMatches[j]);
            }
        }

        //ensure the input treeview is visible
        if ((typeof(show) != 'undefined') && (show == true))
        {
            toggleComponent(this.inputContainerTD, "show");
            toggleComponent(this.inputDivider, "show");
        }
    }
}

/**
 * clears any currently highlighted xpath matches on the input instance document
 */
function clearInputMatches()
{
    //reset the dispaly of currently shown matches
    for (var j = 0; j < this.currentInputMatches.length; ++j)
    {
        this.currentInputMatches[j].className = "";
    }

    //clear the array
    this.currentInputMatches = new Array();
}

/**
 * Highlights all nodes on the output treeview that match the given xpath
 * @param xpath A String containing the XPath to show the matches for
 * @param show A boolean indicating whether the treeview should be shown if it is currently hidden.
 * @param context An optional string conatining the XPath to use as the context point for the main xpath
 */
function showMatchesOnOutput(xpath, show, context)
{
    if (!this.outputGenerated)
    {
        this.genOutput();
    }

    this.clearOutputMatches();

    var matches = this.evaluateXPathAgainstOutput(xpath, context);
    if ((matches != null) && matches.length > 0)
    {
        //add all the matching treeview elements to the array
        for (var i = 0; i < matches.length; ++i)
        {
           this.currentOutputMatches[this.currentOutputMatches.length] =
                    document.getElementById("output_document" + this.findDOMNodeUniqueID(matches[i]));
        }

        for (var j = 0; j < this.currentOutputMatches.length; ++j)
        {
            this.currentOutputMatches[j].className = "selected_treeview_entry";
            if (j == 0)
            {
                scrollViewToShow(this.outputContainer, this.currentOutputMatches[j]);
            }

        }

        //ensure the output treeview is visible
        if ((typeof(show) != 'undefined') && (show == true))
        {
            toggleComponent(this.outputContainerTD, "show");
            toggleComponent(this.outputDivider, "show");
        }
    }
}

/**
 * clears any currently highlighted xpath matches onthe input instance document
 */
function clearOutputMatches()
{
    //reset the dispaly of currently shown matches
    for (var j = 0; j < this.currentOutputMatches.length; ++j)
    {
        this.currentOutputMatches[j].className = "";
    }

    //clear the array
    this.currentOutputMatches = new Array();
}



/**
 * Returns the JavaScript identifier for this object
 */
function getInstanceManagerJavascriptIdentifier()
{
    return this.displayID;
}

/**
 * Returns an XML string describing the current visibility state of this object.
 * @return The XML formatted string containing state information.
 */
function getInstanceManagerStateXML()
{
    var stateXML = '<hy:instances xmlns:hy="http://www.hyfinity.com/xengine">';

    var inputVisible = true;
    var inputWidth = this.inputContainerTD.style.width;
    if (dividerIsHidden(this.inputContainerTD))
    {
        inputVisible = false;
        inputWidth = this.inputContainerTD.hiddenDividerWidth;
    }

    var outputVisible = true;
    var outputWidth = this.outputContainerTD.style.width;
    if (dividerIsHidden(this.outputContainerTD))
    {
        outputVisible = false;
        outputWidth = this.outputContainerTD.hiddenDividerWidth;
    }

    stateXML += '<hy:input ';
    if ((typeof(this.inputDetail.xstore) != 'undefined') && (this.inputDetail.xstore != null))
        stateXML += 'xstore="'+this.inputDetail.xstore+'" ';
    if ((typeof(this.inputDetail.collection) != 'undefined') && (this.inputDetail.collection != null))
        stateXML += 'collection="'+this.inputDetail.collection+'" ';
    if ((typeof(this.inputDetail.document) != 'undefined') && (this.inputDetail.document != null))
        stateXML += 'document="'+this.inputDetail.document+'" ';
    if ((typeof(this.inputDetail.xstoreID) != 'undefined') && (this.inputDetail.xstoreID != null))
        stateXML += 'xstoreID="'+this.inputDetail.xstoreID+'" ';
    if ((typeof(this.inputDetail.collectionID) != 'undefined') && (this.inputDetail.collectionID != null))
        stateXML += 'collectionID="'+this.inputDetail.collectionID+'" ';
    if ((typeof(this.inputDetail.documentID) != 'undefined') && (this.inputDetail.documentID != null))
        stateXML += 'documentID="'+this.inputDetail.documentID+'" ';

    stateXML += 'visible="'+inputVisible+'" width="'+inputWidth+'"/>';

    stateXML += '<hy:output ';
    if ((typeof(this.outputDetail.xstore) != 'undefined') && (this.outputDetail.xstore != null))
        stateXML += 'xstore="'+this.outputDetail.xstore+'" ';
    if ((typeof(this.outputDetail.collection) != 'undefined') && (this.outputDetail.collection != null))
        stateXML += 'collection="'+this.outputDetail.collection+'" ';
    if ((typeof(this.outputDetail.document) != 'undefined') && (this.outputDetail.document != null))
        stateXML += 'document="'+this.outputDetail.document+'" ';
    if ((typeof(this.outputDetail.xstoreID) != 'undefined') && (this.outputDetail.xstoreID != null))
        stateXML += 'xstoreID="'+this.outputDetail.xstoreID+'" ';
    if ((typeof(this.outputDetail.collectionID) != 'undefined') && (this.outputDetail.collectionID != null))
        stateXML += 'collectionID="'+this.outputDetail.collectionID+'" ';
    if ((typeof(this.outputDetail.documentID) != 'undefined') && (this.outputDetail.documentID != null))
        stateXML += 'documentID="'+this.outputDetail.documentID+'" ';

    stateXML += 'visible="'+outputVisible+'" width="'+outputWidth+'"/>';

    stateXML += '</hy:instances>';

    return stateXML;
}




/**
 * Creates a unique ID for the given DOM Node by recursing up the tree
 * and appending the position numbers for each element. eg 1.2.3.2
 * This is the same as the xsl:number function used with <xsl:number level="multiple" format="1" count="*"/>
 * For attributes, the name of the attribute is appended to the end of the ID.
 *
 * @private
 * @param node The DOM Node to find the ID for
 * @return The resulting unique ID
 */
function findDOMNodeUniqueID(node)
{
    if (node == null)
    {
        return "";
    }
    var id ="";
    var tempNode;
    if (node.nodeType == 2) //attribute
    {
        if (typeof(node.ownerElement) != 'undefined')
            tempNode = node.ownerElement; //Standard - FF,Chrome
        else
            tempNode = node.selectSingleNode('..'); //IE
    }
    else
        tempNode = node.parentNode;
    while ((tempNode != null) && (tempNode.parentNode != null))
    {
        id = (this.getPreviousSiblingCount(tempNode) + 1) + "." + id;
        tempNode = tempNode.parentNode;
    }

    if (node.nodeType == 2) //for attribute use the attr name rather than its position as they are not ordered
    {
        id += getBaseName(node);
    }
    else
    {
        id += (this.getPreviousSiblingCount(node) + 1);
    }

    return id;
}

/**
 * Returns a count of the number of previous siblings for the given node.
 * Only counts Element nodes.
 * @private
 * @param node The DOM Node to count the previous siblings for.
 * @return The count of previous element siblings for the given node.
 */
function getPreviousSiblingCount(node)
{
    if (node.previousSibling == null)
    {
        return 0;
    }
    else
    {
        var count = 0;
        var prevNode = node.previousSibling;
        while(prevNode != null)
        {
            if (prevNode.nodeType == 1)
                count++;
            prevNode = prevNode.previousSibling;
        }

        return count;
    }
}






/**
 * Uses the given content as the input instance document.
 * @param xstoreName A string containing the name of the xstore the file is in.
 * @param collectionName A string containing the name of the collection the file is in.
 * @param documentName A string containing the name of the document to load up.
 *
 * @param content The XML DOM containing the content of the document to show.
 */
function setInput(xstoreName, collectionName, documentName, xstoreId, collectionId, documentId, content)
{
    /*this.inputDoc = Sarissa.getDomDocument();
    this.inputDoc.resolveExternals = false;
    this.inputDoc.validateOnParse = false;
    this.inputDoc.loadXML(content.xml);*/
    this.inputDoc = getXMLDocument(content);
    Sarissa.setXpathNamespaces(this.inputDoc, this.rulebase.getNamespaceString());

    this.inputDetail.xstore = xstoreName;
    this.inputDetail.collection = collectionName;
    this.inputDetail.document = documentName;
    this.inputDetail.xstoreID = xstoreId;
    this.inputDetail.collectionID = collectionId;
    this.inputDetail.documentID = documentId;

    this.rulebase.addNamespacesFromDocument(this.inputDoc);

    this.genInput();
}


/**
 * Uses the given content as the output instance document.
 * @param xstoreName A string containing the name of the xstore the file is in.
 * @param collectionName A string containing the name of the collection the file is in.
 * @param documentName A string containing the name of the document to load up.
 *
 * @param content The XML DOM containing the content of the document to show.
 */
function setOutput(xstoreName, collectionName, documentName, xstoreId, collectionId, documentId, content)
{
    /*this.outputDoc = Sarissa.getDomDocument();
    this.outputDoc.resolveExternals = false;
    this.outputDoc.validateOnParse = false;
    this.outputDoc.loadXML(content.xml);*/
    this.outputDoc = getXMLDocument(content);
    Sarissa.setXpathNamespaces(this.outputDoc, this.rulebase.getNamespaceString());

    this.outputDetail.xstore = xstoreName;
    this.outputDetail.collection = collectionName;
    this.outputDetail.document = documentName;
    this.outputDetail.xstoreID = xstoreId;
    this.outputDetail.collectionID = collectionId;
    this.outputDetail.documentID = documentId;

    this.rulebase.addNamespacesFromDocument(this.outputDoc);

    this.genOutput();
}

/**
 * Allows the user to pick the document to show as the input file
 * using XStoreViewer.
 */
function chooseInput()
{
    document.xstoreViewerCaller.xml_permissions.value = document.getElementById('all_xstore_collections').value;

    //set the default collection to showm
    if (this.inputDetail.collection != null)
    {
        document.xstoreViewerCaller.chosen_collection_name.value = this.inputDetail.collection;
        document.xstoreViewerCaller.chosen_document_name.value = this.inputDetail.document;
    }
    else
    {
        document.xstoreViewerCaller.chosen_collection_name.value = document.getElementById('default_message_collection').value;
        document.xstoreViewerCaller.chosen_document_name.value = '';
    }


    document.xstoreViewerCaller.selection_mode.value = 'content';

    tempInstManager = this;
    tempInstType = 'input';

    window.xstoreViewer = instanceXStoreViewerCallback;
    window.open("", "xstoreViewerWindow", 'width=1000, height=600, toolbar=yes, menubar=no, directories=no, scrollbars=yes, resizable=yes, status=yes');
    document.xstoreViewerCaller.submit();
}


/**
 * Allows the user to pick the document to show as the output file
 * using XStoreViewer.
 */
function chooseOutput()
{
    document.xstoreViewerCaller.xml_permissions.value = document.getElementById('all_xstore_collections').value;

    //set the default collection to showm
    if (this.outputDetail.collection != null)
    {
        document.xstoreViewerCaller.chosen_collection_name.value = this.outputDetail.collection;
        document.xstoreViewerCaller.chosen_document_name.value = this.outputDetail.document;
    }
    else
    {
        document.xstoreViewerCaller.chosen_collection_name.value = document.getElementById('default_message_collection').value;
        document.xstoreViewerCaller.chosen_document_name.value = '';
    }

    document.xstoreViewerCaller.selection_mode.value = 'content';

    tempInstManager = this;
    tempInstType = 'output';

    window.xstoreViewer = instanceXStoreViewerCallback;
    window.open("", "xstoreViewerWindow", 'width=1000, height=600, toolbar=yes, menubar=no, directories=no, scrollbars=yes, resizable=yes, status=yes');
    document.xstoreViewerCaller.submit();
}



/** Stores the information about the instance to change when doing an xstore viewer call */
var tempInstManager = null;
var tempInstType = "";

/** XStoreViewer callback function used when selecting a document to display. */
function instanceXStoreViewerCallback(xstoreName, collectionName, documentName, xstoreId, collectionId, documentId, content)
{
    if (tempInstManager != null)
    {
        if (tempInstType == 'input')
        {
            tempInstManager.setInput(xstoreName, collectionName, documentName, xstoreId, collectionId, documentId, content);
        }
        else
        {
            tempInstManager.setOutput(xstoreName, collectionName, documentName, xstoreId, collectionId, documentId, content);
        }
    }
}





function loadInstanceDocuments()
{
    var URL;

    if (this.inputLoadNeeded)
    {
        tempInstManager = this;
        tempInstType = 'input';
        this.inputContainer.innerHTML = 'Loading document.  Please wait...';

        if ((this.inputDetail.xstoreID != null) &&
            (this.inputDetail.collectionID != null) &&
            (this.inputDetail.documentID != null))
            URL = '../xstoreViewer/xstoreviewer?action=call&operation=return_document&chosen_xstore_id='+this.inputDetail.xstoreID+'&chosen_collection_id=' + this.inputDetail.collectionID + '&chosen_document_id='+this.inputDetail.documentID;
        else //assume load by names
            URL = '../xstoreViewer/xstoreviewer?action=call&operation=return_document&chosen_xstore_name='+this.inputDetail.xstore+'&chosen_collection_name=' + this.inputDetail.collection + '&chosen_document_name='+this.inputDetail.document;

        xmlhttp = new XMLHttpRequest();
        xmlhttp.open("POST", URL, true);
        xmlhttp.onreadystatechange = instanceLoadStateChange;
        xmlhttp.send();

        this.inputLoadNeeded = false;
    }
    else if (this.outputLoadNeeded)
    {
        tempInstManager = this;
        tempInstType = 'output';

        this.outputContainer.innerHTML = 'Loading document.  Please wait...';

        if ((this.outputDetail.xstoreID != null) &&
            (this.outputDetail.collectionID != null) &&
            (this.outputDetail.documentID != null))
            URL = '../xstoreViewer/xstoreviewer?action=call&operation=return_document&chosen_xstore_id='+this.outputDetail.xstoreID+'&chosen_collection_id=' + this.outputDetail.collectionID + '&chosen_document_id='+this.outputDetail.documentID;
        else //assume load by names
            URL = '../xstoreViewer/xstoreviewer?action=call&operation=return_document&chosen_xstore_name='+this.outputDetail.xstore+'&chosen_collection_name=' + this.outputDetail.collection + '&chosen_document_name='+this.outputDetail.document;

        xmlhttp = new XMLHttpRequest();
        xmlhttp.open("POST", URL, true);
        xmlhttp.onreadystatechange = instanceLoadStateChange;
        xmlhttp.send();

        this.outputLoadNeeded = false;
    }
}


function instanceLoadStateChange()
{
    if ((xmlhttp != null) && (xmlhttp.readyState == 4))
    {
        //var responseDoc = xmlhttp.responseXML;
        /*var responseDoc = Sarissa.getDomDocument();
        responseDoc.loadXML(xmlhttp.responseText);*/
        var responseDoc = getXMLDocument(xmlhttp.responseText);

        if (responseDoc.selectSingleNode("/html/body/textarea[@id = 'content']/*") != null)
        {
            if (tempInstType == 'input')
            {
                /*tempInstManager.inputDoc = Sarissa.getDomDocument();
                tempInstManager.inputDoc.resolveExternals = false;
                tempInstManager.inputDoc.validateOnParse = false;
                tempInstManager.inputDoc.loadXML(responseDoc.selectSingleNode("/html/body/xml[@id = 'content']/*").xml);*/
                tempInstManager.inputDoc = getXMLDocument(responseDoc.selectSingleNode("/html/body/textarea[@id = 'content']/*"));
                Sarissa.setXpathNamespaces(tempInstManager.inputDoc, tempInstManager.rulebase.getNamespaceString());
                if (tempInstManager.inputDetail.importNamespaces == true)
                {
                    tempInstManager.rulebase.addNamespacesFromDocument(tempInstManager.inputDoc);
                    tempInstManager.inputDetail.importNamespaces = false;
                }
                tempInstManager.genInput();
            }
            else if (tempInstType == 'output')
            {
                /*tempInstManager.outputDoc = Sarissa.getDomDocument();
                tempInstManager.outputDoc.resolveExternals = false;
                tempInstManager.outputDoc.validateOnParse = false;
                tempInstManager.outputDoc.loadXML(responseDoc.selectSingleNode("/html/body/xml[@id = 'content']/*").xml);*/
                tempInstManager.outputDoc = getXMLDocument(responseDoc.selectSingleNode("/html/body/textarea[@id = 'content']/*"));
                Sarissa.setXpathNamespaces(tempInstManager.outputDoc, tempInstManager.rulebase.getNamespaceString());
                if (tempInstManager.outputDetail.importNamespaces == true)
                {
                    tempInstManager.rulebase.addNamespacesFromDocument(tempInstManager.outputDoc);
                    tempInstManager.outputDetail.importNamespaces = false;
                }
                tempInstManager.genOutput();
            }
        }
        tempInstManager.loadInstanceDocuments();
    }
}





/**
 * Called when saving the rulebase, to check that any current docuemnts will be maintained.
 * If the documents are from xstore, then we do not need to do anything here, but if they
 * are just temporary (ie passed through from the log viewer) we need to keep hold of the content
 * through the page refresh operation.
 */
function ensureMaintainDocuments()
{
    if (!((this.inputDetail.xstoreID != null) &&
          (this.inputDetail.collectionID != null) &&
          (this.inputDetail.documentID != null))
        &&
        !((this.inputDetail.xstore != null) &&
          (this.inputDetail.collection != null) &&
          (this.inputDetail.document != null))
        &&
        (this.inputDoc != null))
    {
        document.rulemaker.xml_input_document.value = xmlToString(this.inputDoc);
    }

    if (!((this.outputDetail.xstoreID != null) &&
          (this.outputDetail.collectionID != null) &&
          (this.outputDetail.documentID != null))
        &&
        !((this.outputDetail.xstore != null) &&
          (this.outputDetail.collection != null) &&
          (this.outputDetail.document != null))
        &&
        (this.outputDoc != null))
    {
        document.rulemaker.xml_output_document.value = xmlToString(this.outputDoc);
    }
}

/**
 * Saves the contents of the currently displayed input document to xstore.
 */
function saveInput()
{
    var newName = prompt("Please enter a name for this document", "");
    if (newName != null && newName != '')
    {
        if (newName.indexOf('.') == -1)
            newName += '.xml';

        tempInstManager = this;
        tempInstType = 'input';

        var URL = '../xstoreViewer/xstoreviewer';
        var content = 'action=call&operation=add_document&source_application=rulemaker&chosen_xstore_name='+xstoreViewerCaller.chosen_xstore_name.value+'&chosen_collection_name=' + encodeURI(default_message_collection.value) + '&uploaded_document_filename=' + newName;
        content += '&xml_uploaded_document=' + escape(xmlToString(this.inputDoc));

        //alert(content);

        xmlhttp = new XMLHttpRequest();
        xmlhttp.open("POST", URL, true);
        xmlhttp.onreadystatechange = instanceSaveStateChange;
        xmlhttp.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
        xmlhttp.setRequestHeader("Content-length", content.length);
        xmlhttp.setRequestHeader("Connection", "close");
        xmlhttp.send(content);
    }
}

/**
 * Saves the contents of the currently displayed output document to xstore.
 */
function saveOutput()
{
    var newName = prompt("Please enter a name for this document", "");
    if (newName != null && newName != '')
    {
        if (newName.indexOf('.') == -1)
            newName += '.xml';

        tempInstManager = this;
        tempInstType = 'output';

        var URL = '../xstoreViewer/xstoreviewer';
        var content = 'action=call&operation=add_document&source_application=rulemaker&chosen_xstore_name='+xstoreViewerCaller.chosen_xstore_name.value+'&chosen_collection_name=' + encodeURI(default_message_collection.value) + '&uploaded_document_filename=' + newName;
        content += '&xml_uploaded_document=' + escape(xmlToString(this.outputDoc));

        //alert(content);

        xmlhttp = new XMLHttpRequest();
        xmlhttp.open("POST", URL, true);
        xmlhttp.onreadystatechange = instanceSaveStateChange;
        xmlhttp.setRequestHeader("Content-type", "application/x-www-form-urlencoded");
        xmlhttp.setRequestHeader("Content-length", content.length);
        xmlhttp.setRequestHeader("Connection", "close");
        xmlhttp.send(content);
    }
}




function instanceSaveStateChange()
{
    if ((xmlhttp != null) && (xmlhttp.readyState == 4))
    {
        //var responseDoc = xmlhttp.responseXML;
        /*var responseDoc = Sarissa.getDomDocument();
        responseDoc.loadXML(xmlhttp.responseText);*/
        var responseDoc = getXMLDocument(xmlhttp.responseText);
        Sarissa.setXpathNamespaces(responseDoc, 'xmlns:xstore="http://www.hyfinity.com/xstore"');

        //alert(responseDoc.xml);

        var outcome = getElementText(responseDoc.selectSingleNode("/xstore:xstore/xstore:response/xstore:outcome"));
        var message = getElementText(responseDoc.selectSingleNode("/xstore:xstore/xstore:response/xstore:message"));

        if (tempInstType == 'input')
        {
            if (outcome == 'success')
            {
                document.getElementById('input_temp_message').style.display = 'none';
                tempInstManager.inputDetail.documentID = 'new document';
            }
            else if (message.indexOf('already exists') != -1)
            {
                alert('Your entered name already exists, please choose another');
                tempInstManager.saveInput();
            }
            else
            {
                alert('An unexpected error occurred:\n\n' + message + '\n\n');
            }

        }
        else if (tempInstType == 'output')
        {
            if (outcome == 'success')
            {
                document.getElementById('output_temp_message').style.display = 'none';
                tempInstManager.outputDetail.documentID = 'new document';
            }
            else if (message.indexOf('already exists') != -1)
            {
                alert('Your entered name already exists, please choose another');
                tempInstManager.saveOutput();
            }
            else
            {
                alert('An unexpected error occurred:\n\n' + message + '\n\n');
            }
        }
    }

}




//******************************** Utility Functions ***************************************


var xmlhttp;


/**
 * Convenience method for searching an array for unique names.
 */
function arrayContains(array, value)
{
    var found = false;
    for (var i = 0; i < array.length; ++i)
    {
        if (array[i] == value)
        {
            found = true;
            break;
        }
    }
    return found;
}




/**
 * Drag and drop functions - Dont really belong here!!!!!!!!
 */


//stores the current drag mode - stop - no dragging, move - xpath being dragged, toolSelect - A toolbar option being dragged
var moveMode = 'stop';
//stores the value of the currently dragged component
var moveValue = '';
//represents the HTML container used for the dragging tooltip.  NOTE: This needs to be initialised in an onload function
var moveBox;

var moveTeeeType;

function mouseMove(e, base)
{
    if (moveMode != 'stop')
    {
        if (moveBox.style.visibility == 'visible')
        {
            var coords = getMouseCoords(e, base);
            moveBox.style.top = coords.y + 'px';
            moveBox.style.left = (Number(coords.x) + Number(10)) + 'px';
        }
    }
    dividerMove(e, base);
}
function treeviewMouseDown(value, type, evt)
{
    //alert('mouse Down in xpathBindingsScripts');
    scrollableDivManager.scrollEnabled = true;
    moveMode = 'move';
    moveValue = value;
    moveTeeeType = type;
    moveBox.style.visibility = 'visible';
    var coords = getMouseCoords(evt);
    moveBox.style.top = coords.y + 'px';
    moveBox.style.left = (Number(coords.x) + Number(10)) + 'px';
    if (value.length > 30)
        moveBox.innerHTML = value.substr(0, 30)+'...';
    else
        moveBox.innerHTML = value;

    // cancel out any text selections
    document.body.focus();
    // prevent text selection in IE
    document.onselectstart = function () { return false; };
    // prevent IE from trying to drag an image
    moveBox.ondragstart = function() { return false; };
    // prevent text selection (except IE)
    return false;
}

function mouseUpCancel(e)
{
    //alert('mouse up cancel');
    if (moveMode != 'stop')
    {
        moveMode = 'stop';
        moveBox.style.visibility = 'hidden';

        document.onselectstart = null;
        moveBox.ondragstart = null;
        scrollableDivManager.scrollEnabled = false;
    }

    dividerReleased(e);
}


function saveXPath(container, context)
{
    if (moveMode == 'move')
    {
        moveMode = 'stop';
        moveBox.style.visibility = 'hidden';

        document.onselectstart = null;
        moveBox.ondragstart = null;
        scrollableDivManager.scrollEnabled = false;

        if (typeof(context) != 'undefined')
        {
            if (moveValue.indexOf(context) == 0)
            {
                if (context == moveValue)
                    container.value = '.';
                else
                    container.value = moveValue.substr(context.length+1);
            }
            else
            {
                container.value = moveValue;
            }
        }
        else
            container.value = moveValue;

        container.onchange();
    }
}

function saveXMLFragment(container)
{
    if (moveMode == 'move')
    {
        moveMode = 'stop';
        moveBox.style.visibility = 'hidden';

        document.onselectstart = null;
        moveBox.ondragstart = null;
        scrollableDivManager.scrollEnabled = false;

        var valueToSet = ''
        if (moveTeeeType == 'input_document')
        {
            //tiDoc.setProperty("SelectionNamespaces", hyf.ns.findNamespaceString());
            var selectedNode = rulebase2.instanceManager.inputDoc.selectSingleNode(moveValue);
            valueToSet = xmlToString(selectedNode);
        }
        else
        {
            //xiDoc.setProperty("SelectionNamespaces", hyf.ns.findNamespaceString());
            var selectedNode = rulebase2.instanceManager.outputDoc.selectSingleNode(moveValue);
            valueToSet = xmlToString(selectedNode);
        }

        if (editAreas[container.id])
            editAreaLoader.setValue(container.id, valueToSet);
        else
            container.value = valueToSet;

        container.onchange();
    }
}

document.onmouseup = mouseUpCancel;
document.onmousemove = mouseMove;
