//defines the distance that is maintained between links drawn between the same two elements.
var offsetAmount = 3;


/**
 * Link object
 * Encapsulates a link in the formmaker application XML, provides utility methods to
 * output SVG and Formmaker XML.
 *
 * @param container The parent object of this link
 * @param source The object this link originates from
 * @param target Optional, object this link points to
 */
function Link(id, data, container, source, target)
{
    //alert('in link');
    //alert(id + ', ' + data + ', ' + source._id + ', ' + target._id);
    this._id = id;
    this._container = container;
    this._source = source;
    this._source.addOutgoingLink(this);
    this._mousedown = false;
    this._defaults = 'hidden';

    //this._action = "";
    this._action = id;
    this._conditionXpath = "";
    this._submissionMethod = "form";
    this._nodePath = this._container._projectInfo;
    this._painterNode = "";
    this._painterNodeID = "";
    this._painterPattern = "";

    /** set up the information used for the action binding for from page links. */
    this._sourceFile = '';
    this._sourceType = container._defaults._sourceType;
    this._rootElement = '';
    this._requestOperation = '';
    this._fileContent = null;

    this._instanceType = 'static';

    /** Used to store the details of the structure document selected for a from page link. */
    this._dataFromForm = new Object();
    this._dataFromForm._value = "";
    this._dataFromForm._xstore = "";
    this._dataFromForm._collection = "";

    //Stores the XML string of the data sources listed against this link
    //this is just a quick change to ensure the data doesnt get lost
    //eventuially we will probabaly want to parse this information and allow editing of it.
    this._dataSourcesXML = "";


    this._hasChanges = false;



    /** The offset to use when drawing this link. */
    this._drawOffset = 0;

    /** Stores the rule in the source handler's controller logicsheet that represents this link.
     *  This is used to show the most uptodate condition xpath. */
    this._conditionRule = null;

    /** Will hold the XML http request object needed to retrieve the rule for this link. */
    this._ruleXMLHttp = null;

    if(typeof(target) != 'undefined')
    {
        this._target = target;
        this._target.addIncomingLink(this);

        if((typeof(this._source) != 'undefined') && (this._source.constructor == Page))
        {
            this._painterNode = this._container._defaults._painterNode;
        }
    }
    else
        this._target = undefined;
    if ((data != null) & (typeof(data) != 'undefined'))
    {
        var sourceTypeNode = data.selectSingleNode("fm:source_type");
        if(sourceTypeNode != null) {
          this._sourceType = getElementText(sourceTypeNode);
        }

        var sourceFile = data.selectSingleNode("fm:source_file");
        if(sourceFile != null) {
          this._sourceFile = getElementText(sourceFile);
        }

        var rootElement = data.selectSingleNode("fm:root_element");
        if(rootElement != null) {
          this._rootElement = getElementText(rootElement);
        }
        var requestOperation = data.selectSingleNode("fm:request_operation");
        if(requestOperation != null) {
          this._requestOperation = getElementText(requestOperation);
        }

        var actionNode = data.selectSingleNode("fm:action");
        if(actionNode != null) {
            this._action = getElementText(actionNode);
        }

        var conditionXpathNode = data.selectSingleNode("fm:condition_xpath");
        if(conditionXpathNode != null) {
            this._conditionXpath = getElementText(conditionXpathNode);
        }

        var painterNode_node = data.selectSingleNode("fm:painter_node");
        if(painterNode_node != null)
            this._painterNode = getElementText(painterNode_node);
        //alert(this._painterNode.text);

        if ((this._source.constructor == Page)
              && (this._target.constructor == Page && this._target._type == 'subsection'))
        {
            this._submissionMethod = "subsection";
        }
        if(this._target.constructor == Handler && this._target._type == 'thirdparty')
        {
            this._submissionMethod = "values";
        }

        var submissionMethodNode = data.selectSingleNode("fm:submission_method");
        if(submissionMethodNode != null)
          this._submissionMethod = getElementText(submissionMethodNode);

        var instanceTypeNode = data.selectSingleNode("fm:instance_type");
        if(instanceTypeNode != null)
          this._instanceType = getElementText(instanceTypeNode);

        var dataFromFormNode = data.selectSingleNode("fm:data_from_form");
        if(dataFromFormNode != null) {
          this._dataFromForm._value = getElementText(dataFromFormNode);

          var xstoreAttribute = dataFromFormNode.selectSingleNode("@xstore");
          if(xstoreAttribute != null) {
            this._dataFromForm._xstore = getElementText(xstoreAttribute);
          }

          var collectionAttribute = dataFromFormNode.selectSingleNode("@collection");
          if(collectionAttribute != null) {
            this._dataFromForm._collection = getElementText(collectionAttribute);
          }
        }

        var dataSources = data.selectSingleNode("fm:data_sources");
        if(dataSources != null)
        {
            this._dataSourcesXML = xmlToString(dataSources);
        }
    }

    if (this._painterNode != null)
    {
        var nodeArray = this._painterNode.split('-');
        this._painterNodeID = nodeArray[nodeArray.length - 1];
        this._painterPattern = nodeArray[nodeArray.length - 2];
    }
  //alert(typeof(data));
  //alert(data.xml);

  /*
   * Setup class methods
   */
  if (typeof(__LinkPrototypeCalled) == 'undefined')
  {
    __LinkPrototypeCalled = true;
    Link.prototype.remove = remove;
    Link.prototype.rename = rename;
    Link.prototype.processNewName = processNewName;
    Link.prototype.draw = draw;
    Link.prototype.redraw = redraw;
    Link.prototype.getJavascriptIdentifier = getJavascriptIdentifier;
    Link.prototype.onclick = onclick;
    Link.prototype.onmouseup = onmouseup;
    Link.prototype.addHighlight = addHighlight;
    Link.prototype.removeHighlight = removeHighlight;
    Link.prototype.onmousemove = onmousemove;
    Link.prototype.onmouseover = onmouseover;
    Link.prototype.onmouseout = onmouseout;
    Link.prototype.getApplicationXML = getApplicationXML;
    Link.prototype.getNodeInfo = getNodeInfo;
    Link.prototype.toggleLinkDefaults = toggleLinkDefaults;

    Link.prototype.displayContextPanel = displayContextPanel;

    Link.prototype.calculateOffset = calculateOffset;
    Link.prototype.convertOffsetAmount = convertOffsetAmount;

    Link.prototype.removeExistingRule = removeExistingRule;
    Link.prototype.removeSplice = removeSplice;
    Link.prototype.findConditionRule = findConditionRule;
    Link.prototype.conditionRuleLoaded = conditionRuleLoaded;

    Link.prototype.sourceTypeChanged = linkSourceTypeChanged;
    Link.prototype.sourceFileSelected = linkSourceFileSelected;
    Link.prototype.removeStructureFile = removeLinkStructureFile;
    Link.prototype.loadStructureFile = loadStructureFile;
    Link.prototype.structureLoaded = structureLoaded;
    Link.prototype.checkBindingMethod = checkBindingMethod;

    Link.prototype.removeInstance = removeInstance;
    Link.prototype.checkInstanceType = checkLinkInstanceType;

    Link.prototype.setChanged = setChanged;
    Link.prototype.clearChanged = clearChanged;

    Link.prototype.showActionSources = showActionSources;
  }

  /**
   * Removes this link from the diagram and detaches it from the objects it links to
   */
  function remove(redraw)
    {
    //alert('removing link ' + this._id);

    this._container.storeActionDelete(this._action, this._source, this._dataFromForm);

    this._container.setSelected(undefined);
    if(typeof(this._group) != 'undefined')
        {
      this._container.getSVGLinksContext().removeChild(this._group);
    }

    if(typeof(this._target) != 'undefined')
        {
        this._target.removeIncomingLink(this);

        if ((this._source.constructor == Handler) & (this._source._node != ""))
        {
          if (this._target.constructor == Page)
              this.removeExistingRule('page');
          else if ((this._target.constructor == Handler) & (this._target._node != ""))
              this.removeExistingRule('handler');
        }
        else
        {
            var painterNodes = 0;
            for (linkIndex in this._container._links)
            {
                //alert(this._container._links[linkIndex]._id + ', ' + this._container._links[linkIndex]._painterNode);
                if ((this._container._links[linkIndex]._painterNode == this._painterNode))
                    painterNodes++;
            }
            if ((this._target.constructor == Handler) & (this._target._node != "") & (painterNodes == 1))
                this.removeSplice();
        }
    }

    this._source.removeOutgoingLink(this);

    //Remove link object
    for (link in this._container._links)
    {
        var deletedLink = this._container._links[link];
        if (deletedLink == this)
            delete this._container._links[link];
    }

    if ((typeof(redraw) == 'undefined') || (redraw))
    {
      this._source.redraw();
      if(typeof(this._target) != 'undefined')
            {
        this._target.redraw();
      }
    }

    removeToolTip();
  }

  /**
   * Create SVG for the link and attach it to the containers SVG context
   */
  function draw()
  {

    //alert("drawing link " + this._id);

    //update draw offset
    this._drawOffset = this.calculateOffset();

    SVGDocument = this._container.getSVGDocument();
    SVGContext = this._container.getSVGLinksContext();
    var line = SVGDocument.createElementNS(SVG_NS, 'line');

    if (dojo.isWebKit) //chrome has issues with stroke widths less than 1
        line.setAttribute('stroke-width', 1);
    else
        line.setAttribute('stroke-width', 0.6);

    if((typeof(this._target) == 'undefined'))
    {
        line.setAttribute('style', 'stroke:silver');
    }
    else if(this._submissionMethod == 'subsection')
    {
        line.setAttribute('style', 'stroke:silver;stroke-dasharray:5,2;fill:none');
    }
    else
    {
        line.setAttribute('style', 'stroke:silver');
    }

    /*
        //use different colour for different link type
        if((typeof(this._target) == 'undefined'))
        {
            line.setAttribute('style', 'stroke:gray');
        }
        else if((this._target._type == "normal" || this._target._type == "subsection"))
        {
            line.setAttribute('style', 'stroke:gray');
        }
        else
        {
            line.setAttribute('style', 'stroke:pink;stroke-dasharray:5,2');
        }*/
    line.setAttribute('marker-end', 'url(#Triangle)');
    line.setAttribute('x1', this._source._x + this._drawOffset.x);
    line.setAttribute('y1', this._source._y + this._drawOffset.y);

    //If target exists attach line end to it, otherwise just link it to the source
    if(typeof(this._target) != 'undefined')
    {
      line.setAttribute('x2', this._target._x + this._drawOffset.x);
      line.setAttribute('y2', this._target._y + this._drawOffset.y);

    }
    else
    {
      line.setAttribute('x2', this._source._x + this._drawOffset.x);
      line.setAttribute('y2', this._source._y + this._drawOffset.y);
    }

    //Group
    var group = SVGDocument.createElementNS(SVG_NS, 'g');
    group.setAttribute('id', this._name);
    group.setAttribute('onclick', "handleMouseClick('" +this.getJavascriptIdentifier() + "', evt)" );
    group.setAttribute('onmousedown', "handleMouseDown('" +this.getJavascriptIdentifier() + "', evt)" );
    group.setAttribute('onmousemove', 'parent.' + this.getJavascriptIdentifier() + '.onmousemove(evt)' );
    group.setAttribute('onmouseup', "handleMouseUp('" +this.getJavascriptIdentifier() + "', evt)" );
    group.setAttribute('onmouseover', 'parent.' + this.getJavascriptIdentifier() + '.onmouseover(evt)');
    group.setAttribute('onmouseout', 'parent.' + this.getJavascriptIdentifier() + '.onmouseout(evt)');


    //create an invisble rectangle for firefox 4 to position under the line as the mouse events do not seem to always work otherwise
    if (dojo.isFF >= 4)
    {
        var rect = SVGDocument.createElementNS(SVG_NS, 'rect');
        rect.setAttribute('x', line.getAttribute('x1'));
        rect.setAttribute('y', line.getAttribute('y1') - 0.3);
        var xDiff = line.getAttribute('x2') - line.getAttribute('x1');
        var yDiff = line.getAttribute('y2') - line.getAttribute('y1');
        var lineLength = Math.sqrt((xDiff * xDiff) + (yDiff * yDiff));
        rect.setAttribute('width', lineLength);
        rect.setAttribute('height', 0.6);
        var angle = (180/Math.PI) * Math.atan(yDiff/xDiff);

        if (xDiff < 0)
            angle += 180;

        rect.setAttribute('transform', 'rotate('+angle+', '+line.getAttribute('x1')+', '+line.getAttribute('y1')+')');
        rect.setAttribute('fill', 'white');
        group.appendChild(rect);

    }
    group.appendChild(line);




    SVGContext.appendChild(group);

    this._group = group;
    this._line = line;

    //check if this link should still be highlighted
    if (this._container._selected == this)
    {
        this.addHighlight();
    }

  }

  /** Calculates the offset at which to draw this line.
   * This value is found from the number of links that need to be shown
   * between the source and target of this link.
   *
   * @return the offset value to use.
   */
  function calculateOffset()
  {
    //initialise the new offset to 0
    var newOffset = new Object();
    newOffset.x = 0;
    newOffset.y = 0;

    if (typeof(this._target) != 'undefined')
    {
      var sourceType = objectType(this._source);
      var targetType = objectType(this._target);
      var sourceName = this._source._name;
      var targetName = this._target._name;

      var positionNum = 0;

      var relevantLinks = new Array();
      for(link in this._container._links)
      {
        var link = this._container._links[link];
        var testSourceType = objectType(link._source);
        var testTargetType = objectType(link._target);
        var testSourceName = link._source._name;
        var testTargetName = link._target._name;

        if (((testSourceType == sourceType) && (testSourceName == sourceName) &&
             (testTargetType == targetType) && (testTargetName == targetName)) ||
            ((testSourceType == targetType) && (testSourceName == targetName) &&
             (testTargetType == sourceType) && (testTargetName == sourceName)))
        {
          relevantLinks[relevantLinks.length] = link;
        }
      }

      for (var j = 0; j < relevantLinks.length; ++j)
      {
        if (relevantLinks[j]._id == this._id)
        {
          positionNum = j;
          break;
        }
      }

      var offset = 0;
      var offsetDirection = 1;

      if (positionNum != 0)
      {
        var multiplier = Math.ceil(positionNum/2);
        offset = offsetAmount * multiplier;
        if ((positionNum%2) == 1)
        {
          offset = offset * -1;
        }

        //if this link does not have the same source as the first relevant link, then assume it
        //is going the other way, and so the offset distance needs to be inverted.
        if (!((relevantLinks[positionNum]._source._name == relevantLinks[0]._source._name) &&
            (objectType(relevantLinks[positionNum]._source) == objectType(relevantLinks[0]._source))))
        {
          offsetDirection = -1;
        }
      }
      //if there is an even number of links, then they all need to be offset slightly to center them in relation to the icons
      if ((relevantLinks.length%2) == 0)
      {
          offset = offset + (offsetAmount/2);
      }
      if (offset != 0)
          this.convertOffsetAmount(newOffset, offset * offsetDirection);
    }
    return newOffset;
  }


  /** converts the given offset distance into the x and y offsets needed to achieve it at the required angle. */
  function convertOffsetAmount(offsetResult, distance)
  {
    var xdiff = this._source._x - this._target._x;
    var ydiff = this._source._y - this._target._y;
    var angle = Math.atan(xdiff/ydiff)

    var xsign = -1;
    var ysign = 1;
    //alert(angle);
    if (ydiff >= 0)
    {
      xsign = 1;
      ysign = -1;
    }

    offsetResult.y = (Math.sin(angle) * distance) * ysign;
    offsetResult.x = (Math.cos(angle) * distance) * xsign;

    //alert("distance: " + distance + "   " + offsetResult.x + ", " + offsetResult.y);
  }

  /**
   * Recreate SVG and attach it to containers SVG context
   */
  function redraw()
    {
    if(typeof(this._group) != 'undefined')
        {
      this._container.getSVGLinksContext().removeChild(this._group);
    }
    this.draw();
  }

  /**
   * Returns a string of javascript that identifies this link
   */
  function getJavascriptIdentifier()
    {
    //Use the source of the link for identification, as is always present
    var index;
    for(var i = 0; i < this._source._outgoingLinks.length; ++i)
        {
      if(this == this._source._outgoingLinks[i])
            {
        index = i;
                break;
      }
    }

    return this._source.getJavascriptIdentifier() + "._outgoingLinks[" + index + "]";
  }

  /**
   * Onclick event handler for this line
   */
  function onclick(event)
  {
    var mode = this._container._mode;
    //alert('test on click');
    //alert(mode);
    switch(mode)
    {
        case 'select' :
                if (this._container._selected != this)
                {
                    this._container.setSelected(this);
                }
                break;
        case 'rename' :
                this.rename();
                break;
        case 'delete' :
                this.remove();
                break;
    }
    if (event.detail == 2)
        this.rename();
    if (this._container._dblclick == false)
        this._container.setMode('select');
  }

  function removeExistingRule(type)
  {
      //alert('deleting rule to ' + this._target._id);
      var typeString = "";

      if (type == 'page')
          typeString = 'write_targetPage_' + this._target._id + '_rule';
      else
      {
          /**
           * NOTE: If the rule naming is changed for this process, we also need to update
           * the formmaker part of the delete component service, and maybe other areas!!!
           * This will also cause problems with apps created using the old names as they will
           * no longer have the rules removed if we update this, so prob best to not change these names!!!
           */
          var sourceNode = this._source._pattern + '-' + this._source._nodeID;
          var targetNode = this._target._pattern + '-' + this._target._nodeID;
          typeString = 'call_' + targetNode + '_from_' + sourceNode + '_rule';
      }

      //alert(typeString);
      xmlhttp.open("POST", 'FormMaker?action=process&operation1=remove_existing_rule'
                       //+ '&xstore=blueprints&collection=mvcuser/mvc/Example/logicsheet_pool&document='
                       //+ this._source._pattern + '_' + this._source._nodeID + '_rules.xml'
                       + '&pattern_id=' + this._source._pattern + '&node_id=' + this._source._nodeID
                       + '&rule_id=' + typeString, false);
      xmlhttp.onreadystatechange = showOutcome;
      xmlhttp.send(null);
  }

  function removeSplice()
  {
      xmlhttp.open("POST", 'FormMaker?action=process&operation1=remove_splice&source_pattern_id='+ this._painterPattern
                     + '&source_node_id='+ this._painterNodeID + '&target_pattern_id='+ this._target._pattern
                     + '&target_node_id='+ this._target._nodeID, false);
      xmlhttp.onreadystatechange = showOutcome;
      xmlhttp.send(null);
  }

  /**
   * Onmouseup event handler
   */
  function onmouseup(event)
    {
    //Delegate to the containers event handler
    this._container.onmouseup(event);
  }

  /**
   * On mouse move event handler for this link
   */
  function onmousemove(event)
    {
    this._container._svgCoordinates.reset();
    if(this._moving == true)
        {
      //Need to offset coordinates slightly to prevent erratic mouse events being passed to the line
      this._line.setAttribute('x2', this._container._svgCoordinates.getUserSpaceX(event.clientX) - 3);
      this._line.setAttribute('y2', this._container._svgCoordinates.getUserSpaceY(event.clientY) - 3);
    }
    //if (toolTipVisible())
    if (toolTipVisible(this._id))
    {
      moveToolTip(this._container._svgCoordinates.getUserSpaceX(event.clientX),
                  this._container._svgCoordinates.getUserSpaceY(event.clientY));
    }
  }

  /**
   * On mouse over event handler for this link.
   * Just shows tooltip popup.
   */
  function onmouseover(event)
  {
    if (this._container._displayType == 'mvc')
    {
        this._container._svgCoordinates.reset();
        var x = this._container._svgCoordinates.getUserSpaceX(event.clientX);
        var y = this._container._svgCoordinates.getUserSpaceY(event.clientY);
        var tipArray = new Array("Action: " + this._action);
        var svgDoc = this._container._embeddedSVG;
        //newToolTip(x, y, tipArray, svgDoc);
        if (toolTipVisible(this._id) == false)
            newToolTip(x, y, tipArray, svgDoc, this._id);
    }
  }

  /**
   * On mouse out event handler for this link.
   * hides tooltip popup if needed.
   */
  function onmouseout(event)
  {
      if (toolTipVisible(this._id))
        removeToolTip();
  }

  /**
   * Adds a highlight to the link
   */
  function addHighlight()
    {
    SVGDocument = this._container.getSVGDocument();

    var highlight = SVGDocument.createElementNS(SVG_NS, 'line');
    highlight.setAttribute('stroke-width', 1.5);
    highlight.setAttribute('opacity', 0.4);
    highlight.setAttribute('stroke', 'red');
    highlight.setAttribute('x1', this._source._x + this._drawOffset.x);
    highlight.setAttribute('y1', this._source._y + this._drawOffset.y);
    highlight.setAttribute('x2', this._target._x + this._drawOffset.x);
    highlight.setAttribute('y2', this._target._y + this._drawOffset.y);
    highlight.setAttribute('id', 'highlight');

    this._group.appendChild(highlight);
  }

  /**
   * Removes a highlight from the link
   */
  function removeHighlight()
    {
    for(var i = 0; i < this._group.childNodes.length; ++i)
        {
      if(this._group.childNodes.item(i).getAttribute('id') == 'highlight')
            {
        this._group.removeChild(this._group.childNodes.item(i));
      }
    }
  }

  function toggleLinkDefaults()
  {
      if (this._defaults == 'visible')
      {
          hideComponent(document.getElementById("condition_xpath"));
          //hideComponent(document.getElementById("default_painter_node_header"));
          hideComponent(document.getElementById("default_painter_node"));
          hideComponent(document.getElementById("default_painter_button"));
          hideComponent(document.getElementById("default_submission_method"));
          this._defaults = 'hidden';
          setTextContent(document.getElementById("show_link_defaults"), 'Show Advanced Options');
      }
      else
      {
          showComponent(document.getElementById("condition_xpath"));
          //showComponent(document.getElementById("default_painter_node_header"));
          showComponent(document.getElementById("default_painter_node"));
          showComponent(document.getElementById("default_painter_button"));
          showComponent(document.getElementById("default_submission_method"));
          this._defaults = 'visible';
          setTextContent(document.getElementById("show_link_defaults"), 'Hide Advanced Options');
      }
  }

  /* Gets value of link painter node before screen transform*/
  function getNodeInfo(nodeName)
  {
    //alert('in Link: ' + nodeName);
    if (nodeName == "")
    {
        this._painterNode = "";
        this._painterNodeID = "";
        this._painterPattern = "";
    }
    else
    {
        //alert(this._nodePath);
        var nodeArray = nodeName.split('-');
        this._painterPattern = nodeArray[0];
        this._painterNodeID = nodeArray[1];
        var node = this._nodePath + '-' + this._painterPattern + '-' + this._painterNodeID;
        this._painterNode = node;
    }

    if (this._container._defaults._painterNode == "")
        this._container._defaults._painterNode = node;
/*     alert(this._container._defaults._painterNode);
    if ((this._container._defaults._painterNode == "") || ((node != null) & (nodeChecker(this._container_defaults._painterNode, 'View') == 'false')))
        this._container._defaults._painterNode = node;
    //else
    //    this._container._defaults._painterNode = "";
 */

    this.setChanged();
  }

   /**
   * Serializes the state of the diagram to application XML (as a string)
   */
  function getApplicationXML()
  {
    //alert('in link getAppXML');
    var xml = "<link xmlns='http://www.hyfinity.com/formmaker' ";
    var sourceType = objectType(this._source);
    var targetType = objectType(this._target);
    xml += "id='" + this._id + "'>";
    xml += "<source type='" + sourceType + "'>" + this._source._id + "</source>";
    xml += "<target type='" + targetType + "'>" + this._target._id + "</target>";

    xml += "<action>" + escapeXMLCharacters(this._action) + "</action>";

    if (this._source.constructor == Handler)
    {
        if (this._conditionXpath == "")
            this._conditionXpath = "/";
        xml += "<condition_xpath>" + escapeXMLCharacters(this._conditionXpath) + "</condition_xpath>";
    }

    if ((this._submissionMethod != "") &&
        ((this._source.constructor == Page && this._target.constructor == Handler) ||
         (this._source.constructor == Page && this._target.constructor==Page && this._target._type== 'subsection')))
    {
        xml += "<submission_method>" + this._submissionMethod + "</submission_method>";
    }


    if (this._source.constructor == Page)
    {
        if ((this._painterNode != "") && (this._painterNode != this._container._defaults._painterNode))
            xml += "<painter_node>" + this._painterNode + "</painter_node>";

        /*xml += "<source_type>" + this._sourceType + "</source_type>";
        if(this._sourceFile != '')
        {
            xml += "<source_file>" + this._sourceFile + "</source_file>";
        }

        if (this._sourceType == 'wsdl')
        {
            xml += "<request_operation>" + escapeXMLCharacters(this._requestOperation) + "</request_operation>";
        }
        else
        {
            xml += "<root_element>" + escapeXMLCharacters(this._rootElement) + "</root_element>";
        }*/

        xml += "<data_from_form ";
        xml += "xstore='" + this._dataFromForm._xstore + "' ";
        xml += "collection='" + this._dataFromForm._collection + "' ";
        xml += ">" + this._dataFromForm._value + "</data_from_form>";

        if (this._instanceType != '')
        {
            xml += "<instance_type>" + this._instanceType + "</instance_type>";
        }
    }

    xml += this._dataSourcesXML;

    xml += "</link>";
    return xml;
  }

  function displayContextPanel()
  {
    var controllerNodeID = this._target._nodeID;
    var controllerPattern = this._target._pattern;
    var handlerCheck = true;

    //set form_submission method to be 'subsection' if required
    if(this._target.constructor == Page && this._target._type=="subsection")
    {
        this._submissionMethod = "subsection";
    }

    if (this._conditionRule == null)
    {
        this.findConditionRule();
    }


    if ((this._source.constructor == Page) && (nodeChecker(this._painterNode, 'View') == 'false'))
        this.getNodeInfo("");


    for (index in this._container._handlers)
    {
        var handler = this._container._handlers[index];

        if (((handler._id == this._source._id) | (handler._id == this._target._id)) & ((handler._node == "") || (nodeChecker(handler._node, 'Controller') == 'false')))
        {
            this._container._handlers[index]._node = '';
            handlerCheck = 'false';
        }
    }

    this._container._linkTransform.addParameter('link_javascript_identifier', this.getJavascriptIdentifier());
    this._container._linkTransform.addParameter('link_id', this._id);
    this._container._linkTransform.addParameter('action', this._action);
    this._container._linkTransform.addParameter('painter_node', this._painterPattern + '-' + this._painterNodeID);
    //this._container._linkTransform.addParameter('view_node_list', viewNodeList);
    this._container._linkTransform.addParameter('handler_check', handlerCheck);
    this._container._linkTransform.addParameter('default_source_type',this._container._defaults._sourceType);
    //add the structure file content as a parameter if present
    /*if (this._fileContent != null)
    {
        this._linkTransform.addParameter('source_file', this._fileContent);
    }
    else
    {
        this._linkTransform.addParameter('source_file', null);
        //need to try and get the content and then refresh the display.
        this.loadStructureFile();

    }*/

    if (this._conditionRule != null)
        this._container._linkTransform.addParameter('condition_rule', this._conditionRule);

    var xml = '<container><view_nodes>' + xmlToString(viewNodeList) + '</view_nodes><application>' + this._container.getApplicationXML() + '</application></container>';
    this._container._linkTransform.transform(getXMLDocument(xml));
    //document.result_form.xml_textArea.value = xml;

    if (this._defaults == 'hidden')
        this._defaults = 'visible';
    else
        this._defaults = 'hidden';
    this.toggleLinkDefaults();

    this.checkInstanceType();
    if(this._source.constructor == Page)
    {
        this.checkBindingMethod();
    }

    //check if we need to update the selected root element.
    if ((this._sourceType == 'xsd') && (this._rootElement == '') &&
        (document.result_form.link__root_element != null) &&
        (document.result_form.link__root_element.type == 'select-one') &&
        (document.result_form.link__root_element.options.length == 1))
    {
        this._rootElement = document.result_form.link__root_element.value;
    }



    //try and put the focus in the action name box
    //the delay seems to be needed, as otherwise it does not seem to work when clicking from one link to another.
    //perhaps due to the focus actually being put on the previously displayed text box for the old action?????
    //window.setTimeout("if (document.result_form.action_name) { document.result_form.action_name.focus(); }", 400);

  }

  /**
   * Makes a call to the server to retrieve the rule that defines this link
   * in the runtime application.
   * This only applies to links originating from a handler.
   */
  function findConditionRule()
  {
      if (this._source.constructor == Handler)
      {
          var pattern_id = this._source._pattern;
          var node_id = this._source._nodeID;
          var rule_id = "";

          //This assumes the approach taken in the backend when constructing the rule ids in the first place
          // and therefore must be updated if they are changed.
          if (this._target.constructor == Page)
          {
              rule_id = "write_targetPage_" + this._target._id + "_rule";
          }
          else
          {
              rule_id = "call_" + this._target._pattern + "-" + this._target._nodeID + "_from_" + pattern_id + "-" + node_id + "_rule";
          }

          var queryString = "FormMaker?action=process&operation1=get_rule&rule_id=" + rule_id + "&pattern_id=" + pattern_id + "&node_id=" + node_id;

          this._ruleXMLHttp = new XMLHttpRequest();
          this._ruleXMLHttp.open("POST", queryString, true);
          this._ruleXMLHttp.onreadystatechange = new Function(this.getJavascriptIdentifier() + ".conditionRuleLoaded();");
          this._ruleXMLHttp.send(null);
      }
  }


  /**
   * Called when the http request to load the rule ahs changed state.
   * This is used to retrieve teh rule XML, and refresh the display of the context panel.
   */
  function conditionRuleLoaded()
  {
      if (this._ruleXMLHttp.readyState == 4)
      {
          this._conditionRule = getXMLDocument(this._ruleXMLHttp.responseText);
          Sarissa.setXpathNamespaces(this._conditionRule, "xmlns:xfact='http://www.hyfinity.com/xfactory' xmlns:fm='http://www.hyfinity.com/formmaker' xmlns:hy='http://www.hyfinity.com/xengine'");

          //need to update the displayed condition details
          if (this._container._selected == this)
          {
              this._conditionXpath = document.result_form.condition_xpath_input.value;
              this._action = document.result_form.action_name.value;
              //QUESTION: Is it ok to just refresh the whole panel, or should we only update the condtion part?
              this.displayContextPanel();

              //update the condition xpath stored in FM if it has been overriden on the rule
              if (this._conditionRule != null)
              {
                  var cond = this._conditionRule.selectSingleNode("/hy:rule[@base != 'pattern']/hy:body//hy:clause/hy:target/hy:params/hy:param");
                  if (cond != null)
                  {
                      this._conditionXpath = getElementText(cond);
                  }
              }
          }
      }
  }

  /** Called when the value in the source type drop down is changed.
   * This needs to store the new value, and then refresh the display
   * to ensure the correct options are available.
   */
  function linkSourceTypeChanged(newType)
  {
    this._sourceType = newType;
    this._sourceFile = "";
    this._rootElement = "";
    this._requestOperation = "";
    this._fileContent = null;

    this.displayContextPanel();
    this.setChanged();
  }

  /**
  * Called when a source file has been selected by XStoreViewer
  * @param name the name of the document selected
  * @param content an XML DOM containing the contents of the selected file.
  */
  function linkSourceFileSelected(name, content)
  {
    this._rootElement = "";
    this._requestOperation = "";
    this._sourceFile = name;
    this._fileContent = getXMLDocument(content);

    this.displayContextPanel();
    this.setChanged();
  }

  /**
   * Removes the structure file association for this form.
   */
  function removeLinkStructureFile()
  {
    this._sourceFile = "";
    this._rootElement = "";
    this._requestOperation = "";
    this._fileContent = null;

    this.displayContextPanel();
    this.setChanged();
  }


  function checkLinkInstanceType()
  {
      if (this._source.constructor == Page)
      {
          /*var bindingChecked = document.getElementById('link_data_binding_flag').checked;

          if (bindingChecked)
          {
              showComponent(document.getElementById('link_binding_method_row'));
              showComponent(document.getElementById('static_binding_structure_title'));
              showComponent(document.getElementById('static_binding_generate_document_row'));
              showComponent(document.getElementById('select_static_binding_document_row'));
              showComponent(document.getElementById('configure_action_bindings_link_row'));
              if (this._instanceType == 'none')
              {
                  this._instanceType = 'static';
                  document.getElementById('link_data_binding_method_static').checked = true;
              }
          }
          else
          {
              hideComponent(document.getElementById('link_binding_method_row'));
              hideComponent(document.getElementById('static_binding_structure_title'));
              hideComponent(document.getElementById('static_binding_generate_document_row'));
              hideComponent(document.getElementById('select_static_binding_document_row'));
              hideComponent(document.getElementById('configure_action_bindings_link_row'));
              this._instanceType = 'none';
          }*/
      }
  }

  function checkBindingMethod()
  {
      /*var bindingMethod = '';
      if(document.getElementById('link_data_binding_method_static').checked) bindingMethod = 'static';
      else if(document.getElementById('link_data_binding_method_dynamic').checked) bindingMethod = 'dynamic';

      if(bindingMethod == "static")
      {
          showComponent(document.getElementById('static_binding_structure_title'));
          showComponent(document.getElementById('static_binding_generate_document_row'));
          showComponent(document.getElementById('select_static_binding_document_row'));
          this._instanceType = "static";
      }
      else if(bindingMethod == "dynamic")
      {
          hideComponent(document.getElementById('static_binding_structure_title'));
          hideComponent(document.getElementById('static_binding_generate_document_row'));
          hideComponent(document.getElementById('select_static_binding_document_row'));
          this._instanceType = 'dynamic';
      }*/
  }

    /**
    * Enables inplace renaming of links.
    */
    function rename()
    {
        var old_id = this._action
        var svgCoords = {x: (this._source._x + this._target._x)/2, y: (this._source._y + this._target._y)/2}
        var screenCoords = this._container._svgCoordinates.getScreenCoordinates(svgCoords)
        /* The 'this' param the call back when a new name has been captured. */
        inplaceRenameCapture(screenCoords.x, screenCoords.y, old_id, this);
    }

    /**
    * This is the callback function from the inplaceRenameCapture function after a new name has been entered.
    * @param id The new name.
    */
    function processNewName(id)
    {
        if (checkActionNameUnique(document.getElementById('inplace_rename'), this))
        {
            //Update the RHS panel with the renamed action
            this.displayContextPanel();
            this.setChanged();
            return true;
        }
        else
        {
            return false;
        }

    }

    //change detection/management functions
    function setChanged()
    {
        //alert('setting changed link');
        this._hasChanges = true;
        this._container.setChanged();
    }

    function clearChanged()
    {
        this._hasChanges = false;
    }

    /**
     * This function shows a window to the user indicating which fields/groups etc from the source page contain
     * events that call this action.  Each entry can then be clicked on to go directly to the Field Details page.
     */
    function showActionSources()
    {
        //make sure it has a page or partial page source
        if (this._source.constructor == Page)
        {
            WinManager.displayMessageBox('Loading...', 'Action Sources');

            var pageName = this._source._name;
            var actionName = this._action;

            //get the first (and should be only) form for the page
            var currentForm
            for(form in this._source._forms)
            {
                currentForm = form;
                break;
            }

            if (this._source._forms[currentForm]._structureGenned == "false")
            {
                alert('You have not yet added any fields to the ' + pageName + ' page.');
                WinManager.clearMessageBox();
                return;
            }

            dojo.xhrGet({
                    url: 'FormMaker',
                    handleAs: 'text',
                    preventCache: true,
                    content: {
                        action: 'process',
                        operation1: 'load_form_structure',
                        form_id: currentForm,
                        xml_project_info: document.result_form.xml_project_info.value
                    },
                    error: function(error, ioargs) {
                        alert('Unable to load source details: ' + error);
                        WinManager.clearMessageBox();
                    },
                    load: function(data, ioargs) {
                        var structureDoc = getXMLDocument(data);
                        Sarissa.setXpathNamespaces(structureDoc, "xmlns:fm='http://www.hyfinity.com/formmaker'");

                        if ((Sarissa.getParseErrorText(structureDoc) == Sarissa.PARSED_OK) && (structureDoc.selectSingleNode("/fm:structure") != null))
                        {
                            //can process ok
                            navigationDiagram._actionSourcesTransform.addParameter('action', actionName);
                            navigationDiagram._actionSourcesTransform.addParameter('page', pageName);
                            var result = navigationDiagram._actionSourcesTransform.transformToXML(structureDoc);

                            WinManager.displayMessageBox(xmlToString(result), 'Action Sources');

                        }
                        else
                        {
                            alert('Unable to load source details: ' + data);
                            WinManager.clearMessageBox();
                        }
                    }
            });
        }
    }
}

/**
 * Finds and sets a unique ID for the given link using the source and target as the basis.
 */
function findUniqueLinkID(link)
{
    var testid = link._source._id + '__' + link._target._id + '__link_';
    var pointer = 1;
    var newid = testid + pointer;
    while (navigationDiagram._links[newid] != undefined)
    {
        newid = testid + pointer;
        ++pointer;
    }
    link._id = newid;
}



/**
 * Generic function to return the type of the supplied object.
 * Returns either 'Page', 'Handler' or 'Link'
 */
function objectType(obj)
{
  if (obj.constructor == Page && (obj._type == 'normal' || obj._type == 'subsection'))
      return 'Page';
  else if (obj.constructor == Handler)
      return 'Handler';
  else
      return 'Link';
}
