/**
* Copyright (c) 2009 Hyfinity Limited. All rights reserved.
* Dynamic CSS Palette Creation and Management functionality.
*/

//dojo.require("dojox.widget.ColorPicker");
dojo.require("dijit.ColorPalette");

/**
* Define namespace objects to prevent undefined messages.
*/
if (typeof(hyf) == "undefined")
{
    hyf = {
        version: 1.0
    }
}

if (typeof(hyf.css) == "undefined")
{
    hyf.css = {
        version: 1.0
    }
}

/**
* CSS Palette config file location information. This also keeps track of the main CSS palette object.
* @param strPaletteConfigName Name for the main CSS Palette config (this) object.
* @param monitoring changes - for future use.
*/
hyf.css.PaletteConfig = function(strPaletteConfigName, objChangeManager)
{
    this.strXStore = 'blueprints';
    this.strCollection = 'common';
    this.strFilename = 'css_palette_config.xml'
    this.xmlPaletteConfig = null;	//Palette config file contents.
    this.objPalette = null;			//Main Palette.
    this.strPaletteConfigName = strPaletteConfigName; //Track main palette name to enable retrieval later, especially after async calls.
    this.objChangeManager = objChangeManager;
}

/**
* Control information including complex control such as colour pickers.
*/
hyf.css.Control = function()
{
    this.strName = "";
    this.strIcon = "";
    this.strHint = "";
    this.strAttribName = ""; 		//The CSS attribute name.
    this.strAttribValue = ""; 		//The CSS code represented by this control.
    this.strCurrentAttribValue = "";
    this.bSelected = false;			//Indicates whether this control has been 'clicked-on' (selected).
    this.strRenderFunction = "";	//Rendering function for complex controls such as colour pickers.
    this.strGetValueFunction = "";	//Function to enable retrieval of selected value within the complex control.
    this.strStyle = "";				//Style class for the complex control panel.
}

/**
* Representation of the core palette structure.
* @param objPaletteConfig Main palette config object reference that contains (this) Palette.
*/
hyf.css.Palette = function(objPaletteConfig)
{
    this.strName = "";
    this.objPaletteConfig = objPaletteConfig;		//Reference to the config object that creates the main palette. This is to enable retrieval of the palette object by name after async calls.
    this.objControls = new Object();
    this.objCurrentPaletteUser = null;	//Current page component using this palette.
    this.objPaletteHTML = null;
    this.strComplexControlContainerId = "complex_control_container";
    this.strComplexControlId = "complex_control";
    this.displayFixed = false;
}

//Constant to indicate the height of the palette.
hyf.css.Palette.constPALETTE_XOFFSET = 0;
hyf.css.Palette.constPALETTE_YOFFSET = 2;
hyf.css.Palette.constPALETTE_HEIGHT = 0;

/**
 * Palette Config file load.
 */
hyf.css.PaletteConfig.prototype.load = function()
{
    //Prepare the XStore request.
    var xstoreRequest = '<xstore xmlns="http://www.hyfinity.com/xstore"><query>' +
                            '<action>read</action>' +
                            '<xstore_name>' + this.strXStore + '</xstore_name>' +
                            '<collection_name>' + this.strCollection + '</collection_name>' +
                            '<document_name>' + this.strFilename + '</document_name>' +
                            '</query><content/></xstore>';

    var requestMessage = '<xg:attributes xmlns:xg="http://www.hyfinity.com/xgate">' +
                        '<xg:attribute xg:name="operation1">call_xstore</xg:attribute>' +
                        '<xg:xstore_request>'+xstoreRequest+'</xg:xstore_request>' +
                        '</xg:attributes>'

    //Store 'this' because it's required after async call.
    var objPaletteConfig = this;
    //Make the async call to retrieve config file.
    dojo.rawXhrPost({
        url:        'FormMaker',
        handleAs:   'text',
        postData:   requestMessage,
        headers:    { SOAPAction:    "process"},
        load:       function(response, ioArgs){
                        var xmlPaletteConfig = getXMLDocument();
                        //Ignore the XStore wrapper.
                        if (stripXStoreResponse(response, xmlPaletteConfig))
                        {
                            //Parse the config file.
                            objPaletteConfig.parse(xmlPaletteConfig);
                        }
                        return response;
                    },
        error:      function(response, ioArgs){
                        alert('Error loading css palette config file.  Please try again.\n\n\nError details: ' + response.message);
                        return response;
                    }
    });
}

/**
 * Palette Config file parser. Populates the css palette and controls.
 * @param xmlPaletteConfig XML doc containing the palette config file.
 */
hyf.css.PaletteConfig.prototype.parse = function(xmlPaletteConfig)
{
    //Create Palette.
    this.objPalette = new hyf.css.Palette(this);
    this.xmlPaletteConfig = xmlPaletteConfig;
    Sarissa.setXpathNamespaces(this.xmlPaletteConfig, FMqualifiedNS);
    this.objPalette.strName = this.xmlPaletteConfig.selectSingleNode("/fm:palette_config/fm:palette").getAttribute("name");

    //Parse Controls.
    var xmlControls = this.xmlPaletteConfig.selectNodes("/fm:palette_config/fm:palette/fm:control");

    for (var i = 0; i < xmlControls.length; ++i)
    {
        var objControl = new hyf.css.Control();
        objControl.strName = xmlControls.item(i).getAttribute("name");
        objControl.strIcon = xmlControls.item(i).selectSingleNode("fm:icon").firstChild.nodeValue;
        objControl.strHint = xmlControls.item(i).selectSingleNode("fm:hint").firstChild.nodeValue;
        objControl.strAttribName = xmlControls.item(i).selectSingleNode("fm:css").getAttribute("attribute_name");
        objControl.strAttribValue = xmlControls.item(i).selectSingleNode("fm:css").firstChild.nodeValue;
        objControl.strCurrentAttribValue = objControl.strAttribValue;

        //Determine if a control has complex content.
        var strRenderFunction = xmlControls.item(i).getAttribute("render_function");
        //Each render function must have an accompanying getValue function.
        var strGetValueFunction = xmlControls.item(i).getAttribute("getvalue_function");
        if (strRenderFunction != null && strGetValueFunction != null)
        {
            //Assume this is a complex control and separate function will assist with rendering.
            objControl.strRenderFunction = strRenderFunction;
            objControl.strGetValueFunction = strGetValueFunction;
        }
        var strStyle = xmlControls.item(i).getAttribute("additional_style");
        if (strStyle != null)
        {
            objControl.strStyle = strStyle;
        }

        this.objPalette.objControls[objControl.strName] = objControl;
    }
    this.objPalette.renderCssPalette();
    hyf.css.Palette.hideCssPalette();
}

/**
 * Palette Renderer.
 */
hyf.css.Palette.prototype.renderCssPalette = function()
{
    //Parse the css string to determine selected controls.
    if (this.objCurrentPaletteUser != null)
    {
        this.setSelectedControls(this.objCurrentPaletteUser.value);
    }

    //Create the palette markup.
    var strHTML = '<table onselectstart="return false;" class="table" border="0" cellSpacing="0" cellPadding="0">'
        +	'<tbody><tr><td class="palette_left"></td>';
    //Process the controls that require drawing.
    for (control in this.objControls)
    {
        //Create different class name to enable hover images changes, etc.
        //var strClass = this.objControls[control].strName + "_control";
        var strClass = "css_palette_control";
        if (this.objControls[control].bSelected)
        {
            //strClass = this.objControls[control].strName + "_selected_control";
            strClass = "css_palette_selected_control";
        }
        //Create markup for simple and complex controls.
        var strOnclickHandler = "";
        if (this.objControls[control].strRenderFunction != "")
        {
            strOnclickHandler = 'hyf.css.Palette.renderComplexControlPanel(\''
                + this.objControls[control].strName
                + '\'); return false;" href="#"';
        }
        else
        {
            //If it is a simple control then hide the complex control panel when clicked, if it is open.
            strOnclickHandler = 'hyf.css.Palette.changeCssString(\''
                + this.objControls[control].strName
                + '\' , \'' + this.objControls[control].strAttribValue + '\'); return false;" href="#"';
        }
        strHTML += '<td><a id="' + this.objControls[control].strName
                + '" class="' + strClass + '" title="'
                + this.objControls[control].strHint
                + '" onclick="hyf.css.Palette.hideComplexControlPanel(); ' + strOnclickHandler + '>'
                + '<img src="'+this.objControls[control].strIcon+'"/>'
                + '</a></td>';
    }
    strHTML += '<td class="palette_right"></td></tr></tbody></table>';
    //Create the css palette container to enable loading of the palette html.
    var objCssPaletteContainerHTML = document.createElement("div");
    objCssPaletteContainerHTML.setAttribute("id", this.strName);
    //Allow showing and hiding of the main palette, allowing period for momentary mouseout without hiding.
    dojo.connect(objCssPaletteContainerHTML, 'onmouseout', this, hyf.css.Palette.hideCssPaletteSlowly);
    dojo.connect(objCssPaletteContainerHTML, 'onmouseover', this, hyf.css.Palette.inCssPalette);
    objCssPaletteContainerHTML.className = this.strName;
    objCssPaletteContainerHTML.innerHTML = strHTML;
    if (document.getElementById(this.strName) != null)
    {
        /** Replace an exisiting css palette. */
        var objCssPaletteContainer = document.getElementById(this.strName);
        objCssPaletteContainer.parentNode.replaceChild(objCssPaletteContainerHTML, objCssPaletteContainer);
    }
    else
    {
        /** A css palette doesn't exist yet, so create one. */

        document.body.appendChild(objCssPaletteContainerHTML);
    }
    this.objPaletteHTML = objCssPaletteContainerHTML;
    //Set the coordinates of the css palette relative to the item that is using it.
    if (this.objCurrentPaletteUser != null)
    {
        this.objPaletteHTML.style.top = (getComponentPosition(this.objCurrentPaletteUser, true).y
            + getComponentPosition(this.objCurrentPaletteUser).height
            + hyf.css.Palette.constPALETTE_YOFFSET) + 'px';
        this.objPaletteHTML.style.left = (getComponentPosition(this.objCurrentPaletteUser, true).x
            - hyf.css.Palette.constPALETTE_XOFFSET) + 'px';
        //Adjust to ensure the palette is within the viewport.
        hyf.css.Palette.adjustPaletteWithinViewport(this.objPaletteHTML);
    }
}

/* Constants for the generic button ids on the complex control panel container. */
hyf.css.Palette.COMPLEX_CONTROL_BTN_OK = "cssComplexControlOk";
hyf.css.Palette.COMPLEX_CONTROL_BTN_NONE = "cssComplexControlNone";
hyf.css.Palette.COMPLEX_CONTROL_BTN_CANCEL = "cssComplexControlCancel";

/**
 * Create a panel that hides and shows, coordinating with something else that creates it.
 * @param strControlId Name of the control that has complex content.
 */
hyf.css.Palette.renderComplexControlPanel = function(strControlId)
{
    //remove any existing complex control panel
    hyf.css.Palette.hideComplexControlPanel();
    hyf.css.Palette.inCssPalette();

    var strStyle = objCssPaletteConfig.objPalette.objControls[strControlId].strStyle;
    if (strStyle == "")
    {
        var strStyle = strControlId;
    }

    objCssPaletteConfig.objPalette.strComplexControlContainer = null;

    //Create a new complex control container because one doesn't exist yet.
    //Create the default markup to enable default hiding and showing behaviour. Privide default OK, Cancel and other useful buttons.
    var strComplexControlId = objCssPaletteConfig.objPalette.strComplexControlId
    var strHTML = '<div id="' + strComplexControlId + '"></div>'
        + '<a id="' + hyf.css.Palette.COMPLEX_CONTROL_BTN_OK + '" onclick="hyf.css.Palette.getComplexControlValue(\'' + strControlId + '\', \'' + hyf.css.Palette.COMPLEX_CONTROL_BTN_OK + '\'); return false;" href="#">OK</a>'
        + '<a id="' + hyf.css.Palette.COMPLEX_CONTROL_BTN_NONE + '" onclick="hyf.css.Palette.getComplexControlValue(\'' + strControlId + '\', \'' + hyf.css.Palette.COMPLEX_CONTROL_BTN_NONE + '\'); return false;" href="#">None</a>'
        + '<a id="' + hyf.css.Palette.COMPLEX_CONTROL_BTN_CANCEL + '" onclick="hyf.css.Palette.getComplexControlValue(\'' + strControlId + '\', \'' + hyf.css.Palette.COMPLEX_CONTROL_BTN_CANCEL + '\'); return false;" href="#">Cancel</a>';

    //Create the complex container.
    var strComplexContainerId = objCssPaletteConfig.objPalette.strComplexControlContainerId;
    var objComplexControlContainer = document.createElement("div");
    objComplexControlContainer.setAttribute("id", strComplexContainerId);
    objComplexControlContainer.className = 'css_palette_complex_control ' + strStyle;
    dojo.connect(objComplexControlContainer, 'onmouseout', hyf.css.Palette.hideComplexControlPanelSlowly);
    dojo.connect(objComplexControlContainer, 'onmouseover', hyf.css.Palette.inComplexControlPanel);
    objComplexControlContainer.innerHTML = strHTML;
    var objPaletteHTML = objCssPaletteConfig.objPalette.objPaletteHTML;
    objComplexControlContainer.style.top = (getComponentPosition(objPaletteHTML).y
        + getComponentPosition(objPaletteHTML).height) + 'px';
    objComplexControlContainer.style.left = (getComponentPosition(objPaletteHTML).x
        - hyf.css.Palette.constPALETTE_XOFFSET) + 'px';
    document.body.appendChild(objComplexControlContainer);

    //Call the actual render function to paint the contents of the complex control container.
    var strRenderFunction = objCssPaletteConfig.objPalette.objControls[strControlId].strRenderFunction;
    eval(strRenderFunction + '(strControlId)');
    //Adjust to ensure the palette is within the viewport.
    hyf.css.Palette.adjustPaletteWithinViewport(objComplexControlContainer);
}

/*
* Adjusts the top and left coordinates to ensure the supplied palette container resides within the viewport.
* @param objPalette The palette container that is to be checked to ensure it resides within the viewport.
*/
hyf.css.Palette.adjustPaletteWithinViewport = function(objPalette)
{
    var idealCoords = getComponentPosition(objPalette, true);
    //Now check if it is off the bottom or side of the screen and adjust accordingly.
    var viewportSize = getViewportSize();
    viewportSize.height +=  document.body.scrollTop;
    viewportSize.width += document.body.scrollLeft;

    if ((idealCoords.y + idealCoords.height) > viewportSize.height)
    {
        objPalette.style.top = (idealCoords.y - ((idealCoords.y + idealCoords.height) - viewportSize.height)) + 'px';
    }

    if ((idealCoords.x + idealCoords.width) > viewportSize.width)
    {
        objPalette.style.left = (idealCoords.x - ((idealCoords.x + idealCoords.width) - viewportSize.width)) + 'px';
    }
}

/*
* Generic method for capturing the selected value of a complex control such as the colour picker.
* This method processes the generic parts and then delegates to the control-specific function to process the control-specific value extraction approach.
* @param strControlId The id of the control that has complex content.
* @param strButtonId The button that was pressed (OK|None|Cancel) within the complex control container.
*/
hyf.css.Palette.getComplexControlValue = function(strControlId, strButtonId)
{
    var strGetValueFunction = objCssPaletteConfig.objPalette.objControls[strControlId].strGetValueFunction;
    switch(strButtonId)
    {
        case hyf.css.Palette.COMPLEX_CONTROL_BTN_OK:
        {
            var strComplexControlId = objCssPaletteConfig.objPalette.strComplexControlId;
            hyf.css.Palette.changeCssString(strControlId, eval(strGetValueFunction + '(\'' + strComplexControlId + '\')'));
            break;
        }
        case hyf.css.Palette.COMPLEX_CONTROL_BTN_NONE:
        {
            hyf.css.Palette.changeCssString(strControlId, "");
            break;
        }
        case hyf.css.Palette.COMPLEX_CONTROL_BTN_CANCEL:
        {
            break;
        }
    }
    objCssPaletteConfig.objPalette.displayFixed = false;
    hyf.css.Palette.hideComplexControlPanel();
}

/*
* Dojo-specifc approach for extracting the selected colour from the colour picker control.
* @param strComplexControlId Id of the dojo colour picker control.
*/
hyf.css.Palette.getDojoColourPickerValue = function(strComplexControlId)
{
    return dijit.byId(strComplexControlId).value;
}

/**
 * Rendering function for the dojo colour picker complex control.
 * @param strControlId The Name of the control that created the complex conterl panel.
 */
hyf.css.Palette.renderDojoColourPicker = function(strControlId)
{
    var strCurrentColour = objCssPaletteConfig.objPalette.objControls[strControlId].strCurrentAttribValue;
    //Destroy the existing colour picker to ensure only one is ever shown.
    var strComplexControlId = objCssPaletteConfig.objPalette.strComplexControlId;
    if (dijit.byId(strComplexControlId) != null)
    {
        dijit.byId(strComplexControlId).destroy();
    }

    //Create the dojo colour picker.
    var container = document.getElementById(strComplexControlId);
    container.innerHTML = '<div id="'+strComplexControlId+'_palette"></div>';

    var objDojoColourPicker = new dijit.ColorPalette({id: strComplexControlId, value: strCurrentColour}, dojo.byId(strComplexControlId+'_palette'));
    objDojoColourPicker.startup();
    dojo.connect(objDojoColourPicker, 'onChange', function() {dojo.byId(hyf.css.Palette.COMPLEX_CONTROL_BTN_OK).onclick();});
}

/*
* Retrieves the value entered into the size selection control
* @param strComplexControlId Id of the control.
*/
hyf.css.Palette.getSizeSelectorValue = function(strComplexControlId)
{
    var newNumber = document.getElementById(strComplexControlId + '_number').value;
    var newUnit = document.getElementById(strComplexControlId + '_unit').value;
    return newNumber + newUnit;
}

/**
 * Rendering function for the size selector control, which allows the suer to input a particualr size and unit
 * @param strControlId The Name of the control that created the complex conterl panel.
 */
hyf.css.Palette.renderSizeSelector = function(strControlId)
{
    var strComplexControlId = objCssPaletteConfig.objPalette.strComplexControlId;
    var strCurrentValue = objCssPaletteConfig.objPalette.objControls[strControlId].strCurrentAttribValue;

    var currentNumber = parseInt(strCurrentValue);
    var currentUnit;
    if (isNaN(currentNumber))
    {
        currentNumber = '';
        currentUnit = '';
    }
    else
    {
        currentUnit = strCurrentValue.substr((''+currentNumber).length);
    }

    var htmlStr = '<div style="text-align:center;padding : 3px;">';
    htmlStr += '<input size="5" type="text" id="'+strComplexControlId+'_number" value="'+currentNumber+'" onfocus="hyf.css.Palette.inComplexControlPanel()"/>';
    htmlStr += '<select id="'+strComplexControlId+'_unit" style="margin-left : 3px;" onfocus="hyf.css.Palette.stopComplexControlHide();" onblur="hyf.css.Palette.allowComplexControlHide()" onchange="this.blur();">';

    var units = ['px', '%', 'pt', 'em'];
    for (var i = 0; i < units.length; ++i)
    {
        htmlStr += '<option value="'+units[i]+'"';
        if (units[i] == currentUnit)
            htmlStr += ' selected="selected"';
        htmlStr += '>'+units[i]+'</option>';
    }

    htmlStr += '</select></div>';

    var container = document.getElementById(strComplexControlId);
    container.innerHTML = htmlStr;
}


/*
* Retrieves the value selected from the font selection control
* @param strComplexControlId Id of the control.
*/
hyf.css.Palette.getFontSelectorValue = function(strComplexControlId)
{
    return document.getElementById(strComplexControlId + '_font').value;
}

/**
 * Rendering function for the font selection control
 * @param strControlId The Name of the control that created the complex conterl panel.
 */
hyf.css.Palette.renderFontSelector = function(strControlId)
{
    var strComplexControlId = objCssPaletteConfig.objPalette.strComplexControlId;
    var strCurrentValue = objCssPaletteConfig.objPalette.objControls[strControlId].strCurrentAttribValue;

    var htmlStr = '<div style="text-align:center;padding : 3px;">';
    htmlStr += '<select id="'+strComplexControlId+'_font" onfocus="hyf.css.Palette.stopComplexControlHide();" onblur="hyf.css.Palette.allowComplexControlHide()" onchange="this.blur();">';

    var fonts = ['Arial', 'Comic Sans MS', 'Courier', 'Courier New', 'Georgia', 'Helvetica', 'Impact', 'Times New Roman', 'Trebuchet MS', 'Verdana', 'sans-serif', 'serif', 'monospace', 'cursive'];
    for (var i = 0; i < fonts.length; ++i)
    {
        htmlStr += '<option value="'+fonts[i]+'"';
        if (fonts[i].toLowerCase() == strCurrentValue.toLowerCase())
            htmlStr += ' selected="selected"';
        htmlStr += '>'+fonts[i]+'</option>';
    }

    htmlStr += '</select></div>';

    var container = document.getElementById(strComplexControlId);
    container.innerHTML = htmlStr;
}

/** Stops the complex control and palette from hiding, even if you move the mouse out of the popup area */
hyf.css.Palette.stopComplexControlHide = function()
{
    objCssPaletteConfig.objPalette.displayFixed = true;
}

/** Enables the complex control area nad palette to hide again when needed */
hyf.css.Palette.allowComplexControlHide = function()
{
    setTimeout("objCssPaletteConfig.objPalette.displayFixed = false;", 990)
    ComplexControlPopdownTimeout = setTimeout("hyf.css.Palette.hideComplexControlPanelSlowly();", 1000)
}


/*
* Changes the current Css String by parsing it and determining whether to add, delete or replace the new name/value pair.
* @param strControlId Name of the css control.
* @param strAttibValue Value associated with the css attribute.
*/
hyf.css.Palette.changeCssString = function(strControlId, strAttribValue)
{
    var strAttribName = objCssPaletteConfig.objPalette.objControls[strControlId].strAttribName;
    //Locate the user of the palette.
    var objCurrentPaletteUser = objCssPaletteConfig.objPalette.objCurrentPaletteUser;
    var strCssString = (objCurrentPaletteUser.value).toLowerCase();
    strAttribName = strAttribName.toLowerCase();
    strAttribValue = strAttribValue.toLowerCase();

    //Search for css attribute name.
    var iAttribNameStart = hyf.css.Palette.findAttribNameInString(strAttribName);

    //Allow for partial name matches such as 'color' within 'background-color'.
    if (iAttribNameStart != -1)
    {
        //Found attribute, now search for attribute value, starting from the end of the attribute name to the next semi-colon
        var iAttribValueStart = iAttribNameStart + strAttribName.length;
        var iAttribValueEnd = strCssString.indexOf(';', iAttribValueStart);
        if (iAttribValueEnd == -1)
        {
            //No semi-colon, therefore assume last item.
            iAttribValueEnd = strCssString.length;
        }
        //The strAttribValuePart will contain ':' and spaces
        var strAttribValuePart = strCssString.substring(iAttribValueStart, iAttribValueEnd);
        //only remove attribute if the new value is blank, or the new value is the same as the current, and not complex control
        if (strAttribValue == '' || ((strAttribValuePart.indexOf(strAttribValue) != -1) && (objCssPaletteConfig.objPalette.objControls[strControlId].strRenderFunction == "")))
        {
            //Found attribute value, therefore remove this name/value pair.
            var strAttribNameAndValuePart = strAttribName + strAttribValuePart;
            objCssPaletteConfig.objPalette.objControls[strControlId].strCurrentAttribValue = "";
            //Make a copy of the existing css string and omit the attribute name/value pair.
            strCssString = strCssString.substring(0, iAttribNameStart) + strCssString.substring(iAttribValueEnd+1, strCssString.length);
        }
        else
        {
            //Attribute value not found. Therefore replace with current value;
            strCssString = strCssString.substring(0, iAttribValueStart) + ':' + strAttribValue + ';' + strCssString.substring(iAttribValueEnd+1, strCssString.length);
            objCssPaletteConfig.objPalette.objControls[strControlId].strCurrentAttribValue = strAttribValue;
        }
    }

    if (iAttribNameStart == -1 && strAttribValue != '')
    {
        //Attribute not found. Therefore insert new one.
        if (strCssString != "" && strCssString.charAt(strCssString.length-1) != ';')
        {
            strCssString += ";";
        }
        strCssString += strAttribName + ':' + strAttribValue + ';';
        objCssPaletteConfig.objPalette.objControls[strControlId].strCurrentAttribValue = strAttribValue;
    }

    objCurrentPaletteUser.value = strCssString;
    objCssPaletteConfig.objPalette.renderCssPalette();

    //return false to stop href on the anchor that called this function.
    return false;
}

/*
* Determine which controls should be highlighted(selected) based on the Css string.
*/
hyf.css.Palette.prototype.setSelectedControls = function()
{
    var objCurrentPaletteUser = this.objCurrentPaletteUser;
    var strCssString = (objCurrentPaletteUser.value).toLowerCase();

    //Determine which controls are selected based on the css string.
    for (control in this.objControls)
    {
        var strAttribName = this.objControls[control].strAttribName.toLowerCase();
        var strAttribValue = this.objControls[control].strAttribValue.toLowerCase();

        var iAttribNameStart = hyf.css.Palette.findAttribNameInString(strAttribName);

        if (iAttribNameStart != -1)
        {
            //Found attribute, now search for attribute value, starting from the end of the attribute name to the next semi-colon
            var iAttribNameStart = strCssString.indexOf(strAttribName);
            var iAttribValueStart = iAttribNameStart + strAttribName.length;
            var iAttribValueEnd = strCssString.indexOf(';', iAttribValueStart);
            if (iAttribValueEnd == -1)
            {
                //No semi-colon, therefore assume last item.
                iAttribValueEnd = strCssString.length;
            }

            //The strAttribValuePart will contain ':' and spaces
            var strAttribValuePart = strCssString.substring(iAttribValueStart, iAttribValueEnd);

            //check for complex control
            if (this.objControls[control].strRenderFunction != "")
            {
                //Complex control found, which means attribute value is unknown, but assume selected.
                this.objControls[control].bSelected = true;
                var colonPos = strAttribValuePart.indexOf(':');

                this.objControls[control].strCurrentAttribValue = strAttribValuePart.substr(colonPos+1).replace(/(^\s*)|(\s*$)/gi,"");
                //alert(this.objControls[control].strCurrentAttribValue);
                continue;
            }


            if (strAttribValuePart.indexOf(strAttribValue) != -1)
            {
                //Attribute name and value have been located. Assume selected.
                this.objControls[control].bSelected = true;
            }
            else
            {
                this.objControls[control].bSelected = false;
            }
        }
        else
        {
            this.objControls[control].bSelected = false;
            if (this.objControls[control].strRenderFunction != "")
            {
                //rest to the default value
                this.objControls[control].strCurrentAttribValue = this.objControls[control].strAttribValue;
            }
        }
    }
}

hyf.css.Palette.findAttribNameInString = function(strAttribName)
{
    //Locate the user of the palette.
    var objCurrentPaletteUser = objCssPaletteConfig.objPalette.objCurrentPaletteUser;
    var strCssString = (objCurrentPaletteUser.value).toLowerCase();
    strAttribName = strAttribName.toLowerCase();
    //Search for css attribute name.
    var iAttribNameStart = -1;
    do
    {
        iAttribNameStart += 1;
        iAttribNameStart = strCssString.indexOf(strAttribName, iAttribNameStart);
        //Allow for partial name matches such as 'color' within 'background-color'.
        if (iAttribNameStart != -1 &&
            (iAttribNameStart == 0 || strCssString.charAt(iAttribNameStart-1) == ' ' || strCssString.charAt(iAttribNameStart-1) == ';'))
        {
            return iAttribNameStart;
        }

    }
    while (iAttribNameStart != -1)

    return iAttribNameStart;
}

//Timer used to control the showing and hiding of the css palette.
var cssPalettePopdownTimeout;

/**
 * Called when mouse moved over the Css Palette window.
 * Used to cancel any timers to hide the Css Palette.
 */
hyf.css.Palette.inCssPalette = function()
{
    if(cssPalettePopdownTimeout)
    {
        clearTimeout(cssPalettePopdownTimeout);
        cssPalettePopdownTimeout=null;
    }
}

/*
* This is invoked from the html css override edit boxes.
* @param objComponent The page field, group, etc. that raised the need for the Css Palette.
*/
hyf.css.Palette.showCssPalette = function(objComponent)
{
    if (!objCssPaletteConfig.objPalette.displayFixed)
    {
        hyf.css.Palette.hideCssPalette();
        if (objComponent != null)
        {
            var objPalette = objCssPaletteConfig.objPalette;
            objPalette.objCurrentPaletteUser = objComponent;
            objPalette.renderCssPalette();
            showComponent(objPalette.objPaletteHTML);
        }
    }
}

/**
 * Hides the Css Palette window.
 */
hyf.css.Palette.hideCssPalette = function()
{
    if (!objCssPaletteConfig.objPalette.displayFixed)
    {
        var objCssPaletteDiv = objCssPaletteConfig.objPalette.objPaletteHTML;
        if (objCssPaletteDiv != null)
        {
            hideComponent(objCssPaletteDiv);
            hyf.css.Palette.hideComplexControlPanel();
            if(cssPalettePopdownTimeout)
            {
                clearTimeout(cssPalettePopdownTimeout);
                cssPalettePopdownTimeout=null;
            }

        }
    }
}

/**
 * Called when the mouse moves off of the trigger container.
 * Initalise the timeout to hide the Css Palette
 */
hyf.css.Palette.hideCssPaletteSlowly = function(e)
{
    if(cssPalettePopdownTimeout)
    {
        clearTimeout(cssPalettePopdownTimeout);
    }
    cssPalettePopdownTimeout=setTimeout("hyf.css.Palette.hideCssPalette()", 500);
}

//Global var to control the hiding and showing of the conplex control container.
var ComplexControlPopdownTimeout;

/*
* This is called when the mouse moves over the complex control.
*/
hyf.css.Palette.inComplexControlPanel = function()
{
    //Reset popdown timers to ensure any deplayed calls to close the complex control panel are cancelled
    if(ComplexControlPopdownTimeout)
    {
        clearTimeout(ComplexControlPopdownTimeout);
        ComplexControlPopdownTimeout = null;
    }
    //Must also reset the parent palette timer that opened the complex control.
    if(cssPalettePopdownTimeout)
    {
        clearTimeout(cssPalettePopdownTimeout);
        cssPalettePopdownTimeout = null;
    }
}

/*
* This is called when the user 'drifts' off (mouseout) the complex control container.
* Need to start closing the container, but allow for the fact that the user may 'drift' back in (mouseover).
*/
hyf.css.Palette.hideComplexControlPanelSlowly = function(e)
{
    if(ComplexControlPopdownTimeout)
    {
        clearTimeout(ComplexControlPopdownTimeout);
    }
    if(cssPalettePopdownTimeout)
    {
        clearTimeout(cssPalettePopdownTimeout);
        cssPalettePopdownTimeout = null;
    }
    //Start to close the container
    ComplexControlPopdownTimeout=setTimeout("hyf.css.Palette.hideComplexControlPanel()", 500);
}

/*
* Close the complex control container without delay. This is called when specifically requested to do so.
* Such as clicking one of the buttons or using mouseout.
*/
hyf.css.Palette.hideComplexControlPanel = function()
{
    if (!objCssPaletteConfig.objPalette.displayFixed)
    {
        var objComplexControlDiv = dojo.byId(objCssPaletteConfig.objPalette.strComplexControlContainerId);
        if (objComplexControlDiv != null)
        {
            objComplexControlDiv.parentNode.removeChild(objComplexControlDiv);
            if(ComplexControlPopdownTimeout)
            {
                clearTimeout(ComplexControlPopdownTimeout);
                ComplexControlPopdownTimeout = null;
            }
            //start to close the main css palette if no activity
            hyf.css.Palette.hideCssPaletteSlowly();
        }
    }
}
