/*
 * formStructureScript.js
 *
 * Contains all the general scripts used by the form structure page
 *
 * @author Rafel Bordas
 *
 */

var structureXML;
var tempStructureXML;
var treeStructureTransform;
var layoutStructureTransform;
var structureBindingsXML;
var defaultStructureXML;
var defaultStructureTransform;
var defaultStructureBindingsXML;
var mouseDown = false;
var mouseDownSource = 'structure';
var selectedElement = null;
//var originalControlColour;
var selectedElementPath;
var addMode = "into";
var form;

//the transform used to merge details from the palette onto existing controls
var mergeStructureTransform
//the transform used to merge details from abstract data source onto existing controls
var mergeStructureFromDSTransform;

//The ChangeManager that will be used to keep track of which changes have been applied.
var changeManager;

var FMqualifiedNS = 'xmlns:fm="http://www.hyfinity.com/formmaker"';
var FMdefaultNS = 'http://www.hyfinity.com/formmaker';

/** Set up an array of all the names that the user cannot use for element names.
 *  Should keep this in sync with the list in formMapScripts.js and the configGenerator.xsl */
var invalidNames = new Array();
invalidNames[invalidNames.length] = 'submit';
invalidNames[invalidNames.length] = 'action';
invalidNames[invalidNames.length] = 'new';
invalidNames[invalidNames.length] = 'try';
invalidNames[invalidNames.length] = 'do';
invalidNames[invalidNames.length] = 'if';
invalidNames[invalidNames.length] = 'delete';

var objPaletteConfig;

/**
 * Sets up the data islands and XSL processor needed for screen.
 * Puts input forms in to the correct state.
 */
//window.onload =
dojo.addOnLoad(init);
function init()
{
    setTableHeight();
    hideContextMenu();

    changeManager = new hyf.fm.ChangeManager('changeManager');
    objPaletteConfig = new hyf.palette.PaletteConfig("objPaletteConfig", changeManager);
    objPaletteConfig.load();

    form = document.formStructure_form;

    //Create and draw structure treeview
    structureXML = getXMLIsland("structureXML");
    Sarissa.setXpathNamespaces(structureXML, FMqualifiedNS);

    treeStructureTransform = new SimpleTransform("xsl/clientside_structure_treeview.xsl", document.getElementById('StructureTreeview'));
    layoutStructureTransform = new SimpleTransform("xsl/clientside_structure_layout_view.xsl", document.getElementById('StructureTreeview'));

    mergeStructureTransform = new SimpleTransform("xsl/clientside_structure_merge.xsl", null);
    mergeStructureFromDSTransform = new SimpleTransform("xsl/clientside_structure_merge_from_ds.xsl", null);

    drawStructureTreeView();

    scrollableDivManager.attachContainer(document.getElementById('StructureTreeview'));

    //Create bindings for structure treeview
    if (document.getElementById("structureBindingsXML") != null)
    {
        structureBindingsXML = getXMLIsland("structureBindingsXML");
        Sarissa.setXpathNamespaces(structureBindingsXML, FMqualifiedNS);
    }

    defaultStructureTransform = new SimpleTransform("xsl/clientside_default_structure_treeview.xsl", document.getElementById('DefaultStructureTreeview'));

    //initialise default structure
    initDefaultStructure();

    //Disable text selection on treeviews
    document.getElementById('StructureTreeview').onselectstart = new Function("return false");
    document.getElementById('DefaultStructureTreeview').onselectstart = new Function("return false");


    //initialise the state manager
    stateManager = new FormMakerStateManager(document.getElementById('formmakerStateIsland').value, FormMakerStateManager.PAGE_STRUCTURE_PAGE);
}

function setTableHeight()
{
    var requiredHeight = (document.documentElement.clientHeight - 140) + 'px';
    var heightExcludingTabs = (document.documentElement.clientHeight - 166) + 'px';
    document.getElementById('pageTable').style.height = requiredHeight;
    document.formStructure_form.style.height = requiredHeight;
    document.getElementById('StructureDisplay').style.height = requiredHeight;
    document.getElementById('StructureTreeview').style.height = heightExcludingTabs;
    document.getElementById('DefaultStructure').style.height = requiredHeight;
    document.getElementById('DefaultStructureContent').style.height = heightExcludingTabs;
    document.getElementById('divider').style.height = requiredHeight;
    document.getElementById('divider2').style.height = requiredHeight;

    //resize controls palette
    if ((typeof(objPaletteConfig) == 'object') && (dijit.byId(objPaletteConfig.objPalette.strName)))
    {
        var paletteHeight = (document.documentElement.clientHeight - 140 - 55);
        dijit.byId(objPaletteConfig.objPalette.strName).resize({h:paletteHeight});
    }
}
window.onresize = setTableHeight;

/**
 * Starts the initialisation of the default structure view.
 * The information for this is actually loaded in via an AJAX call
 * started here, rather than with the main page load to improve performance.
 *
 * @private
 */
function initDefaultStructure()
{

    //select the correct tab
    switch (form.default_structure_type.value)
    {
        case 'Template' :   document.getElementById('defaultStructureTabDataSource').className = '';
                            document.getElementById('defaultStructureTabTemplate').className = 'current';
                            document.getElementById('defaultStructureTabStructure').className = '';
                            hideComponent(document.getElementById('DefaultStructureDataSoruceSelection'));

                            break;
        case 'Structure':   document.getElementById('defaultStructureTabDataSource').className = '';
                            document.getElementById('defaultStructureTabTemplate').className = '';
                            document.getElementById('defaultStructureTabStructure').className = 'current';
                            hideComponent(document.getElementById('DefaultStructureDataSoruceSelection'));
                            loadAppMap();
                            break;
        case 'Schema' :
        case 'WSDL' :
        case 'DB' :
        case 'DataSource':  showComponent(document.getElementById('DefaultStructureDataSoruceSelection'));
                            document.getElementById('defaultStructureTabDataSource').className = 'current';
                            document.getElementById('defaultStructureTabTemplate').className = '';
                            document.getElementById('defaultStructureTabStructure').className = '';
                            dataSourceDisplayInit();
                            break;
    }


    if (form.default_structure_type.value == 'DataSource')
    {
        hideComponent(dojo.byId('DefaultStructureTreeview'));
    }
    else
    {
        loadDefaultStructure();
    }
}

function loadDefaultStructure()
{
    showComponent(dojo.byId('DefaultStructureTreeview'));


    //show the loading message
    defaultStructureXML = getXMLDocument("<loading/>");
    drawDefaultStructureTreeView();

    //start loading the actual files
    dojo.xhrPost({
        url:        "FormMaker",
        handleAs:   "text",
        preventCache:"true",
        content:    {   action:                     'process',
                        operation1:                 'load_default_structure',
                        schema_filename:            document.formStructure_form.schema_filename.value,
                        schema_root_element:        document.formStructure_form.schema_root_element.value,
                        request_operation:          document.formStructure_form.request_operation.value,
                        response_operation:         document.formStructure_form.response_operation.value,
                        page_source_type:           document.formStructure_form.page_source_type.value,
                        default_structure_filename: document.formStructure_form.default_structure_filename.value,
                        default_structure_type:     document.formStructure_form.default_structure_type.value,
                        template_collection_name:   document.formStructure_form.template_collection_name.value,
                        form_id:                    document.formStructure_form.form_id.value,
                        gen_control:                document.formStructure_form.gen_control.value,
                        binding_type:               document.formStructure_form.binding_type.value,
                        form_map_filename:          document.formStructure_form.form_map_filename.value,
                        xml_project_info:           document.formStructure_form.xml_project_info.value},
        error:  function(response, ioArgs) {
                    alert('An error occurred while trying to load the schema/WSDL for this page.\n\nPlease try your request again, and if the problem persists, please contact Hyfinity.\n\n\nError details: ' + response.message);

                    /*var errMsg = '';
                    for (prop in response)
                        errMsg += prop + " : " + response[prop] + "\n";

                    alert(errMsg);*/

                    return response;
                },
        load:   function(response, ioArgs) {

                    //window.clipboardData.setData("Text",response);

                    var tempDoc = getXMLDocument(response);
                    Sarissa.setXpathNamespaces(tempDoc, FMqualifiedNS);

                    var structureNode = tempDoc.selectSingleNode("/fm:default_structure/fm:structure");
                    var bindingsNode = tempDoc.selectSingleNode("/fm:default_structure/fm:xforms");
                    if (structureNode != null)
                    {
                        defaultStructureXML = getXMLDocument(new XMLSerializer().serializeToString(structureNode));
                        Sarissa.setXpathNamespaces(defaultStructureXML, FMqualifiedNS);
                    }
                    else
                    {
                        defaultStructureXML = getXMLDocument("<empty/>");
                    }
                    if (bindingsNode != null)
                    {
                        defaultStructureBindingsXML = getXMLDocument(new XMLSerializer().serializeToString(bindingsNode));
                        Sarissa.setXpathNamespaces(defaultStructureBindingsXML, FMqualifiedNS);
                    }
                    else
                    {
                        defaultStructureBindingsXML = getXMLDocument("<empty/>");
                    }

                    //reset the namespacesCopied flag so that any new namespaces in this
                    //default structure's bindings will be copied across as and when needed.
                    namespacesCopied = false;

                    drawDefaultStructureTreeView();


                    return response;
                }
    });

}


/**
 * Convenience method for searching the xml structure for unique names.
 */
function searchXMLStructure(name, showAlert)
{
    var uniqName = false;
    //alert(name);
    var testElement = structureXML.selectSingleNode("//*[(@name = '" + name + "') and not(ancestor::fm:path)]");
    if (testElement == null)
        uniqName = true;
    else
    {
        if (showAlert)
            alert("Name already in use! Please specify a unique name.");
        else
           uniqName = false;// alert("Structure already includes element of this name.");
    }

    //alert(uniqBool);
    return uniqName
}

/**
 * Convenience method for searching an array for unique names.
 */

function arrayContains(array, value)
{
    var found = false;
    for (var i = 0; i < array.length; ++i)
    {
        if (array[i] == value)
        {
            found = true;
            break;
        }
    }
    return found;
}

/**
 * Convenience method to check for invalid name entries
 */
function checkInvalidNames(name)
{
    var nameInvalid = false;
    if(arrayContains(invalidNames, name))
    {
        nameInvalid = true;
        alert("The name of the element you have chosen is invalid.  Please choose another.");
    }

    //alert(nameInvalid);
    return nameInvalid;
}

/* Checks for invalid entry characters */
function checkInvalidChars(id)
{
  //alert('in checkInvalidChars');
  var filter = /[\W]/;
  if((id == '') || (filter.test(id)) || !isNaN(id.charAt(0)))
  {
    alert('Names cannot be blank, include non-word characters, or start with a number. Please specify a new unique name.');
    return true;
  }
  else
      return false;
}

/**
 * Responsible for ensuring that the bindings file associated with the current structure
 * is updated as needed during structure manipulation operations.
 *
 * This function will be recursively called as needed to deal with copying groups/repeats etc.
 *
 * @param currentElement The DOM node in the structure file for the element being manipulated
 *      For rename (or copy with new name) operations, this should have already been renamed.
 * @param type The type of componenet represented by the currentElement - structure, element, group, repeat
 * @param manageType The type of manipualtion requested -
 *      copy - Copy bindings from the defaultStructure binding file into the current one
 *      delete - Delete the binding information for the currentElement
 *      rename - Rename the binding section for the currentElement
 * @param nameMap An associative array of name mappings.  This is provided in the cases where the data in the
 *      currentElement fragment has been renamed from its original name, and so can be used to find the
 *      old name of the component that the bindings will still be found under.
 *      The mapping is from new name to old name, ie nameMap[newName] = oldName
 */
function manageBindings(currentElement, type, manageType, nameMap)
{
    var name = currentElement.getAttribute('name');
    var oldName = name;
    if ((typeof(nameMap) != 'undefined') && (nameMap != null) && (nameMap[name] != null))
        oldName = nameMap[name];
    //alert('beginning bindings management: \n' + name + ', ' + type + '\n' + currentElement.xml);

    var currentBinding;

    if (type != 'structure')
    {
        if (manageType == 'copy')
        {
            currentBinding = defaultStructureBindingsXML.selectSingleNode("//*[./@name = \'" + oldName + "\']");
        }
        else
        {
            currentBinding = structureBindingsXML.selectSingleNode("//*[./@name = \'" + oldName + "\']");
        }

        //alert(currentBinding);

        if (currentBinding != null)
        {
            if (manageType == 'copy')
            {
                copyNamespaces();

                //need to check if the binding uses any conflicting namespaces
                var bindingString = xmlToString(currentBinding);
                log.debug("binding before conflict processing: " + bindingString);
                for (ns in namespaceConflicts)
                {
                    bindingString = bindingString.replace(new RegExp(ns+":", "g"), namespaceConflicts[ns]+":");
                    //handle the case the namespace prefix being changed is actually also
                    //used to set the namespace of the tags themselves, not just the contained xpath
                    //values.  This extra replace should ensure that the namespace definition on this
                    //element (if any) also gets updated for the new prefix.
                    bindingString = bindingString.replace(new RegExp("xmlns:"+ns+"=", "g"), "xmlns:"+namespaceConflicts[ns]+"=");
                }
                log.debug("binding after conflict processing: " + bindingString);
                var tempDoc = getXMLDocument(bindingString);

                var newBinding = tempDoc.documentElement;

                //check if we need to update to the new name
                if (name != oldName)
                {
                    log.debug('updating binding fragment name from ' + oldName + ' to ' + name);
                    newBinding.setAttribute('name', name);
                }

                //check if there are any action bindings for actions that dont apply to this page, and if so remove
                //them from the xml fragment
                Sarissa.setXpathNamespaces(tempDoc, FMqualifiedNS);
                var newBindingActions = newBinding.selectNodes("fm:action");
                for (var i = 0; i < newBindingActions.length; i++)
                {
                    //check if this action name is in the lists of those that apply to this page
                    var actionElem = newBindingActions.item(i);
                    //parent actions is just a simple string array so can be easily checked
                    if (!arrayContains(parentActions, actionElem.getAttribute('name')))
                    {
                        //pageActiosn is an object array so we need to loop through it
                        var existsInPageActions = false
                        for (var j = 0; j < pageActions.length; ++j)
                        {
                            if (actionElem.getAttribute('name') == pageActions[j].name)
                            {
                                existsInPageActions = true;
                                break;
                            }
                        }

                        if (!existsInPageActions)
                        {
                            newBinding.removeChild(actionElem);
                        }
                    }
                }

                //import the new binding fragment into the main docuemnt if needed
                if (typeof(structureBindingsXML.importNode) == 'function')
                    newBinding = structureBindingsXML.importNode(newBinding, true);

                //check if there is an existing binding section for this name
                //This shouldn't actually ever happen any more as it can't exist in the structure to have got
                //this far, but check just in case
                var existingBinding = structureBindingsXML.selectSingleNode("//*[./@name = \'" + name + "\']");

                if (existingBinding == null)
                    structureBindingsXML.documentElement.appendChild(newBinding);
                else
                    structureBindingsXML.documentElement.replaceChild(newBinding, existingBinding);

            }
            else if (manageType == 'delete')
            {
                structureBindingsXML.documentElement.removeChild(currentBinding);
            }
            else //rename
            {
                currentBinding.setAttribute('name', name);
            }
        }
    }

    if ((manageType != 'rename') && (type != 'element'))
    {
        var nestedBindings = currentElement.selectNodes("./*[(local-name() = 'element') or (local-name() = 'group') or (local-name() = 'repeat')]");
        for (var i = 0; i < nestedBindings.length; i++)
        {
            //var selectedNodeName = nestedBindings(i).getAttribute('name');
            var selectedNodeType = getBaseName(nestedBindings.item(i));
            //alert('node: ' + i + ':\nparameters to process:\n    ' + selectedNodeName + '\n    ' + selectedNodeType + '\n' + nestedBindings(i).xml)
            manageBindings(nestedBindings.item(i), selectedNodeType, manageType, nameMap);
        }
    }
}

//boolean flag to indicate whether or not the namespaces have been copied across yet
var namespacesCopied = false;
/*stores all namexpaces that conflicted between the current structure binding file,
 * and the default structure bindings file.
 * The 'key' of stored values is the old namespace prefix in the default structure bindings,
 * and the value is the new prefix it has been assigned in the current bindings file.
 * eg namespaceConflicts[oldPrefix] = newPrefix
 */
var namespaceConflicts = new Object();

/**
 * Copies any namespaces from the default structure binding file into the
 * current structure binding file if they are not present.
 *
 * This is needed when copying bindings from existing sections as they may be using
 * different namespace prefixes to those defined in the current bindings.
 * @private
 */
function copyNamespaces()
{
    if (namespacesCopied)
    {
        log.debug("namespaces already copied, so don't need to do it again");
        return;
    }

    log.debug("copying namespaces from the default structure bindings file");
    namespacesCopied = true;

    //We use the hyf.ns functionality for this, so initially clear out the current namespaces
    //so we are starting from a clean slate.
    hyf.ns.namespaces = new Object();

    //first add all namespaces from the current bindings file
    hyf.ns.addNamespacesFromDocument(structureBindingsXML, true, true);

    log.debug("found " + getPropertyCount(hyf.ns.namespaces) + " existing namespaces");


    //loop through all namespaces in the default structure bindings file and add to namespaces object if missing
    namespaceConflicts = hyf.ns.addNamespacesFromDocument(defaultStructureBindingsXML, true, true)

    log.debug("found " + getPropertyCount(namespaceConflicts) + " namespace conflicts");

    //update the current bindings file to use the new namespaces
    var newString = '<xforms xmlns="http://www.hyfinity.com/formmaker" ' + hyf.ns.findNamespaceString() + '/>';

    tempDoc = getXMLDocument(newString);

    var copyNodes = structureBindingsXML.selectNodes("/fm:xforms/*");
    for (var i=0; i < copyNodes.length; ++i)
    {
        tempDoc.documentElement.appendChild(copyNodes.item(i).cloneNode(true));
    }

    structureBindingsXML = tempDoc;

}

/**
 * Returns the number of properties for the given object.
 */
function getPropertyCount(obj)
{
    var count = 0;
    for (prop in obj)
    {
        ++count;
    }
    return count;
}

/**
 * Finds the next available number to be used for a namespace prefix.
 * This assumes a prefix format of 'ns' + number, so works recursively up from the given
 * number until it finds one that gives a prefix that isnt used in any of the three documents.
 * @param current The last used prefix to start checking from  (The first value checked will be one higher than this.)
 * @param namespaces The object containing all the namespaces to check against, keyed on prefix value.
 * @return The number that can be used (along with the 'ns' start) to give a unique namespace prefix.
 * @private
 */
function findNextAvailableNSNumber(current, namespaces)
{
    var checkPrefix = 'ns' + (current + 1);

    if (namespaces[checkPrefix])
    {
        return findNextAvailableNSNumber(current + 1, namespaces);
    }

    return (current + 1);

}

/**
 * Checks if the given object has a property defined with the given value
 * @private
 */
function propertyValueDefined(value, object)
{
    for (prop in object)
    {
        if (object[prop] == value)
        {
            return true;
        }
    }
    return false;
}

/**
 * Eventhandler for the structure treeview mouse up event
 *
 * @param xpath The xpath to the element generating the event
 * @param setRow Optional parameter to indicate the row (of a grid group) to insert into
 * @param setCol Optional parameter to indicate the col (of a grid group) to insert into
 *        These two parameters will only be present if the mouse was released on an 'empty cell' field
 *        In this case, the target will be the grid group to insert into, and the setRow, and setCol fields
 *        Indicate the location of the empty cell to replace.
 */
function structureTreeviewMouseup(xpath, setRow, setCol)
{
    log.trace('in structureTreeviewMouseup. row: ' + setRow + ', col: ' + setCol);

    var mouseUpElement = structureXML.selectSingleNode(xpath);

    if((selectedElement != null) && typeof(selectedElement) != 'undefined')
    {
        if (mouseDownSource == 'structure')
        {
            log.debug('moving within structure');
            if (selectedElement == mouseUpElement)
                log.info('source and target components are the same!');
            else
            {
                var container = selectedElement.parentNode;
                if (appendToStructure(mouseUpElement, selectedElement, setRow))
                {
                    checkGridMove(container, selectedElement, mouseUpElement, setRow, setCol);
                    changeManager.storeMove(selectedElement, container);
                }
            }
        }
        else
        {
            processAdditionToStructure(mouseUpElement, selectedElement, setRow, setCol);
        }
    }

    selectedElement = null;

    //Pass the event back up to the container
    document.onmouseup();

    //Redisplay the structure treeview
    drawStructureTreeView();
}



function processAdditionToStructure(target, nodeToAdd, setRow, setCol)
{
    //create an associative array to map any names that need to be changed due to conflicts
    //This will map from new name to old one, and will be used for the bindings work later.
    var nameMap = new Object();

    log.debug('Checking name conflicts for ' + getBaseName(nodeToAdd) + ' ' + nodeToAdd.getAttribute('name'));
    if ((getBaseName(nodeToAdd) == 'element') || (getBaseName(nodeToAdd) == 'custom'))
    {
        var currentName = nodeToAdd.getAttribute("name");
        var elementName = uniqueNameGenerator(currentName);
        if (elementName != currentName)
        {
            nameMap[elementName] = currentName;
            nodeToAdd.setAttribute("name", elementName);
            log.info('updated element name from ' + currentName + ' to ' + elementName);
        }
    }
    else
    {
        var elementNodeList = nodeToAdd.selectNodes("descendant-or-self::*[((local-name(.) = 'element') or (local-name(.) = 'group') or (local-name(.) = 'repeat') or (local-name(.) = 'custom') or (local-name(.) = 'structure')) and not(ancestor::fm:path)]");
        var currentElement = "";
        var elementName = "";
        //alert('length' + elementNodeList.length);
        for (var i = 0; i < elementNodeList.length; i++)
        {
            //alert(elementNodeList[i].xml);
            currentElement = elementNodeList[i].selectSingleNode("@name");
            if (currentElement == null) //meaning it is a structure tag
                elementCheck = true;
            else
            {
                //sets the unique name
                var currentName = currentElement.nodeValue;
                elementName = uniqueNameGenerator(currentName)
                if (elementName != currentName)
                {
                    //update to new name, and store reference in map for later use
                    nameMap[elementName] = currentName;
                    currentElement.nodeValue = elementName;
                    log.info('updated component name from ' + currentName + ' to ' + elementName);

                    //if this component is contained within a 'grid' group, we need to also update
                    //the name in the grid specific metadata

                    if ((elementNodeList[i].parentNode != null) &&
                        (getBaseName(elementNodeList[i].parentNode) == 'group')  &&
                        (elementNodeList[i].parentNode.getAttribute('gen_method') == 'grid'))
                    {
                        log.debug("Updating associated grid_component entry");
                        var group = elementNodeList[i].parentNode;
                        var gridComp = group.selectSingleNode("fm:grid_structure/fm:grid_component[@name = '" + currentName + "']")
                        if (gridComp != null)
                            gridComp.setAttribute("name", elementName);
                    }

                }
            }
        }
    }

    processPageLevelFeatures();

    log.debug("nodeToAdd baseName is " + getBaseName(nodeToAdd));

    if (getBaseName(nodeToAdd) != 'structure')
    {
        nodeToAdd = nodeToAdd.cloneNode(true);
        appendToStructure(target, nodeToAdd, setRow);
        checkGridAdd(nodeToAdd, target, setRow, setCol);
        changeManager.storeAddition(nodeToAdd);
        checkPathDataSourceIdChange(nodeToAdd);
    }
    else
    {
        var nodeToAdds = nodeToAdd.selectNodes("./*[local-name() != 'form_details' and local-name() != 'template_resources']");
        for (var i = 0; i < nodeToAdds.length; i++)
        {
            nodeToAdd = nodeToAdds.item(i).cloneNode(true);
            appendToStructure(target, nodeToAdd, setRow);
            checkGridAdd(nodeToAdd, target, setRow, setCol);
            changeManager.storeAddition(nodeToAdd);
            checkPathDataSourceIdChange(nodeToAdd);
        }
    }
    if ((typeof(structureBindingsXML) != 'undefined') && (typeof(defaultStructureBindingsXML) != 'undefined'))
    {
        manageBindings(nodeToAdd, getBaseName(nodeToAdd), 'copy', nameMap);
    }
}


/**
 * checks if we need to add any page level settings (eg events or resources)
 * when dragging a new component across.
 */
function processPageLevelFeatures()
{
    if (mouseDownSource == 'palette')
    {
        //Store any resources that are used by the dragged control.
        if (objPaletteConfig.objPalette.strCurrentControlResources != "")
        {
            objPaletteConfig.objChangeManager.setResources(objPaletteConfig.objPalette.strCurrentControlResources);
            log.debug('Resources cached for this control' + Sarissa.escape(objPaletteConfig.objPalette.strCurrentControlResources));
        }
    }

    //Check if we need to include any page level events
    //we only do this when we are dragging the whole structure element across
    if (getBaseName(selectedElement) == 'structure')
    {
        if (selectedElement.selectSingleNode("*[local-name() = 'form_details']/*[local-name() = 'page_events']/*"))
        {
            var targetContainer = structureXML.selectSingleNode("/fm:structure/fm:form_details/fm:page_events");
            if (targetContainer == null)
            {
                targetContainer = createNamespaceElement(structureXML, 'page_events', FMdefaultNS);
                structureXML.selectSingleNode("/fm:structure/fm:form_details").appendChild(targetContainer);
            }
            addMissingEventActions(selectedElement.selectSingleNode("*[local-name() = 'form_details']/*[local-name() = 'page_events']"), targetContainer);
        }
        if (selectedElement.selectSingleNode("*[local-name() = 'form_details']/*[local-name() = 'external_events']/*"))
        {
            var targetContainer = structureXML.selectSingleNode("/fm:structure/fm:form_details/fm:external_events");
            if (targetContainer == null)
            {
                targetContainer = createNamespaceElement(structureXML, 'external_events', FMdefaultNS);
                structureXML.selectSingleNode("/fm:structure/fm:form_details").appendChild(targetContainer);
            }
            addMissingEventActions(selectedElement.selectSingleNode("*[local-name() = 'form_details']/*[local-name() = 'external_events']"), targetContainer);
        }
    }
}



var waitingForAppMapAddedElem;
/**
 * Checks if the given element has been added to this page from an existing page.
 * If so, this then checks if any of the components being added have a path section that
 * provides a data source id.
 * If this data source is also defined for the current page, then the id value is updated accordingly.
 * If the data source has not been used by this page, then the path section is removed for these fields.
 * @param elem The DOM Element in the structure file that has just been added.
 */
function checkPathDataSourceIdChange(elem)
{
    if (mouseDownSource == 'existing_page')
    {
        log.debug('new additions came from existing page');
        var paths = elem.selectNodes(".//fm:path[@data_source_id != '']");
        if (paths.length > 0)
        {
            log.debug('have ' + paths.length + ' paths to check data source id for.');
            //check that the app map has been loaded
            if (appMapDoc != null)
            {
                //clear out the cached reference while the app map loaded
                waitingForAppMapAddedElem = null;

                var pageName = form.default_structure_filename.value;
                //remove the '_structure.xml' from the end of the filename
                pageName = pageName.substring(0, (pageName.length - 14))

                log.debug('existing page name is ' + pageName);

                //find the existing page in the app map
                var existingPage = appMapDoc.selectSingleNode("/fm:application/fm:pages/fm:page[@id = '"+pageName+"']");

                //for each path
                for (var i = 0; i < paths.length; ++i)
                {
                    var path = paths.item(i);
                    var dsId = path.getAttribute('data_source_id');
                    log.debug('checking path with existing id ' + dsId);
                    //find the matching data source details
                    var dsDetails = null;
                    if (existingPage != null)
                        dsDetails = existingPage.selectSingleNode("fm:data_sources/fm:data_source[@id = '"+dsId+"']");

                    log.debug('found existing ds details: ' + dsDetails.getAttribute('type') + ', ' + dsDetails.getAttribute('name') + ', ' + dsDetails.getAttribute('action') + ', ' + dsDetails.getAttribute('use'));

                    var newId = findCurrentPageIdForDS(dsDetails);
                    //check if it exists in the current page
                    if (newId != null)
                    {
                        //if so update the source id
                        log.debug('updating to new id ' + newId);
                        path.setAttribute('data_source_id', newId);
                    }
                    else
                    {
                        //otherwise remove the path section
                        log.debug('no matching data source found for this page so removing path info');
                        //QUESTION: Does this cause any issues with iterating through the nodeset???
                        path.parentNode.removeChild(path);
                    }
                }
            }
            else
            {
                //wait for app map to load.
                waitingForAppMapAddedElem = elem;
                window.setTimeout("checkPathDataSourceIdChange(waitingForAppMapAddedElem)", 500);
            }
        }
    }
}


/**
 * checks through all the data sources used by this page to see if any of them
 * match the details of the provided element.
 * If so then the ID (in the app map) of the match is returned.
 * If no match, then null is returned.
 */
function findCurrentPageIdForDS(ds)
{
    if (ds == null)
    {
        return null;
    }
    for (var i=0; i < pageDataSources.length; ++i)
    {
        pds = pageDataSources[i];
        log.debug('checking against page ds ' + pds.type + ', ' + pds.name + ', ' + pds.action + ', ' + pds.use)
        if ((pds.type == ds.getAttribute('type')) &&
            (pds.name == ds.getAttribute('name')) &&
            (pds.action == ds.getAttribute('action')) &&
            (pds.use == ds.getAttribute('use')))
        {
            return pds.id;
        }
    }

    return null;
}


/**
 * Checks through all the events defined under the source container, and makes sure that
 * they are defined under the target container, by adding them as needed.
 */
function addMissingEventActions(sourceContainer, targetContainer)
{
    log.trace("in addMissingEventActions ");

    if ((sourceContainer == null) || (targetContainer == null))
        return;

    var sourceActions = sourceContainer.selectNodes("*[local-name() = 'events']/*[local-name() = 'event']/*[local-name() = 'action']");

    for (var i = 0; i < sourceActions.length; ++i)
    {
        var action = sourceActions.item(i);
        var event = action.parentNode;

        log.debug("Checking event " + event.getAttribute('id') + ", action " + action.getAttribute('id'));

        var existingEvent = targetContainer.selectSingleNode("fm:events/fm:event[@id = '" + event.getAttribute('id') + "']");
        if (existingEvent == null)
        {
            //just copy the whole source event section
            var eventsContainer = targetContainer.selectSingleNode("fm:events");
            if (eventsContainer == null)
            {
                eventsContainer = createNamespaceElement(targetContainer.ownerDocument, "events", FMdefaultNS);
                targetContainer.appendChild(eventsContainer);
            }

            eventsContainer.appendChild(event.cloneNode(true));
        }
        else
        {
            //check if there is an existing action that matches
            var matchingAction = false;

            var existingActions = existingEvent.selectNodes("fm:action[@id = '" + action.getAttribute('id') + "']");
            var serializer = new XMLSerializer()
            for (var j = 0; j < existingActions.length; ++j)
            {
                //is it okay to just do a string match on the XML??
                if (serializer.serializeToString(action) == serializer.serializeToString(existingActions.item(j)))
                {
                    log.debug("action already exists");
                    matchingAction = true;
                    break;
                }
            }

            if (!matchingAction)
            {
                existingEvent.appendChild(action.cloneNode(true));
            }
        }
    }
}


/** Convenience method which takes in an element. If the name is already unique
* it will simply do nothing. If it already exists, it will return a new name
*/
function uniqueNameGenerator(elementName)
{
  var counter = 2;
  var newName = elementName;
  while(!searchXMLStructure(newName))
  {
    newName = elementName+'_'+counter;
    counter++;
  }
  return newName;
}

//////////////////////////////////////////////////////////////////////////
/* Start of functions relating to manipulation of grid structure details.*/
//////////////////////////////////////////////////////////////////////////

/**
 * Checks if the given element to remove is within a grid group, and if so
 * removes the relevant section of the grid_structure XML
 * @param group The Node in the structure XML that contained the component removed
 * @param compName The name of the component being removed.
 */
function checkGridRemove(group, compName)
{
    //check if the elem is within a group
    if (getBaseName(group) == 'group')
    {
        var gridComp = group.selectSingleNode("fm:grid_structure/fm:grid_component[@name = '"+compName+"']");
        if (gridComp != null)
        {
            //alert('removing grid component');
            gridComp.parentNode.removeChild(gridComp);
            //alert(group.selectSingleNode("fm:grid_structure").xml);
        }
    }
}

/**
 * Checks if the given element to rename is within a grid group, and if so
 * updates the relevant section of the grid_structure XML
 * @param renameElem The Node in the structure XML representing the component being renamed.
 * @param newName the new name being given to the component
 */
function checkGridRename(renameElem, newName)
{
    //check if the elem is within a group
    var group = renameElem.parentNode;
    if (getBaseName(group) == 'group')
    {
        var gridComp = group.selectSingleNode("fm:grid_structure/fm:grid_component[@name = '"+renameElem.getAttribute('name')+"']");
        if (gridComp != null)
        {
            //alert('renaming grid component');
            gridComp.setAttribute('name', newName);
            //alert(group.selectSingleNode("fm:grid_structure").xml);
        }
    }
}

/**
 * Handles manipulation of the grid_structure details on a move operation.
 * This includes checking whether either the source or target locations are within grid groups.
 * @param sourceContainer The DOM element that contained the source before it was moved
 * @param source The DOM element being moved.
 * @param target the DOM element dropped on - The addMode will affect the actual target location
 *               in relation to this.
 * @param setRow Optional parameter to indicate the row to insert into
 * @param setCol Optional parameter to indicate the col to insert into
 *        These two parameters will only be present if the mouse was released on an 'empty cell' field
 *        In this case, the target will be the gris group to insert into, and the setRow, and setCol fields
 *        Indicate the location of the empty cell to replace.
 */
function checkGridMove(sourceContainer, source, target, setRow, setCol)
{
    //remove any source grid details
    checkGridRemove(sourceContainer, source.getAttribute('name'));

    //check if new grid details needed
    checkGridAdd(source, target, setRow, setCol);
}


/**
 * Checks if any grid_structure details need to be set up for adding the given componenet
 * when dropped on the specified target
 * @param comp The DOM element being added.
 * @param target the DOM element dropped on - The addMode will affect the actual target location
 *               in relation to this.
 * @param setRow Optional parameter to indicate the row to insert into
 * @param setCol Optional parameter to indicate the col to insert into
 *        These two parameters will only be present if the mouse was released on an 'empty cell' field
 *        In this case, the target will be the grid group to insert into, and the setRow, and setCol fields
 *        Indicate the location of the empty cell to replace.
 */
function checkGridAdd(comp, target, setRow, setCol)
{
    //handle the insert onto empty cell special case.
    if ((typeof(setRow) != 'undefined') && (setRow != null) && (typeof(setCol) != undefined) && (setCol != null))
    {
        var gridStruc = target.selectSingleNode("fm:grid_structure");
        var newComp = createNamespaceElement(structureXML, "grid_component", FMdefaultNS);
        newComp.setAttribute("row", setRow);
        newComp.setAttribute("col", setCol);
        newComp.setAttribute("name", comp.getAttribute('name'));
        gridStruc.appendChild(newComp);

        //if needed, move the new component into the correct position, so that the order of child
        //nodes matches the grid layout
        var next = findFollowingComponent(gridStruc, setRow, setCol);
        if (next != null)
        {
            comp.parentNode.insertBefore(comp, next);
        }
        else
        {
            comp.parentNode.appendChild(comp);
        }

        return;
    }


    var targetGroup;
    var insertMode;
    if (addMode == 'below')
    {
        targetGroup = target.parentNode;
        insertMode = 'after';
    }
    else if (addMode == 'above')
    {
        targetGroup = target.parentNode;
        insertMode = 'before';
    }
    //addMode == 'into'
    else if ((getBaseName(target) == 'repeat') || (getBaseName(target) == 'group'))
    {
        targetGroup = target;
        insertMode = 'into';
    }
    else
    {
        targetGroup = target.parentNode;
        insertMode = 'after';
    }

    if ((getBaseName(targetGroup) == 'group') && (targetGroup.getAttribute('gen_method') == 'grid'))
    {
        var gs = targetGroup.selectSingleNode("fm:grid_structure");
        var rows = gs.getAttribute('rows');
        var cols = gs.getAttribute('cols');

        var insertPoint = new Object();

        if (insertMode == 'into')
        {
            insertPoint = findFreePosition(gs);
        }
        else
        {
            var targetGC = gs.selectSingleNode("fm:grid_component[@name = '"+target.getAttribute('name')+"']");
            var targetRow = Number(targetGC.getAttribute('row'));
            var targetCol = Number(targetGC.getAttribute('col'));

            if (insertMode == 'before')
            {
                //alert('inserting before');
                //check if the previous cell is empty
                var prevCol, prevRow;
                if (targetCol == 1)
                {
                    if (targetRow == 1)
                    {   //no previous cell
                        prevCol = targetCol;
                        prevRow = targetRow;
                    }
                    else
                    {
                        prevCol = cols;
                        prevRow = targetRow - 1;
                    }
                }
                else
                {
                    prevCol = targetCol - 1;
                    prevRow = targetRow;
                }

                //alert('previous cell is: ' + prevRow + ', ' + prevCol);

                if (checkGridCellEmpty(gs, prevRow, prevCol))
                { //if empty add comp at previous cell
                    insertPoint.row = prevRow;
                    insertPoint.col = prevCol;
                }
                else
                {//if not insert comp at target position, and shift all following cells
                    insertPoint.row = targetRow;
                    insertPoint.col = targetCol;
                    shiftCells(gs, targetRow, targetCol);
                }
            }
            else if (insertMode == 'after')
            {
                //alert('inserting after');
                //check if the following cell is empty
                var nextCol, nextRow;
                if (targetCol == cols)
                {
                    nextCol = 1;
                    nextRow = targetRow + 1;
                }
                else
                {
                    nextCol = targetCol + 1;
                    nextRow = targetRow;
                }

                if (checkGridCellEmpty(gs, nextRow, nextCol))
                { //if empty add comp at following cell
                    insertPoint.row = nextRow;
                    insertPoint.col = nextCol;
                }
                else
                { //if not insert comp after target position, and shift all following cells
                    insertPoint.row = nextRow;
                    insertPoint.col = nextCol;
                    shiftCells(gs, nextRow, nextCol);
                }
            }
        }

        var newComp = createNamespaceElement(structureXML, "grid_component", FMdefaultNS);
        newComp.setAttribute("row", insertPoint.row);
        newComp.setAttribute("col", insertPoint.col);
        newComp.setAttribute("name", comp.getAttribute('name'));
        gs.appendChild(newComp);

        //if needed, move the new component into the correct position, so that the order of child
        //nodes matches the grid layout
        var next = findFollowingComponent(gs, insertPoint.row, insertPoint.col);
        if (next != null)
        {
            comp.parentNode.insertBefore(comp, next);
        }
        else
        {
            comp.parentNode.appendChild(comp);
        }

        //check if the grid size needs to be increased
        var maxRow = 0;// = gs.selectSingleNode("fm:grid_component[not(../fm:grid_component/@row > @row)][1]").getAttribute('row');
        var maxCol = 0;// = gs.selectSingleNode("fm:grid_component[not(../fm:grid_component/@col > @col)][1]").getAttribute('col');
        //Need to allow for an element spanning beyond the bottom row
        var components = gs.selectNodes("fm:grid_component");
        for (var i=0; i<components.length; i++)
        {
            var colspan = (components.item(i).getAttribute('colspan')) ? components.item(i).getAttribute('colspan') : 1;
            if ((Number(components.item(i).getAttribute('col')) + Number(colspan) - Number(1)) > maxCol)
                maxCol = (Number(components.item(i).getAttribute('col')) + Number(colspan) - Number(1));

            var rowspan = (components.item(i).getAttribute('rowspan')) ? components.item(i).getAttribute('rowspan') : 1;
            if ((Number(components.item(i).getAttribute('row')) + Number(rowspan) - Number(1)) > maxRow)
                maxRow = (Number(components.item(i).getAttribute('row')) + Number(rowspan) - Number(1));
        }

        if (maxRow > rows)
            gs.setAttribute('rows', maxRow);
        if (maxCol > cols)
            gs.setAttribute('cols', maxCol);

        //alert(gs.xml);
    }
}

/**
 * Finds the posiiton of the first free cell in the grid
 * and returns it.
 * @private
 * @param gs The grid_structure element
 * @return An object contains two properties, 'row' and 'col', detailing the found location.
 */
function findFreePosition(gs)
{
    var numCols = gs.getAttribute('cols');
    var currentRow = 1;
    var currentCol = 1;
    while (true)
    {
        if (checkGridCellEmpty(gs, currentRow, currentCol))
        {
            var position = new Object();
            position.row = currentRow;
            position.col = currentCol;
            return position;
        }

        if (currentCol == numCols)
        {
            currentRow++;
            currentCol = 1;
        }
        else
        {
            currentCol++;
        }

    }
}

/**
 * Checks if the given grid cell is empty or not
 *
 * @private
 * @param gs The grid_structure element
 * @param row The row of the cell to check
 * @param col The col of the cell to check
 * @return true if the specified cell is empty, false otherwise.
 */
function checkGridCellEmpty(gs, row, col)
{
    if (findGridCellComponent(gs, row, col))
    {
        return false;
    }
    else
    {
        return true;
    }
}

/**
 * finds the compoennt that fills the specified cell in the grid.
 *
 * @private
 * @param gs The grid_structure element
 * @param row The row of the cell to check
 * @param col The col of the cell to check
 * @return the grid_component node if one exists, or null if the cell is currently empty
 */
function findGridCellComponent(gs, row, col)
{
    if (gs.selectSingleNode("fm:grid_component[(@row = "+row+") and (@col = "+col+")]"))
    {
        return gs.selectSingleNode("fm:grid_component[(@row = "+row+") and (@col = "+col+")]");
    }
    else
    {
        //cell empty, so check for span onto this cell

        //first check above
        var cell = checkGridSpan(gs, 2, 1, row-1, col, true)
        if (cell)
        {
            return cell
        }
        else
        {
            //then check left
            return checkGridSpan(gs, 1, 2, row, col-1, false);
        }
    }
}

/**
 * Checks if there is a component starting in the test cell specified, that spans the
 * specified distance.
 * This calls itself recursively to check any cells that could possibly span.
 *
 * @private
 * @param gs The grid_structure element
 * @param rowSpan The amount of rows the component needs to span
 * @param colSpan The amount of cols the component needs to span
 * @param testRow The row of the component to check for
 * @param testcol The col of the component to check for
 * @param checkAbove Boolean indicating whether we should check cells above as well as left or not.
 * @return A boolean indicating whether there is a component that spans correctly or not.
 */
function checkGridSpan(gs, rowSpan, colSpan, testRow, testCol, checkAbove)
{
    if ((testRow > 0) && (testCol > 0))
    {
        var xpath = "fm:grid_component[(@row = "+testRow+") and (@col = "+testCol+")";
        if (rowSpan > 1)
            xpath +=  "and (@rowspan >= " + rowSpan + ")";

        if (colSpan > 1)
            xpath +=  "and (@colspan >= " + colSpan + ")";

        xpath += "]";
        if (gs.selectSingleNode(xpath))
        {
            return gs.selectSingleNode(xpath);
        }
        else
        {
            //check left
            var left = checkGridSpan(gs, rowSpan, colSpan + 1, testRow, testCol - 1, checkAbove)
            if (left)
            {
                return left;
            }
            else
            {
                //check above
                if (checkAbove)
                {
                    var above = checkGridSpan(gs, rowSpan + 1, colSpan, testRow - 1, testCol, checkAbove);
                    if (above)
                    {
                        return above;
                    }
                }
            }
        }
    }
    return null;
}

/**
 * Shifts the position of all components in the grid starting from the
 * given grid position.
 * All components are moved one position to the right, wrapping on to new rows if needed,
 * until either an empty cell is found, or all components are moved.
 *
 * @private
 * @param gs The grid_structure element
 * @param row The row number of the first element to shift
 * @param col The column number of the first element to shift
 */
function shiftCells(gs, row, col)
{
    var cell = gs.selectSingleNode("fm:grid_component[(@row = "+row+") and (@col = "+col+")]");

    if (cell)
    {
        //there is a component in this cell to move
        shiftCellsMoveComponent(gs, row, col, cell);
    }
}


function shiftCellsMoveComponent(gs, row, col, cell)
{
    log.trace("In shiftCellsMoveComponent");
    log.debug("row: " + row + ", col: " + col + ", cell: " + cell.getAttribute('name'));

    //determine the next cell location
    var newRow, newCol;
    if (col == gs.getAttribute('cols'))
    {
        newRow = Number(row) + 1;
        newCol = 1;
    }
    else
    {
        newRow = row;
        newCol = Number(col) + 1;
    }

    log.debug("newRow: "+ newRow + ", newCol: " + newCol);

    //blank out this cells location so it doesnt affect the searching
    cell.setAttribute('row', '');
    cell.setAttribute('col', '');

    //find the position to move the cell into
    var pos = findTargetGridPosition(gs, newRow, newCol, cell);

    log.debug("Found new position: " + pos.row + ", " + pos.col);

    //check for any components that this move will effect
    var cellsToShift = findAffectedGridComponents(gs, pos.row, pos.col, cell);

    log.debug("found "+cellsToShift.length+" components to shift");

    //update the position of this component
    cell.setAttribute('row', pos.row);
    cell.setAttribute('col', pos.col);

    //shift affected components out the way
    for (var i = cellsToShift.length - 1; i >= 0; --i)
    {
        var detail = cellsToShift[i];
        shiftCellsMoveComponent(gs, detail.row, detail.col, detail.gc);
    }
}

/**
 * Finds the first possible position (starting with the specified location)
 * that the provided component could be moved into.
 * This ignores existing components, with the exception of those that span from above into
 * a location.
 *
 * @private
 */
function findTargetGridPosition(gs, row, col, comp)
{
    log.trace("in findTargetGridPosition");
    log.debug("row: " + row + ", col: " + col + ", comp: " + comp.getAttribute('name'));

    if (row > 10)
        return null;

    var numCols = gs.getAttribute('cols');
    var colspan = (comp.getAttribute('colspan')) ? comp.getAttribute('colspan') : 1;
    var endPoint = Number(col) + Number(colspan) - Number(1);

    log.debug("endPoint is " + endPoint + ", numCols is " + numCols);

    if ((endPoint <= numCols) && gs.selectSingleNode("fm:grid_component[(@row = "+row+") and (@col = "+col+")]"))
    {
        var position = new Object();
        position.row = row;
        position.col = col;
        return position;
    }
    else if ((endPoint <= numCols) && checkGridCellEmpty(gs, row, col))
    {
        var position = new Object();
        position.row = row;
        position.col = col;
        return position;
    }
    else
    {
        //something spans onto this cell, or our component wouldn't fit,  so check the next one.
        var newRow, newCol;
        if (col == numCols)
        {
            newRow = Number(row) + 1;
            newCol = 1;
        }
        else
        {
            newRow = row;
            newCol = Number(col) + 1;
        }

        return findTargetGridPosition(gs, newRow, newCol, comp);

    }
}


/**
 * Finds all the current components in the grid that will be affected if the specified
 * component is moved inot the specified position.
 * This returns an array containing details of all the affected components
 * @private
 */
function findAffectedGridComponents(gs, row, col, comp)
{
    log.trace("in findAffectedGridComponents");
    var cellsToShift = new Array();

    var colspan = (comp.getAttribute('colspan')) ? comp.getAttribute('colspan') : 1;
    var rowspan = (comp.getAttribute('rowspan')) ? comp.getAttribute('rowspan') : 1;

    log.debug("row: " + row + ", col: " + col + ", colspan: " + colspan + ", rowspan: " + rowspan);

    //check every cell the component will span over when placed in this position
    for (var i = row; i < (Number(row) + Number(rowspan)); ++i)
    {
        for (var j = col; j < (Number(col) + Number(colspan)); ++j)
        {
            log.debug("Checking for component at " + i + "," + j);

            var comp = findGridCellComponent(gs, i, j);

            if (comp)
            {
                var cellDetail = new Object();
                cellDetail.gc = comp;
                cellDetail.row = comp.getAttribute('row');
                cellDetail.col = comp.getAttribute('col');
                if (cellDetail.row != '' && cellDetail.col != '')
                {
                    comp.setAttribute('row', '');
                    comp.setAttribute('col', '');
                    cellsToShift[cellsToShift.length] = cellDetail;
                }
            }
        }
    }

    return cellsToShift;
}

/**
 * Returns the DOM Node representing the first element in the grid after the
 * given position.
 * @param gs The grid_structure element from the structure file.
 * @param row The row number to find the following component for.
 * @param col The column number to find the following component for.
 * @return The node in the structure file for the found component, or null if no following component
 */
function findFollowingComponent(gs, row, col)
{
    var numRows = gs.getAttribute('rows');
    var numCols = gs.getAttribute('cols');
    var newRow, newCol;
    if (col == numCols)
    {
        newRow = Number(row) + 1;
        newCol = 1;
    }
    else
    {
        newRow = row;
        newCol = Number(col) + 1;
    }
    //if we have reached the end of the grid return null
    if (newRow > numRows)
    {
        return null;
    }
    var nextCell = gs.selectSingleNode("fm:grid_component[(@row = "+newRow+") and (@col = "+newCol+")]");
    //if next cell is empty check the following one
    if (nextCell == null)
    {
        return findFollowingComponent(gs, newRow, newCol);
    }
    else  //otherwise return the component contained in it
    {
        return gs.selectSingleNode("../*[@name = '" + nextCell.getAttribute('name') + "']");
    }

}

//////////////////////////////////////////////////////////////////////////
/* End of functions relating to manipulation of grid structure details.*/
//////////////////////////////////////////////////////////////////////////


var pageStructureTreeDDID = null;

/**
 * Draws an updated structure treeview on screen.
 */
function drawStructureTreeView(type)
{
    if (pageStructureTreeDDID != null)
    {
        dragDropHandler.detachContainer(pageStructureTreeDDID);
        pageStructureTreeDDID = null;
    }


    log.debug('Redrawing structure');
    if (typeof(type) != 'undefined')
    {
        if (form.structureDisplayType.value != type)
        {
            form.structureDisplayType.value = type;
            document.getElementById('StructureTreeview').scrollTop = 0;
        }
    }

    if (form.structureDisplayType.value == 'tree')
    {
        var xmlToTransform = '<container><namespaces><namespace prefix="fm" uri="http://www.hyfinity.com/formmaker"/></namespaces>'
                            + xmlToString(structureXML) + '</container>';
        var docToTransform = getXMLDocument(xmlToTransform);
        treeStructureTransform.transform(docToTransform);
        document.getElementById('layoutViewTab').className = '';
        document.getElementById('treeViewTab').className = 'current';

        //initialise drag drop container
        pageStructureTreeDDID = dragDropHandler.attachContainer(document.getElementById('StructureTreeview'), 'page_component', "dragDropComplete", "dragdrop-item", "both", -2.5);
    }
    else
    {
        layoutStructureTransform.transform(structureXML);
        document.getElementById('layoutViewTab').className = 'current';
        document.getElementById('treeViewTab').className = '';

        //initialise drag drop container
        pageStructureTreeDDID = dragDropHandler.attachContainer(document.getElementById('StructureTreeview'), 'page_component', "dragDropComplete", "dragdrop-item", "both", 0, {top:true,bottom:true,left:true,right:true,into:true});

        //size the spacer element accordingly so that the user can easily drag components
        //into a relatively empty page.  this spacer provides a drop target to drag components
        //to the end of the current content
        if (document.getElementById('section_structure_spacer') != null)
        {

            var reqSize = document.getElementById('section_structure').offsetHeight -
                    document.getElementById('section_structure_content').offsetHeight;
            if (reqSize > 0)
            {
                document.getElementById('section_structure_spacer').style.height = reqSize + 'px';
            }
        }
    }
}

/**
 * Function to handle processing a drag and drop operation using the newer dragdrop api.
 * @param sc The HTML component that provides the container from with the drag initiated.
 * @param se The specific HTML element in the source container that was dragged.
 * @param tc The HTML component that provides the container into which the component was dragged.
 * @param te The specific element in the target container that the drag was finished on.
 * @param loc A string specifying the location requested relative to the target element.
 * @param e The browser event object for the mouseup event
 */
function dragDropComplete(sc, se, tc, te, loc, e)
{
    log.info('in dragDropComplete');

    var targetXPath = te.getAttribute('_xpath');
    if (targetXPath == null)
        return false;

    if (targetXPath == '/*')
        loc = 'into';


    if ((selectedElement == null) || (typeof(selectedElement) == 'undefined'))
    {
        return false;
    }

    if (loc == 'merge')
    {
        log.info('merge requested!');
        processMergeRequest(selectedElement, structureXML.selectSingleNode(targetXPath));
        return true;
    }


    switch (loc)
    {
        case 'top'      :
        case 'left'     : addMode = 'above'; break;
        case 'bottom'   :
        case 'right'    : addMode = 'below'; break;
        case 'into'     : addMode = 'into'; break;
    }


    if (dojo.hasClass(te, 'sv_empty_grid_element'))
        structureTreeviewMouseup(targetXPath, te.getAttribute('_row'), te.getAttribute('_col'), e);
    else
        structureTreeviewMouseup(targetXPath, null, null, e);

    return true;
}

/**
 * This function processes a merge operation.
 * This is used to combine the details from the source element onto the existing
 * target element.
 * The exact details of the combination process will depend on the different types of source
 * and target elements.
 * @param sourceElem The node in the source XML structure file for the component to merge from
 * @param targetElem The node in the current page XML structure file for the component to merge to
 */
function processMergeRequest(sourceElem, targetElem)
{
    if (mouseDownSource == 'palette')
    {
        log.debug('merging from palette');
        //when merging from the palette, we are only interested in visual properties.


        //Check if any palette resources or page level events need to be added.
        if (getBaseName(sourceElem) == 'structure')
        {
            processPageLevelFeatures();

            //process the first component in the structure
            sourceElem = sourceElem.selectSingleNode("(fm:element|fm:group|fm:repeat|fm:custom)[1]");
        }

        //combine the XML for both the source and target into a single docuemnt, which can then be transformed
        var xmlStr = '<merge_request xmlns="http://www.hyfinity.com/formmaker"><source>'
                + xmlToString(sourceElem)
                + '</source><target>'
                + xmlToString(targetElem)
                + '</target></merge_request>';

        log.debug('about to perform merge transform');
        var resultDoc = mergeStructureTransform.transformToXML(xmlStr);
        if (log.isDebugEnabled())
            log.debug('received transform output' + xmlToString(resultDoc));

        if ((resultDoc != null) && (resultDoc.documentElement != null))
        {
            var updatedElem = resultDoc.documentElement;
            //replace the target element xml with the merged one
            if (typeof(structureXML.importNode) == 'function')
                updatedElem = structureXML.importNode(updatedElem, true);

            targetElem.parentNode.replaceChild(updatedElem, targetElem);
            log.debug('structure file updated');
            changeManager.setChanged(false);

            //if a goup has just been changed to have the 'grid' display method
            //then check to make sure that the grid structure meta data isFinite correct
            if ((getBaseName(updatedElem) == 'group') && (updatedElem.getAttribute('gen_method') == 'grid'))
            {
                svCheckGridStructure(updatedElem);
            }

            //check if te control typeof of a field has been changed, and if so store the details
            //for later processing of the instance documetns on the server
            //eg buttons wont need an entry in the to page document.
            if ((getBaseName(updatedElem) == 'element') &&
                (getElementText(updatedElem.selectSingleNode('fm:field/fm:input')) != getElementText(targetElem.selectSingleNode('fm:field/fm:input'))))
            {
                changeManager.storeFieldTypeChange(updatedElem, getElementText(targetElem.selectSingleNode('fm:field/fm:input')));
            }

            //redraw the centre panel
            drawStructureTreeView();
        }
    }
    else if (mouseDownSource == 'datasource')
    {
        log.debug('merging from data source');
        //when merging fom a data source, we are mainly interested in binding related details which
        //are handled on the server, so just need to store the details away to process later.

        if (getBaseName(sourceElem) == 'element')
        {
            //We do however make some changes to the structure file as well when dealing with
            //merging of individual fields, so still use a transform


            //combine the XML for both the source and target into a single docuemnt, which can then be transformed
            var xmlStr = '<merge_request xmlns="http://www.hyfinity.com/formmaker"><source>'
                    + xmlToString(sourceElem)
                    + '</source><target>'
                    + xmlToString(targetElem)
                    + '</target></merge_request>';

            log.debug('about to perform merge transform');
            var resultDoc = mergeStructureFromDSTransform.transformToXML(xmlStr);
            if (log.isDebugEnabled())
                log.debug('received transform output' + xmlToString(resultDoc));
            if ((resultDoc != null) && (resultDoc.documentElement != null))
            {
                var updatedElem = resultDoc.documentElement;
                //replace the target element xml with the merged one
                if (typeof(structureXML.importNode) == 'function')
                    updatedElem = structureXML.importNode(updatedElem, true);

                targetElem.parentNode.replaceChild(updatedElem, targetElem);
                log.debug('structure file updated');
                targetElem = updatedElem;
            }

            //update field name
            var oldName = targetElem.getAttribute('name');
            var newName = uniqueNameGenerator(sourceElem.getAttribute('name'));
            if (oldName != newName)
            {
                targetElem.setAttribute('name', newName);

                //keep the bindings in sync
                if (typeof(structureBindingsXML) != 'undefined')
                {
                    var nameMapping = new Object();
                    nameMapping[newName] = oldName;
                    manageBindings(targetElem, getBaseName(targetElem), 'rename', nameMapping);
                }

                changeManager.storeRename(targetElem, oldName);
            }

            //store data source change for future processing on the server to
            //update bindings and instance documents.
            changeManager.storeDataSourceChange(targetElem);
        }
        else if (getBaseName(sourceElem) == 'repeat')
        {
            if (getBaseName(targetElem) == 'repeat')
            {
                //we only handle merging repeats with a single group child (ie used for table structures)
                //but we can assume this due tot he check already having been done in the overriden
                //dragDropHandler.getInsertLocations method

                //update repeat name
                var oldName = targetElem.getAttribute('name');
                var newName = uniqueNameGenerator(sourceElem.getAttribute('name'));
                if (oldName != newName)
                {
                    targetElem.setAttribute('name', newName);

                    //keep the bindings in sync
                    if (typeof(structureBindingsXML) != 'undefined')
                    {
                        var nameMapping = new Object();
                        nameMapping[newName] = oldName;
                        manageBindings(targetElem, getBaseName(targetElem), 'rename', nameMapping);
                    }

                    changeManager.storeRename(targetElem, oldName);
                }
                //update repeat path section
                var oldPath = targetElem.selectSingleNode('fm:path');
                if (oldPath != null)
                    targetElem.removeChild(oldPath);
                var newPath = sourceElem.selectSingleNode('fm:path');
                if (newPath != null)
                {
                    if (typeof(structureXML.importNode) == 'function')
                        newPath = structureXML.importNode(newPath, true);

                    targetElem.appendChild(newPath);
                }
                changeManager.storeDataSourceChange(targetElem);

                //switch variables to point to the child groups
                targetElem = targetElem.selectSingleNode("fm:group");
                sourceElem = sourceElem.selectSingleNode("fm:group");

                //update group name
                oldName = targetElem.getAttribute('name');
                newName = uniqueNameGenerator(sourceElem.getAttribute('name'));
                if (oldName != newName)
                {
                    targetElem.setAttribute('name', newName);

                    //keep the bindings in sync
                    if (typeof(structureBindingsXML) != 'undefined')
                    {
                        var nameMapping = new Object();
                        nameMapping[newName] = oldName;
                        manageBindings(targetElem, getBaseName(targetElem), 'rename', nameMapping);
                    }

                    changeManager.storeRename(targetElem, oldName);
                }
                changeManager.storeDataSourceChange(targetElem);

                //update group path section
                oldPath = targetElem.selectSingleNode('fm:path');
                if (oldPath != null)
                    targetElem.removeChild(oldPath);
                newPath = sourceElem.selectSingleNode('fm:path');
                if (newPath != null)
                {
                    if (typeof(structureXML.importNode) == 'function')
                        newPath = structureXML.importNode(newPath, true);

                    targetElem.appendChild(newPath);
                }


                //add in any children of the source group
                var childrenToAdd = sourceElem.selectNodes("*[(local-name() = 'repeat') or (local-name() = 'group') or (local-name() = 'custom') or ((local-name() = 'element') and not(ancestor::fm:path))]");
                for(var i = 0; i < childrenToAdd.length; ++i)
                {
                    //add all children into the group
                    addMode = 'into';
                    var childToAdd = childrenToAdd.item(i).cloneNode(true);

                    //handle the specific case of merging with an editable table repeat, and so make sure all
                    //the new fields are editable controls
                    if (targetElem.parentNode.getAttribute('createNewRowContent') == 'true')
                    {
                        var fieldType = getElementText(childToAdd.selectSingleNode("fm:field/fm:input"));
                        if (fieldType == 'output')
                        {
                            setElementText(childToAdd.selectSingleNode("fm:field/fm:input"), 'text');
                        }
                    }

                    processAdditionToStructure(targetElem, childToAdd);
                }
            }

            if (getBaseName(targetElem) == 'element')
            {
                //handle the special case where a repeat is dragged from the data source onto a
                //select type control - this is used to convert the control to show dynamic options
                //using the repeat data

                var valueElems = sourceElem.selectNodes(".//fm:element[not(ancestor::fm:path)]");

                var selectOptionsHTML = '';
                for(var i = 0; i < valueElems.length; ++i)
                {
                    var opName = valueElems.item(i).getAttribute('name');
                    selectOptionsHTML += '<option value="' + opName + '">' + opName + '</option>';
                }

                var popupHTML = '<div class="message_popup"><p>Please select which fields you would like to use to provide the display text and associated data values for each entry in the dynamic list of options:</p>';
                popupHTML += '<table style="margin: 5px 0px 10px 0px;"><thead><tr><td>Data Value</td><td>Display Text</td></tr></thead><tbody><tr>';
                popupHTML += '<td><select id="doc_data_value">' + selectOptionsHTML + '</select></td>'
                popupHTML += '<td><select id="doc_display_text">' + selectOptionsHTML + '</select></td>'
                popupHTML += '</tr></tbody></table>';

                popupHTML += '<p><a href="#" onclick="processDynamicOptionsFieldSelection()">Continue</a></p>';
                popupHTML += '</div>';

                WinManager.displayMessageBox(popupHTML);


                //store the source and target away for processing once the user has selected the elements.
                tempDynamicOptionsChangeSource = sourceElem;
                tempDynamicOptionsChangeTarget = targetElem;
            }
        }

        //redraw the centre panel
        drawStructureTreeView();
    }
}


var tempDynamicOptionsChangeSource = null;
var tempDynamicOptionsChangeTarget = null;

function processDynamicOptionsFieldSelection()
{
    //change options type to dynamic
    tempDynamicOptionsChangeTarget.selectSingleNode("fm:field/fm:input").setAttribute("options", "dynamic");

    //deteremine fields to use for data and display values

    var dataName = document.getElementById('doc_data_value').value;
    var displayName = document.getElementById('doc_display_text').value;

    var dataElem = tempDynamicOptionsChangeSource.selectSingleNode(".//fm:element[not(ancestor::fm:path) and (@name = '" + dataName + "')]");
    var displayElem = tempDynamicOptionsChangeSource.selectSingleNode(".//fm:element[not(ancestor::fm:path) and (@name = '" + displayName + "')]");

    //store details for server processing
    changeManager.storeDynamicOptionsChange(tempDynamicOptionsChangeTarget,
                                                tempDynamicOptionsChangeSource.selectSingleNode('fm:path'),
                                                dataElem.selectSingleNode('fm:path'),
                                                displayElem.selectSingleNode('fm:path'));

    tempDynamicOptionsChangeSource = null;
    tempDynamicOptionsChangeTarget = null;

    WinManager.clearMessageBox();
}



var defaultStructureTreeDDID = null;
/**
 * Draws an updated default structure treeview on screen.
 */
function drawDefaultStructureTreeView()
{
    if (defaultStructureTreeDDID != null)
    {
        dragDropHandler.detachContainer(defaultStructureTreeDDID);
        defaultStructureTreeDDID = null;
    }

    defaultStructureTransform.addParameter('structure_type', form.default_structure_type.value);
    defaultStructureTransform.addParameter('structure_name', form.default_structure_filename.value);
    defaultStructureTransform.addParameter('request_operation', form.request_operation.value);
    defaultStructureTransform.addParameter('response_operation', form.response_operation.value);
    dojo.byId('DefaultStructureContent').appendChild(dojo.byId('DefaultStructureTreeview'));

    var xmlToTransform = '<container><namespaces><namespace prefix="fm" uri="http://www.hyfinity.com/formmaker"/></namespaces>'
                            + xmlToString(defaultStructureXML) + '</container>';
    var docToTransform = getXMLDocument(xmlToTransform);
    defaultStructureTransform.transform(docToTransform);

    //initialise drag drop container
    defaultStructureTreeDDID = dragDropHandler.attachContainer(document.getElementById('DefaultStructureTreeview'), 'page_component', "dragDropComplete", "dragdrop-item", "source", 0, {top:true,bottom:true,left:true,right:true,into:true});
}

/**
 * Appends an element to the given parent node (or its container)
 *
 * This method will attempt to insert the given node into the structure
 * xml at the specified location.
 * This will take account of the cuirrent add mode setting to determine how to insert the
 * node relative to the location node specified.
 *
 * @param newParentNode The location on the current structure to append to
 * @param newChildNode The XML Node to add to the struture file
 * @param forceInto (optional) if present, the 'into' add mode will be used regardless of the current setting.
 *
 * @return true if the new node was successfully inserted, false otherwise.
 */
function appendToStructure(newParentNode, newChildNode, forceInto)
{
    var parentGroup = newParentNode.parentNode;
    var tempNode = null;

    try
    {
        log.debug("Appending to structure, add mode is " + addMode + ", forceInto: " + forceInto);
        if ((addMode == 'into')  || ((typeof(forceInto) != 'undefined') && (forceInto != null)))
        {
            log.debug('using into mode');
            if(getBaseName(newParentNode) == 'element' || getBaseName(newParentNode) == 'custom')
                parentGroup.insertBefore(newChildNode, newParentNode.nextSibling);
            else
                newParentNode.appendChild(newChildNode);
        }
        else if (addMode == 'below')
        {
            log.debug('using below mode');
            parentGroup.insertBefore(newChildNode, newParentNode.nextSibling);
        }
        else
        {
            log.debug('using above mode');
            tempNode = newParentNode.previousSibling;
            if (tempNode == null)
                parentGroup.insertBefore(newChildNode, parentGroup.firstChild);
            else
                parentGroup.insertBefore(newChildNode, tempNode.nextSibling);
        }

        return true;
    }
    catch(e)
    {
        //Alert user to what they are trying to do
        var errorMsg;
        if (e.description)
            errorMsg = e.description;
        else
            errorMsg = e.message;
        alert(errorMsg);
        return false;
    }
}


function openPage(page, pageName, configuration) {
    if (configuration == '' || configuration == null)
    {
        window.open(page, pageName, config='toolbar=yes, menubar=no, directories=no, scrollbars=yes, resizable=yes, status=yes');
    }
    else
    {
        window.open(page, pageName, config=configuration);
    }
}

/*
*   Copies XML path section from schema to form.
*
*   newParentNode = MouseUpNode
*   newChildNode = MouseDownNode
*/
function replaceFormPath(newParentNode, newChildNode) {

  log.trace('in replaceFormPath');

  try {
      if(getBaseName(newParentNode) == 'element' || getBaseName(newParentNode) == 'custom')
      {
          if (getBaseName(selectedElement) == 'element')
          {
              var oNodeList;
              var Item;
              var removed;
              objNodeList = newParentNode.selectNodes("fm:path");
              for (var i=0; i<objNodeList.length; i++)
              {
                  newParentNode.removeChild(objNodeList.item(i));
              }
              newParentNode.insertBefore(selectedElementPath, newParentNode.firstChild);
           }
           else
               alert("Adding Schema Links from Groups is not permitted");
      }
      else
      {
          alert("Adding Schema Links to Groups or Repeats is not permitted");
      }
  }
  catch(e)
  {
    //Alert user to what they are trying to do
    alert(e.description+ " error");
  }
}

/** Global variables to simplify communication with xstore call back function */
var documentNameField;
var templateCollectionName;

var selectedStructureType;

/**
 * Function for selecting type of default structure file
 */
function selectDefaultStructure(structureType)
{
    if (structureType == 'Template')
    {
        selectedStructureType = 'Template';
        selectDocumentName(form.default_structure_filename, formTemplatesXStore, formTemplatesCollections);
    }
    else if (structureType == 'Structure')
    {
        selectedStructureType = 'Structure';
        selectDocumentName(form.default_structure_filename, structuresXStore, structuresCollection);
        //when dragging fields from an existing page we need to check the data soruce information in the app map file
        //so make sure this has been loaded up.
        loadAppMap();
    }
    else //data source
    {
        form.default_structure_type.value = 'DataSource';
        initDefaultStructure();
    }
}


var appMapDoc = null;

/**
 * checks if the app map file has been loaded up, and if not
 * starts the ajax call to load it.
 */
function loadAppMap()
{
    log.trace('checking if app map load needed');
    if (appMapDoc == null)
    {
        log.debug('loading app map');
        dojo.xhrPost({
            url:        "FormMaker",
            handleAs:   "text",
            preventCache:"true",
            content:    {   action:                     'process',
                            operation1:                 'load_form_map',
                            form_map_filename:          document.formStructure_form.form_map_filename.value,
                            xml_project_info:           document.formStructure_form.xml_project_info.value},
            error:  function(response, ioArgs) {
                        alert('An error occurred while trying to load the application file.\n\nPlease try your request again, and if the problem persists, please contact Hyfinity.\n\n\nError details: ' + response.message);

                        log.error('Unable to load application file:');
                        var errMsg = '';
                        for (prop in response)
                            errMsg += prop + " : " + response[prop] + "\n";
                        log.error(errMsg);

                        return response;
                    },
            load:   function(response, ioArgs) {
                        log.debug('received app map response');
                        //window.clipboardData.setData("Text",response);

                        var tempDoc = getXMLDocument(response);
                        Sarissa.setXpathNamespaces(tempDoc, FMqualifiedNS);

                        var appMapNode = tempDoc.selectSingleNode("/fm:application");
                        if (appMapNode != null)
                        {
                            appMapDoc = getXMLDocument(new XMLSerializer().serializeToString(appMapNode));
                            Sarissa.setXpathNamespaces(appMapDoc, FMqualifiedNS);
                            log.debug('setup app map document');
                        }
                        else
                        {
                            alert('An error occurred while trying to load the application file.\n\nPlease try your request again, and if the problem persists, please contact Hyfinity.');
                            log.error('unable to load applicaiton file');
                            appMapDoc = getXMLDocument('<empty/>');
                        }

                        return response;
                    }
        });
    }
}

/**
 * Makes a call to XStore to select a single document name
 */
function selectDocumentName(field, xstore, collectionList) {

    //alert(field + ', ' + xstore + ', ' + collections + ', ' + structureType);
    document.xstoreViewerCaller.chosen_xstore_name.value = xstore;

    document.xstoreViewerCaller.xml_permissions.value =
        "<xstores filter='on' xmlns='http://www.hyfinity.com/xengine'>" +
            "<xstore name='" + xstore + "'>" +
                "<collections>";

    if (selectedStructureType == 'Structure')
    {
        document.xstoreViewerCaller.chosen_collection_name.value = collectionList;
        document.xstoreViewerCaller.xml_permissions.value += "<collection name='" + collectionList + "'/>";
    }
    else
    {
        //alert(form.template_collection_name.value);
        if (form.template_collection_name.value != '')
            templateCollectionName = form.template_collection_name.value;
        else
            templateCollectionName = collectionList[0];
        //alert('templateCollectionName is ' + templateCollectionName);

        document.xstoreViewerCaller.chosen_collection_name.value = templateCollectionName;
        for(i in collectionList) {
            document.xstoreViewerCaller.xml_permissions.value += "<collection name='" + collectionList[i] + "'/>";
        }
    }

    document.xstoreViewerCaller.xml_permissions.value +=
                "</collections>" +
            "</xstore>" +
        "</xstores>";

    //alert(document.xstoreViewerCaller.xml_permissions.value);
    window.open('about:blank','xstoreViewerWindow','width=1000, height=600, toolbar=yes, menubar=no, location=no, directories=no, scrollbars=yes, resizable=yes, status=yes');

    documentNameField = field;
    document.xstoreViewerCaller.selection_mode.value = 'id';
    document.xstoreViewerCaller.name_filter.value = '';
    window.xstoreViewer = documentXstoreViewer;
    document.xstoreViewerCaller.submit();
}

/**
 * Simple function to provide xstore call back functionality
 */
function documentXstoreViewer(xstoreName, collectionName, documentName, xstoreId, collectionId, documentId)
{
    documentNameField.value = documentName;
    document.xstoreViewerCaller.chosen_collection_name.value = collectionName;

    form.default_structure_type.value = selectedStructureType;
    if (form.default_structure_type.value == 'Template')
    {
        form.template_collection_name.value = collectionName;
    }
    else
        form.template_collection_name.value = '';

    initDefaultStructure();
}



var ucletters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
var lcletters = 'abcdefghijklmnopqrstuvwxyz';
/**
 * Takes the given string and returns a new version with each
 * underscore replaced by a space, and a space inserted before each capital letter
 */
function insertSpaces(str)
{
    var newString = ''
    for (var i = 0; i < str.length; ++i)
    {
        if (((i == 0) || str.charAt(i-1) == '_') && (lcletters.indexOf(str.charAt(i)) != -1))
        {
            newString += ucletters.substr(lcletters.indexOf(str.charAt(i)), 1);
        }
        else if ((i > 0) && ((ucletters.indexOf(str.charAt(i)) != -1) && (ucletters.indexOf(str.charAt(i-1)) == -1)))
        {
            newString += ' ' + str.charAt(i);
        }
        else if (str.charAt(i) == '_')
        {
            newString += ' ';
        }
        else
        {
            newString += str.charAt(i);
        }
    }
    return newString;
}


/**
 * Deletes a component from the structure.
 * @param element The Element within the structure XML file representing the component to delete.
 * @private
 */
function deleteComponent(element)
{
    log.trace('in deleteComponent');
    if (getBaseName(element) == 'structure')
        alert('Removal of the base Structure tag is not allowed.');
    else
    {
        checkGridRemove(element.parentNode, element.getAttribute('name'));

        changeManager.storeDelete(element);

        element.parentNode.removeChild(element);

        if (typeof(structureBindingsXML) != 'undefined')
        {
            manageBindings(element, getBaseName(element), 'delete');
        }
    }
}

/**
 * Global on mouse move function to allow a div to be displayed when the user drags and drops
 */
function globalMouseMove(e, base)
{
    dividerMove(e, base);
}
document.onmousemove = globalMouseMove;

/**
 * Global on mouse up function to allow a div to be hidden when the user drags and drops
 */
function globalMouseUp(e) {
  mouseDown = false;
  mouseDownSource = 'structure';
  scrollableDivManager.scrollEnabled = false;
  dividerReleased(e);
}
document.onmouseup = globalMouseUp;


function saveData()
{
    //output the XML in memory to a hidden field
    form.xml_newPageConfig.value = xmlToString(structureXML);
    form.xml_config_xforms.value = xmlToString(structureBindingsXML);

    outputStateDocument();

    //output the changes information
    form.xml_pageChanges.value = changeManager.getChangesXML();
}


function outputStateDocument()
{
    //update the state document
    stateManager.setPageStructureControlsWidth(getCurrentStyle(document.getElementById('controls_td'), 'width'));
    stateManager.setPageStructureSourceWidth(getCurrentStyle(document.getElementById('default_structure_td'), 'width'));

    if (getCurrentStyle(document.getElementById('controls_td'), 'display') == 'none')
        stateManager.setPageStructureControlsVisibility('false');
    else
        stateManager.setPageStructureControlsVisibility('true');

    if (getCurrentStyle(document.getElementById('default_structure_td'), 'display') == 'none')
        stateManager.setPageStructureSourceVisibility('false');
    else
        stateManager.setPageStructureSourceVisibility('true');

    form.xml_fm_state.value = xmlToString(stateManager.getStateDocument());
}

/* Submits the form to send the new XML content back to the server to be saved,
 * setting the operations so that the same screen is returned*/
function save()
{

    saveData();

    //populate the hidden fields
    form.presentation_xsl.value = "form_structure_html.xsl";
    resetOperationsList();
    addOperation("save_field_details");
    addOperation("save_xpath_bindings_details");

    outputGenProcessesRequired();

    addOperation("load_form_map");
    addOperation("load_form_structure");
    addOperation("load_bindings_file");

    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);

    //submit the form
    form.submit();
}

/**
 * Resets the list of operations that will be performed on the next submit.
 * This should always be called before you start defining the required operations.
 */
function resetOperationsList()
{

    var i = 1;
    var opField = form.operation1;

    while (opField != null)
    {
        opField.value = '';
        i++;
        opField = eval('form.operation' + i);
    }
}

/**
 * Adds the given operation to the list of operations to be performed when the page is submitted.
 */
function addOperation(opName)
{
    var i = 1;
    var opField = form.operation1;

    while (opField != null && opField.value != '')
    {
        i++;
        opField = eval('form.operation' + i);
    }

    if (opField == null)
    {
        if (dojo.isIE < 8)
            opField = document.createElement('<input name="operation'+i+'"/>');
        else
            opField = document.createElement('input');
        opField.type = 'hidden';
        opField.name = 'operation' + i;
        form.appendChild(opField);
    }

    opField.value = opName;
}

/**
 * Checks if any geration processes will be required when saving the data
 * and if so adds the appropraite actions.
 */
function outputGenProcessesRequired()
{
    if (changeManager.hasImportantChanges)
    {
        addOperation("configure_blank_form_binding_structure");
    }
    if (changeManager.hasResources)
    {
        addOperation("process_additional_page_resources");
    }
}

function pageSelectionChanged(select)
{
    saveData();

    form.new_display_page.value = select.value;

    //populate the hidden fields
    form.presentation_xsl.value = "form_structure_html.xsl";
    resetOperationsList();
    addOperation("save_field_details");
    addOperation("save_xpath_bindings_details");
    outputGenProcessesRequired();
    addOperation("load_form_map");
    addOperation("switch_page");
    addOperation("load_form_structure");
    addOperation("load_bindings_file");

    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);

    //submit the form
    form.submit();
}

/* resets all groups/elements to how they were initially (when the page was loaded) */
function cancelAll()
{

    if (confirm('This will reset all details to their initial values upon loading the page.\nDo you wish to continue?'))
    {
        form.presentation_xsl.value = "form_structure_html.xsl";
        resetOperationsList();
        addOperation("load_form_structure");
        addOperation("load_form_map");
        addOperation("load_bindings_file");

        outputStateDocument();

        //stop the win manager prompting about losing changes
        WinManager.setGenuineCloseFlag(false);

        //submit the form
        form.submit();
    }
}

/**
 * Save data and go to the showFormMap screen.
 */
function showFormMap() {

    saveData();

    form.presentation_xsl.value = 'form_map_html.xsl';

    resetOperationsList();
    addOperation("save_field_details");
    addOperation("save_xpath_bindings_details");
    outputGenProcessesRequired();
    addOperation("load_form_map");

    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);

    form.submit();
}

/**
 * Save data and go to the Field Details screen.
 */
function showFieldDetails() {

    saveData();

    //populate the hidden fields
    form.presentation_xsl.value = 'field_details_html.xsl';
    resetOperationsList();
    addOperation("save_field_details");
    addOperation("save_xpath_bindings_details");
    outputGenProcessesRequired();
    addOperation("load_form_structure");
    addOperation("load_form_map");

    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);

    form.submit();
}

/**
 * Save data and go to the XPath bindings screen
 */
function showXPathBindings() {

    saveData();

    //populate the hidden fields
    form.presentation_xsl.value="xpath_bindings_html.xsl";
    resetOperationsList();
    addOperation("save_field_details");
    addOperation("save_xpath_bindings_details");
    outputGenProcessesRequired();
    addOperation("load_form_structure");
    addOperation("load_bindings_file");
    addOperation("load_form_map");

    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);

    form.submit();
}

/*
 * Saves the current details and calls to the server to generate the current page
 */
function generatePage()
{
    saveData();

    //populate the hidden fields
    form.presentation_xsl.value="form_structure_html.xsl";
    resetOperationsList();
    addOperation("save_field_details");
    addOperation("save_xpath_bindings_details");
    outputGenProcessesRequired();
    addOperation("load_form_map");
    addOperation("generate_output");
    addOperation("load_form_structure");
    addOperation("load_bindings_file");

    showStatusMessage("Generation in process...");

    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);

    //submit the form
    form.submit();

}

function generateApp()
{
    saveData();

    //populate the hidden fields
    form.presentation_xsl.value="form_structure_html.xsl";
    resetOperationsList();
    addOperation("save_field_details");
    addOperation("save_xpath_bindings_details");
    outputGenProcessesRequired();

    form.new_display_page.value = form.gen_control.value;
    //form.gen_control.value = 'all';
    addOperation("switch_gen_control_to_all");

    addOperation("load_form_map");
    addOperation("generate_output");
    addOperation('generate_handlers');
    addOperation("switch_page");
    addOperation("load_form_structure");
    addOperation("load_bindings_file");

    showStatusMessage("Generation in process...");

    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);

    //submit the form
    form.submit();

}


//-------------------------------------------------------------------------------------------------
/*      Start of Custom Context Menu releated functions.                                         */
//-------------------------------------------------------------------------------------------------

var contextMenuXPath = '';

/**
 * Shows the custom right click context menu, providing the contextMenuXPath has been set. */
function showContextMenu(e)
{
    var evt;
    if (window.event)
    {
        evt = window.event;
    }
    else
    {
        evt = e;
    }

    log.trace('in showContextMenu');
    log.debug('xpath is ' + contextMenuXPath);
    //check if click occurred on structure
    if(contextMenuXPath != '')
    {
        log.info('showing custom menu...');

        //cancel any in progress drag operations
        dragDropHandler.onmouseup(evt);

        var menu = document.getElementById("context_menu");

        checkSpanMenuItems(contextMenuXPath);

        checkVisibilityMenuItems(contextMenuXPath);

        showComponent(menu);

        moveObjToCoords(menu, getMouseCoords(evt));
        return false;
    }
    else
    {
        return true;
    }
}


/** Hides the custom right click context menu. */
function hideContextMenu()
{
    log.trace('in hideContextMenu');
    hideComponent(document.getElementById("context_menu"));
    contextMenuXPath = '';
}

/**
 * Checks if the menu is being shown for a component that supports the visibility options
 * (eg a field or group), and if so shows the menu options for toggling the appropriate settings,
 * and updates the sate display in the menu accordingly
 * @param xpath The XPath to the component that the menu is being shown for.
 */
function checkVisibilityMenuItems(xpath)
{
    var elem = structureXML.selectSingleNode(xpath);
    var elemType = getBaseName(elem);
    if (elemType == 'group' || elemType == 'element')
    {
        showComponent(document.getElementById('content_menu_visibility_options'));

        if (elem.getAttribute('hide') == 'true')
            dojo.addClass(document.getElementById('context_menu_visibility_hide'), 'menu_option_selected');
        else
            dojo.removeClass(document.getElementById('context_menu_visibility_hide'), 'menu_option_selected');

        showComponent(document.getElementById('context_menu_visibility_disable'));
        if (elem.getAttribute('disable') == 'true')
            dojo.addClass(document.getElementById('context_menu_visibility_disable'), 'menu_option_selected');
        else
            dojo.removeClass(document.getElementById('context_menu_visibility_disable'), 'menu_option_selected');

        if (elemType == 'element')
        {
            //show label option
            var labelElem = elem.selectSingleNode("fm:field/fm:label");
            if (labelElem.getAttribute('required') == 'true')
                dojo.addClass(document.getElementById('context_menu_visibility_label'), 'menu_option_selected');
            else
                dojo.removeClass(document.getElementById('context_menu_visibility_label'), 'menu_option_selected');

            setTextContent(document.getElementById('context_menu_visibility_label'), 'Enable Field Label');
        }
        else
        {
            //show label option
            var labelElem = elem.selectSingleNode("fm:label");
            if (labelElem.getAttribute('required') == 'true')
                dojo.addClass(document.getElementById('context_menu_visibility_label'), 'menu_option_selected');
            else
                dojo.removeClass(document.getElementById('context_menu_visibility_label'), 'menu_option_selected');

            setTextContent(document.getElementById('context_menu_visibility_label'), 'Enable Group Label');
        }

    }
    else
    {
        hideComponent(document.getElementById('content_menu_visibility_options'));
    }

}


/** Called when the 'Field Details' option is selected from the custom context menu.
 * Uses the svViewFieldDetails method to go to the details page.
 */
function menuViewFieldDetails(e)
{
    if(contextMenuXPath != '')
    {
        svViewFieldDetails(contextMenuXPath, e);
    }
    hideContextMenu();
}

/** Called when the 'Data Bindings' option is selected from the custom context menu.
 * Uses the svViewDataBindings method to go to the details page.
 */
function menuViewDataBindings(e)
{
    if(contextMenuXPath != '')
    {
        svViewDataBindings(contextMenuXPath, e);
    }
    hideContextMenu();
}

/** Called when the 'Delete' option is selected from the custom context menu.
 * Uses the deleteComponent method to actually delete the selected component from the structure
 */
function menuDeleteComponent()
{
    log.trace('in menuDeleteComponent');
    if(contextMenuXPath != '')
    {
        deleteComponent(structureXML.selectSingleNode(contextMenuXPath));
        //Redisplay the structure treeview
        drawStructureTreeView();
    }
    hideContextMenu();
}


/** Called when the 'Rename' option is selected from the custom context menu.
 * Uses the svStartRename method to actually rename the selected component in the structure
 */
function menuRenameComponent()
{
    log.trace('in menuRenameComponent');
    if(contextMenuXPath != '')
    {
        var elem = structureXML.selectSingleNode(contextMenuXPath);
        var type = getBaseName(elem);
        if (getBaseName(elem) == 'custom')
            type = constCUSTOM_ELEMENT;

        svStartRename(elem, type)

        hideContextMenu();
    }
    else
        hideContextMenu();
}

function menuToggleHide()
{
    log.trace('in menuToggleHide');
    if(contextMenuXPath != '')
    {
        var elem = structureXML.selectSingleNode(contextMenuXPath);
        if (elem.getAttribute('hide') == 'true')
            elem.setAttribute('hide', 'false');
        else
            elem.setAttribute('hide', 'true');

        document.getElementById('context_menu_visibility_hide').blur();
        changeManager.setChanged(false);
        drawStructureTreeView();
    }
    hideContextMenu();
}
function menuToggleDisable()
{
    log.trace('in menuToggleDisable');
    if(contextMenuXPath != '')
    {
        var elem = structureXML.selectSingleNode(contextMenuXPath);
        if (elem.getAttribute('disable') == 'true')
            elem.setAttribute('disable', 'false');
        else
            elem.setAttribute('disable', 'true');

        document.getElementById('context_menu_visibility_disable').blur();
        changeManager.setChanged(false);
        drawStructureTreeView();
    }
    hideContextMenu();
}

function menuToggleLabel()
{
    log.trace('in menuToggleLabel');
    if(contextMenuXPath != '')
    {
        var elem = structureXML.selectSingleNode(contextMenuXPath);

        var labelElem = elem.selectSingleNode("fm:field/fm:label | fm:label");

        if (labelElem.getAttribute('required') == 'true')
            labelElem.setAttribute('required', 'false');
        else
            labelElem.setAttribute('required', 'true');

        document.getElementById('context_menu_visibility_label').blur();
        changeManager.setChanged(false);
        drawStructureTreeView();
    }
    hideContextMenu();
}

document.oncontextmenu = showContextMenu;
document.onmousedown = hideContextMenu;


//-------------------------------------------------------------------------------------------------
/*      End of Custom Context Menu releated functions.                                         */
//-------------------------------------------------------------------------------------------------

