/*
 * Script functions for the XPath Binding screen
 *
 * Company : Hyfinity Ltd.
 * Copyright (c) 2003
 *
 * @author Gerard Smyth
 */

//set up the init function to be called on page load
//window.onload = init;
dojo.addOnLoad(init);


/* DOM Document variables for the data islands*/
var applicationDoc, configDoc, bindingsDoc, tiDoc, xiDoc, xisDoc, eventConfigDoc;

/** XSL Processor to use for rendering the instance treeviews. */
var instanceXSLProc;

var FMqualifiedNS = 'xmlns:fm="http://www.hyfinity.com/formmaker"';
var FMdefaultNS = 'http://www.hyfinity.com/formmaker';

var XStoreQualifiedNS = 'xmlns:xstore="http://www.hyfinity.com/xstore"';

/* text shown when an xpath hasn't been entered */
var blank_xpath_text = 'enter_xpath_here';

var EFORM_WRAPPER_XPATH = "/mvc:eForm/mvc:Data";
var EFORM_ROOT_ELEMENT_NAME = "eForm";

/** The static strings to use to indicate which tab is currently displayed */
var HANDLER_TO_PAGE_TAB = 'handler-to-page';
var PAGE_TO_HANDLER_TAB = 'page-to-handler';
var ACTION_CONFIG_TAB = 'action-config';

var currentTab = HANDLER_TO_PAGE_TAB;

/** Global variable to store the name of the page_to_handler action that the screen is currently working on.
    This does not apply when the currentTab is the HANDLER_TO_PAGE_TAB. */
var currentAction = '';

/** FM State manager used to maintain the current visual state of the UI. */
var stateManager;

/** Global boolean value to indicate whether any chanegs have been made to the applciation file.
 *  This is then used to determine whether it needs to be saved when leaving the page. */
var applicationMapChanged = false;


/* onload function used to setup the display. */
function init()
{
    setTableHeight();
    initialiseXML();
    loadEventConfig();

    currentTab = document.XPath_Bindings_form.currentTab.value;
    currentAction = document.XPath_Bindings_form.currentAction.value;

    if (document.XPath_Bindings_form.currentType.value == 'form')
        document.XPath_Bindings_form.currentElement.value = 'details'

    hyf.ns.container = document.getElementById('xpathBindings__namespace_group');
    hyf.ns.addNamespacesFromDocument(bindingsDoc, true, true);

    showNamespaces();

    //attach an event to each accordion pane so we can detect when it is selected.
    if (dijit.byId('actionSubmissionAccordion') != null)
    {
        dojo.forEach(dijit.byId('actionSubmissionAccordion').getChildren(),
                    function(pane) {
                        dojo.connect(pane, 'onSelected', accordionActionChanged);
                    });
    }

    checkCurrentFieldXPathsSpecified();

    currentTab = 'none';
    changeBindingType(document.XPath_Bindings_form.currentTab.value, true);

    //showNewElement(document.XPath_Bindings_form.currentType.value, document.XPath_Bindings_form.currentElement.value, true);

    document.getElementById('TreeList2').onselectstart=new Function ("return false");
    document.onmousemove = mouseMove;
    document.onmouseup = mouseUpCancel;

    //updateDisplayedInstanceDocument();

    //set up a call to check for missing xpaths and highlight on the treeview
    //this is done using setTimeout so that this init method returns and the page
    //gets displayed corectly without this possibly long processing needing to have completed.
    setTimeout("checkXPathsSpecified()", 50);

    dojo.connect(null, "updateDividerSize", function(){
            if (dijit.byId('actionSubmissionAccordion'))
                dijit.byId('actionSubmissionAccordion').resize();
        });

    log.debug('init complete');
}

function setTableHeight()
{
    var requiredHeight = (document.documentElement.clientHeight - 165) + 'px';
    document.getElementById('pageTable').style.height = requiredHeight;
    document.getElementById('TreeList').style.height = requiredHeight;
    document.getElementById('content_container').style.height = requiredHeight;
    document.getElementById('TreeList2').style.height = requiredHeight;
    document.getElementById('divider').style.height = requiredHeight;
    document.getElementById('divider2').style.height = requiredHeight;

    adjustAccordionHeight();
}
window.onresize = setTableHeight;

/* loads up the data in the XML islands */
function initialiseXML()
{
    log.debug('about to load up xml islands');
    configDoc = getXMLIsland('pageConfig');
    Sarissa.setXpathNamespaces(configDoc, FMqualifiedNS);

    bindingsDoc = getXMLIsland('hyfinityBindings');
    Sarissa.setXpathNamespaces(bindingsDoc, FMqualifiedNS);

    //Actions Load the links data island to enable dynamic loading of page to handler instance docs
    applicationDoc = getXMLIsland('application');
    Sarissa.setXpathNamespaces(applicationDoc, FMqualifiedNS);

    //initialise the DOMs that the instance documents will be loaded into
    tiDoc = getXMLDocument('<empty/>');
    xiDoc = getXMLDocument('<empty/>');
    xisDoc = getXMLDocument('<empty/>');

    //create the XSL processor used to render the instance documents.
    instanceXSLProc = new XSLTProcessor();
    var instanceXslDoc = getXMLDocumentFromURL("xsl/xpathBindingsInstanceTreeView.xsl");
    instanceXSLProc.importStylesheet(instanceXslDoc);

    //initialise the state manager
    stateManager = new FormMakerStateManager(document.getElementById('formmakerStateIsland').value, FormMakerStateManager.DATA_BINDINGS_PAGE);

    log.debug('XML init completed');
}


function loadEventConfig()
{
    eventConfigDoc = null;

    /** Prepare the XStore request. */
    //QUESTION: Is it okay to hard code the xstore details for the config file?
    var xstoreRequest = '<xstore xmlns="http://www.hyfinity.com/xstore"><query>' +
                            '<action>read</action>' +
                            '<xstore_name>blueprints</xstore_name>' +
                            '<collection_name>common</collection_name>' +
                            '<document_name>event_config.xml</document_name>' +
                            '</query><content/></xstore>';

    var requestMessage = '<xg:attributes xmlns:xg="http://www.hyfinity.com/xgate">' +
                        '<xg:attribute xg:name="operation1">call_xstore</xg:attribute>' +
                        '<xg:xstore_request>'+xstoreRequest+'</xg:xstore_request>' +
                        '</xg:attributes>'

    /** Make the async call to retrieve config file. */
    dojo.rawXhrPost({
        url:        'FormMaker',
        handleAs:   'text',
        postData:   requestMessage,
        headers:    { SOAPAction:    "process"},
        load:       function(response, ioArgs){
                        eventConfigDoc = getXMLDocument();
                        /** Ignore the XStore wrapper.	*/
                        if (stripXStoreResponse(response, eventConfigDoc))
                        {
                            Sarissa.setXpathNamespaces(eventConfigDoc, FMqualifiedNS);

                            log.debug('event config file loaded');
                            //log.debug(Sarissa.escape(eventConfigDoc.xml));

                            //update the currently displayed screen if needed
                            //check if the currently displayed element has events.
                            var configElement;
                            switch (document.XPath_Bindings_form.currentType.value)
                            {
                                case 'element' : configElement = configDoc.selectSingleNode("/fm:structure//fm:element[@name = '"+document.XPath_Bindings_form.currentElement.value+"' and not(ancestor::fm:path)]"); break;
                                case 'group'   : configElement = configDoc.selectSingleNode("/fm:structure//fm:group[@name = '"+document.XPath_Bindings_form.currentElement.value+"']"); break;
                                case 'repeat'  : configElement = configDoc.selectSingleNode("/fm:structure//fm:repeat[@name = '"+document.XPath_Bindings_form.currentElement.value+"']"); break;
                                case 'form'    : configElement = configDoc.selectSingleNode("/fm:structure/fm:form_details"); break;
                            }
                            if (configElement.selectSingleNode("fm:events/fm:event | */fm:events/fm:event | fm:field/fm:label/fm:events/fm:event"))
                                showNewElement(document.XPath_Bindings_form.currentType.value, document.XPath_Bindings_form.currentElement.value);
                        }
                        return response;
                    },
        error:      function(response, ioArgs){
                        alert('Error loading events config file.  Please try again.\n\n\nError details: ' + response.message);
                        return response;
                    }
    });
}




/**
 * Checks the type and name of the current componenet, and then makes sure
 * that only the XPaths required for that componenet are visible on the screen.
 * Also updates the XPath Guidelines display message.
 */
function showRelevantXpaths()
{
    //alert("about to hide unneeded xpaths");
    var type = document.XPath_Bindings_form.currentType.value;
    var name = document.XPath_Bindings_form.currentElement.value;

    log.trace('in showRelevantXpaths for ' + type + ' ' + name);

    var displayType = type;
    if (displayType == 'element')
        displayType = 'field';

    enableComponent(document.getElementById('group_xpaths_group'), true);
    enableComponent(document.getElementById('dynamic_action_xpaths_group'), true);
    enableComponent(document.getElementById('dynamic_group_label_style_xpaths'), true);
    enableComponent(document.getElementById('dynamic_group_style_xpaths'), true);
    enableComponent(document.getElementById('repeat_xpaths_group'), true);
    enableComponent(document.getElementById('element_xpaths_group'), true);
    enableComponent(document.getElementById('dynamic_action_xpaths'), true);
    enableComponent(document.getElementById('dynamic_label_style_xpaths'), true);
    enableComponent(document.getElementById('dynamic_style_xpaths'), true);
    enableComponent(document.getElementById('dynamic_element_style_xpaths'), true);
    enableComponent(document.getElementById('dynamic_disabled_style_xpaths'), true);

    /* All the show and hide component calls below find the label element, which is the TH tag in these cases
     * Therefore the parentNode is the TR tag which contains the row of the table to hide.
     */
    var configElement;
    var needXPaths = false;
    var defaultSort = false;
    var message='<br/>';

    //alert(type + " : " + name);

    if (currentTab == ACTION_CONFIG_TAB)
    {
        hideGroupDetails();
        hideRepeatDetails();
        hideElementDetails();
        hidePageDetails();

        showFormDetails();

        return;
    }
    else
    {
        if (type == 'element')
        {
            hideGroupDetails();
            hideRepeatDetails();
            hideFormDetails();
            hidePageDetails();
            configElement = configDoc.documentElement.selectSingleNode("/fm:structure//fm:element[@name = '"+name+"' and not(ancestor::fm:path)]");
            if (configElement != null)
                needXPaths = showElementDetails(configElement);
        }
        else if (type == 'group')
        {
            hideElementDetails();
            hideRepeatDetails();
            hideFormDetails();
            hidePageDetails();
            configElement = configDoc.documentElement.selectSingleNode("/fm:structure//fm:group[@name = '"+name+"']");
            if (configElement != null)
                needXPaths = showGroupDetails(configElement);
        }
        else if (type == 'repeat') //type is repeat
        {
            hideElementDetails();
            hideGroupDetails();
            hideFormDetails();
            hidePageDetails();
            configElement = configDoc.documentElement.selectSingleNode("/fm:structure//fm:repeat[@name = '"+name+"']");

            //repeats always require at least 1 xpath for handler to page
            //for page to handler one xpath is always required unless action instance type is none
            if (currentTab == HANDLER_TO_PAGE_TAB)
                needXPaths = true;
            else  if (applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[fm:action = \'" + currentAction + "\']/fm:instance_type[. = 'static' or . = 'dynamic']") != null)
                needXPaths = true;

            //method instead returns whether the default sorting behaviour is required
            if (configElement != null)
                defaultSort = showRepeatDetails(configElement);
        }
        else if (type == 'form')
        {
            hideGroupDetails();
            hideRepeatDetails();
            hideElementDetails();
            hideFormDetails();

            configElement = configDoc.selectSingleNode("/fm:structure");

            needXPaths = showPageDetails();
        }


        //create the display message
        if (needXPaths)
        {
            var repeats = configElement.selectNodes("ancestor::fm:repeat");
            //alert(repeats.length)
            if (repeats.length > 0)
            {
                var context = "";
                message += "This "+displayType+" is contained within one (or more) repeating groups.";
                if (currentTab == HANDLER_TO_PAGE_TAB)
                    message += "Therefore you can use the following XSLT variable names in your XPaths to refer to the current entry in each node-set:";
                message += "<br/><br/>";
                for (i=0; i<repeats.length; ++i)
                {
                    var repeatName = repeats.item(i).getAttribute('name');

                    if (currentTab == HANDLER_TO_PAGE_TAB)
                        message += "$"+repeatName+"LoopEntry - Variable containing the current entry in the "+repeatName+" group<br/>";

                    var repeat_xpath;
                    if (currentTab == HANDLER_TO_PAGE_TAB)
                        repeat_xpath = bindingsDoc.selectSingleNode("/fm:xforms/fm:repeat[@name = '"+repeatName+"']/fm:repeat_xpath");
                    else
                        repeat_xpath = bindingsDoc.selectSingleNode("/fm:xforms/fm:repeat[@name = '"+repeatName+"']/fm:action[@name = '"+currentAction+"']/fm:xform_xpath");

                    if ((repeat_xpath == null) || (getElementText(repeat_xpath) == blank_xpath_text))
                    {
                        if (currentTab == HANDLER_TO_PAGE_TAB)
                            context += "/ ( "+repeatName+" 'Repeating Data Location' not yet set ) ";
                        else
                            context += "/ ( "+repeatName+" 'Repeat Submission Binding' not yet set ) ";
                    }
                    else
                    {
                        if (getElementText(repeat_xpath).charAt(0) != '/')
                            context += '/' + getElementText(repeat_xpath);
                        else
                        {
                            if ((currentTab == PAGE_TO_HANDLER_TAB)  &&
                                (getElementText(repeat_xpath).indexOf(EFORM_WRAPPER_XPATH) != 0))
                            {
                                context = EFORM_WRAPPER_XPATH + getElementText(repeat_xpath);
                            }
                            else
                            {
                                context = getElementText(repeat_xpath);
                            }
                        }
                    }
                }
                message +="<br/>The context for these XPaths is : "+context+"<br/><br/>";
            }
            if ((type=='element') && (currentTab == HANDLER_TO_PAGE_TAB) &&
                ((configElement.selectSingleNode("fm:field/fm:input").getAttribute("options") == 'dynamic') &&
                 ((getElementText(configElement.selectSingleNode("fm:field/fm:input")) == 'select') ||
                  (getElementText(configElement.selectSingleNode("fm:field/fm:input")) == 'combobox') ||
                  (getElementText(configElement.selectSingleNode("fm:field/fm:input")) == 'radio') ||
                  (getElementText(configElement.selectSingleNode("fm:field/fm:input")) == 'tabs') ||
                  (getElementText(configElement.selectSingleNode("fm:field/fm:input")) == 'multipleCheckbox'))))
            {
                message +="The context for the 'Data Value' and 'Display Text' XPaths will depend upon the value entered for the 'Options Location'.<br/><br/>";
            }

            if ((type == 'element') && (currentTab != HANDLER_TO_PAGE_TAB))
            {
                //check if multiple values supported
                if (((getElementText(configElement.selectSingleNode("fm:field/fm:input")) == 'multipleCheckbox') ||
                     ((getElementText(configElement.selectSingleNode("fm:field/fm:input")) == 'select') && (configElement.selectSingleNode("fm:field/fm:input").getAttribute("multiple_selection") == 'true'))) &&
                    (bindingsDoc.selectSingleNode("/fm:xforms/fm:element[@name = '"+name+"']").getAttribute('binding_format') == 'structure'))
                {
                    message += "This field supports multiple values using the XML Structure binding format, and so two submission XPaths are required.<br/>The Context Location should point to the repeating element in your data.  This element will be repeated for each value selected.  For each context, the Field Value binding will be used to determine the actual element to bind the value to.<br/><br/>"
                }
                else
                    message += "The 'Field Submission Binding' is evaluated against the data submitted from the page.<br/><br/>";
            }
            else if ((type == 'repeat') && (currentTab != HANDLER_TO_PAGE_TAB))
            {
                message += "The 'Repeat Submission Binding' is evaluated against the data submitted from the page.<br/><br/>";
            }
            else if ((type == 'group') && (currentTab != HANDLER_TO_PAGE_TAB))
            {
                message += "The 'Context Location' is evaluated against the data submitted from the page.<br/><br/>";
            }
            else
            {
                message += "All XPaths are evaluated against the data going to the page.<br/><br/>";
            }

            if(defaultSort)
            {
                message += "You have selected to use the default sorting behaviour, and so the sort XPaths cannot be altered.  In order for this to work correctly you must ensure that the required control elements are present in both instance documents.<br/><br/>";
            }


            //add in the binding type guide information for the action bindings
            //NOTE: This isn't how this process works anymore, so need to change
            /*if (currentTab == PAGE_TO_HANDLER_TAB)
            {
                //check instance type for the current action
                if (applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[fm:action = \'" + currentAction + "\']/fm:instance_type[. = 'static']") != null)
                {   //static
                    message += '<table><tr><td>The binding type for this action is <b>static</b>.<br/><br/>This will use the contents of the Static Binding Structure document defined for this link as the XML structure to bind to at runtime.</td>';
                    message += '<td><img id="binding_asb_static" alt="static action submission bindings" src="images/binding_asb_static_icon.png"></img></td></tr></table>';
                }
                else if (applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[fm:action = \'" + currentAction + "\']/fm:instance_type[. = 'dynamic']") != null)
                {   //dynamic
                    message += '<table><tr><td>The binding type for this action is <b>dynamic</b>.<br/><br/>This will use dynamic data retrieved at runtime to define the binding structure.</td>';
                    message += '<td><img id="binding_asb_dynamic" alt="dynamic action submission bindings" src="images/binding_asb_dynamic_icon.png"></img></td></tr></table>';
                }

            }*/

        }
        else
        {
            if (currentTab == HANDLER_TO_PAGE_TAB)
                message += 'There are no page display bindings required for this ' + displayType + '.';
            else
                message += 'There are no ' + currentAction + ' submission bindings required for this ' + displayType + '.';

            //ensure the message is visible in this case
            if (type == 'group')
            {
                setMessageGroupDisplay('visible');
            }
            else if (type == 'repeat')
            {
                setMessageRepeatDisplay('visible');
            }
            else if (type == 'form')
            {
                setMessagePageDisplay('visible');
            }
            else
            {
                setMessageDisplay('visible');
            }
        }
    }

    //check if this is an action that is actually based on another action
    if ((currentTab == PAGE_TO_HANDLER_TAB) && (bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = \'" + currentAction + "\']/@base") != null))
    {
        var actionBase = bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = \'" + currentAction + "\']").getAttribute('base');

        message = "<br/>The bindings for this action are being set by the " +actionBase+ " action.<br/><br/>";
        message += "Change these bindings by altering the value under the " + actionBase + " action accordion pane.<br/>";
        message += "Alternatively, use the 'Action Configuration' tab to no longer set this action's bindings to those of action "+actionBase+".<br/><br/>";

        //ensure the message is visible in this case
        if (type == 'group')
        {
            disableComponent(document.getElementById('group_xpaths_group'), true);
            disableComponent(document.getElementById('dynamic_action_xpaths_group'), true);
            disableComponent(document.getElementById('dynamic_group_label_style_xpaths'), true);
            disableComponent(document.getElementById('dynamic_group_style_xpaths'), true);
            setMessageGroupDisplay('visible');
        }
        else if (type == 'repeat')
        {
            disableComponent(document.getElementById('repeat_xpaths_group'), true);
            setMessageRepeatDisplay('visible');
        }
        else
        {
            disableComponent(document.getElementById('element_xpaths_group'), true);
            disableComponent(document.getElementById('dynamic_action_xpaths'), true);
            disableComponent(document.getElementById('dynamic_label_style_xpaths'), true);
            disableComponent(document.getElementById('dynamic_style_xpaths'), true);
            disableComponent(document.getElementById('dynamic_element_style_xpaths'), true);
            disableComponent(document.getElementById('dynamic_disabled_style_xpaths'), true);
            setMessageDisplay('visible');
        }

    }

    //alert(message);

    document.getElementById(type+'_message_location').innerHTML = message;
}

function hideElementDetails()
{
    //hide the element level xpath table rows
    hideComponent(document.getElementById('element_group'));
    hideComponent(document.getElementById('element_group_title'));
    hideComponent(document.getElementById('description_container_group'));
    hideComponent(document.getElementById('message_container_group'));
    hideComponent(document.getElementById('message_container'));
}

function hideGroupDetails()
{
    //Destroy the edit area instance if it is visible for the current component.
    //This is needed because we are about to hide the contents,w hich will stop this working
    if (editAreas['xpathBindings__group_xpaths__choice_fragment'])
        editAreaLoader.delete_instance('xpathBindings__group_xpaths__choice_fragment');

    //hide all the group details
    hideComponent(document.getElementById('group_group'));
    hideComponent(document.getElementById('group_group_title'));
    hideComponent(document.getElementById('message_group_container_group'));
    hideComponent(document.getElementById('message_group_container'));
}

function hideRepeatDetails()
{
    //hide all the repeat details
    hideComponent(document.getElementById('repeat_group'));
    hideComponent(document.getElementById('repeat_group_title'));
    hideComponent(document.getElementById('message_repeat_container_group'));
    hideComponent(document.getElementById('message_repeat_container'));
}

/** hides all the form componenets concerning form level details.
 *  This is the action configuration information. */
function hideFormDetails()
{
    hideComponent(document.getElementById('action_group_title'));
    hideComponent(document.getElementById('form_details_group'));
}

/**
 * Hides any page display information relating to the page itself.
 * This currently only includes any needed dynamic event xpaths
 */
function hidePageDetails()
{
    hideComponent(document.getElementById('page_group'));
    hideComponent(document.getElementById('page_group_title'));
}


/**
 * Creates the needed HTML componenets to provide an xpath entry control
 * and places it in the given container.
 * @param container The HTML element to contain the created controls
 * @param id The id to give to the input control
 * @param label The text label to display
 * @param value The current xpath value to show (can either be a string,
 *                      or a DOM node whose string content should be used.)
 * @param showMatches (optional) Indicates whether matches should be shown on the
 *                      input or output documents. - 'input', 'output', 'both', 'none'
 * @param addRemoveLink (optional) Boolean indicating whether a remove button should
 *                      also be output. default false.
 */
function addDisplayXPathField(container, id, label, value, showMatches, addRemoveLink)
{
    var strValue = '';
    if (typeof(value) == 'string')
        strValue = value;
    else
        if (value != null)
            strValue = getElementText(value);

    if (strValue == '')
        strValue = blank_xpath_text;

    //QUESTION: do we need to ensure the eform wrapper is on the xpaths??
    /*if ((strValue.indexOf(EFORM_WRAPPER_XPATH) != 0) &&
        (strValue.indexOf("/") == 0))
    {
        strValue = EFORM_WRAPPER_XPATH + strValue;
    }*/


    var labelContainer, detailsContainer, removeContainer;

    if (container.tagName.toLowerCase() == 'table')
    {
        var newRow = container.insertRow(-1);
        labelContainer = newRow.insertCell(-1);
        detailsContainer = newRow.insertCell(-1);
        if (addRemoveLink)
        {
            removeContainer = newRow.insertCell(-1);
            removeContainer.vAlign = 'middle';
            removeContainer.style.width = '20px';
        }
    }
    else
    {
        var entry = document.createElement('div');
        labelContainer = document.createElement('span');
        detailsContainer = document.createElement('span');
        entry.appendChild(labelContainer);
        entry.appendChild(detailsContainer);
        container.appendChild(entry);
        if (addRemoveLink)
        {
            removeContainer = document.createElement('span');
            entry.appendChild(removeContainer);
        }
    }


    labelContainer.id = id + '_label';
    labelContainer.className = 'tableColumnHeading';
    labelContainer.innerHTML = label;

    detailsContainer.innerHTML = createXPathTextAreaString(id, strValue);

    if (addRemoveLink)
    {
        removeContainer.innerHTML = '<a href="javascript:removeAdditionalSubmissionBinding(\''+id+'\')" class="remove_button" title="Remove binding" style="height : auto;"><span>Remove</span></a>';
    }

    if (showMatches == 'input')
        setTimeout("validateXPathImpl('" + id + "', true, false)", 50);
    else if (showMatches == 'output')
        setTimeout("validateXPathImpl('" + id + "', false, true)", 50);
    else if (showMatches == 'both')
        setTimeout("validateXPathImpl('" + id + "', true, true)", 50);
    else
        setTimeout("validateXPathImpl('" + id + "', false, false)", 50);
}

/**
 * Show any page display information relating to the page itself.
 * This currently only includes any needed dynamic event xpaths
 */
function showPageDetails()
{
    showComponent(document.getElementById('page_group'));
    showComponent(document.getElementById('page_group_title'));

    //currently only have event xpaths that can apply to the page
    var actionContainer = document.getElementById("dynamic_action_xpaths_page");
    actionContainer.innerHTML = "";

    var needXPaths = false;

    if (currentTab == HANDLER_TO_PAGE_TAB)
    {
        //show page event xpaths
        var dynamicEvents = configDoc.selectNodes("/fm:structure/fm:form_details/fm:page_events/fm:events/fm:event[fm:action/fm:param[@option = 'Dynamic']]");
        if (dynamicEvents.length > 0)
        {
            showNewFormatEventXPaths(actionContainer, dynamicEvents, '', 'page');
            needXPaths = true;
        }

        //show external event xpaths
        dynamicEvents = configDoc.selectNodes("/fm:structure/fm:form_details/fm:external_events/fm:events/fm:event[fm:action/fm:param[@option = 'Dynamic']]");
        if (dynamicEvents.length > 0)
        {
            showNewFormatEventXPaths(actionContainer, dynamicEvents, '', 'external');
            needXPaths = true;
        }
    }

    setMessagePageDisplay(messagePageMode);

    return needXPaths;
}

/* shows/hides all the relevant xpaths text areas for the element specified by configElement
 * @param configElement The element in the structure file that represents the element to display.
 */
function showElementDetails(configElement)
{
    showComponent(document.getElementById('element_group'));
    showComponent(document.getElementById('element_group_title'));
    showComponent(document.getElementById('description_container_group'));
    showComponent(document.getElementById('message_container_group'));
    showComponent(document.getElementById('message_container'));

    var needXPaths = false;

    //set up element name display
    var elemName = configElement.getAttribute("name");
    document.getElementById("xpathBindings__element_name").innerHTML = elemName;

    var container = document.getElementById('element_xpaths_group');
    //delete any existing table content
    for (var i = container.rows.length - 1; i >= 0; --i)
    {
        container.deleteRow(i);
    }
    document.getElementById("dynamic_label_style_xpaths").innerHTML = "";
    document.getElementById("dynamic_style_xpaths").innerHTML = "";
    document.getElementById("dynamic_element_style_xpaths").innerHTML = "";
    document.getElementById("dynamic_disabled_style_xpaths").innerHTML = "";
    document.getElementById("dynamic_action_xpaths").innerHTML = "";


    var bindingElement = bindingsDoc.selectSingleNode("/fm:xforms/fm:element[@name = '"+elemName+"']");

    var controlType = getElementText(configElement.selectSingleNode("fm:field/fm:input"));

    //check which tab we are on
    if (currentTab == HANDLER_TO_PAGE_TAB)
    {

        //check if this field can support multiple values, and so will need the binding format
        //selection controls.
        if ((controlType == 'multipleCheckbox') ||
            ((controlType == 'select') && (configElement.selectSingleNode("fm:field/fm:input").getAttribute("multiple_selection") == 'true')))
        {
            var newRow = container.insertRow(-1);
            labelContainer = newRow.insertCell(-1);
            detailsContainer = newRow.insertCell(-1);

            var detailsHTML = '<select id="xpathBindings__element_xpaths__data_format">';
            detailsHTML += '<option value="csv"';
            if (bindingElement.getAttribute('binding_format') == 'csv')
                detailsHTML += ' selected="selected"';
            detailsHTML += '>CSV String</option>';
            detailsHTML += '<option value="structure"';
            if (bindingElement.getAttribute('binding_format') == 'structure')
                detailsHTML += ' selected="selected"';
            detailsHTML += '>XML Structure</option>';
            detailsHTML += '</select>';

            //add in a hint message
            detailsHTML += '<span style="vertical-align:top;" class="hintIcon" onmouseover="showTipMessage(\'TR\');" onmouseout="hideTipMessage();" >';
            detailsHTML += '<span class="tipMessage" style="display:none"><u><b>Data format</b></u><br/> <br/>This control can be used to select multiple values, and so the data format option is needed to indicate how the values should be stored in the XML data.<br/><br/><b>CSV String</b> - If this option is chosen, then the values will be stored as a single, comma separated, string. eg: <br/><br/> &lt;element&gt;value1,value2,value3&lt;/element&gt; <br/><br/> In this case the bindings should point to the element that contains this string value.<br/><br/><b>XML Structure</b> - This option means that there will be a separate element in the XML data for every selected value. eg:<br/><br/>&lt;element&gt;value1&lt;/element&gt;<br/>&lt;element&gt;value2&lt;/element&gt;<br/>&lt;element&gt;value3&lt;/element&gt;<br/><br/>In this case, the Field Value binding should select all the elements that contain values for this control.</span> &#160;&#160;</span>';

            labelContainer.className = 'tableColumnHeading';
            labelContainer.innerHTML = 'Data format';
            detailsContainer.innerHTML = detailsHTML;
        }

        //show the transform xpath
        if (!(((controlType == 'output') ||
               (controlType == 'button') ||
               (controlType == 'htmlButton') ||
               (controlType == 'image') ||
               (controlType == 'text') ||
               (controlType == 'hidden')) &&
              (configElement.selectSingleNode("fm:field/fm:input").getAttribute("options") == 'static')))
        {
            addDisplayXPathField(container,
                                'xpathBindings__element_xpaths__transform_xpath',
                                'Field Value',
                                bindingElement.selectSingleNode("fm:transform_xpath"),
                                'input');
            needXPaths = true;
        }
        else
        {
            setTimeout("clearInputMatches()", 50);
        }

        //check if label xpath needed
        if ((configElement.selectSingleNode("fm:field/fm:label[not(./@required) or (./@required = 'true')]") != null) &&
            (configElement.selectSingleNode("fm:field/fm:label").getAttribute("type") == 'dynamic'))
        {
            addDisplayXPathField(container,
                                'xpathBindings__element_xpaths__label_xpath',
                                'Label Value',
                                bindingElement.selectSingleNode("fm:label_xpath"));
            needXPaths = true;
        }

        //check repeat xpaths
        if ((configElement.selectSingleNode("fm:field/fm:input").getAttribute("options") == 'dynamic') &&
            ((controlType == 'select') ||
             (controlType == 'combobox') ||
             (controlType == 'radio') ||
             (controlType == 'tabs') ||
             (controlType == 'multipleCheckbox')))
        {
            var headingRow = container.insertRow(-1);
            var headingCell = headingRow.insertCell(-1);
            headingCell.colSpan = 2;
            headingCell.className = 'groupHeading';
            headingCell.innerHTML = 'Data Constraints';


            switch (controlType)
            {
                case 'select'   :   fieldType = 'Select Box'; break;
                case 'combobox' :   fieldType = 'Combo Box'; break;
                case 'radio'    :   fieldType = 'Radio Button'; break;
                case 'tabs'     :   fieldType = 'Tab'; break;
                case 'multipleCheckbox'     :   fieldType = 'Checkbox'; break;
                default         :   fieldType = controlType;
            }

            addDisplayXPathField(container,
                                'xpathBindings__element_xpaths__repeat_xpath',
                                fieldType + ' Options Location',
                                bindingElement.selectSingleNode("fm:repeat_xpath"));

            addDisplayXPathField(container,
                                'xpathBindings__element_xpaths__value_xpath',
                                fieldType + ' Options Data Value',
                                bindingElement.selectSingleNode("fm:value_xpath"));

            addDisplayXPathField(container,
                                'xpathBindings__element_xpaths__text_xpath',
                                fieldType + ' Options Display Text',
                                bindingElement.selectSingleNode("fm:text_xpath"));

            needXPaths = true;

        }

        if ((configElement.getAttribute("hide") == 'true') || (configElement.getAttribute("disable") == 'true'))
        {
            var headingRow = container.insertRow(-1);
            var headingCell = headingRow.insertCell(-1);
            headingCell.colSpan = 2;
            headingCell.className = 'groupHeading';
            headingCell.innerHTML = 'Visibility Details';

            //check if hide xpath needed
            if (configElement.getAttribute("hide") == 'true')
            {
                addDisplayXPathField(container,
                                    'xpathBindings__element_xpaths__hide_xpath',
                                    'Hide If',
                                    bindingElement.selectSingleNode("fm:hide_xpath"));
                needXPaths = true;
            }

            //check if disable xpath needed
            if (configElement.getAttribute("disable") == 'true')
            {
                addDisplayXPathField(container,
                                    'xpathBindings__element_xpaths__disable_xpath',
                                    'Disable If',
                                    bindingElement.selectSingleNode("fm:disable_xpath"));
                needXPaths = true;
            }
        }

        if (((configElement.selectSingleNode("fm:constraints") != null) &&
             (configElement.selectSingleNode("fm:constraints").getAttribute("validate") == 'dynamic')) ||
            ((configElement.selectSingleNode("fm:error/fm:server") != null) &&
             (configElement.selectSingleNode("fm:error/fm:server").getAttribute("use") == 'true')))
        {
            var headingRow = container.insertRow(-1);
            var headingCell = headingRow.insertCell(-1);
            headingCell.colSpan = 2;
            headingCell.className = 'groupHeading';
            headingCell.innerHTML = 'Hints and Errors';

            //check if validate xpath needed
            if ((configElement.selectSingleNode("fm:constraints") != null) &&
                (configElement.selectSingleNode("fm:constraints").getAttribute("validate") == 'dynamic'))
            {
                addDisplayXPathField(container,
                                    'xpathBindings__element_xpaths__validate_xpath',
                                    'Validate If',
                                    bindingElement.selectSingleNode("fm:validate_xpath"));
                needXPaths = true;
            }


            //check if server_error_xpath is needed
            if ((configElement.selectSingleNode("fm:error/fm:server") != null) &&
                (configElement.selectSingleNode("fm:error/fm:server").getAttribute("use") == 'true'))
            {
                addDisplayXPathField(container,
                                    'xpathBindings__element_xpaths__server_message_xpath',
                                    'Error from Server Value',
                                    bindingElement.selectSingleNode("fm:server_message_xpath"));
                needXPaths = true;
            }
        }

        //show the value xpath for a checkbox with a dynamic value
        if ((configElement.selectSingleNode("fm:field/fm:input").getAttribute("options") == 'dynamic') &&
            (controlType == 'checkbox'))
        {
            addDisplayXPathField(container,
                                'xpathBindings__element_xpaths__value_xpath',
                                'Checkbox Value',
                                bindingElement.selectSingleNode("fm:value_xpath"));
            needXPaths = true;
        }

        document.getElementById("dynamic_label_style_xpaths").innerHTML = "";
        document.getElementById("dynamic_style_xpaths").innerHTML = "";
        document.getElementById("dynamic_element_style_xpaths").innerHTML = "";
        document.getElementById("dynamic_disabled_style_xpaths").innerHTML = "";
        var actionContainer = document.getElementById("dynamic_action_xpaths");
        actionContainer.innerHTML = "";

        //show any needed label styles Xpaths
        var dynamicStyles = configElement.selectSingleNode("fm:field/fm:label");
        if (dynamicStyles)
        {
            showDynamicStylesXPaths(document.getElementById("dynamic_label_style_xpaths"), dynamicStyles,configElement.getAttribute('name'), 'label');
        }
        //show any needed label container styles Xpaths
        var dynamicStyles = configElement.selectSingleNode("fm:field/fm:label/fm:container");
        if (dynamicStyles)
        {
            showDynamicStylesXPaths(document.getElementById("dynamic_label_style_xpaths"), dynamicStyles, configElement.getAttribute('name'), 'labelcontainer');
        }

        //show any needed background styles Xpaths
        var dynamicStyles = configElement.selectSingleNode("fm:field/fm:style");
        if (dynamicStyles)
        {
            showDynamicStylesXPaths(document.getElementById("dynamic_style_xpaths"), dynamicStyles,configElement.getAttribute('name'), 'background');
        }

        //show any needed element styles Xpaths
        var dynamicElementStyles = configElement.selectSingleNode("fm:field/fm:element_style");
        if (dynamicElementStyles)
        {
            showDynamicStylesXPaths(document.getElementById("dynamic_element_style_xpaths"), dynamicElementStyles,configElement.getAttribute('name'), 'element');
        }

        //show any needed disabled styles Xpaths
        if (configElement.getAttribute("disable") == 'true')
        {
            var dynamicDisabledStyles = configElement.selectSingleNode("fm:field/fm:disabled_style");
            if (dynamicDisabledStyles)
            {
                showDynamicStylesXPaths(document.getElementById("dynamic_disabled_style_xpaths"), dynamicDisabledStyles,configElement.getAttribute('name'), 'disabled');
            }
        }

        //now handle the new events format
        dynamicEvents = configElement.selectNodes("fm:events/fm:event[fm:action/fm:param[@option = 'Dynamic']]");
        if (dynamicEvents.length > 0)
        {
            showNewFormatEventXPaths(actionContainer, dynamicEvents, configElement.getAttribute('name'), 'field');
            needXPaths = true;
        }
        dynamicEvents = configElement.selectNodes("fm:field/fm:label/fm:events/fm:event[fm:action/fm:param[@option = 'Dynamic']]");
        if (dynamicEvents.length > 0)
        {
            showNewFormatEventXPaths(actionContainer, dynamicEvents, configElement.getAttribute('name'), 'label');
            needXPaths = true;
        }
    }

    //check if xform xpath needed
    if ((currentTab == HANDLER_TO_PAGE_TAB) ||
        (currentAction == '') ||
        (controlType == 'output') ||
        (controlType == 'button') ||
        (controlType == 'htmlButton') ||
        (applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[fm:action = \'" + currentAction + "\']/fm:instance_type[. = 'none']") != null))
    {
        setTimeout("clearOutputMatches()", 50);
    }
    else
    {
        var xformXPaths = null, valueXPaths = null;
        var baseAction = null;

        //check if the current action is set from a different one
        if (bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = '" + currentAction + "']/@base") != null)
        {
            baseAction = bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = '" + currentAction + "']").getAttribute('base');
            xformXPaths = bindingElement.selectNodes("fm:action[@name = '"+baseAction+"']/fm:xform_xpath");
            valueXPaths = bindingElement.selectNodes("fm:action[@name = '"+baseAction+"']/fm:value_xpath");
        }
        else
        {
            xformXPaths = bindingElement.selectNodes("fm:action[@name = '"+currentAction+"']/fm:xform_xpath");
            valueXPaths = bindingElement.selectNodes("fm:action[@name = '"+currentAction+"']/fm:value_xpath");
        }

        //check if this field supports multiple values, and so both a context and value xpath are needed
        if (((controlType == 'multipleCheckbox') ||
             ((controlType == 'select') &&
              (configElement.selectSingleNode("fm:field/fm:input").getAttribute("multiple_selection") == 'true'))) &&
            (bindingElement.getAttribute('binding_format') == 'structure'))
        {
            addDisplayXPathField(container,
                                'xpathBindings__element_xpaths__xform_xpath0',
                                'Submission Context Location',
                                xformXPaths.item(0),
                                'output');

            addDisplayXPathField(container,
                                'xpathBindings__element_xpaths__submission_value_xpath0',
                                'Field Value Binding',
                                valueXPaths.item(0),
                                'none');

            for (var i = 1; i < xformXPaths.length; ++i)
            {
                addDisplayXPathField(container,
                                    'xpathBindings__element_xpaths__xform_xpath' + i,
                                    'Submission Context Location',
                                    xformXPaths.item(i),
                                    'none',
                                    true);
                addDisplayXPathField(container,
                                    'xpathBindings__element_xpaths__submission_value_xpath' + i,
                                    'Field Value Binding',
                                    valueXPaths.item(i),
                                    'none');
            }
        }
        else
        {
            //must always have one xpath field
            addDisplayXPathField(container,
                                'xpathBindings__element_xpaths__xform_xpath0',
                                'Field Submission Binding',
                                xformXPaths.item(0),
                                'output');

            for (var i = 1; i < xformXPaths.length; ++i)
            {
                addDisplayXPathField(container,
                                    'xpathBindings__element_xpaths__xform_xpath' + i,
                                    '',
                                    xformXPaths.item(i),
                                    'none',
                                    true);
            }
        }

        //add new binding button
        if (baseAction == null)
        {
            var newBindingRow = container.insertRow(-1);
            var newBindingCell = newBindingRow.insertCell(-1);
            newBindingCell.colSpan = 2;
            newBindingCell.innerHTML = '<a href="javascript:addAdditionalSubmissionBinding()">Add Additional Binding</a>'
        }




        needXPaths = true;
    }

    setMessageDisplay(messageMode);

    return needXPaths;
}

/**
 * Removes an additional submission binding field from the currently displayed element,
 * and updates the display.
 * @param id The id of the textbox containing the binding to remove.
 */
function removeAdditionalSubmissionBinding(id)
{
    saveToDOM();
    var configElement;
    switch(document.XPath_Bindings_form.currentType.value)
    {
        case 'element' : configElement = bindingsDoc.selectSingleNode("/fm:xforms/fm:element[@name = '"+document.XPath_Bindings_form.currentElement.value+"']");
    }

    if (configElement != null)
    {
        var xpathValue = document.getElementById(id).value;

        var xpaths = configElement.selectNodes("fm:action[@name = '"+currentAction+"']/fm:xform_xpath");
        for (var i = 0; i < xpaths.length; ++i)
        {
            if (getElementText(xpaths.item(i)) == xpathValue)
            {
                xpaths.item(i).parentNode.removeChild(xpaths.item(i));

                //check for an associated value xpath for multi value controls
                var values = configElement.selectNodes("fm:action[@name = '"+currentAction+"']/fm:value_xpath");
                if (values.item(i))
                    values.item(i).parentNode.removeChild(values.item(i));

                break;
            }
        }

        showRelevantXpaths();
    }
}

/**
 * Adds an additional submission binding field to the currently displayed element,
 * and updates the display.
 */
function addAdditionalSubmissionBinding()
{
    saveToDOM();
    var configElement;
    switch(document.XPath_Bindings_form.currentType.value)
    {
        case 'element' : configElement = bindingsDoc.selectSingleNode("/fm:xforms/fm:element[@name = '"+document.XPath_Bindings_form.currentElement.value+"']");
    }

    if (configElement != null)
    {
        saveActionBindingElem(configElement, currentAction, "xform_xpath", blank_xpath_text);
        if (document.getElementById('xpathBindings__element_xpaths__submission_value_xpath0'))
            saveActionBindingElem(configElement, currentAction, "value_xpath", blank_xpath_text);
        showRelevantXpaths();
    }
}

/**
 * Creates an HTML formatted string to show the textarea for entering xpaths for the specified type of dynamic styling
 * @param container The HTML container in which to display the textareas
 * @param dynamicStyles the Style node within the structure file.
 * @param fieldname name of the selected field
 * @param styleType The type of dynamic styles to output. 'label' 'background' 'element' or 'disabled'
 */
function showDynamicStylesXPaths(container, dynamicStyles, fieldName, styleType)
{
    var textareas  = new Array();


    var displayType;
    switch(styleType)
    {
        case 'label'            :   displayType = 'Label'; break;
        case 'background'       :   displayType = 'Background'; break;
        case 'element'          :   displayType = 'Element'; break;
        case 'disabled'         :   displayType = 'Disabled'; break;
        case 'group'            :   displayType = 'Group'; break;
        case 'grouplabel'       :   displayType = 'Label'; break;
        case 'labelcontainer'   :
        case 'grouplabelcontainer': displayType = 'Label Background'; break;
        default                 :   displayType = styleType;
    }

    var styleHTML = "";

    //check if styles are dynamic
    if (dynamicStyles.getAttribute('options') == 'dynamic')
    {

        var classes = dynamicStyles.selectNodes("fm:class");
        styleHTML += '<div id="'+styleType+'_style_xpath_heading" class="groupHeading">Dynamic '+displayType+' Style XPaths</div>';
        styleHTML += '<table border="0" cellspacing="0" cellpadding="0" class="table" id="'+styleType+'_style_xpaths_group">';
        for (var i=0; i < (classes.length); ++i)
        {
            var id = classes.item(i).getAttribute('id');
            // if dynamic
            if (classes.item(i).getAttribute('type') == 'dynamic')
            {
                var xpathNode = "";
                if(bindingsDoc.selectSingleNode('/fm:xforms/*[@name = "'+fieldName+'"]/fm:dynamic_style_xpath[@style_id = "'+id+'" and @style_type = "'+styleType+'"]'))
                {
                    xpathNode = getElementText(bindingsDoc.selectSingleNode('/fm:xforms/*[@name = "'+fieldName+'"]/fm:dynamic_style_xpath[@style_id = "'+id+'" and @style_type = "'+styleType+'"]'));
                }
                styleHTML += '<tr style="padding-top:5px;"><th class="tableColumnHeading">Apply Dynamic '+displayType+' Style If</th><td>';
                styleHTML += createXPathTextAreaString('xpathBindings__'+styleType+'_style_if_'+id, xpathNode);;
                styleHTML += '</td></tr>';

                textareas[textareas.length] = 'xpathBindings__'+styleType+'_style_if_'+id;

                xpathNode="";
                if(bindingsDoc.selectSingleNode('/fm:xforms/*[@name = "'+fieldName+'"]/fm:dynamic_style_xpath_value[@style_id = "'+id+'" and @style_type = "'+styleType+'"]'))
                {
                    xpathNode = getElementText(bindingsDoc.selectSingleNode('/fm:xforms/*[@name = "'+fieldName+'"]/fm:dynamic_style_xpath_value[@style_id = "'+id+'" and @style_type = "'+styleType+'"]'));
                }
                styleHTML += '<tr style="padding-bottom:5px;"><th class="tableColumnHeading">Dynamic '+displayType+' Style Value</th><td>';
                styleHTML += createXPathTextAreaString('xpathBindings__'+styleType+'_style_xpath_'+id, xpathNode);;
                styleHTML += '</td></tr>';

                textareas[textareas.length] = 'xpathBindings__'+styleType+'_style_xpath_'+id;
            }
            // if static
            if (classes.item(i).getAttribute('type') == 'static')
            {
                // Background Styling if...
                var xpathNode="";
                if(bindingsDoc.selectSingleNode('/fm:xforms/*[@name = "'+fieldName+'"]/fm:dynamic_style_xpath[@style_id = "'+id+'" and @style_type = "'+styleType+'"]'))
                {
                     xpathNode = getElementText(bindingsDoc.selectSingleNode('/fm:xforms/*[@name = "'+fieldName+'"]/fm:dynamic_style_xpath[@style_id = "'+id+'" and @style_type = "'+styleType+'"]'));
                }
                styleHTML += '<tr style="padding-top:5px;padding-bottom:5px;"><th class="tableColumnHeading">Apply \''+getElementText(classes.item(i))+'\' '+displayType+' Style If</th><td>';
                styleHTML += createXPathTextAreaString('xpathBindings__'+styleType+'_style_if_'+id, xpathNode);;
                styleHTML += '</td></tr>';
                textareas[textareas.length] = 'xpathBindings__'+styleType+'_style_if_'+id;
            }
        }
        styleHTML += '</table>';
    }
    container.innerHTML += styleHTML;

    //validate the xpath details for each event textarea shown.
    for (var ta = 0; ta < textareas.length; ++ta)
    {
        setTimeout("validateXPathImpl('"+textareas[ta]+"', false, false)", 50);
    }
}


/**
 * Creates an HTML formatted string to show the textareas for entering xpaths for setvalue event actions.
 * @param container The HTML container in which to display the textareas
 * @param dynamicEvents The nodeList of all appropriate event trigger elements from the structure file.
 * @param fieldName The name of the field the events are for.
 * @param eventSource 'field' or 'label'
 */
function showEventXPaths(container, dynamicEvents, fieldName, eventSource)
{
    var textareas  = new Array();

    var displaySource;
    switch(eventSource)
    {
        case 'field'    :   displaySource = 'Field'; break;
        case 'label'    :   displaySource = 'Label'; break;
        default         :   displaySource = eventSource;
    }

    var actionHTML = "";
    if (dynamicEvents.length > 0)
    {

        actionHTML += '<div id="event_xpath_heading" class="groupHeading">'+displaySource+' Set Value Events</div>';
        actionHTML += '<table border="0" cellspacing="0" cellpadding="0" class="table" id="element_event_xpaths_group">';
        for (var i = 0; i < dynamicEvents.length; ++i)
        {
            var trigger = dynamicEvents.item(i);
            actionHTML += '<tr><th class="tableColumnHeading">Event Trigger: </th><td>';
            if (getBaseName(trigger) == 'href')
            {
                actionHTML += 'anchor activated';
            }
            else
            {
                actionHTML += getBaseName(trigger);
            }

            actionHTML += '</td></tr>';

            var dynamicActions = trigger.selectNodes("fm:setvalue[@value_type = 'dynamic']");
            for (var j = 0; j < dynamicActions.length; ++j)
            {

                var action = dynamicActions.item(j);

                if(eventSource=="field" || eventSource=="label")
                {
                    var xpathNode = bindingsDoc.selectSingleNode("/fm:xforms/fm:element[@name = '"+fieldName+"']/fm:setvalue_action_xpath[@event_source = '"+eventSource+"' and @event_type = '"+getBaseName(trigger)+"' and @target_field = '"+action.getAttribute('target')+"']");
                }
                else
                {
                    var xpathNode = bindingsDoc.selectSingleNode("/fm:xforms/fm:group[@name = '"+fieldName+"']/fm:setvalue_action_xpath[@event_source = '"+eventSource+"' and @event_type = '"+getBaseName(trigger)+"' and @target_field = '"+action.getAttribute('target')+"']");
                }

                /*actionHTML += '<tr><th class="tableColumnHeading">Target Field:</th><td>';
                actionHTML += action.getAttribute('target');*/
                actionHTML += '<tr><th class="tableColumnHeading">Value to set for field <span class="emphasis">'+action.getAttribute('target')+'</span></th><td>';

                var value = "";
                if (xpathNode != null)
                {
                    value = getElementText(xpathNode);
                }

                actionHTML += createXPathTextAreaString('setvalue_action_xpath_'+eventSource+'_'+action.getAttribute('target'), value);

                actionHTML += '</td></tr>';
                textareas[textareas.length] = 'setvalue_action_xpath_'+eventSource+'_'+action.getAttribute('target');
            }
        }
        actionHTML += '</table>';
        //alert(actionHTML);
    }
    container.innerHTML += actionHTML;


    //validate the xpath details for each event textarea shown.
    for (var ta = 0; ta < textareas.length; ++ta)
    {
        setTimeout("validateXPathImpl('"+textareas[ta]+"', false, false)", 50);
    }
}

/**
 * Creates an HTML formatted string to show the textareas for entering xpaths for
 * any dynamic event information in the new format.
 * @param container The HTML container in which to display the textareas
 * @param dynamicEvents The nodeList of all appropriate event elements from the structure file.
 * @param fieldName The name of the field the events are for.
 * @param eventSource 'field', 'label', 'group', 'grouplabel' or 'page'
 */
function showNewFormatEventXPaths(container, dynamicEvents, fieldName, eventSource)
{
    var textareas  = new Array();

    var displaySource;
    switch(eventSource)
    {
        case 'field'    :   displaySource = 'Field'; break;
        case 'label'    :   displaySource = 'Label'; break;
        case 'group'    :   displaySource = 'Group'; break;
        case 'grouplabel':  displaySource = 'Label'; break;
        case 'page'     :   displaySource = 'Page'; break;
        case 'external' :   displaySource = 'External'; break;
        default         :   displaySource = eventSource;
    }

    var actionHTML = "";
    if (dynamicEvents.length > 0)
    {

        actionHTML += '<div id="event_xpath_heading" class="groupHeading">'+displaySource+' Events</div>';
        actionHTML += '<table border="0" cellspacing="0" cellpadding="0" class="table" id="'+eventSource+'_event_xpaths_group">';
        for (var i = 0; i < dynamicEvents.length; ++i)
        {
            var trigger = dynamicEvents.item(i);
            actionHTML += '<tr><th class="tableColumnHeading">Event Trigger: </th><td>';
            //Need to look up the event id in the config file to determine the display name
            actionHTML += getEventActionScreenName(trigger.getAttribute('id'), 'event');

            actionHTML += '</td></tr>';

            var dynamicActions = trigger.selectNodes("fm:action[fm:param/@option = 'Dynamic']");
            for (var j = 0; j < dynamicActions.length; ++j)
            {
                var action = dynamicActions.item(j);

                var dynamicParams = action.selectNodes("fm:param[@option = 'Dynamic']");

                if (dynamicParams.length == 1)
                {
                    actionHTML += '<tr><th class="tableColumnHeading">'
                    actionHTML += getEventActionDisplayMessage(action);
                    actionHTML += '</th><td>';

                    var value = getEventActionExistingValue(fieldName, eventSource, trigger, action, dynamicParams.item(0));
                    var fieldID = getEventActionFieldId(eventSource, trigger, action, dynamicParams.item(0));

                    actionHTML += createXPathTextAreaString(fieldID, value);

                    actionHTML += '</td></tr>';
                    textareas[textareas.length] = fieldID;
                }
                else
                {
                    actionHTML += '<tr><th colspan="2" class="tableColumnHeading" style="width : auto;">'
                    actionHTML += getEventActionDisplayMessage(action);
                    actionHTML += '</th></tr>';

                    //more than one dynamic param in this action
                    for (var k = 0; k < dynamicParams.length; ++k)
                    {
                        var param = dynamicParams.item(k);

                        actionHTML += '<tr><th class="tableColumnHeading">'
                        //Show screen name here from the event config file
                        actionHTML += getEventActionScreenName(param.getAttribute('name'), 'param', param.getAttribute('id'));
                        actionHTML += '</th><td>';

                        var value = getEventActionExistingValue(fieldName, eventSource, trigger, action, param);
                        var fieldID = getEventActionFieldId(eventSource, trigger, action, param);

                        actionHTML += createXPathTextAreaString(fieldID, value);

                        actionHTML += '</td></tr>';
                        textareas[textareas.length] = fieldID;

                    }
                }
            }
        }
        actionHTML += '</table>';
        //alert(actionHTML);
    }
    container.innerHTML += actionHTML;


    //validate the xpath details for each event textarea shown.
    for (var ta = 0; ta < textareas.length; ++ta)
    {
        setTimeout("validateXPathImpl('"+textareas[ta]+"', false, false)", 50);
    }
}

/**
 * Returns the display message to output for the given action
 */
function getEventActionDisplayMessage(action)
{
    var msg = '';
    switch (action.getAttribute('id'))
    {
        case 'SetValue' : msg = 'Value to set for field <span class="emphasis">'+action.selectSingleNode("fm:param[@name = 'TargetToSet']").getAttribute('value')+'</span>';
                          break;
        case 'formSubmission' : msg = 'Validate flag for action <span class="emphasis">'+action.selectSingleNode("fm:param[@name = 'Action']").getAttribute('value')+'</span>';
                          break;
        default         : msg = getEventActionScreenName(action.getAttribute('id'), 'action') + ' XPath';
    }
    return msg;
}

/**
 * Checks in the bindings file to retreive the existing value that applies to the given event.
 * @param fieldName The name of the field this event applies to.
 * @param eventSource 'field', 'label', 'group', 'grouplabel' or 'page'
 * @param event The 'event' element in the structure file to get the value for.
 * @param action The 'action' element in the structure file to get the value for.
 * @param param The 'param' element in the structure file to get the value for.
 * @return The existing value, or an empty string if no value present.
 */
function getEventActionExistingValue(fieldName, eventSource, event, action, param)
{
    var value = "";
    var containerNode = null;
    if (eventSource == "field" || eventSource == "label")
    {
        containerNode = bindingsDoc.selectSingleNode("/fm:xforms/fm:element[@name = '"+fieldName+"']");
    }
    else if (eventSource == "group" || eventSource == "grouplabel")
    {
        containerNode = bindingsDoc.selectSingleNode("/fm:xforms/fm:group[@name = '"+fieldName+"']");
    }
    else if (eventSource == "page" || eventSource == "external")
    {
        containerNode = bindingsDoc.selectSingleNode("/fm:xforms/fm:page");
    }

    if (containerNode != null)
    {
        var aid = getEventActionAdditionalIdentifier(action);

        var predicate = "@event_source = '"+eventSource+"' and @event = '"+event.getAttribute('id')+"' and @action = '"+action.getAttribute('id')+"' and @param = '"+param.getAttribute('name')+"'";
        if (aid != '')
        {
            predicate += " and @action_identifier = '" + aid + "'";
        }

        var xpathNode = containerNode.selectSingleNode("fm:event_xpath[" + predicate + "]");

        if (xpathNode != null)
        {
            value = getElementText(xpathNode);
        }
    }

    return value;
}

/**
 * Returns the additional identification information (if any) that applies to the
 * given action.  This is used to handle the situation that a given event could have multiple
 * instances of the same action type, and so this value can seperate them.
 */
function getEventActionAdditionalIdentifier(action)
{
    var aid = '';
    switch (action.getAttribute('id'))
    {
        case 'SetValue'       : aid = action.selectSingleNode("fm:param[@name = 'TargetToSet']").getAttribute('value');
                                break;
        case 'formSubmission' : aid = action.selectSingleNode("fm:param[@name = 'Action']").getAttribute('value');
                                break;
        default               : aid = action.selectNodes("preceding-sibling::fm:action[@id = '"+action.getAttribute('id')+"']").length;
    }
    return aid;
}

/**
 * Returns the id value to use for the xpath textarea for the given event information.
 * @param eventSource 'field', 'label', 'group', 'grouplabel' or 'page'
 * @param event The 'event' element in the structure file to get the value for.
 * @param action The 'action' element in the structure file to get the value for.
 * @param param The 'param' element in the structure file to get the value for.
 * @return The id string to use for the text area field.
 */
function getEventActionFieldId(eventSource, event, action, param)
{
    var id = eventSource + '_' + event.getAttribute('id') + '_' + action.getAttribute('id') + '_' + param.getAttribute('name');
    var aid = getEventActionAdditionalIdentifier(action);
    if (aid != '')
    {
        id += '_' + aid;
    }
    return id;
}


/**
 * Finds the screen name for the given id in the event config file and returns it.
 * @param id The id value to look up
 * @param type The type of id to look up - 'event', 'action', 'param'
 * @param paramActionId If the type is param, then this must contain the id of the action that is
 *              the parent of the param for which the screen name is requried
 * @return The matching screen name value, or the given id if no value was found.
 */
function getEventActionScreenName(id, type, paramActionId)
{
    if (eventConfigDoc != null)
    {
        var configNode = null;
        switch (type)
        {
            case 'event' :  configNode = eventConfigDoc.selectSingleNode("/fm:event_config/fm:events/*/*/fm:event[@id = '"+id+"']");
                            break;
            case 'action':  configNode = eventConfigDoc.selectSingleNode("/fm:event_config/fm:actions/fm:action[@id = '"+id+"']");
                            break;
            case 'param' :  configNode = eventConfigDoc.selectSingleNode("/fm:event_config/fm:actions/fm:action[@id = '"+paramActionId+"']/fm:params/fm:param[@name = '"+id+"']");
                            break;

        }

        if (configNode != null)
        {
            var value = configNode.getAttribute('screen_name');
            if ((value != null) && (value != ''))
            {
                return value;
            }
        }
    }
    return id;
}

/**
 * creates a String containing the HTML to show an xpath textarea with all the events initialised for xpath validation etc.
 * @param id The ID to use for the textarea
 * @param value The value to place in the textarea.
 * @param onchange Optional javascript string to be used for the onchange event
 * @return The created HTML string
 */
function createXPathTextAreaString(id, value, onchange)
{
    var htmlString = '';

    htmlString += '<span><span class="textarea">';

    htmlString += '<textarea wrap="soft" name="'+id+'" id="'+id+'" class="xpath" onmouseup="mouseUpSave(this);validateXPath(\''+id+'\');" onfocus="validateXPath(\''+id+'\');" onkeyup="validateXPath(\''+id+'\');event.cancelBubble = true;" onkeypress="event.cancelBubble = true;" onkeydown="event.cancelBubble = true;"';

    if (typeof(onchange) == 'string')
        htmlString += ' onchange="setChanged();' + onchange + '">';
    else
        htmlString += ' onchange="setChanged();">';
    htmlString += value + '</textarea>';
    htmlString += '<span id="'+id+'_match" class="xpath_match_container"></span>';
    htmlString += '<div id="'+id+'_error" class="error_container"></div>';

    htmlString += '</span></span>';

    return htmlString;
}


/* shows or hides the group xpath text areas as required by the given group element */
function showGroupDetails(configElement)
{
    //Destroy the edit area instance if it is visible for the current component.
    //This is needed because we are about to remove and then recreate the group contents.
    if (editAreas['xpathBindings__group_xpaths__choice_fragment'])
        editAreaLoader.delete_instance('xpathBindings__group_xpaths__choice_fragment');

    //alert("showing group xpaths");
    showComponent(document.getElementById('group_group'));
    showComponent(document.getElementById('group_group_title'));
    showComponent(document.getElementById('message_group_container_group'));
    showComponent(document.getElementById('message_group_container'));

    var needXPaths = false;

    //set up group name display
    var elemName = configElement.getAttribute("name");
    document.getElementById("xpathBindings__group_name").innerHTML = elemName;

    clearInputMatches();
    clearOutputMatches();

    var container = document.getElementById('group_xpaths_group');
    //delete any existing table content
    for (var i = container.rows.length - 1; i >= 0; --i)
    {
        container.deleteRow(i);
    }
    document.getElementById("dynamic_action_xpaths_group").innerHTML = "";
    document.getElementById("dynamic_group_label_style_xpaths").innerHTML = "";
    document.getElementById("dynamic_group_style_xpaths").innerHTML = "";

    var bindingElement = bindingsDoc.selectSingleNode("/fm:xforms/fm:group[@name = '"+elemName+"']");

    if (currentTab == HANDLER_TO_PAGE_TAB)
    {
        //check if hide xpath needed
        if (configElement.getAttribute('hide') == 'true')
        {
            addDisplayXPathField(container,
                                'xpathBindings__group_xpaths__hide_xpath',
                                'Hide If',
                                bindingElement.selectSingleNode("fm:hide_xpath"));
            needXPaths = true;
        }
        //check if disable xpath needed
        if (configElement.getAttribute('disable') == 'true')
        {
            addDisplayXPathField(container,
                                'xpathBindings__group_xpaths__disable_xpath',
                                'Disable If',
                                bindingElement.selectSingleNode("fm:disable_xpath"));
            needXPaths = true;
        }

        //check if label xpath needed
        if ((configElement.selectSingleNode("fm:label[not(./@required) or (./@required = 'true')]") != null ) &&
            (configElement.selectSingleNode("fm:label").getAttribute('type') == 'dynamic'))
        {
            addDisplayXPathField(container,
                                'xpathBindings__group_xpaths__label_xpath',
                                'Label Value',
                                bindingElement.selectSingleNode("fm:label_xpath"));
            needXPaths = true;
        }


        var actionContainer = document.getElementById("dynamic_action_xpaths_group");

        //now handle the new events format
        var dynamicEvents = configElement.selectNodes("fm:events/fm:event[fm:action/fm:param[@option = 'Dynamic']]");
        if (dynamicEvents.length > 0)
        {
            showNewFormatEventXPaths(actionContainer, dynamicEvents, configElement.getAttribute('name'), 'group');
            needXPaths = true;
        }
        dynamicEvents = configElement.selectNodes("fm:label/fm:events/fm:event[fm:action/fm:param[@option = 'Dynamic']]");
        if (dynamicEvents.length > 0)
        {
            showNewFormatEventXPaths(actionContainer, dynamicEvents, configElement.getAttribute('name'), 'grouplabel');
            needXPaths = true;
        }

        //show any needed dynamic group label styles Xpaths
        var dynamicStyles = configElement.selectSingleNode("fm:label");
        if (dynamicStyles)
        {
            showDynamicStylesXPaths(document.getElementById("dynamic_group_label_style_xpaths"), dynamicStyles,configElement.getAttribute('name'), 'grouplabel');
            if (document.getElementById("dynamic_group_label_style_xpaths").innerHTML != "")
                needXPaths = true;
        }
        var dynamicStyles = configElement.selectSingleNode("fm:label/fm:container");
        if (dynamicStyles)
        {
            showDynamicStylesXPaths(document.getElementById("dynamic_group_label_style_xpaths"), dynamicStyles,configElement.getAttribute('name'), 'grouplabelcontainer');
            if (document.getElementById("dynamic_group_label_style_xpaths").innerHTML != "")
                needXPaths = true;
        }

        //show any needed dynamic group styles Xpaths
        var dynamicStyles = configElement.selectSingleNode("fm:style");
        if (dynamicStyles)
        {
            showDynamicStylesXPaths(document.getElementById("dynamic_group_style_xpaths"), dynamicStyles,configElement.getAttribute('name'), 'group');
            if (document.getElementById("dynamic_group_style_xpaths").innerHTML != "")
                needXPaths = true;
        }
    }
    else if (currentTab == PAGE_TO_HANDLER_TAB)
    {

        //check if choice_fragment needed
        if ((currentAction != '') &&
            (configElement.getAttribute('submit_if_hidden') == 'false') &&
            ((configElement.getAttribute('hide') == 'true') ||
             (configElement.getAttribute('conditional_display') == 'true')) &&
            (applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[fm:action = \'" + currentAction + "\']/fm:instance_type[. = 'static' or . = 'dynamic']") != null))
        {

            var storedXPath = '';
            var storedFragment = '';

            //check if the current action is set from a different one
            if (bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = '" + currentAction + "']/@base") != null)
            {
                var baseAction = bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = '" + currentAction + "']").getAttribute('base');
                if (bindingElement.selectSingleNode("fm:action[@name = '"+baseAction+"']/fm:xform_xpath") != null)
                    storedXPath = getElementText(bindingElement.selectSingleNode("fm:action[@name = '"+baseAction+"']/fm:xform_xpath"));

                if (bindingElement.selectSingleNode("fm:action[@name = '"+baseAction+"']/fm:choice_fragment") != null)
                {
                    var children = bindingElement.selectSingleNode("fm:action[@name = '"+baseAction+"']/fm:choice_fragment").childNodes;
                    for (var i = 0; i < children.length; ++i)
                    {
                        storedFragment += xmlToString(children.item(i));
                    }
                }
            }
            else
            {
                if (bindingElement.selectSingleNode("fm:action[@name = '"+currentAction+"']/fm:xform_xpath") != null)
                    storedXPath = getElementText(bindingElement.selectSingleNode("fm:action[@name = '"+currentAction+"']/fm:xform_xpath"));

                if (bindingElement.selectSingleNode("fm:action[@name = '"+currentAction+"']/fm:choice_fragment") != null)
                {
                    var children = bindingElement.selectSingleNode("fm:action[@name = '"+currentAction+"']/fm:choice_fragment").childNodes;
                    for (var i = 0; i < children.length; ++i)
                    {
                        storedFragment += xmlToString(children.item(i));
                    }
                }
            }

            //show context location
            addDisplayXPathField(container,
                                'xpathBindings__group_xpaths__xform_xpath',
                                'Context Location',
                                storedXPath);

            //show group fragment
            var newRow = container.insertRow(-1);
            var label = newRow.insertCell(-1);
            label.className = 'tableColumnHeading';
            label.id = 'xpathBindings__group_xpaths__choice_fragment_label';
            label.innerHTML = 'Group XML Fragment';
            var detail = newRow.insertCell(-1);
            var textareaId = 'xpathBindings__group_xpaths__choice_fragment';
            detail.innerHTML = '<span id="'+textareaId+'_container">'
                    + '<textarea name="'+textareaId+'" id="'+textareaId+'" onmouseup="mouseUpSave(this);validateXMLFragment(\''+textareaId+'\');" onkeyup="validateXMLFragment(\''+textareaId+'\'); event.cancelBubble = true;" onkeypress="event.cancelBubble = true;" onkeydown="event.cancelBubble = true;" onfocus="validateXMLFragment(\''+textareaId+'\');" class="fragment" onchange="setChanged();">'
                    + storedFragment
                    + '</textarea>'
                    + '<div class="error_container" id="'+textareaId+'_error">'
                    + '</div>'
                + '</span>';

            //convert to an edit area
            //this is done using a timeout, as sometimes this method gets called
            //in quick succession (eg when docs loading) so want to wait for the screen
            //to settle down before doing the conversion.
            if (editAreaInitialiserTimeout != null)
            {
                clearTimeout(editAreaInitialiserTimeout);
            }
            setTimeout("editAreaInitialiser()", 50);

            needXPaths = true;
        }
    }

    setMessageGroupDisplay(messageGroupMode);

    return needXPaths
}

editAreaInitialiserTimeout = null;

function editAreaInitialiser()
{
    if (!editAreas['xpathBindings__group_xpaths__choice_fragment'])
    {
        editAreaLoader.init({
            id : "xpathBindings__group_xpaths__choice_fragment"     // textarea id
            ,syntax: "xml"          // syntax to be uses for highgliting
            ,start_highlight: true  // to display with highlight mode on start-up
            ,toolbar: "search, go_to_line, |, undo, redo, |, select_font,|, change_smooth_selection, highlight, reset_highlight, word_wrap, pretty_print, |, help"
            ,plugins: "validator, xmlhelper"
            ,font_size: 8
            ,replace_tab_by_spaces: 4
            ,change_callback: 'setChanged'
        });
    }
}


/* shows or hides the repeat xpath text areas as required by the given repeat element */
function showRepeatDetails(configElement)
{
    showComponent(document.getElementById('repeat_group'));
    showComponent(document.getElementById('repeat_group_title'));
    showComponent(document.getElementById('message_repeat_container_group'));
    showComponent(document.getElementById('message_repeat_container'));

    var defaultSort = false;

    //set up repeat name display
    var elemName = configElement.getAttribute("name");
    document.getElementById("xpathBindings__repeat_name").innerHTML = elemName;

    var container = document.getElementById('repeat_xpaths_group');
    //delete any existing table content
    for (var i = container.rows.length - 1; i >= 0; --i)
    {
        container.deleteRow(i);
    }

    var bindingElement = bindingsDoc.selectSingleNode("/fm:xforms/fm:repeat[@name = '"+elemName+"']");

    if (currentTab == HANDLER_TO_PAGE_TAB)
    {
        //show repeat xpath
        addDisplayXPathField(container,
                            'xpathBindings__repeat_xpaths__repeat_xpath',
                            'Repeating Data Location',
                            bindingElement.selectSingleNode("fm:repeat_xpath"),
                            'input');

        //check if sort xpath needed
        if (configElement.selectSingleNode("fm:sort") != null)
        {
            addDisplayXPathField(container,
                                'xpathBindings__repeat_xpaths__sort_xpath',
                                'Sort element within Repeating Data Location',
                                bindingElement.selectSingleNode("fm:sort_xpath"));

            if (configElement.selectSingleNode("fm:sort").getAttribute("default") == 'true')
            {
                defaultSort = true;
                disableComponent(document.XPath_Bindings_form.xpathBindings__repeat_xpaths__sort_xpath);
                disableComponent(document.getElementById('xpathBindings__repeat_xpaths__sort_xpath_label'));
                if (document.XPath_Bindings_form.xpathBindings__repeat_xpaths__sort_xpath.value == '')
                    document.XPath_Bindings_form.xpathBindings__repeat_xpaths__sort_xpath.value = '*[local-name(.) = ../../'+configElement.getAttribute('name')+'_sort_field]';
            }
            else
            {
                enableComponent(document.XPath_Bindings_form.xpathBindings__repeat_xpaths__sort_xpath);
                enableComponent(document.getElementById('xpathBindings__repeat_xpaths__sort_xpath_label'));
            }
        }

        //check if sort order xpath needed
        if ((configElement.selectSingleNode("fm:sort") != null ) &&
            (configElement.selectSingleNode("fm:sort").getAttribute('order') == 'dynamic'))
        {
            addDisplayXPathField(container,
                                'xpathBindings__repeat_xpaths__sort_order_xpath',
                                'Sort Order Value',
                                bindingElement.selectSingleNode("fm:sort_order_xpath"));

            if (configElement.selectSingleNode("fm:sort").getAttribute("default") == 'true')
            {
                disableComponent(document.XPath_Bindings_form.xpathBindings__repeat_xpaths__sort_order_xpath);
                disableComponent(document.getElementById('xpathBindings__repeat_xpaths__sort_order_xpath_label'));
                if (document.XPath_Bindings_form.xpathBindings__repeat_xpaths__sort_order_xpath.value == '')
                    document.XPath_Bindings_form.xpathBindings__repeat_xpaths__sort_order_xpath.value = '../'+configElement.getAttribute('name')+'_sort_order"]';
            }
            else
            {
                enableComponent(document.XPath_Bindings_form.xpathBindings__repeat_xpaths__sort_order_xpath);
                enableComponent(document.getElementById('xpathBindings__repeat_xpaths__sort_order_xpath_label'));
            }
        }

        //check if sort data-type xpath needed
        if ((configElement.selectSingleNode("fm:sort") != null ) &&
            (configElement.selectSingleNode("fm:sort").getAttribute('data-type') == 'dynamic'))
        {
            addDisplayXPathField(container,
                                'xpathBindings__repeat_xpaths__sort_data_type_xpath',
                                'Sort Datatype Value',
                                bindingElement.selectSingleNode("fm:sort_data-type_xpath"));

            if (configElement.selectSingleNode("fm:sort").getAttribute("default") == 'true')
            {
                disableComponent(document.XPath_Bindings_form.xpathBindings__repeat_xpaths__sort_data_type_xpath);
                disableComponent(document.getElementById('xpathBindings__repeat_xpaths__sort_data_type_xpath_label'));
                if (document.XPath_Bindings_form.xpathBindings__repeat_xpaths__sort_data_type_xpath.value == '')
                    document.XPath_Bindings_form.xpathBindings__repeat_xpaths__sort_data_type_xpath.value = '../'+configElement.getAttribute('name')+'_sort_data_type"]';
            }
            else
            {
                enableComponent(document.XPath_Bindings_form.xpathBindings__repeat_xpaths__sort_data_type_xpath);
                enableComponent(document.getElementById('xpathBindings__repeat_xpaths__sort_data_type_xpath_label'));
            }
        }

        //check if row style change xpath needed
        if ((configElement.selectSingleNode("fm:row_styling") != null ) &&
            (configElement.selectSingleNode("fm:row_styling").getAttribute('alternate_styles') == 'dynamic'))
        {
            addDisplayXPathField(container,
                                'xpathBindings__repeat_xpaths__row_style_change_xpath',
                                'Alternate Row Style If',
                                bindingElement.selectSingleNode("fm:row_style_change_xpath"));
        }

        //check if column style change xpath needed
        if ((configElement.selectSingleNode("fm:column_styling") != null ) &&
            (configElement.selectSingleNode("fm:column_styling").getAttribute('alternate_styles') == 'dynamic'))
        {
            addDisplayXPathField(container,
                                'xpathBindings__repeat_xpaths__column_style_change_xpath',
                                'Alternate Column Style If',
                                bindingElement.selectSingleNode("fm:column_style_change_xpath"));
        }
    }
    else if (currentTab == PAGE_TO_HANDLER_TAB)
    {
        //show xform xpath
        if (applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[fm:action = \'" + currentAction + "\']/fm:instance_type[. = 'static' or . = 'dynamic']") != null)
        {
            //update xform_xpath
            var storedValue = '';

            //check if the current action is set from a different one
            if (bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = '" + currentAction + "']/@base") != null)
            {
                var baseAction = bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = '" + currentAction + "']").getAttribute('base');
                if (bindingElement.selectSingleNode("fm:action[@name = '"+baseAction+"']/fm:xform_xpath") != null)
                    storedValue = getElementText(bindingElement.selectSingleNode("fm:action[@name = '"+baseAction+"']/fm:xform_xpath"));
            }
            else
            {
                if (bindingElement.selectSingleNode("fm:action[@name = '"+currentAction+"']/fm:xform_xpath") != null)
                    storedValue = getElementText(bindingElement.selectSingleNode("fm:action[@name = '"+currentAction+"']/fm:xform_xpath"));
            }

            addDisplayXPathField(container,
                                'xpathBindings__repeat_xpaths__xform_xpath',
                                'Repeat Submission Binding',
                                storedValue,
                                'output');
        }
    }


    setMessageRepeatDisplay(messageRepeatMode);

    return defaultSort;
}

/** Shows all the form level binding details.
 * This is the action configuration information. */
function showFormDetails()
{
    if (currentTab == ACTION_CONFIG_TAB)
    {
        showComponent(document.getElementById('form_details_group'));
        showComponent(document.getElementById('action_group_title'));

        document.getElementById("xpathBindings__action_name").innerHTML = currentAction;

        if (currentAction == '')
        {
            //no actions apply to this page.
            document.getElementById('form_details_message_container').innerHTML = 'There are no actions applicable to this page to define bindings for.';
            showComponent(document.getElementById('form_details_message_container'));
            hideComponent(document.getElementById('form_details_xpaths_group'));
            hideComponent(document.getElementById('form_details_action_base_container'));
        }
        else
        {
            var action_elem = bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = \'" + currentAction + "\']");

            showComponent(document.getElementById('form_details_action_base_container'));
            //check if this action is based on another, and display accordingly
            if (action_elem != null && action_elem.getAttribute('base') != null)
            {
                document.getElementById('xpathBindings__actions__binding_base_flag').checked = true;
                document.getElementById('xpathBindings__actions__binding_base').value = action_elem.getAttribute('base');
                showComponent(document.getElementById('xpathBindings__actions__binding_base_row'));
            }
            else
            {
                document.getElementById('xpathBindings__actions__binding_base_flag').checked = false;
                document.getElementById('xpathBindings__actions__binding_base').selectedIndex = 0;
                hideComponent(document.getElementById('xpathBindings__actions__binding_base_row'));
            }

            //check if this action is defined directly against this page, or a parent page.
            if (applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[(fm:source/@type = 'Page') and (fm:action = '" +currentAction + "') and (fm:source = '"+document.XPath_Bindings_form.gen_control.value+"')]"))
            {   //is on this page so instance stuff applies
                hideComponent(document.getElementById('form_details_message_container'));
                showComponent(document.getElementById('form_details_xpaths_group'));

                var instance_type = 'none';
                var instance_type_elem = applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[fm:action = \'" + currentAction + "\']/fm:instance_type");
                if (instance_type_elem != null)
                    instance_type = getElementText(instance_type_elem);
                if (instance_type == 'static')
                {
                    document.getElementById('xpathBindings__instances__instance_bindingRequired').checked = true;
                    toggleInstanceBindingInfo(true, instance_type);
                }
                else if (instance_type == 'dynamic')
                {
                    document.getElementById('xpathBindings__instances__instance_bindingRequired').checked = true;
                    //setTimeout("validateXPathImpl('xpathBindings__instances__instance_xpath', true, true)", 50);
                    toggleInstanceBindingInfo(true, instance_type);
                }
                else
                {
                    document.getElementById('xpathBindings__instances__instance_bindingRequired').checked = false;
                    toggleInstanceBindingInfo(false);
                }

                //check maintain form data flag
                if (action_elem != null && action_elem.getAttribute('maintain_form_data') == 'true')
                    document.getElementById('xpathBindings__instances__keep_form_data').checked = true;
                else
                    document.getElementById('xpathBindings__instances__keep_form_data').checked = false;


                //check for additional data to maintain
                showKeepDataXPaths();

            }
            else
            {   //is on parent page, so instance stuff doesnt apply here
                document.getElementById('form_details_message_container').innerHTML = 'Please configure the instance binding information for this action against the page where this action originates.';
                showComponent(document.getElementById('form_details_message_container'));
                hideComponent(document.getElementById('form_details_xpaths_group'));
            }
        }
    }
}



/* saves the information currently in the text boxes to the DOM for the current group/element*/
function saveToDOM()
{
    findNamespaces();

    //alert('saving data..');
    var type = document.XPath_Bindings_form.currentType.value;
    var name = document.XPath_Bindings_form.currentElement.value;

    var success = true;

    //dont save action information for actions that are set by another action
    if ((currentTab != PAGE_TO_HANDLER_TAB) || (bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = \'" + currentAction + "\']/@base") == null))
    {
        if ((type == 'element') && (currentTab != ACTION_CONFIG_TAB))
        {
            saveElement(name);
        }
        else if ((type == 'group') && (currentTab != ACTION_CONFIG_TAB))
        {
            success = saveGroup(name);
        }
        else if ((type == 'repeat') && (currentTab != ACTION_CONFIG_TAB))
        {
            saveRepeat(name);
        }
        else if ((type == 'form') && (currentTab != ACTION_CONFIG_TAB))
        {
            savePage();
        }
        else // we are on the action config tab
        {
            success = saveInstanceBindingInfo();
        }

        checkCurrentFieldXPathsSpecified();
    }

    //update action and tab hidden fields to contain the current data
    document.XPath_Bindings_form.currentTab.value = currentTab;
    document.XPath_Bindings_form.currentAction.value = currentAction;

    //update the state document - should only really do this if the save has been triggerd by a page change action
    stateManager.setStructureTreeWidth(getCurrentStyle(document.getElementById('treeview_td'), 'width'));
    stateManager.setDocumentTreeWidth(getCurrentStyle(document.getElementById('instance_td'), 'width'));

    if (getCurrentStyle(document.getElementById('treeview_td'), 'display') == 'none')
        stateManager.setStructureTreeVisibility('false');
    else
        stateManager.setStructureTreeVisibility('true');

    if (getCurrentStyle(document.getElementById('instance_td'), 'display') == 'none')
        stateManager.setDocumentTreeVisibility('false');
    else
        stateManager.setDocumentTreeVisibility('true');

    document.XPath_Bindings_form.xml_fm_state.value = xmlToString(stateManager.getStateDocument());

    return success;
}




//----------------------------------------------------------------
//----------------------------------------------------------------
//----------------------------------------------------------------
/* NAMESPACE STUFF WAS HERE                                     */
//----------------------------------------------------------------
//----------------------------------------------------------------
//----------------------------------------------------------------
function showNamespaces()
{
    hyf.ns.showNamespaces();
}

function findNamespaces()
{
    var nsString = hyf.ns.findNamespaceString();

    var newString = '<xforms xmlns="http://www.hyfinity.com/formmaker" ' + nsString + '/>';

    //alert(newString);
    tempDoc = getXMLDocument(newString);
    Sarissa.setXpathNamespaces(tempDoc, FMqualifiedNS);

    var copyNodes = bindingsDoc.selectNodes("/fm:xforms/*");
    for (var i=0; i < copyNodes.length; ++i)
    {
        tempDoc.documentElement.appendChild(copyNodes.item(i).cloneNode(true));
    }

    bindingsDoc = tempDoc;
}

/**
 * define the function to be called when a namespace change occurrs.
 * This is needed so that we can update the xpaths if a prefix value is changed.
 */
hyf.ns.namespaceChangeFunction = function(operation, oldPrefix, oldUri, newPrefix, newUri)
{
    if ((operation == 'edit') && (oldPrefix != newPrefix))
    {
        //update all the XPaths
        saveToDOM();

        var bindingsString = xmlToString(bindingsDoc);
        bindingsString = bindingsString.replace(new RegExp(oldPrefix+":", "g"), newPrefix+":");
        bindingsDoc = getXMLDocument(bindingsString);

        showNewElement(document.XPath_Bindings_form.currentType.value, document.XPath_Bindings_form.currentElement.value, true);
    }
    if (tiDoc != null)
        Sarissa.setXpathNamespaces(tiDoc, hyf.ns.findNamespaceString());
    if (xiDoc != null)
        Sarissa.setXpathNamespaces(xiDoc, hyf.ns.findNamespaceString());
    setTimeout("updateInstanceTreeList('"+instanceTypeVisible+"');", 50);
    setChanged();
    return true;
}


//----End namespace stuf----------------------------


/** Saves the entries in the currently visible element xpaths to the dom for the given element.*/
function saveElement(elementName)
{
    var configElement = bindingsDoc.selectSingleNode("/fm:xforms/fm:element[@name = '"+elementName+"']");

    if (configElement == null)
    {
        //alert('need to create new config element');
        configElement = createNamespaceElement(bindingsDoc, 'element', FMdefaultNS);
        configElement.setAttribute("name", elementName);
        bindingsDoc.selectSingleNode("/fm:xforms").appendChild(configElement);
    }

    if (currentTab == HANDLER_TO_PAGE_TAB)
    {
        //save the binding_format attribute if needed
        if (document.getElementById('xpathBindings__element_xpaths__data_format'))
        {
            configElement.setAttribute('binding_format', document.getElementById('xpathBindings__element_xpaths__data_format').value)
        }


        var transform = document.getElementById('xpathBindings__element_xpaths__transform_xpath');
        var label = document.getElementById('xpathBindings__element_xpaths__label_xpath');
        var repeat = document.getElementById('xpathBindings__element_xpaths__repeat_xpath');
        var value = document.getElementById('xpathBindings__element_xpaths__value_xpath');
        var text = document.getElementById('xpathBindings__element_xpaths__text_xpath');
        var hide = document.getElementById('xpathBindings__element_xpaths__hide_xpath');
        var disable = document.getElementById('xpathBindings__element_xpaths__disable_xpath');
        var validate = document.getElementById('xpathBindings__element_xpaths__validate_xpath');
        var server_error = document.getElementById('xpathBindings__element_xpaths__server_message_xpath');
        var description = document.XPath_Bindings_form.xpathBindings__description;

        //remove all existing (non action) children
        //we leave the transform_xpaths around if we are not creating a new one.
        //This is because when changing field type we do not regenerate the xpaths
        //as this is relatively trickey, and instead rely on the xpaths still being
        //present in the underlying bindings file.
        numChildren = configElement.childNodes.length;
        for (var i = 0; i < numChildren; ++i)
        {
            var baseName = getBaseName(configElement.childNodes[i]);
            if ((baseName != 'action') && ((baseName != 'transform_xpath') || (transform != null)))
            {
                configElement.removeChild(configElement.childNodes[i]);
                --i;
                --numChildren;
            }
        }

        //recreate elements for all the XPaths on the screen

        if (transform != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'transform_xpath', FMdefaultNS);
            setElementText(newXPath, transform.value);
            configElement.appendChild(newXPath);
        }
        if (label != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'label_xpath', FMdefaultNS);
            setElementText(newXPath, label.value);
            configElement.appendChild(newXPath);
        }
        if (repeat != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'repeat_xpath', FMdefaultNS);
            setElementText(newXPath, repeat.value);
            configElement.appendChild(newXPath);
        }
        if (value != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'value_xpath', FMdefaultNS);
            setElementText(newXPath, value.value);
            configElement.appendChild(newXPath);
        }
        if (text != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'text_xpath', FMdefaultNS);
            setElementText(newXPath, text.value);
            configElement.appendChild(newXPath);
        }
        if (hide != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'hide_xpath', FMdefaultNS);
            setElementText(newXPath, hide.value);
            configElement.appendChild(newXPath);
        }
        if (disable != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'disable_xpath', FMdefaultNS);
            setElementText(newXPath, disable.value);
            configElement.appendChild(newXPath);
        }
        if (validate != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'validate_xpath', FMdefaultNS);
            setElementText(newXPath, valdiate.value);
            configElement.appendChild(newXPath);
        }
        if (server_error != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'server_message_xpath', FMdefaultNS);
            setElementText(newXPath, server_error.value);
            configElement.appendChild(newXPath);
        }
        if (description != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'description', FMdefaultNS);
            setElementText(newXPath, description.value);
            configElement.appendChild(newXPath);
        }

        var structureElem = configDoc.selectSingleNode("/fm:structure//fm:element[@name = '"+elementName+"' and not(ancestor::fm:path)]");
        //now handle the new format event xpaths
        var dynamicEvents = structureElem.selectNodes("fm:events/fm:event[fm:action/fm:param[@option = 'Dynamic']]");
        if (dynamicEvents.length > 0)
        {
            saveNewFormatEventXPaths(dynamicEvents, configElement, 'field');
        }
        dynamicEvents = structureElem.selectNodes("fm:field/fm:label/fm:events/fm:event[fm:action/fm:param[@option = 'Dynamic']]");
        if (dynamicEvents.length > 0)
        {
            saveNewFormatEventXPaths(dynamicEvents, configElement, 'label');
        }



        //need to save label styles_xpaths here
        var dynamicLabelStyles = structureElem.selectNodes("fm:field/fm:label[@options = 'dynamic']/fm:class");
        if (dynamicLabelStyles.length > 0)
        {
            saveDynamicStylesXPaths(dynamicLabelStyles, configElement, 'label');
        }
        dynamicLabelStyles = structureElem.selectNodes("fm:field/fm:label/fm:container[@options = 'dynamic']/fm:class");
        if (dynamicLabelStyles.length > 0)
        {
            saveDynamicStylesXPaths(dynamicLabelStyles, configElement, 'labelcontainer');
        }

        //need to save background styles_xpaths here
        var dynamicStyles = structureElem.selectNodes("fm:field/fm:style[@options = 'dynamic']/fm:class");

        if (dynamicStyles.length > 0)
        {
            saveDynamicStylesXPaths(dynamicStyles, configElement, 'background');
        }


        //need to save element styles_xpaths here
        var dynamicElementStyles = structureElem.selectNodes("fm:field/fm:element_style[@options = 'dynamic']/fm:class");
        if (dynamicElementStyles.length > 0)
        {
            saveDynamicStylesXPaths(dynamicElementStyles, configElement, 'element');
        }


        //need to save disabled styles_xpaths here
        var dynamicDisabledStyles = configDoc.selectNodes("/fm:structure//fm:element[(@name = '"+elementName+"') and (@disable = 'true')]/fm:field/fm:disabled_style[@options = 'dynamic']/fm:class");
        if (dynamicDisabledStyles.length > 0)
        {
            saveDynamicStylesXPaths(dynamicDisabledStyles, configElement, 'disabled');
        }
    }
    else
    {
        //have been editing the page to handler bindings for a particular action.
        //there could be multiple bindings, so need to loop through each.
        resetActionBindingElem(configElement, currentAction);
        var i=0;
        var textarea = document.getElementById('xpathBindings__element_xpaths__xform_xpath' + i);
        while (textarea != null)
        {
            saveActionBindingElem(configElement, currentAction, "xform_xpath", textarea.value);

            //check for a submission value xpath as well for the case where the field supports multiple values
            //and so defines abstract context and value xpath for submission.
            if (document.getElementById('xpathBindings__element_xpaths__submission_value_xpath' + i))
            {
                saveActionBindingElem(configElement, currentAction, "value_xpath", document.getElementById('xpathBindings__element_xpaths__submission_value_xpath' + i).value);
            }

            ++i;
            textarea = document.getElementById('xpathBindings__element_xpaths__xform_xpath' + i);
        }
    }


}

/**
 * removes all the binding elements relating to the given action
 * from the specified container element.
 */
function resetActionBindingElem(bindingElem, actionName)
{
    var actionElem = bindingElem.selectSingleNode("fm:action[@name = '"+actionName+"']");
    if (actionElem != null)
    {
        while (actionElem.firstChild != null)
            actionElem.removeChild(actionElem.firstChild);
    }
}

/**
 * Updates the binding file to add in teh specified action submission binding.
 */
function saveActionBindingElem(bindingElem, actionName, xpathName, xpathValue)
{
    //check if the action element already exists
    var actionElem = bindingElem.selectSingleNode("fm:action[@name = '"+actionName+"']");
    if (actionElem == null)
    {
        actionElem = createNamespaceElement(bindingsDoc, 'action', FMdefaultNS);
        actionElem.setAttribute('name', actionName);
        bindingElem.appendChild(actionElem);
    }

    var xformXPathElem = null;
    /*if (xpathName == "choice_fragment")
        xformXPathElem = actionElem.selectSingleNode("fm:" + xpathName);
    if (xformXPathElem == null)
    {*/
        xformXPathElem = createNamespaceElement(bindingsDoc, xpathName, FMdefaultNS);
        actionElem.appendChild(xformXPathElem);
    /*}*/

    if (xpathName == "choice_fragment")
        actionElem.replaceChild(xpathValue, xformXPathElem)
    else
        setElementText(xformXPathElem, xpathValue);

}

/**
 * saves the xpaths for the dynamic styles defined in the dynamicStyles NodeList as nodes under the bindingElement node.
 * @param dynamicStyles The NodeList of all relevant class elements from the structure file
 * @param bindingElement The element in the bindings file for this field
 * @param styleType 'label', 'background', 'element', or 'disabled'
 */
function saveDynamicStylesXPaths(dynamicStyles, bindingElement, styleType)
{

    for (var i = 0; i < dynamicStyles.length; ++i)
    {
        var trigger = dynamicStyles.item(i);
        var id = trigger.getAttribute('id');

        var newXPath = createNamespaceElement(bindingsDoc, 'dynamic_style_xpath', FMdefaultNS);

        newXPath.setAttribute('style_type', styleType);
        newXPath.setAttribute('style_id', id);
        setElementText(newXPath, eval('document.XPath_Bindings_form.xpathBindings__'+styleType+'_style_if_'+id).value);

        bindingElement.appendChild(newXPath);

        if (trigger.getAttribute('type') == 'dynamic')
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'dynamic_style_xpath_value', FMdefaultNS);

            newXPath.setAttribute('style_type', styleType);
            newXPath.setAttribute('style_id', id);
            setElementText(newXPath, eval('document.XPath_Bindings_form.xpathBindings__'+styleType+'_style_xpath_'+id).value);

            bindingElement.appendChild(newXPath);
        }
    }
}



/**
 * saves the xpaths for the events defined in the dynamicEvents NodeList as nodes under the bindingElement node.
 * @param dynamicEvents The NodeList of all relevant setvalue elements from the structure file
 * @param bindingElement The element in the binfings file for this field
 * @param eventSource 'field' or 'label'
 */
function saveEventXPaths(dynamicEvents, bindingElement, eventSource)
{
    for (var i = 0; i < dynamicEvents.length; ++i)
    {
        var trigger = dynamicEvents.item(i);
        var dynamicActions = trigger.selectNodes("fm:setvalue[@value_type = 'dynamic']");
        for (var j = 0; j < dynamicActions.length; ++j)
        {
            var action = dynamicActions.item(j);

            var newXPath = createNamespaceElement(bindingsDoc, 'setvalue_action_xpath', FMdefaultNS);

            newXPath.setAttribute('event_source', eventSource);
            newXPath.setAttribute('event_type', getBaseName(trigger));
            newXPath.setAttribute('target_field', action.getAttribute('target'));

            setElementText(newXPath, eval('document.XPath_Bindings_form.setvalue_action_xpath_' + eventSource + '_' + action.getAttribute('target')).value);

            bindingElement.appendChild(newXPath);
        }
    }
}


/**
 * saves the new format xpaths for the events defined in the dynamicEvents NodeList as nodes under the bindingElement node.
 * @param dynamicEvents The NodeList of all relevant event elements from the structure file
 * @param bindingElement The element in the bindings file for this field
 * @param eventSource 'field', 'label', 'group', 'grouplabel' or 'page'
 */
function saveNewFormatEventXPaths(dynamicEvents, bindingElement, eventSource)
{
    log.trace('saving new format event xpaths for ' + eventSource)
    for (var i = 0; i < dynamicEvents.length; ++i)
    {
        var trigger = dynamicEvents.item(i);
        var dynamicActions = trigger.selectNodes("fm:action[fm:param/@option = 'Dynamic']");
        for (var j = 0; j < dynamicActions.length; ++j)
        {
            var action = dynamicActions.item(j);
            var dynamicParams = action.selectNodes("fm:param[@option = 'Dynamic']");

            for (var k = 0; k < dynamicParams.length; ++k)
            {
                var param = dynamicParams.item(k);

                var newXPath = createNamespaceElement(bindingsDoc, 'event_xpath', FMdefaultNS);

                newXPath.setAttribute('event_source', eventSource);
                newXPath.setAttribute('event', trigger.getAttribute('id'));
                newXPath.setAttribute('action', action.getAttribute('id'));
                newXPath.setAttribute('param', param.getAttribute('name'));

                var aid = '' + getEventActionAdditionalIdentifier(action)
                if (aid != '')
                    newXPath.setAttribute('action_identifier', aid);

                setElementText(newXPath, document.getElementById(getEventActionFieldId(eventSource, trigger, action, param)).value);

                bindingElement.appendChild(newXPath);
            }
        }
    }
}

/**
 * saves all the information currently displayed on screen for the given group
 * to the XML DOM.
 * @return A boolean indicating whether the save was successful or not.
 */
function saveGroup(groupName)
{
    var configElement = bindingsDoc.selectSingleNode("/fm:xforms/fm:group[@name = '"+groupName+"']");
    if (configElement == null)
    {
        //alert('need to create new config element');
        configElement = createNamespaceElement(bindingsDoc, 'group', FMdefaultNS);
        configElement.setAttribute("name", groupName);
        bindingsDoc.selectSingleNode("/fm:xforms").appendChild(configElement);
    }

    if (currentTab == HANDLER_TO_PAGE_TAB)
    {
        var hide = document.getElementById('xpathBindings__group_xpaths__hide_xpath');
        var disable = document.getElementById('xpathBindings__group_xpaths__disable_xpath');
        var label = document.getElementById('xpathBindings__group_xpaths__label_xpath');

        //remove all existing (non action) children
        numChildren = configElement.childNodes.length;
        for (var i = 0; i < numChildren; ++i)
        {
            if (getBaseName(configElement.childNodes[i]) != 'action')
            {
                configElement.removeChild(configElement.childNodes[i]);
                --i;
                --numChildren;
            }
        }

        if (hide != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'hide_xpath', FMdefaultNS);
            setElementText(newXPath, hide.value);
            configElement.appendChild(newXPath);
        }
        if (disable != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'disable_xpath', FMdefaultNS);
            setElementText(newXPath, disable.value);
            configElement.appendChild(newXPath);
        }
        if (label != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'label_xpath', FMdefaultNS);
            setElementText(newXPath, label.value);
            configElement.appendChild(newXPath);
        }

        var structureElem = configDoc.selectSingleNode("/fm:structure//fm:group[@name = '"+groupName+"' and not(ancestor::fm:path)]");
        //now handle the new format event xpaths
        var dynamicEvents = structureElem.selectNodes("fm:events/fm:event[fm:action/fm:param[@option = 'Dynamic']]");
        if (dynamicEvents.length > 0)
        {
            saveNewFormatEventXPaths(dynamicEvents, configElement, 'group');
        }
        dynamicEvents = structureElem.selectNodes("fm:field/fm:label/fm:events/fm:event[fm:action/fm:param[@option = 'Dynamic']]");
        if (dynamicEvents.length > 0)
        {
            saveNewFormatEventXPaths(dynamicEvents, configElement, 'grouplabel');
        }


        //need to save label styles_xpaths here
        var dynamicLabelStyles = structureElem.selectNodes("fm:label[@options = 'dynamic']/fm:class");
        if (dynamicLabelStyles.length > 0)
        {
            saveDynamicStylesXPaths(dynamicLabelStyles, configElement, 'grouplabel');
        }
        //need to save label container styles_xpaths here
        dynamicLabelStyles = structureElem.selectNodes("fm:label/fm:container[@options = 'dynamic']/fm:class");
        if (dynamicLabelStyles.length > 0)
        {
            saveDynamicStylesXPaths(dynamicLabelStyles, configElement, 'grouplabelcontainer');
        }

        //need to save background styles_xpaths here
        var dynamicStyles = structureElem.selectNodes("fm:style[@options = 'dynamic']/fm:class");
        if (dynamicStyles.length > 0)
        {
            saveDynamicStylesXPaths(dynamicStyles, configElement, 'group');
        }

    }
    else
    {
        resetActionBindingElem(configElement, currentAction);

        var xform = document.getElementById('xpathBindings__group_xpaths__xform_xpath');
        var fragment = '';
        //check whether to get the fragment content from the editarea
        if (editAreas['xpathBindings__group_xpaths__choice_fragment'])
            fragment = editAreaLoader.getValue('xpathBindings__group_xpaths__choice_fragment');
        else if (document.getElementById('xpathBindings__group_xpaths__choice_fragment') != null)
            fragment = document.XPath_Bindings_form.xpathBindings__group_xpaths__choice_fragment.value;

        if (xform != null)
        {
            saveActionBindingElem(configElement, currentAction, "xform_xpath", xform.value);
        }

        //save choice XML fragment
        if (fragment != '')
        {
            var tempDoc = getXMLDocument('<fm:choice_fragment xmlns:fm="' + FMdefaultNS + '">' + fragment + '</fm:choice_fragment>');
            if (Sarissa.getParseErrorText(tempDoc) == Sarissa.PARSED_OK)
            {
                var imported = tempDoc.documentElement;
                if (typeof(bindingsDoc.importNode) == 'function')
                    imported = bindingsDoc.importNode(imported, true);
                saveActionBindingElem(configElement, currentAction, "choice_fragment", imported);
            }
            else
            {
                alert('Please correct the error with the XML fragment before continuing.');
                return false;
            }
        }
    }


    return true;
}


function saveRepeat(repeatName)
{

    var configElement = bindingsDoc.selectSingleNode("/fm:xforms/fm:repeat[@name = '"+repeatName+"']");
    if (configElement == null)
    {
        //alert('need to create new config element');
        configElement = createNamespaceElement(bindingsDoc, 'repeat', FMdefaultNS);
        configElement.setAttribute("name", repeatName);
        bindingsDoc.selectSingleNode("/fm:xforms").appendChild(configElement);
    }


    if (currentTab == HANDLER_TO_PAGE_TAB)
    {
        var repeat = document.getElementById('xpathBindings__repeat_xpaths__repeat_xpath');
        var sort = document.getElementById('xpathBindings__repeat_xpaths__sort_xpath');
        var sort_order = document.getElementById('xpathBindings__repeat_xpaths__sort_order_xpath');
        var sort_type = document.getElementById('xpathBindings__repeat_xpaths__sort_data_type_xpath');
        var row_style = document.getElementById('xpathBindings__repeat_xpaths__row_style_change_xpath');
        var column_style = document.getElementById('xpathBindings__repeat_xpaths__column_style_change_xpath');

        //remove all existing (non action) children
        numChildren = configElement.childNodes.length;
        for (var i = 0; i < numChildren; ++i)
        {
            if (getBaseName(configElement.childNodes[i]) != 'action')
            {
                configElement.removeChild(configElement.childNodes[i]);
                --i;
                --numChildren;
            }
        }

        if (repeat != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'repeat_xpath', FMdefaultNS);
            setElementText(newXPath, repeat.value);
            configElement.appendChild(newXPath);
        }
        if (sort != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'sort_xpath', FMdefaultNS);
            setElementText(newXPath, sort.value);
            configElement.appendChild(newXPath);
        }
        if (sort_order != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'sort_order_xpath', FMdefaultNS);
            setElementText(newXPath, sort_order.value);
            configElement.appendChild(newXPath);
        }
        if (sort_type != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'sort_data-type_xpath', FMdefaultNS);
            setElementText(newXPath, sort_type.value);
            configElement.appendChild(newXPath);
        }
        if (row_style != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'row_style_change_xpath', FMdefaultNS);
            setElementText(newXPath, row_style.value);
            configElement.appendChild(newXPath);
        }
        if (column_style != null)
        {
            var newXPath = createNamespaceElement(bindingsDoc, 'column_style_change_xpath', FMdefaultNS);
            setElementText(newXPath, column_style.value);
            configElement.appendChild(newXPath);
        }

    }
    else
    {
        resetActionBindingElem(configElement, currentAction);
        var xform = document.XPath_Bindings_form.xpathBindings__repeat_xpaths__xform_xpath;
        if (xform)
            saveActionBindingElem(configElement, currentAction, "xform_xpath", xform.value);
    }
}


function savePage()
{
    log.debug('saving page level details');
    var configElement = bindingsDoc.selectSingleNode("/fm:xforms/fm:page");
    if (configElement == null)
    {
        configElement = createNamespaceElement(bindingsDoc, 'page', FMdefaultNS);
        bindingsDoc.selectSingleNode("/fm:xforms").appendChild(configElement);
    }
    if (currentTab == HANDLER_TO_PAGE_TAB)
    {
        //remove all existing (non action) children
        numChildren = configElement.childNodes.length;
        for (var i = 0; i < numChildren; ++i)
        {
            if (getBaseName(configElement.childNodes[i]) != 'action')
            {
                configElement.removeChild(configElement.childNodes[i]);
                --i;
                --numChildren;
            }
        }

        //recreate elements for all the XPaths on the screen

        //handle the new format page event xpaths
        var dynamicEvents = configDoc.selectNodes("/fm:structure/fm:form_details/fm:page_events/fm:events/fm:event[fm:action/fm:param[@option = 'Dynamic']]");
        if (dynamicEvents.length > 0)
        {
            saveNewFormatEventXPaths(dynamicEvents, configElement, 'page');
        }

        //now handle the new format external event xpaths
        dynamicEvents = configDoc.selectNodes("/fm:structure/fm:form_details/fm:external_events/fm:events/fm:event[fm:action/fm:param[@option = 'Dynamic']]");
        if (dynamicEvents.length > 0)
        {
            saveNewFormatEventXPaths(dynamicEvents, configElement, 'external');
        }
    }
}

/* save any changes made to the current text boxes
   update all their values to show the new group/element/repeat
   then set which should be shown or hidden using showRelevantXPaths function*/
function showNewElement(type, name, dontSave)
{
    log.trace('in showNewElement for ' + type + ' ' + name);

    //update the current info
    if ((typeof(dontSave) == 'undefined') || !dontSave)
    {
        var success = saveToDOM();
        if (!success)
            return;
    }

    document.XPath_Bindings_form.currentType.value = type;
    document.XPath_Bindings_form.currentElement.value = name;

    //MOST OF THE PROCESSING THAT WAS IN THIS METHOD NOW OCCURS IN showRelevantXpaths()!!!
    //This should make it cleaner, as all the processing to determine what xpaths to show
    //and what values to use is now all in one place!


    // update the values in the text boxes for all the xpaths specified for this new element
    if ((type == 'element') && (currentTab != ACTION_CONFIG_TAB))
    {
        //update the description
        document.XPath_Bindings_form.xpathBindings__description.value = '';
        var configElement = bindingsDoc.selectSingleNode("/fm:xforms/fm:element[@name = '"+name+"']");
        if ((configElement != null) && (configElement.selectSingleNode("fm:description") != null))
        {
            document.XPath_Bindings_form.xpathBindings__description.value = getElementText(configElement.selectSingleNode("fm:description"));
        }
    }

    if (currentTab != ACTION_CONFIG_TAB)
    {
        updateDisplayedInstanceDocument();
    }

    //work out which xpaths should be shown
    showRelevantXpaths();

    if (currentTab == ACTION_CONFIG_TAB)
    {
        //for the other types, this call has happened earlier (in the ifs above)
        //but for the action config we need to wait until after the showRelevantXPaths call
        //before doing this. (For the others it needs to be earlier to resest the document loaded flag)
        updateDisplayedInstanceDocument();
    }

    if (currentTab == ACTION_CONFIG_TAB)
        markActionOnTreeview(currentAction);
    else
        markTreeView(type, name);

    stateManager.displayOpenSections();
}


/*
 * Highlights the given entry in the page structure treeview.
 */
function markTreeView(type, name)
{
    var links = document.getElementById('PageTreeList').getElementsByTagName("a");
    for ( var i = 0; i< links.length; ++i)
    {
        if (links[i].id == type+'_'+name)
        {
            if (type == 'repeat' || type == 'group')
                dojo.addClass(links[i].parentNode.parentNode, "selectedTreeViewEntryExistingBorder");
            else
                dojo.addClass(links[i], "selectedTreeViewEntry");
        }
        else
        {
            if (dojo.hasClass(links[i],"selectedTreeViewEntry"))
                dojo.removeClass(links[i],"selectedTreeViewEntry");
            if (dojo.hasClass(links[i].parentNode.parentNode,"selectedTreeViewEntryExistingBorder"))
                dojo.removeClass(links[i].parentNode.parentNode,"selectedTreeViewEntryExistingBorder");
        }
    }
}

/*
 * Highights the given action in the action list treeview.
 */
function markActionOnTreeview(actionName)
{
    var links = document.getElementById('ActionTreeList').getElementsByTagName("a");
    for ( var i = 0; i< links.length; ++i)
    {
        if (links[i].id == 'action_treeview_entry_' + actionName)
        {
            links[i].className = "selectedTreeViewEntry";
        }
        else
        {
            links[i].className = "unselectedTreeViewEntry";
        }
    }
}


/*******************************************************************
 * code to do the drag and drop of the xpaths
 *******************************************************************/

var moveMode = 'stop';
var moveValue = '';
function mouseMove(e, base)
{
    if (moveMode == 'move')
    {
        var coords = getMouseCoords(e, base);
        document.getElementById('xpathBox').style.top = coords.y + 'px';
        document.getElementById('xpathBox').style.left = (Number(coords.x) + Number(10)) + 'px'
    }
    dividerMove(e, base);
}
function mouseDown(value, e)
{
    var evt = (window.event) ? window.event : e;
    moveMode = 'move';
    moveValue = value;
    var coords = getMouseCoords(evt);
    document.getElementById('xpathBox').style.visibility = 'visible';
    document.getElementById('xpathBox').style.background = 'yellow';
    document.getElementById('xpathBox').style.color = 'black';
    document.getElementById('xpathBox').style.top = coords.y + 'px';
    document.getElementById('xpathBox').style.left = (Number(coords.x) + Number(10)) + 'px'
    if (value.length > 30)
        document.getElementById('xpathBox').innerHTML = value.substr(0, 30)+'...';
    else
        document.getElementById('xpathBox').innerHTML = value;

    // cancel out any text selections
    document.body.focus();
    // prevent text selection in IE
    document.onselectstart = function () { return false; };
    // prevent IE from trying to drag an image
    document.getElementById('xpathBox').ondragstart = function() { return false; };
    // prevent text selection (except IE)
    return false;

}
function mouseUpCancel(e, base)
{
    //alert('mouse up cancel');
    if (moveMode == 'move')
    {
        moveMode = 'stop';
        document.getElementById('xpathBox').style.visibility = 'hidden';
        document.onselectstart = null;
        document.getElementById('xpathBox').ondragstart = null;
    }

    dividerReleased(e, base);
}

function mouseUpSave(box, alternate)
{
    if (moveMode == 'move')
    {
        moveMode = 'stop';
        document.getElementById('xpathBox').style.visibility = 'hidden';
        document.onselectstart = null;
        document.getElementById('xpathBox').ondragstart = null;

        //handle the special case where the selected textbox is on the page details or action configuration screens
        //and so do not need to check for repeats etc
        if ((document.XPath_Bindings_form.currentType.value == 'form') || (currentTab == ACTION_CONFIG_TAB))
        {
            box.value = moveValue;
            if (typeof(box.onchange) == 'function')
                box.onchange();
        }
        //handle the special case where it is the choice fragment box for a group, and so we need
        //to insert an XML fragement, not an xpath
        else if (box.name == 'xpathBindings__group_xpaths__choice_fragment')
        {
            var valueToSet = ''
            //check which instance is currently shown
            if (instanceTypeVisible == 'transform')
            {
                var selectedNode = tiDoc.selectSingleNode(moveValue);
                valueToSet = xmlToString(selectedNode);
            }
            else
            {
                var selectedNode = xiDoc.selectSingleNode(moveValue);
                valueToSet = xmlToString(selectedNode);
            }
            if (editAreas['xpathBindings__group_xpaths__choice_fragment'])
            {
                editAreaLoader.setValue('xpathBindings__group_xpaths__choice_fragment', valueToSet);
                //NOTE: The editarea automatically calls the change function we specified when setting it up
            }
            else
            {
                box.value = valueToSet;
                if (typeof(box.onchange) == 'function')
                    box.onchange();
            }
        }
        else
        {
            //find the structure config element for this text box
            var configElement = configDoc.selectSingleNode("/fm:structure//*[(local-name() = '"+document.XPath_Bindings_form.currentType.value+"') and (@name = '"+document.XPath_Bindings_form.currentElement.value+"') and not(ancestor::fm:path)]");
            if (configElement != null)
            {
                //store the repeat xpaths for this element
                var repeatXPath = "";
                var repeatXformXPath = "";

                //check if there are any repeats as ancestors of this entry
                var repeats = configElement.selectNodes("ancestor::fm:repeat");
                if (repeats.length != 0)
                {
                    //find the repeat xpaths
                    for(var i = 0; i<repeats.length; ++i)
                    {
                        var repeat = repeats.item(i);
                        var rxp = bindingsDoc.selectSingleNode("/fm:xforms/fm:repeat[@name = '"+repeat.getAttribute("name")+"']/fm:repeat_xpath");
                        var rxformxp = bindingsDoc.selectSingleNode("/fm:xforms/fm:repeat[@name = '"+repeat.getAttribute("name")+"']/fm:action[@name ='"+currentAction+"']/fm:xform_xpath");
                        if (rxp != null)
                        {
                            var rxpText = getElementText(rxp);
                            if (rxpText.indexOf("/") == 0)
                            {
                                repeatXPath = rxpText;
                            }
                            else
                            {
                                repeatXPath += "/" + rxpText;
                            }
                        }
                        if (rxformxp != null)
                        {
                            var rxformxpText = getElementText(rxformxp);
                            if (rxformxpText.indexOf("/") == 0)
                            {
                                if ((rxformxpText.indexOf(EFORM_WRAPPER_XPATH) != 0) &&
                                    (rxformxpText.indexOf("/") == 0))
                                {
                                    rxformxpText = EFORM_WRAPPER_XPATH + rxformxpText;
                                }

                                repeatXformXPath = rxformxpText;
                            }
                            else
                            {
                                repeatXformXPath += "/" + rxformxpText;
                            }
                        }
                    }
                }

                //alert(box.name);

                var context="";
                if (document.XPath_Bindings_form.currentType.value == 'group')
                {
                    if (box.name.indexOf("xform_xpath") != -1)
                        context = repeatXformXPath;
                    else
                        context = repeatXPath;
                }
                else if (document.XPath_Bindings_form.currentType.value == 'repeat')
                {
                    if (box.name.indexOf("xform_xpath") != -1)
                    {
                        context = repeatXformXPath;
                    }
                    else if (box.name.indexOf("sort_xpath") != -1)
                    { //the sort xpath is relative to the context of the repeat location for this repeat
                        var repeatLoc = document.XPath_Bindings_form.xpathBindings__repeat_xpaths__repeat_xpath.value;
                        if (repeatLoc.indexOf("/") == 0)
                            context = repeatLoc;
                        else
                            context = repeatXPath + "/" + repeatLoc
                    }
                    else
                        context = repeatXPath;
                }
                else if (document.XPath_Bindings_form.currentType.value == 'element')
                {
                    if (box.name.indexOf("xform_xpath") != -1)
                    {
                        context = repeatXformXPath;
                    }
                    else if (box.name.indexOf("submission_value_xpath") != -1)
                    {
                        //This is the special case of a field that supports multiple values (eg multicheck) which is
                        //set to use xml structure as the binding format, and so needs both a repeating context
                        //and a value xpath.
                        //The context for this value xpath should be set by the xform_xpath one

                        //each action can have multiple submission bindings, so need to make sure we link to the correct one
                        var bindingNumber = box.name.substring(box.name.indexOf("submission_value_xpath") + 22);
                        var repeatLoc = document.getElementById('xpathBindings__element_xpaths__xform_xpath' + bindingNumber).value;
                        if (repeatLoc.indexOf("/") == 0)
                            context = repeatLoc;
                        else
                            context = repeatXformXPath + "/" + repeatLoc
                    }
                    else if ((box.name.indexOf("text_xpath") != -1) || (box.name.indexOf("value_xpath") != -1))
                    { //the text and value xpaths are relative to the repeat xpath for the controls showing a repeating list of options
                        var elementType = getElementText(configElement.selectSingleNode("fm:field/fm:input"));
                        if ((elementType == 'select') ||
                            (elementType == 'radio') ||
                            (elementType == 'combobox') ||
                            (elementType == 'multipleCheckbox'))
                        {
                            var repeatLoc = document.XPath_Bindings_form.xpathBindings__element_xpaths__repeat_xpath.value;
                            if (repeatLoc.indexOf("/") == 0)
                                context = repeatLoc;
                            else
                                context = repeatXPath + "/" + repeatLoc
                        }
                        else
                            context = repeatXPath;
                    }
                    else
                        context = repeatXPath;
                }

                //check if the new xpath starts with the context
                if ((context != '') && (moveValue.indexOf(context) == 0))
                {
                    if (context == moveValue)
                        moveValue = '.';
                    else
                        moveValue = moveValue.substr(context.length+1);
                }

                box.value = moveValue;
                if (typeof(box.onchange) == 'function')
                    box.onchange();

                //check if the alternate text box should be populated as well
                if (alternate != undefined)
                {
                    var emptyInstance = 'false';
                    if ((transformInstance.document == "") || (xformsInstance.document == ""))
                        emptyInstance = 'true';

                    if (((alternate.value == '') || (alternate.value == blank_xpath_text)) && ((transformInstance.document == xformsInstance.document) || (emptyInstance == 'true')))
                    {
                        alternate.value = moveValue;
                        if (typeof(alternate.onchange) == 'function')
                            alternate.onchange();
                    }
                }
            }
        }
    }
}


/*******************************************************************
 * End code to do the drag and drop of the xpaths
 *******************************************************************/



/* resets all groups/elements to how they were initially (when the page was loaded) */
function cancelAll()
{
    if (confirm('This will reset the details for all fields, groups and repeats to their initial values upon loading the page.\nDo you wish to continue?'))
    {
        //relaod the pageConfig from the data island
        configDoc = getXMLIsland('pageConfig');
        Sarissa.setXpathNamespaces(configDoc, FMqualifiedNS);
        bindingsDoc = getXMLIsland('hyfinityBindings');
        Sarissa.setXpathNamespaces(bindingsDoc, FMqualifiedNS);
        applicationDoc = getXMLIsland('application');
        Sarissa.setXpathNamespaces(applicationDoc, FMqualifiedNS);

        showNewElement(document.XPath_Bindings_form.currentType.value, document.XPath_Bindings_form.currentElement.value, true);

        hyf.ns.namespaces = new Object(),
        hyf.ns.addNamespacesFromDocument(bindingsDoc, true, true);
        showNamespaces();

        clearChanged();
    }
}

/* resets the current group/element to how it was initially (when the page was loaded) */
function cancel()
{
    var displayType = document.XPath_Bindings_form.currentType.value;
    if (displayType == 'element')
        displayType = 'field';


    if (confirm('This will reset the details for this ' + displayType + ' to their initial values upon loading the page.\nDo you wish to continue?'))
    {
        //reset the structure section
        configDoc = getXMLIsland('pageConfig');
        Sarissa.setXpathNamespaces(configDoc, FMqualifiedNS);

        //load the initial bindings file from the data island into a temporary DOM document
        tempDoc = getXMLIsland('hyfinityBindings');
        Sarissa.setXpathNamespaces(tempDoc, FMqualifiedNS);

        if (currentTab == ACTION_CONFIG_TAB)
        {
            var oldInstanceConfig = tempDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = '"+currentAction+"']");
            if (oldInstanceConfig != null)
            {
                var imported = oldInstanceConfig;
                if (typeof(bindingsDoc.importNode) == 'function')
                    imported = bindingsDoc.importNode(imported, true);
                var newInstanceConfig = bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = '"+currentAction+"']");
                newInstanceConfig.parentNode.replaceChild(imported, newInstanceConfig);
            }
        }
        else if (document.XPath_Bindings_form.currentType.value == 'element')
        {
            var oldElement = tempDoc.selectSingleNode("/fm:xforms/fm:element[@name = '"+document.XPath_Bindings_form.currentElement.value+"']");
            var currentElement = bindingsDoc.selectSingleNode("/fm:xforms/fm:element[@name = '"+document.XPath_Bindings_form.currentElement.value+"']");

            if (oldElement != null)
            {
                var imported = oldElement;
                if (typeof(bindingsDoc.importNode) == 'function')
                    imported = bindingsDoc.importNode(imported, true);

                //replace the current element with the old one
                currentElement.parentNode.replaceChild(imported, currentElement);
            }
            else
            {
                if (currentElement != null)
                {
                    //remove current element
                    currentElement.parentNode.removeChild(currentElement);
                }
            }
        }
        else if (document.XPath_Bindings_form.currentType.value == 'group')
        {
            var oldGroup = tempDoc.selectSingleNode("/fm:xforms/fm:group[@name = '"+document.XPath_Bindings_form.currentElement.value+"']");
            var currentGroup = bindingsDoc.selectSingleNode("/fm:xforms/fm:group[@name = '"+document.XPath_Bindings_form.currentElement.value+"']");

            if (oldGroup != null)
            {
                var imported = oldGroup;
                if (typeof(bindingsDoc.importNode) == 'function')
                    imported = bindingsDoc.importNode(imported, true);

                //replace the current group with the old one
                currentGroup.parentNode.replaceChild(imported, currentGroup);
            }
            else
            {
                if (currentGroup != null)
                {
                    //remove current group
                    currentGroup.parentNode.removeChild(currentGroup);
                }
            }
        }
        else if (document.XPath_Bindings_form.currentType.value == 'repeat') //is repeat
        {
            var oldRepeat = tempDoc.selectSingleNode("/fm:xforms/fm:repeat[@name = '"+document.XPath_Bindings_form.currentElement.value+"']");
            var currentRepeat = bindingsDoc.selectSingleNode("/fm:xforms/fm:repeat[@name = '"+document.XPath_Bindings_form.currentElement.value+"']");

            if (oldRepeat != null)
            {
                var imported = oldRepeat;
                if (typeof(bindingsDoc.importNode) == 'function')
                    imported = bindingsDoc.importNode(imported, true);

                //replace the current repeat with the old one
                currentRepeat.parentNode.replaceChild(imported, currentRepeat);
            }
            else
            {
                if (currentRepeat != null)
                {
                    //remove current repeat
                    currentRepeat.parentNode.removeChild(currentRepeat);
                }
            }
        }

        showNewElement(document.XPath_Bindings_form.currentType.value, document.XPath_Bindings_form.currentElement.value, true);
    }
}

/*Submits the form to send the new XML content back to the server to be saved*/
function save()
{
    //add the changes to the current page to the DOM
    saveToDOM();

    //output the XML in memory to a hidden field
    document.XPath_Bindings_form.xml_config_xforms.value = xmlToString(bindingsDoc);

    //alert('will submit data back to the server now!!!');

    //populate the hidden fields
    document.XPath_Bindings_form.presentation_xsl.value="xpath_bindings_html.xsl";

    resetOperationsList();
    addOperation("save_xpath_bindings_details");
    checkAppMapSaveRequired();
    addOperation("load_form_structure");
    addOperation("load_bindings_file");
    addOperation("load_form_map");

    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);

    //submit the form
    document.XPath_Bindings_form.submit();

}

/**
 * Resets the list of operations that will be performed on the next submit.
 * This should always be called before you start defining the required operations.
 */
function resetOperationsList()
{
    var i = 1;
    var opField = document.XPath_Bindings_form.operation1;

    while (opField != null)
    {
        opField.value = '';
        i++;
        opField = eval('document.XPath_Bindings_form.operation' + i);
    }
}

/**
 * Adds the given operation to the list of operations to be performed when the page is submitted.
 */
function addOperation(opName)
{
    var i = 1;
    var opField = document.XPath_Bindings_form.operation1;

    while (opField != null && opField.value != '')
    {
        i++;
        opField = eval('document.XPath_Bindings_form.operation' + i);
    }

    if (opField == null)
    {
        if (dojo.isIE < 8)
            opField = document.createElement('<input name="operation'+i+'"/>');
        else
            opField = document.createElement('input');
        opField.type = 'hidden';
        opField.name = 'operation' + i;
        document.XPath_Bindings_form.appendChild(opField);
    }

    opField.value = opName;
}
/**
 * Checks if the application file has been modified, and if so sets the correct hidden fields so the
 * details will be saved.
 */
function checkAppMapSaveRequired()
{
    if (applicationMapChanged)
    {
        document.XPath_Bindings_form.xml_formMap.value = xmlToString(applicationDoc);
        addOperation("save_form_map");
    }
}


function pageSelectionChanged(select)
{
    //add the changes to the current page to the DOM
    saveToDOM();

    //output the XML in memory to a hidden field
    document.XPath_Bindings_form.xml_config_xforms.value = xmlToString(bindingsDoc);

    document.XPath_Bindings_form.new_display_page.value = select.value;

    //populate the hidden fields
    document.XPath_Bindings_form.presentation_xsl.value="xpath_bindings_html.xsl";

    resetOperationsList();
    addOperation("save_xpath_bindings_details");
    checkAppMapSaveRequired();
    addOperation("load_form_map");
    addOperation("switch_page");
    addOperation("load_form_structure");
    addOperation("load_bindings_file");

    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);

    //submit the form
    document.XPath_Bindings_form.submit();

}

/* Submits the form so that the changes are made, and then loads the details to
 * show the Form Map screen*/
function showFormMap()
{
    //add the changes to the current page to the DOM
    saveToDOM();

    //output the XML in memory to a hidden field
    document.XPath_Bindings_form.xml_config_xforms.value = xmlToString(bindingsDoc);

    //populate the hidden fields
    document.XPath_Bindings_form.presentation_xsl.value="form_map_html.xsl";
    resetOperationsList();
    addOperation("save_xpath_bindings_details");
    checkAppMapSaveRequired();
    addOperation("load_form_map");

    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);

    //submit the form
    document.XPath_Bindings_form.submit();
}

/* Submits the form so that the changes are made, and then loads the details to
 * show the Field Details screen*/
function showFieldDetails()
{
    //add the changes to the current page to the DOM
    saveToDOM();

    //output the XML in memory to a hidden field
    document.XPath_Bindings_form.xml_config_xforms.value = xmlToString(bindingsDoc);

    //populate the hidden fields
    document.XPath_Bindings_form.presentation_xsl.value="field_details_html.xsl";
    resetOperationsList();
    addOperation("save_xpath_bindings_details");
    checkAppMapSaveRequired();
    addOperation("load_form_structure");
    addOperation("load_form_map");

    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);

    //submit the form
    document.XPath_Bindings_form.submit();
}

/* Submits the form so that the changes are made, and then loads the details to
 * show the Form Structure screen*/
function showFormStructure()
{
    //add the changes to the current page to the DOM
    saveToDOM();

    //output the XML in memory to a hidden field
    document.XPath_Bindings_form.xml_config_xforms.value = xmlToString(bindingsDoc);

    //populate the hidden fields
    document.XPath_Bindings_form.presentation_xsl.value="form_structure_html.xsl";
    resetOperationsList();
    addOperation("save_xpath_bindings_details");
    checkAppMapSaveRequired();
    addOperation("load_form_structure");
    addOperation("load_bindings_file");
    addOperation("load_form_map");

    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);

    //submit the form
    document.XPath_Bindings_form.submit();
}

/*
 *function to toggle the description field
*/
function toggleDescription()
{
    stateManager.toggleSection('xpathBindings__description');
}

function toggleNamespaces()
{
    stateManager.toggleSection('xpathBindings__namespace');
}

var messageMode = 'hidden';
function toggleMessage()
{
    if (messageMode == 'visible')
        messageMode = 'hidden';
    else
        messageMode = 'visible';

    setMessageDisplay(messageMode);
}

function setMessageDisplay(visible)
{
    if (visible == 'visible')
    {
        showComponent(document.getElementById("message_container"));
        document.getElementById('xpathBindings__message_button').className = 'collapsible_open';
    }
    else
    {
        hideComponent(document.getElementById("message_container"));
        document.getElementById('xpathBindings__message_button').className = 'collapsible_hidden';
    }
}


var messageRepeatMode = 'hidden';
function toggleMessageRepeat()
{
    if (messageRepeatMode == 'visible')
        messageRepeatMode = 'hidden';
    else
        messageRepeatMode = 'visible';

    setMessageRepeatDisplay(messageRepeatMode);
}
function setMessageRepeatDisplay(visible)
{
    if (visible == 'visible')
    {
        showComponent(document.getElementById("message_repeat_container"));
        document.getElementById('xpathBindings__message_repeat_button').className = 'collapsible_open';
    }
    else
    {
        hideComponent(document.getElementById("message_repeat_container"));
        document.getElementById('xpathBindings__message_repeat_button').className = 'collapsible_hidden';
    }
}

var messageGroupMode = 'hidden';
function toggleMessageGroup()
{
    if (messageGroupMode == 'visible')
        messageGroupMode = 'hidden';
    else
        messageGroupMode = 'visible';

    setMessageGroupDisplay(messageGroupMode);
}

function setMessageGroupDisplay(visible)
{
    if (visible == 'visible')
    {
        showComponent(document.getElementById("message_group_container"));
        document.getElementById('xpathBindings__message_group_button').className = 'collapsible_open';
    }
    else
    {
        hideComponent(document.getElementById("message_group_container"));
        document.getElementById('xpathBindings__message_group_button').className = 'collapsible_hidden';
    }
}

var messagePageMode = 'hidden';
function toggleMessagePage()
{
    if (messagePageMode == 'visible')
        messagePageMode = 'hidden';
    else
        messagePageMode = 'visible';

    setMessagePageDisplay(messagePageMode);
}

function setMessagePageDisplay(visible)
{
    if (visible == 'visible')
    {
        showComponent(document.getElementById("message_page_container"));
        document.getElementById("xpathBindings__message_page_button").className = 'collapsible_open';
    }
    else
    {
        hideComponent(document.getElementById("message_page_container"));
        document.getElementById("xpathBindings__message_page_button").className = 'collapsible_hidden';
    }
}

/*
 * Saves the current details and calls to the server to generate the current page
 */
function generatePage()
{
    //add the changes to the current page to the DOM
    saveToDOM();

    //output the XML in memory to a hidden field
    document.XPath_Bindings_form.xml_config_xforms.value = xmlToString(bindingsDoc);

    //populate the hidden fields
    document.XPath_Bindings_form.presentation_xsl.value="xpath_bindings_html.xsl";
    resetOperationsList();
    addOperation("save_xpath_bindings_details");
    checkAppMapSaveRequired();
    addOperation("generate_output");
    addOperation("load_form_structure");
    addOperation("load_bindings_file");
    addOperation("load_form_map");

    showStatusMessage("Generation in process...");

    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);

    //submit the form
    document.XPath_Bindings_form.submit();
}

function generateApp()
{
    //add the changes to the current page to the DOM
    saveToDOM();

    //output the XML in memory to a hidden field
    document.XPath_Bindings_form.xml_config_xforms.value = xmlToString(bindingsDoc);

    //populate the hidden fields
    document.XPath_Bindings_form.presentation_xsl.value="xpath_bindings_html.xsl";
    resetOperationsList();
    addOperation("save_xpath_bindings_details");
    checkAppMapSaveRequired();

    document.XPath_Bindings_form.new_display_page.value = document.XPath_Bindings_form.gen_control.value;
    //document.XPath_Bindings_form.gen_control.value = 'all';
    addOperation("switch_gen_control_to_all");

    addOperation("generate_output");
    addOperation('generate_handlers');

    addOperation("load_form_map");
    addOperation("switch_page");

    addOperation("load_form_structure");
    addOperation("load_bindings_file");

    showStatusMessage("Generation in process...");

    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);

    //submit the form
    document.XPath_Bindings_form.submit();
}

/**
 * Trys to populate default xpaths for the from and to page binding fields.
 */
function generateDefaultXPaths()
{
    //add the changes to the current page to the DOM
    saveToDOM();

    //output the XML in memory to a hidden field
    document.XPath_Bindings_form.xml_config_xforms.value = xmlToString(bindingsDoc);

    //populate the hidden fields
    document.XPath_Bindings_form.presentation_xsl.value="xpath_bindings_html.xsl";
    resetOperationsList();
    addOperation("save_xpath_bindings_details");
    checkAppMapSaveRequired();
    addOperation("generate_default_bindings");
    addOperation("load_form_structure");
    addOperation("load_bindings_file");
    addOperation("load_form_map");

    //stop the win manager prompting about losing changes
    WinManager.setGenuineCloseFlag(false);

    //submit the form
    document.XPath_Bindings_form.submit();
}

function openPage(page, pageName, configuration) {
    if (configuration == '' || configuration == null)
    {
        window.open(page, pageName, config='toolbar=yes, menubar=no, directories=no, scrollbars=yes, resizable=yes, status=yes');
    }
    else
    {
        window.open(page, pageName, config=configuration);
    }
}



function editDocument()
{
    var tempObj;
    if (instanceTypeVisible == 'transform')
        tempObj = transformInstance;
    else
        tempObj = xformsInstance;

    xmlhttp = new XMLHttpRequest();

    var requestURL = "/xstoreViewer/xstoreviewer?action=call&operation=edit_document";
    requestURL += "&chosen_xstore_name=" + tempObj.xstore;
    requestURL += "&chosen_collection_name=" + tempObj.collection;
    requestURL += "&chosen_document_name=" + tempObj.document;

    xmlhttp.open("POST", requestURL, true);

    xmlhttp.onreadystatechange = editDocumentStateChange;

    xmlhttp.send();

}

/** Called when a change in state has occurred for the edit docuemnt request. */
function editDocumentStateChange()
{
    if ((xmlhttp != null) && (xmlhttp.readyState == 4))
    {
        var responseDoc = getXMLDocument(xmlhttp.responseText);
        Sarissa.setXpathNamespaces(responseDoc, 'xmlns:xstore="http://www.hyfinity.com/xstore"');

        //check for failure
        if (responseDoc.selectSingleNode("/xstore:xstore/xstore:response/xstore:outcome[. = 'success']") == null)
        {
            var error1 = responseDoc.selectSingleNode("/xstore:error");
            var error2 = responseDoc.selectSingleNode("/xstore:xstore/xstore:response/xstore:message");

            var alertMessage = "An error occurred while trying to edit the document.\n\n";
            if (error1 != null)
                alertMessage += getElementText(error1);
            if (error2 != null)
                alertMessage += getElementText(error2);

            alert(alertMessage);
        }
    }
}

function editDocumentInBrowser()
{
    var tempObj;
    if (instanceTypeVisible == 'transform')
        tempObj = transformInstance;
    else
        tempObj = xformsInstance;

    openXStoreDocument(tempObj.xstore, tempObj.collection, tempObj.document)
}





// Indicates whether or not the instance treeview is currently displayed.
var instanceVisible = true;
var instanceTypeVisible = 'transform';
// Stores the name of the document currently rendered in the instance treeview.
var currentTreeViewDocumentName = "";
// Stores the name of the action associated with the currently rendered document.
var currentTreeViewAction = '';
var currentTreeViewTab = '';


/**
 * Uses dojo to start loading the specified instance document as an AJAX request.
 * This means that it shouldn't lock the interface while the documents are being loaded.
 */
function loadInstance(type)
{
    log.trace('in loadInstance, type is ' + type);

    var tempInstObj;
    var tempDocObj;
    if (type == 'transform')
    {
        tempInstObj = transformInstance;
        tempDocObj = tiDoc;
    }
    else if (type == 'xforms')
    {
        tempInstObj = xformsInstance;
        tempDocObj = xiDoc;
    }
    else if (type == 'xforms-source')
    {
        tempInstObj = xformsSourceInstance;
        tempDocObj = xisDoc;
    }

    tempInstObj.loaded = false;

    if (type != 'xforms-source')
        updateInstanceTreeList(type);


    if (tempInstObj.document != '')
    {
        var xstoreRequest = '<xstore xmlns="http://www.hyfinity.com/xstore"><query>' +
                            '<action>read</action>' +
                            '<xstore_name>' + tempInstObj.xstore + '</xstore_name>' +
                            '<collection_name>' + tempInstObj.collection + '</collection_name>' +
                            '<document_name>' + tempInstObj.document + '</document_name>' +
                            '</query><content/></xstore>';

        var requestMessage = '<xg:attributes xmlns:xg="http://www.hyfinity.com/xgate">' +
                            '<xg:attribute xg:name="operation1">call_xstore</xg:attribute>' +
                            '<xg:xstore_request>'+xstoreRequest+'</xg:xstore_request>' +
                            '</xg:attributes>'

        dojo.rawXhrPost({
            url:        'FormMaker',
            handleAs:   'text',
            postData:   requestMessage,
            headers:    { SOAPAction:    "process"},
            load:       function(response, ioArgs){
                            if (stripXStoreResponse(response, tempDocObj))
                            {
                                tempInstObj.loaded = true;
                                processLoadedInstance(tempDocObj);
                            }
                            return response;
                        },
            error:      function(response, ioArgs){
                            alert('Error loading document.  Please try again.\n\n\nError details: ' + response.message);
                            return response;
                        }
        });
    }
    else
    {
        tempInstObj.loaded = true;
    }
}

/**
 * This is just a simple wrapper to call the loadInstance method above, but determines the
 * type parameter based upon the current action value.
 */
function refreshInstance()
{
    loadInstance(instanceTypeVisible);
}


var currentActionDocComponents =
{
    baseDocument: false,
    dynamicFragmentsChecked: false,
    dynamicFragmentsNeeded: false,
    dynamicXPaths: new Array(),
    xpathNamespaces: ''
};

/**
 * Called when one of the instance documents has been succesfully loaded.
 * this then updates the namespaces,
 * and treelists for the new document as needed.
 */
function processLoadedInstance(doc)
{

    log.trace('in processLoadedInstance');

    hyf.ns.addNamespacesFromDocument(doc);
    showNamespaces();
    Sarissa.setXpathNamespaces(doc, hyf.ns.findNamespaceString());


    //if we are on the action tab, then we need to check for any dynamic data to maintain
    if (currentTab == PAGE_TO_HANDLER_TAB)
    {
        log.debug('is page to handler tab');


        log.debug('Need base document: ' + currentActionDocComponents.baseDocument);
        log.debug('base doc name: ' + xformsInstance.document + ', loaded: ' + xformsInstance.loaded);
        log.debug('Dynamic fragments checked: ' + currentActionDocComponents.dynamicFragmentsChecked);
        log.debug('Dynamic fragments needed: ' + currentActionDocComponents.dynamicFragmentsNeeded);
        log.debug('source doc name: ' + xformsSourceInstance.document + ', loaded: ' + xformsSourceInstance.loaded);

        //check if all the required documents have been loaded
        if (((!currentActionDocComponents.baseDocument) || (xformsInstance.document == '') || (xformsInstance.loaded)) &&
            (currentActionDocComponents.dynamicFragmentsChecked) &&
            ((!currentActionDocComponents.dynamicFragmentsNeeded) || (xformsSourceInstance.document == '') || (xformsSourceInstance.loaded)))
        {
            //all the required information has been loaded, so we can now construct the document to display

            log.debug('all details loaded, so constructing document');

            //firstly, if we are not using a base document structure, then we need to manually create the eform container
            if ((!currentActionDocComponents.baseDocument) || (xformsInstance.document == ''))
            {
                log.debug('base document not being used, so generating eform wrapper');
                //this is related to the specific action, so we will palce the combined document in the xiDoc object
                xiDoc = getXMLDocument('<mvc:eForm xmlns:mvc="http://www.hyfinity.com/mvc"><mvc:Data/></mvc:eForm>');
                Sarissa.setXpathNamespaces(xiDoc, hyf.ns.findNamespaceString());
                xformsInstance.loaded = true;
            }


            //now check if there are any dynamic fragments required
            if ((currentActionDocComponents.dynamicFragmentsNeeded) && (xformsSourceInstance.loaded))
            {
                log.debug('have xpaths to process');
                //log.debug('source doc is: ' + Sarissa.escape(xisDoc.xml));
                //find the container element to copy the fragments to
                var targetContainer = xiDoc.selectSingleNode("/*[local-name() = 'eForm']/*[local-name() = 'Data']");

                if (targetContainer != null)
                {
                    Sarissa.setXpathNamespaces(xisDoc, currentActionDocComponents.xpathNamespaces);
                    for (var i = 0; i < currentActionDocComponents.dynamicXPaths.length; ++i)
                    {
                        log.debug('processing ' + currentActionDocComponents.dynamicXPaths[i]);
                        var match = xisDoc.selectSingleNode(currentActionDocComponents.dynamicXPaths[i]);
                        if (match != null)
                        {
                            log.debug('found match');
                            //MSXML6 has the importNode function, but MSXML3 doesnt understand this, and doesnt seem to need it.
                            var clone = match.cloneNode(true);
                            var imported = clone;
                            if (typeof(xiDoc.importNode) == 'function')
                                imported = xiDoc.importNode(clone, true);

                            targetContainer.appendChild(imported);
                        }
                    }
                }
            }
        }
        else
        {
            //not all the information has been loaded yet so cant do anything
            log.debug('not all information loaded, so returning');
            return;
        }
    }


    if (instanceTypeVisible == 'transform')
    {
        updateInstanceTreeList('transform');
    }
    else
    {
        updateInstanceTreeList('xforms');
    }

    /** Why are we calling this here?? Is this to force recheking all the displayed XPaths?? */
    showRelevantXpaths();
}

function findCurrentActionInstanceXPaths()
{
    log.trace('in findCurrentActionInstanceXPaths');

    dojo.xhrPost({
        url:        "FormMaker",
        handleAs:   "text",
        preventCache:"true",
        content:    {   action:                     'process',
                        operation1:                 'load_bindings_file',
                        form_id:                    getElementText(applicationDoc.selectSingleNode("/fm:application/fm:pages/fm:page[@id = /fm:application/fm:links/fm:link[(fm:source/@type = 'Page') and (fm:action = '"+currentAction+"')]/fm:source]/fm:form_ref[1]")),
                        binding_type:               document.XPath_Bindings_form.binding_type.value,
                        xml_project_info:           document.XPath_Bindings_form.xml_project_info.value},
        error:  function(response, ioArgs) {
                    alert('An error occurred while trying to load the information needed to show the appropriate document.\n\nPlease try your request again, and if the problem persists, please contact Hyfinity.\n\n\nError details: ' + response.message);

                    return response;
                },
        load:   function(response, ioArgs) {

                    //window.clipboardData.setData("Text",response);

                    //log.debug('loaded source page binding doc: ' + Sarissa.escape(response));
                    log.debug('loaded source page binding doc');

                    var tempDoc = getXMLDocument(response);

                    Sarissa.setXpathNamespaces(tempDoc, FMqualifiedNS);

                    var xpathNodes = tempDoc.selectNodes("/fm:xforms/fm:instance_configuration/fm:action[@name = \'" + currentAction + "\']/fm:instance_xpath")

                    if (xpathNodes.length > 0)
                    {
                        log.debug('source page defines dynamic fragments for this action');
                        currentActionDocComponents.dynamicFragmentsNeeded = true;
                        currentActionDocComponents.dynamicXPaths = new Array();
                        for (var i = 0; i < xpathNodes.length; ++i)
                        {
                            currentActionDocComponents.dynamicXPaths[currentActionDocComponents.dynamicXPaths.length] = getElementText(xpathNodes.item(i));
                        }

                        //need to also find the namespace defintions from this bindings file so that we can
                        //evaluate the instance xpath just retrieved
                        var nsString = '';

                        var atts = tempDoc.documentElement.attributes;
                        for (var i = 0; i < atts.length; ++i)
                        {
                            var att = atts.item(i);
                            //check if this attribute is actually a namespace definition
                            if (att.nodeName.indexOf('xmlns') == 0)
                            {
                                var prefix = '';
                                if (att.nodeName.indexOf(':') != -1)
                                    prefix = att.nodeName.substring(att.nodeName.indexOf(':') + 1);

                                //check if the prefix is blank
                                if ((prefix != 'xmlns')  && (prefix != ''))
                                {
                                    nsString += 'xmlns:' + prefix + '="' + att.nodeValue + '" ';
                                }
                            }
                        }

                        currentActionDocComponents.xpathNamespaces = nsString;
                    }

                    currentActionDocComponents.dynamicFragmentsChecked = true;

                    //if we have found xpaths indicating some dyanmic data is needed, then we now need to load up
                    //the source file for the page this action comes from
                    if (currentActionDocComponents.dynamicFragmentsNeeded)
                    {
                        log.debug('dynamic fragments are needed, so loading this page\'s binding document');
                        //load up the to page document for the source page
                        xformsSourceInstance = {
                            xstore: getElementText(applicationDoc.selectSingleNode("/fm:application/fm:forms/fm:form[@id = /fm:application/fm:pages/fm:page[@id = /fm:application/fm:links/fm:link[fm:source/@type = \'Page\' and fm:action = \'" +currentAction + "\']/fm:source]/fm:form_ref[1]]/fm:data_to_form/@xstore")),
                            collection: getElementText(applicationDoc.selectSingleNode("/fm:application/fm:forms/fm:form[@id = /fm:application/fm:pages/fm:page[@id = /fm:application/fm:links/fm:link[fm:source/@type = \'Page\' and fm:action = \'" +currentAction + "\']/fm:source]/fm:form_ref[1]]/fm:data_to_form/@collection")),
                            document: getElementText(applicationDoc.selectSingleNode("/fm:application/fm:forms/fm:form[@id = /fm:application/fm:pages/fm:page[@id = /fm:application/fm:links/fm:link[fm:source/@type = \'Page\' and fm:action = \'" +currentAction + "\']/fm:source]/fm:form_ref[1]]/fm:data_to_form")),
                            loaded: false
                        };

                        xisDoc = getXMLDocument('<empty/>');

                        if (xformsSourceInstance.document != '')
                        {
                            loadInstance('xforms-source');
                        }
                        else
                        {
                            document.getElementById('instance_missing_message').innerHTML = "This action is set to maintain some additional information, but no example Data Binding file has been defined for the source page!<br/>In order to gain help with entering these binding XPaths and to check for matches, please define or generate an example file for the appropriate page on the Application Map screen."
                            showComponent(document.getElementById('instance_missing_message'));
                        }
                    }
                    else
                    {
                        //still need to call the processLoadedInstance method even though we are not actually loading any
                        //new documents, to tell it that we have finished woring out wether any dynamic fragments are needed
                        processLoadedInstance(xiDoc);
                    }

                    return response;
                }
    });

}

/**
 * Renders a tree list to display the specified instance document.
 */
function updateInstanceTreeList(instanceToShow)
{
    instanceXSLProc.clearParameters();

    //ensure all namespaces are placed on the bindings document
    findNamespaces();

    //instanceXSLProc.setParameter(null, 'namespace_document', bindingsDoc);
    //alert('setting input document');
    if (instanceToShow == 'transform')
    {
        var xmlToTransform = '<container><namespaces>'+hyf.ns.getNamespaceDocument()+'</namespaces>'
                            + '<document>' + xmlToString(tiDoc) + '</document></container>';
        var docToTransform = getXMLDocument(xmlToTransform);

        document.getElementById('treeListSelectorDIV').innerHTML = transformInstance.document;
        currentTreeViewDocumentName = transformInstance.document;

        instanceXSLProc.setParameter(null, "output_prefix", "input_document");
        instanceXSLProc.setParameter(null, "instance_loaded", transformInstance.loaded);

        //transform to create the content
        Sarissa.updateContentFromNode(docToTransform, document.getElementById('instanceTreeList'), instanceXSLProc);

    }
    else if (instanceToShow == 'xforms')
    {
        if (currentTab == PAGE_TO_HANDLER_TAB)
            document.getElementById('treeListSelectorDIV').innerHTML = "Action Binding Fragment";
        else
            document.getElementById('treeListSelectorDIV').innerHTML = xformsInstance.document;

        currentTreeViewDocumentName = xformsInstance.document;

        instanceXSLProc.setParameter(null, "output_prefix", "output_document");
        instanceXSLProc.setParameter(null, "instance_loaded", xformsInstance.loaded);

        var xmlToTransform = '<container><namespaces>'+hyf.ns.getNamespaceDocument()+'</namespaces>'
                            + '<document>' + xmlToString(xiDoc) + '</document></container>';
        var docToTransform = getXMLDocument(xmlToTransform);

        //transform to create the content
        Sarissa.updateContentFromNode(docToTransform, document.getElementById('instanceTreeList'), instanceXSLProc);
    }

    instanceTypeVisible = instanceToShow;

    //ensure the rendered instance is visible
    if (!instanceVisible)
        toggleInstanceVisibility();


    document.getElementById('TreeList2').scrollLeft = 0;
    document.getElementById('TreeList2').scrollTop = 0;
}

/** Toggles the visibility of the instance treeview.
 * Although we have two different types of instance, based on the current action,
 * only one can be shown, so this just toggles the display on or off, regardless of which is currently visible.
 * This does not check which document should be displayed - this is done by updateDisplayedInstanceDocument()
 */
function toggleInstanceVisibility()
{
    //if something is currently visible, hide it.
    if (instanceVisible)
    {
        instanceVisible = false;
        hideComponent(document.getElementById('instance_td'));
        hideComponent(document.getElementById('divider2_td'));

        /*xpathBindings__instances__show_button.innerHTML = "Show " + currentTreeViewDocumentName;*/
    }
    else
    {
        //otherwise show the currently rendered document.
        instanceVisible = true;
        showComponent(document.getElementById('instance_td'));
        showComponent(document.getElementById('divider2_td'));

        /*xpathBindings__instances__show_button.innerHTML = "Hide " + currentTreeViewDocumentName;*/
    }
}


/**
 * Works out which instance document should be displayed based on the current tab/action,
 * and then calls the relevant methods to make sure it is loaded up and displayed correctly.
 */
function updateDisplayedInstanceDocument()
{
    log.trace('in updateDisplayedInstanceDocument');

    //first check if the document we now want to display is the same as the current one.
    //if so we dont have to do anything!
    if ((currentTreeViewAction != currentAction) ||
        (currentTab != currentTreeViewTab) ||
        (currentTab == ACTION_CONFIG_TAB))
    {
        hideComponent(document.getElementById('instance_missing_message'));
        hideComponent(document.getElementById('action_instance_missing_message'));

        //if we are on the main handler to page tab, always show the transform instance document
        if (currentTab == HANDLER_TO_PAGE_TAB)
        {
            renderToPageDocument();
        }
        else if (currentTab == ACTION_CONFIG_TAB)
        {
            //for the action config tab, we check the binding type value to determine which document to display
            /*if (getRadioButtonValue(document.XPath_Bindings_form.xpathBindings__instances__instance_binding_option) == 'useInstanceBinding')
            {*/
                renderToPageDocument();
            /*}
            else
            {
                renderCurrentActionDocument();
            }*/
        }
        else if (currentTab == PAGE_TO_HANDLER_TAB)
        {
            log.debug('current tab is page to handler (action submission)');

            //for the action bindings, we need to look at the binding type to work out what instance to base on
            //we then use the instance xpath to retrieve the fragment to use, and then render this.
            var instType = 'none';
            var appMapInstType = applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[fm:source/@type = \'Page\' and fm:action = \'" +currentAction + "\']/fm:instance_type");
            if (appMapInstType != null)
                instType = getElementText(appMapInstType);

            if (instType == 'none')
            {
                log.debug('binding not required, so no document to display');
                //ensure the RHS is hidden
                instanceVisible = true;
                toggleInstanceVisibility();
            }
            else
            {
                //reset the flags detailing how this document will be built up
                currentActionDocComponents.baseDocument = false;
                currentActionDocComponents.dynamicFragmentsChecked = false;
                currentActionDocComponents.dynamicFragmentsNeeded = false;

                //make sure we are viewing the action structure rather than the to page one
                instanceTypeVisible = 'xforms';

                if (instType == 'static')
                {
                    log.debug("A base document structrue is being used");
                    //we are using the action document as the base structrue, so load up
                    currentActionDocComponents.baseDocument = true;
                    renderCurrentActionDocument();
                }

                //now check if we need to take any additional dynamic data form the to page document.
                //the current action might not actually originate from this page, so the xpaths could be defined in a
                //completely different bindings file.  Therefore we need to check for this case, and if so temporarily load up the file

                if (getElementText(applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[(fm:source/@type = 'Page') and (fm:action = '"+currentAction+"')]/fm:source")) == document.XPath_Bindings_form.gen_control.value)
                {
                    log.debug("action does originate from the current page");
                    //from this page so dont need to load up a different bindings file
                    var dynamicXPaths = bindingsDoc.selectNodes("/fm:xforms/fm:instance_configuration/fm:action[@name = \'" + currentAction + "\']/fm:instance_xpath");

                    if (dynamicXPaths.length > 0)
                    {
                        currentActionDocComponents.dynamicFragmentsNeeded = true;
                        currentActionDocComponents.dynamicXPaths = new Array();
                        for (var i = 0; i < dynamicXPaths.length; ++i)
                        {
                            currentActionDocComponents.dynamicXPaths[currentActionDocComponents.dynamicXPaths.length] = getElementText(dynamicXPaths.item(i));
                        }

                        currentActionDocComponents.xpathNamespaces = hyf.ns.findNamespaceString();
                    }

                    currentActionDocComponents.dynamicFragmentsChecked = true;

                    if (currentActionDocComponents.dynamicFragmentsNeeded)
                    {
                        log.debug('dynamic fragments are needed');
                        //we need to add some details fromt he page document for this action to create the action binding fragment
                        //as this action comes formt he page curerntly being editted, we shoukld already have this file loaded up
                        //so just reference it here.
                        xformsSourceInstance = transformInstance;
                        xisDoc = tiDoc;

                        if ((xformsSourceInstance.document != '') && !xformsSourceInstance.loaded)
                        {
                            loadInstance('xforms-source');
                        }

                        if (xformsSourceInstance.document == '')
                        {
                            document.getElementById('instance_missing_message').innerHTML = "This action is set to maintain some additional information, but no example Data Binding file has been defined for the source page!<br/>In order to gain help with entering these binding XPaths and to check for matches, please define or generate an example file for the appropriate page on the Application Map screen."
                            showComponent(document.getElementById('instance_missing_message'));
                        }
                    }

                    //still need to call the processLoadedInstance method even though we are not actually loading
                    //any new documents, so that the correct document structrue gets created and displayed.
                    processLoadedInstance(xisDoc)
                }
                else
                {
                    //from a different page so need to first load up the bindings file to check if any dynamic xpaths are specified.
                    log.debug("action originates from a different page");
                    findCurrentActionInstanceXPaths();
                }
            }
        }

        currentTreeViewAction = currentAction;
        currentTreeViewTab = currentTab;
    }

}


function renderToPageDocument()
{
    log.trace('in renderToPageDocument');

    //check if we have a document defined to display
    if (transformInstance.document != '')
    {
        if (!transformInstance.loaded)
            loadInstance('transform');
        else
            updateInstanceTreeList('transform');
    }
    else
    {
        //ensure the RHS is hidden
        instanceVisible = true;
        toggleInstanceVisibility();
        if (currentTab == ACTION_CONFIG_TAB)
            var mc = document.getElementById('action_instance_missing_message');
        else
            var mc = document.getElementById('instance_missing_message');

        mc.innerHTML = "An Example Data Binding file has not been defined for this page!<br/>In order to gain help with entering these binding XPaths and to check for matches, please define or generate a file from the Application Map screen."
        showComponent(mc);
    }
}


function renderCurrentActionDocument()
{
    log.trace('in renderCurrentActionDocument');

    if (currentAction == '')
    {
        //ensure the RHS is hidden
        instanceVisible = true;
        toggleInstanceVisibility();
    }
    else
    {
        //We need to set the data_from_form information for each action. xformsInstance var is set in the page xsl
        xformsInstance = {
            xstore: getElementText(applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[fm:source/@type = \'Page\' and fm:action = \'" +currentAction + "\']/fm:data_from_form/@xstore")),
            collection: getElementText(applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[fm:source/@type = \'Page\' and fm:action = \'" +currentAction + "\']/fm:data_from_form/@collection")),
            document: getElementText(applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[fm:source/@type = \'Page\' and fm:action = \'" +currentAction + "\']/fm:data_from_form")),
            loaded: false
        };

        if (xformsInstance.document != '')
        {
            loadInstance('xforms');
        }
        else
        {
            //ensure the RHS is hidden
            instanceVisible = true;
            toggleInstanceVisibility();

            if (currentTab == ACTION_CONFIG_TAB)
                var mc = document.getElementById('action_instance_missing_message');
            else
                var mc = document.getElementById('instance_missing_message');

            mc.innerHTML = "The Static Binding Structure file has not been defined for this action!<br/>In order to gain help with entering these binding XPaths and to check for matches, please define or generate a file from the Application Map screen."
            showComponent(mc);
        }
    }
}





/**
 * Toggles the instance binding info if the checkbox is ticked to indicate binding is required
 * @param checked Indicates whether binding is required
 * @param instance_type (Optional) If binding is required, then this can be used to set what instance type
 *                      should be defaulted to.
 */
function toggleInstanceBindingInfo(checked, instance_type)
{
    //Make sure bindings are required before checking for bindings instance
    if (!checked)
    {
        hideComponent(document.getElementById("xpathBindings__instances__instance_bindingMethod_container"));
        hideComponent(document.getElementById("form_details_action_base_container"));
        //document.getElementById("xpathBindings__instances__instance_bindingMethod_container").innerHTML = '';
    }
    else
    {
        showComponent(document.getElementById("xpathBindings__instances__instance_bindingMethod_container"));
        showComponent(document.getElementById("form_details_action_base_container"));
        //We are only toggling the binding type, which means we have to cater for toggle from 'none' to something else
        //Choose static by default when binding is required for previously 'none' setting
        if (typeof(instance_type) == 'undefined' || instance_type == 'static')
            document.getElementById('xpathBindings__instances__use_base_structure').checked = true;
        else
            document.getElementById('xpathBindings__instances__use_base_structure').checked = false;
    }
}

function toggleInstanceBindingInfoWrapper(checked)
{
    toggleInstanceBindingInfo(checked);
    if (checked)
    {
        saveInstanceBindingInfo();
        updateDisplayedInstanceDocument();
    }
}




/**
 * Creates an HTML formatted string to show the textareas for entering xpaths for maintaining dynamic data.
 */
function showKeepDataXPaths()
{
    var textareas  = new Array();

    var existingEntries = bindingsDoc.selectNodes("/fm:xforms/fm:instance_configuration/fm:action[@name = \'" + currentAction + "\']/fm:instance_xpath");

    var actionHTML = "";
    if (existingEntries.length > 0)
    {
        actionHTML += '<table border="0" cellspacing="0" cellpadding="0" class="table" id="xpathBindings__instances__keep_xpaths">';
        for (var i = 0; i < existingEntries.length; ++i)
        {
            var entry = existingEntries.item(i);

            actionHTML += '<tr><th class="tableColumnHeading">Additional data to maintain:</th><td>';

            actionHTML += createXPathTextAreaString('xpathBindings__instances__keep_' + i, getElementText(entry), "setKeepXPathValue('"+i+"', this.value);");
            actionHTML += '</td>'

            actionHTML += '<td style="width : 15px;" valign="middle"><a style="height : auto;" href="javascript:removeKeepXPath(\''+i+'\')" class="remove_button"><span>Remove</xpan></a></td>';

            actionHTML += '</tr>';
            textareas[textareas.length] = 'xpathBindings__instances__keep_' + i;
        }
        actionHTML += '</table>';
        //alert(actionHTML);
    }
    document.getElementById('xpathBindings__instances__keep_xpaths_container').innerHTML = actionHTML;


    //validate the xpath details for each event textarea shown.
    for (var ta = 0; ta < textareas.length; ++ta)
    {
        setTimeout("validateXPathImpl('"+textareas[ta]+"', false, false)", 50);
    }
}

/**
 * Removes the keep xpath with the given id for the current action, and
 * updates the display.  The id should be the zero based index of the xpath to remove.
 */
function removeKeepXPath(id)
{
    var xpathNum = Number(id) + 1;

    var entry = bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = \'" + currentAction + "\']/fm:instance_xpath["+xpathNum+"]");

    if (entry != null)
    {
        entry.parentNode.removeChild(entry);
        setChanged();
    }

    showKeepDataXPaths();
}

/**
 * Adds a keep xpath for the current action, and updates the display.
 */
function addKeepXPath()
{
    var container = bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = \'" + currentAction + "\']");

    if (container == null)
    {
        var icElem = bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration");
        //assume the instance_configuration element will always be present!
        container = createNamespaceElement(bindingsDoc, 'action', FMdefaultNS);
        container.setAttribute('name', currentAction);
        icElem.appendChild(container);
    }

    if (container != null)
    {
        var newEntry = createNamespaceElement(bindingsDoc, 'instance_xpath', FMdefaultNS);
        setElementText(newEntry, blank_xpath_text);
        container.appendChild(newEntry);
        setChanged();
    }

    showKeepDataXPaths();
}

/**
 * Sets the value stored in the DOm for the given keep xpath to the provided value.
 * The id should be the zero based index of the xpath to remove.
 */
function setKeepXPathValue(id, value)
{
    var xpathNum = Number(id) + 1;

    var entry = bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = \'" + currentAction + "\']/fm:instance_xpath["+xpathNum+"]");

    if (entry != null)
    {
        setElementText(entry, value);
        setChanged();
    }
}





/**
 * Saves the binding info for a particular action. This updates the client-side DOMs only for the app and bindings info
 * This needs to capture the settings for the previously selected action to enable saving of dependent binding info
 * @param newAction selected action. This method is triggered when the actions select is changed
 */
function saveInstanceBindingInfo()
{
    //Using the current action, save the settings on binding required checkbox, the binding instance_type and also the instance xpath
    if ((currentTab == ACTION_CONFIG_TAB) &&  //only do this for a page to handler action
        (currentAction != ''))
    {
        //find the appropariate action element in thebindings file
        var bActionElem = bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = \'" + currentAction + "\']");
        if (bActionElem == null)
        {
            var icElem = bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration");
            //assume the instance_configuration element will always be present!
            bActionElem = createNamespaceElement(bindingsDoc, 'action', FMdefaultNS);
            bActionElem.setAttribute('name', currentAction);
            icElem.appendChild(bActionElem);
        }

        //check that the action is actually defined from this page
        if (applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[(fm:source/@type = 'Page') and (fm:source = '"+document.XPath_Bindings_form.gen_control.value+"') and (fm:action = '"+currentAction+"')]") != null)
        {
            var bindingRequired = document.getElementById("xpathBindings__instances__instance_bindingRequired").checked;
            var instance_type = 'none';
            if (bindingRequired)
            {
                //Try and access the instance_type
                if (document.getElementById('xpathBindings__instances__use_base_structure').checked)
                {
                    instance_type = 'static';
                }
                else
                {
                    instance_type = 'dynamic';
                }
            }
            //Save server-side binding required flag instance_type and xpath info to the DOMs
            var instanceTypeElem = applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[fm:action = \'" + currentAction + "\']/fm:instance_type");
            if (instanceTypeElem == null)
            {
                var actionElem = applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[fm:action = \'" + currentAction + "\']");
                instanceTypeElem = createNamespaceElement(applicationDoc, 'instance_type', FMdefaultNS);
                actionElem.appendChild(instanceTypeElem);
            }
            if (getElementText(instanceTypeElem) != instance_type)
            {
                setElementText(instanceTypeElem, instance_type);
                applicationMapChanged = true;
            }

            //save keep form data flag
            if(document.getElementById('xpathBindings__instances__keep_form_data').checked)
                bActionElem.setAttribute('maintain_form_data', 'true');
            else
                bActionElem.setAttribute('maintain_form_data', 'false');

            //NOTE: maintain other data xpaths will have already been saved as they were updated.
        }

        var currentBase = bActionElem.getAttribute('base');

        //always save the 'base on' action value
        if (document.getElementById('xpathBindings__actions__binding_base_flag').checked)
        {
            var newBase = document.getElementById('xpathBindings__actions__binding_base').value;
            if ((newBase == currentAction) || (bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = '" + newBase + "']/@base") != null))
            {
                alert('You have selected an invliad base action.\n\nIt is not possible to base this action upon itself, or another action that is already itself set by a different action.');
                return false;
            }

            bActionElem.setAttribute('base', newBase);
        }
        else if (bActionElem.getAttribute('base') != null)
        {
            bActionElem.removeAttribute('base');
        }

        if (currentBase != bActionElem.getAttribute('base'))
        {
            //update all the locations showing the action name string
            var fullTitle = findActionNameString(currentAction, 'full');
            var normalTitle = findActionNameString(currentAction, 'normal');
            var minTitle = findActionNameString(currentAction, 'min');

            //update accordion title
            var ap = dijit.byId('ap_' + currentAction);
            if (ap != null)
            {
                ap.title = fullTitle;
                document.getElementById('ap_' + currentAction + '_button_title').innerHTML = fullTitle;
            }

            //update action selection treeview
            document.getElementById("action_treeview_entry_" + currentAction).innerHTML = normalTitle;

            //update base action selection dropdown.
            document.getElementById("base_action_select_option_" + currentAction).innerHTML = minTitle;

            //check if we should delete all this action's xpaths
            if (bActionElem.getAttribute('base') != null)
            {
                deleteActionBindings(currentAction);
            }
        }
    }
    return true;
}

/**
 * Returns a string that can be used for displaying the specified action.
 * Any changes to this method should also be reflected in the 'outputActionNameString'
 * template in the xpath_bindings_html.xsl file
 */
function findActionNameString(action, mode)
{
    var actionName = action;
    var actionLink = applicationDoc.selectSingleNode("/fm:application/fm:links/fm:link[fm:action = '"+action+"']");
    if (actionLink != null)
    {
        if (mode == 'full' && getElementText(actionLink.selectSingleNode("fm:source")) != document.XPath_Bindings_form.gen_control.value)
        {
            actionName += ' (from ';
            actionName += getElementText(actionLink.selectSingleNode("fm:source"));
            actionName += ' page to ';
        }
        else if (mode == 'normal' || mode == 'full')
        {
            actionName += ' (to ';
        }

        if (mode == 'normal' || mode == 'full')
        {
            var target = actionLink.selectSingleNode("fm:target");
            actionName += getElementText(target) + " ";
            if (target.getAttribute('type') == 'Handler')
                actionName += 'Controller';
            else
                actionName += target.getAttribute('type');

            actionName += ')';
        }

        if (bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = '" + currentAction + "']/@base") != null)
        {
            actionName += ' - set by ';
            actionName += getElementText(bindingsDoc.selectSingleNode("/fm:xforms/fm:instance_configuration/fm:action[@name = '" + currentAction + "']/@base"));
        }
    }
    return actionName;
}

/**
 * Sets all the action bindings for the target action to those of the source action.
 */
function copyActionBindings(targetAction, sourceAction)
{
    var nodes = bindingsDoc.selectNodes("/fm:xforms/*[local-name() = 'element' or local-name() = 'group' or local-name() = 'repeat']");
    for (var i = 0; i < nodes.length; ++i)
    {
        var entry = nodes.item(i);

        var existing = entry.selectSingleNode("fm:action[@name = '"+targetAction+"']");
        if (existing != null)
            entry.removeChild(existing);

        var source = entry.selectSingleNode("fm:action[@name = '"+sourceAction+"']");
        if (source != null)
        {
            var copy = source.cloneNode(true);
            copy.setAttribute('name', targetAction);
            entry.appendChild(copy);
        }
    }
}

/**
 * Deletes all the action bindings for the given action
 */
function deleteActionBindings(targetAction)
{
    var nodes = bindingsDoc.selectNodes("/fm:xforms/*[local-name() = 'element' or local-name() = 'group' or local-name() = 'repeat']");
    for (var i = 0; i < nodes.length; ++i)
    {
        var entry = nodes.item(i);

        var existing = entry.selectSingleNode("fm:action[@name = '"+targetAction+"']");
        if (existing != null)
            entry.removeChild(existing);
    }
}


/**
 * This function is called when the user selects one of the static or dynamic binding raio buttons.
 * This is responsible for showing the correct textual labels, and ensuring the correct docuemnt is
 * avaialble to drag xpaths from/check against.
 * @param newValue the value of the clicked radio button.
 */
/*function actionInstanceBindingTypeChanged(newValue, dontUpdateDoc)
{
    if (newValue == 'useDocumentBinding')  //static binding
    {
        hideComponent(document.getElementById('xpathBindings__instances__dynamic_binding_xpath_label'));
        hideComponent(document.getElementById('xpathBindings__instances__static_binding_xpath_label'));
        hideComponent(document.getElementById('xpathBindings__instances__instance_xpath_container'));
    }
    else //dynamic binding
    {
        showComponent(document.getElementById('xpathBindings__instances__dynamic_binding_xpath_label'));
        hideComponent(document.getElementById('xpathBindings__instances__static_binding_xpath_label'));

        showComponent(document.getElementById('xpathBindings__instances__instance_xpath_container'));
    }
    if ((typeof(dontUpdateDoc) == 'undefined') || !dontUpdateDoc)
    {
        //save the current status, before we update the intance doc, as this can cause the displayed screen
        //to be updated based on the data in the XML
        saveInstanceBindingInfo();

        updateDisplayedInstanceDocument();
        if (newValue != 'useDocumentBinding')
            setTimeout("validateXPathImpl('xpathBindings__instances__instance_xpath', true, true)", 50);
    }
}*/

/** Called when a new action is selected from the Action Configuration tab's
 * action treeview list.
 */
function showNewActionConfig(action)
{
    if (!saveToDOM())
        return;

    currentAction = action;

    showNewElement(document.XPath_Bindings_form.currentType.value, document.XPath_Bindings_form.currentElement.value, true);

    //actionInstanceBindingTypeChanged(getRadioButtonValue(document.XPath_Bindings_form.xpathBindings__instances__instance_binding_option), true);

    markActionOnTreeview(currentAction);
}

/**
 * Called when one of the tabs is clicked, to change between showing the handler to page bindings,
 * or the bindings for one of the actions.
 */
function changeBindingType(newType, dontSave)
{
    if (newType == currentTab)
        return;

    if (newType == ACTION_CONFIG_TAB)
    {
        if ((typeof(dontSave) == 'undefined') || (dontSave == false))
            if (!saveToDOM())
                return;

        currentTab = ACTION_CONFIG_TAB;

        showNewElement(document.XPath_Bindings_form.currentType.value, document.XPath_Bindings_form.currentElement.value, true);

        //actionInstanceBindingTypeChanged(getRadioButtonValue(document.XPath_Bindings_form.xpathBindings__instances__instance_binding_option), true);

        markActionOnTreeview(currentAction);

        updateTabHighlighting(currentTab);
    }
    else if (newType == HANDLER_TO_PAGE_TAB)
    {
        if ((typeof(dontSave) == 'undefined') || (dontSave == false))
            if (!saveToDOM())
                return;

        currentTab = HANDLER_TO_PAGE_TAB;

        showNewElement(document.XPath_Bindings_form.currentType.value, document.XPath_Bindings_form.currentElement.value, true);
        document.getElementById('PageDisplayBindingsTabContents').appendChild(document.getElementById('bindingsContent'));
        updateTabHighlighting(currentTab);
    }
    else //page to handler
    {
        var paneToSelect = dijit.byId('ap_'+currentAction);

        if ((paneToSelect == null) || (paneToSelect.selected))
        {
            if ((typeof(dontSave) == 'undefined') || (dontSave == false))
                if (!saveToDOM())
                    return;

            currentTab = PAGE_TO_HANDLER_TAB;
            updateTabHighlighting(currentTab);
            adjustAccordionHeight();

            if (paneToSelect != null)
            {
                showNewElement(document.XPath_Bindings_form.currentType.value, document.XPath_Bindings_form.currentElement.value, true);
                paneToSelect.setContent(dojo.byId('bindingsContent'));
            }
            else
            {
                updateDisplayedInstanceDocument();
            }
        }
        else
        {
            //about to select pane
            if (dontSave == true)
                saveOnAPChange = false;
            dijit.byId('actionSubmissionAccordion').selectChild(paneToSelect);
            //selecting the pane will trigger the 'accordionActionChanged' method below, which will
            //ensure the correct page layout and display
        }

        if (dijit.byId('actionSubmissionAccordion'))
            dijit.byId('actionSubmissionAccordion').resize();
    }

}


function updateTabHighlighting(tab)
{
    if (tab == ACTION_CONFIG_TAB)
    {
        if(document.getElementById('pageConfigTab') != null)
            document.getElementById('pageConfigTab').className = '';
        document.getElementById('actionConfigTab').className = 'current';

        document.getElementById('handlerToPageTab').className = '';
        document.getElementById('pageToHandlerTab').className = '';

        showComponent(document.getElementById('actionConfigTabContents'));
        hideComponent(document.getElementById('pageTabContents'));

        showComponent(document.getElementById('ActionTreeList'));
        hideComponent(document.getElementById('PageTreeList'));
    }
    else
    {
        document.getElementById('pageConfigTab').className = 'current';
        document.getElementById('actionConfigTab').className = '';

        hideComponent(document.getElementById('actionConfigTabContents'));
        showComponent(document.getElementById('pageTabContents'));

        hideComponent(document.getElementById('ActionTreeList'));
        showComponent(document.getElementById('PageTreeList'));

        if (tab == HANDLER_TO_PAGE_TAB)
        {
            document.getElementById('handlerToPageTab').className = 'current';
            document.getElementById('pageToHandlerTab').className = '';

            hideComponent(document.getElementById('ActionSubmissionBindingsTabContents'));
            showComponent(document.getElementById('PageDisplayBindingsTabContents'));
        }
        else
        {
            document.getElementById('handlerToPageTab').className = '';
            document.getElementById('pageToHandlerTab').className = 'current';

            showComponent(document.getElementById('ActionSubmissionBindingsTabContents'));
            hideComponent(document.getElementById('PageDisplayBindingsTabContents'));
        }
    }
}





dojo.require("dojo.parser");
dojo.require("dijit.layout.AccordionContainer");

var ignoreAPChange = false;
var saveOnAPChange = true;

/**
 * Called when one of the action accordion panes is selected.
 */
function accordionActionChanged()
{
    if (!ignoreAPChange)
    {
        if(saveOnAPChange)
        {
            if (!saveToDOM())
            {
                ignoreAPChange = true;
                setTimeout("var paneToSelect = dijit.byId('ap_'+currentAction);dijit.byId('actionSubmissionAccordion').selectChild(paneToSelect);ignoreAPChange = false;", 300);
                return;
            }
        }
        saveOnAPChange = true;

        dijit.byId('ap_'+currentAction).selected = false;

        currentAction = this.id.substr(3);
        currentTab = PAGE_TO_HANDLER_TAB;
        updateTabHighlighting(currentTab);

        showNewElement(document.XPath_Bindings_form.currentType.value, document.XPath_Bindings_form.currentElement.value, true);

        this.setContent(dojo.byId('bindingsContent'));
    }
}


function toggleActionBaseInfo(visible)
{
    if (visible)
    {
        showComponent(document.getElementById('xpathBindings__actions__binding_base_row'));
    }
    else
    {
        hideComponent(document.getElementById('xpathBindings__actions__binding_base_row'));
    }
}



/*
 * Extracts the currently selected value from a set of radio buttons
 */
function getRadioButtonValue(field)
{
    for(var i = 0; i < field.length; ++i)
    {
        if(field[i].checked == true)
        {
            return field[i].value;
        }
    }
}

/**
 * Sets the size of the accordion as needed so that
 * the currently visible entry will have a height of 300 px.
 */
function adjustAccordionHeight()
{
    var currentPane = dijit.byId('ap_'+currentAction);
    if (currentPane != null)
    {
        var height = getCurrentStyle(currentPane.containerNode,'height');
        //height = height.substring(0, height.length - 2);
        height = parseInt(height);

        var diff = height - 300;

        var totalHeight = getCurrentStyle(dijit.byId('actionSubmissionAccordion').domNode,'height');
        //totalHeight = totalHeight.substring(0, totalHeight.length - 2);
        totalHeight = parseInt(totalHeight);

        var newSize = new Object();
        newSize.h = totalHeight - diff;

        /* Ensure a min overall size of 400px
         * Is this needed??????
         */
        /*if (newSize.h < 400)
            newSize.h = 400;*/

        dijit.byId('actionSubmissionAccordion').resize(newSize);
    }
}


/**
 * Functionality to detect and manage whether any changes have been made on the screen
 *
 */

var bindingScreenChanges = false;

function setChanged()
{
    if (bindingScreenChanges == false)
    {
        showStatusMessage("Unsaved Changes");
        if(document.title.charAt(0)!="*")
        {
            document.title = "* " + document.title + " (modified)";
        }
        //notify win manger of changes
        WinManager.setUnsaved(window);
    }

    bindingScreenChanges = true;
}

function clearChanged()
{
    bindingScreenChanges = false;

    hideStatusMessage();
    if(document.title.charAt(0) == "*")
    {
        document.title = document.title.substring(2, (document.title.length - 11));
    }

    //clear winmanger changed setting
    WinManager.clearUnsaved(window);
}
